﻿' Title    : sdxBaseEngine Framework
' Author   : Sascha Bajus
' Created  : 09/2009
' Version  : 1.0.0.4
' Copyright © 2009 Sascha Bajus

#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region


Imports SlimDX
Imports SlimDX.Direct3D9
Imports sdxBaseEngine
Imports System.ComponentModel


''' <summary>
''' BaseEngine stellt ein Grundgerüst zum Rendern von 3D-Objekten zur Verfügung.
''' </summary>
''' <remarks></remarks>
Public Class SDX9BaseEngine : Inherits Form
    '#Events for Keyboard and Render
    ''' <summary>
    ''' Wird vor dem Beenden des Rendervorgangs ausgelöst um eine Auswertung
    ''' der Tastatureingaben zu erstellen.
    ''' </summary>
    Public Shared Event dxKeyboardState()

    ''' <summary>
    ''' Wird vor dem Beenden des Rendervorgangs ausgelöst um eine Auswertung
    ''' der Mauseingaben und Position zu erhalten
    ''' </summary>
    ''' <remarks></remarks>
    Public Shared Event dxMouseState()
    ''' <summary>
    ''' Wird zwischen Renderstart und Renderend ausgelöst um die eigentliche Szene
    ''' zu rendern.
    ''' </summary>
    ''' <remarks></remarks>
    Public Shared Event dxRenderScene()
    ''' <summary>
    ''' Wird nach dem Laden der Engine ausgelöst um alle Ressourcen in einem Thread zu laden.
    ''' Wärende der Zeit wird ein Splashlogo gerendert und nach ablauf des Ladens die Szene 
    ''' gestartet.
    ''' </summary>
    Public Shared Event dxLoadObjectsSettings()
    Public Shared Event dxDeviceLost()
    Public Shared Event dxDeviceReset()
    Public Shared Event dxEngineClosing()
    Public Shared Event dxLoadDeviceObjects()
    Public Shared Event dxLoadObjectsTextures()
    Public Shared Event dxEngineClosed()

    Private Shared _ScreenshotCounter As Integer              '# Adds Number to Filename


    Private Shared _dxGraphicDevice As Device                 '# GraficDevice
    Private Shared _dxWindowHandle As IntPtr                  '# WindowHandle
    Private Shared _dxCam As engineCamera                     '# Camera
    Private Shared _dxTexture As engineTexture                '# TextureEffects
    Private Shared _dxTransform As engineTransform            '# Transformations
    Private Shared _dxTimer As engineTimer                    '# DXTimer
    Private Shared _dxText2d As engineText2D                  '# 2DText
    Private Shared _dxLights As engineLights                  '# dxLights
    Private Shared _dxResourceLoader As engineResourceLoader  '# Resourceloader

    Private Shared _dxKeyboard As DirectInput.Keyboard        '# dxKeyboard
    Private Shared _dxKeyboardState As New DirectInput.KeyboardState      '# current State

    Private Shared _dxMouse As DirectInput.Mouse              '# dxMouse
    Private Shared _dxMouseState As DirectInput.MouseState    '# current State
    Private Shared _dxMousePosition As Point                  '# current 2D Position

    Private Shared _dxEnum As sdxEnumeration.deviceEnumeration '# Parameter for Device
    Private Shared _dxAdapterinfo As AdapterInformation
    Private Shared _direct3d As New Direct3D                  '# New Direct3D


    Private Shared _dxMouseCooperativeLevel As DirectInput.CooperativeLevel = Nothing
    Private Shared _dxKeyboardCooperativeLevel As DirectInput.CooperativeLevel = Nothing

    Private Shared _dxTargetbufferColor As Color = Color.Black  '#Targetbuffer Color
    Private Shared _dxTargetbufferSize As Point                 '#Targetbuffersize
    Private Shared _dxTargetbufferCenter As Vector2             '#Targetbuffercenter
    Private Shared _dxScreenScaling As Vector2
    Private Shared _dxTargetbufferRatiofactor As Single
    Private Shared _dxTargetSurface(2) As Surface               '#Surfaces
    Private Shared _dxTexturesurface(1) As TextureSurface       '#Surface - Texture
    Private Shared _dxCurrentTarget As Surface
    Private Shared _dxDefaultTargetIndex As Integer
    Private Shared _dxcurrentTargetIndex As Integer

    Private Shared _isKEYreleased(144) As Boolean              '#for KEYswitches
    Private Shared _isBUTTONreleased(10) As Boolean            '#for MOUSEswitches

    '#Default ClearFlags
    Private Shared _dxClearFlags As ClearFlags = ClearFlags.Target Or ClearFlags.ZBuffer

    '#current Worldmatrix
    Private Shared _dxCurrentWorldMatrix As Matrix



    '#Property Classes
    Private Shared _dxLightFX As shaderLight
    Private Shared _dxVBuffer As enginevbuffers
    Private Shared _dxEngine As New clsEngine
    Private Shared _dxWorld As New clsWorld


    Private Shared _dxStencil As New engineStencilBuffer
    Private Shared _dxZbuffer As New engineZBuffer


    '#Loading Thread
    Private Shared _dxLoading As New BackgroundWorker
    Private Shared _dxLoadingStatusText As String = ""
    '#ReLoading Thread
    Private Shared _dxReLoading As New BackgroundWorker


    '#SplashMesh
    Private Shared _splashmesh As sdxMesh
    Private _material(2) As shaderLight.MaterialSet
    Private Shared _currentScene As SceneManager
    Private Shared _deviceStatus As DeviceStatus
    Private Shared _EngineErrorDescription As String
    Private Shared _EngineError As Boolean


    Public Enum EngineSurfaces As Integer
        ''' <summary>
        ''' Wechselt auf den Backbuffer
        ''' </summary>
        Backbuffer = 0
        ''' <summary>
        ''' Textursurface in voller Backbuffergröße für Posteffekte
        ''' </summary>
        Texture0_Full = 1
        ''' <summary>
        ''' Textursurface in voller Backbuffergröße für Posteffekte
        ''' </summary>
        Texture1_Full = 2
        ''' <summary>
        ''' Default Target
        ''' </summary> 
        DefaultTarget = 3
    End Enum
    Public Enum RenderTarget As Integer
        ''' <summary>
        ''' Beendet das aktuelle EngineSurface
        ''' </summary>
        EndScene = 0
        ''' <summary>
        ''' Startet das aktuelle EngineSurface
        ''' </summary>
        BeginScene = 1
    End Enum
    Public Enum SceneManager As Integer
        drawSplashScreen = 0
        drawProject = 1
    End Enum
    Public Enum DeviceStatus As Integer
        DeviceIsOK = 0
        DeviceIsLost = 1
        DeviceIsReset = 2
        DeviceNotReset = 3
        DeviceIsClosing = 4
    End Enum

    Private Sub loadSplashMeshTextures()
        _material(0) = New shaderLight.MaterialSet
        _material(1) = New shaderLight.MaterialSet
        _material(2) = New shaderLight.MaterialSet

        _material(0).MapColor = dxTexture.FromImage(My.Resources.sdxlogo, Color.Magenta)
        _material(1).MapColor = dxTexture.FromImage(My.Resources.sdxtext, Color.Magenta)
        _material(2).MapColor = dxTexture.FromImage(My.Resources.sdxcircle, Color.Black)

        _splashmesh.MaterialFX(0) = _material(0)
        _splashmesh.MaterialFX(1) = _material(1)
        _splashmesh.MaterialFX(2) = _material(2)
    End Sub



#Region "Create Engine"
    ''' <summary>
    ''' Erstellt eine Enumeration und startet die Engine, wenn der Vorgang erfolgreich abgeschlossen wurde.
    ''' </summary>
    ''' <param name="WindowText">
    ''' Legt den Application WindowText fest.</param>
    ''' <param name="ShaderVersion">
    ''' Legt die zu prüfende PixelShader Version fest</param>

    Public ReadOnly Property dxEngine(ByVal WindowText As String, Optional ByVal ShaderVersion As String = "0.0", Optional ByVal forceStencil As Boolean = False) As Boolean
        Get


            Dim _dxenumeration As New sdxEnumeration(ShaderVersion, forceStencil)

            '#show enumeration Window
            If _dxenumeration.ShowDialog(Me) = System.Windows.Forms.DialogResult.OK Then
                '#get WindowHandle
                _dxWindowHandle = Me.Handle
            Else
                _deviceStatus = DeviceStatus.DeviceIsClosing
                Return False
            End If
            'more Power     



            '#make me visible
            Me.Left = CInt((Screen.PrimaryScreen.Bounds.Width - Me.Width) / 2)   '#Set Window to Screensenter
            Me.Top = CInt((Screen.PrimaryScreen.Bounds.Height - Me.Height) / 2)  '#Set Window to Screensenter
            Me.BackColor = Color.Black
            Me.BackgroundImage = My.Resources.baseengine
            Me.BackgroundImageLayout = ImageLayout.Center
            Me.Visible = True
            Me.Show()
            Me.Refresh()
            DirectInputKeyboard()
            DirectInputMouse()
            AquiereInputDevices()


            '#add values to pp
            _dxEnum = _dxenumeration.EnumerationResult
            _dxEnum.dxPresentParameters.DeviceWindowHandle = _dxWindowHandle

            '#adapterinfo
            _dxAdapterinfo = _dxenumeration.Adapterinfo

            '#get Buffersize and Center
            _dxTargetbufferSize = New Point(CInt(_dxEnum.dxPresentParameters.BackBufferWidth), CInt(_dxEnum.dxPresentParameters.BackBufferHeight))
            _dxTargetbufferCenter = New Vector2(CSng(_dxEnum.dxPresentParameters.BackBufferWidth / 2), CSng(_dxEnum.dxPresentParameters.BackBufferHeight / 2))
            _dxTargetbufferRatiofactor = CSng(_dxTargetbufferSize.X / _dxTargetbufferSize.Y)

            Me.Text = WindowText
            Me.ClientSize = New System.Drawing.Size(CInt(_dxTargetbufferSize.X), CInt(_dxTargetbufferSize.Y))
            Me.Left = CInt((Screen.PrimaryScreen.Bounds.Width - Me.Width) / 2)   '#Set Window to Screensenter
            Me.Top = CInt((Screen.PrimaryScreen.Bounds.Height - Me.Height) / 2)  '#Set Window to Screensenter
            Me.ShowInTaskbar = True
            Me.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle
            Me.MinimizeBox = True
            Me.MaximizeBox = False
            Me.Icon = My.Resources.ico_SlimDX

            '#disable autopaint
            Me.SetStyle(ControlStyles.AllPaintingInWmPaint, True)                '#Turn off repainting
            '#Turn off Opaque Forms
            Me.SetStyle(ControlStyles.Opaque, True)


            '#create the Device and BaseEngine objects
            Try
                _dxGraphicDevice = initDevice(_dxWindowHandle, _dxEnum.dxDeviceFlags, _dxEnum.dxPresentParameters)
            Catch ex As Exception
                MsgBox(ex.Message)
                Me.Close()
            End Try

            'Dim t As Surface = Surface.CreateDepthStencil(_dxGraphicDevice, 512, 512, Format.D16, MultisampleType.None, 0, True)

            '_dxGraphicDevice.DepthStencilSurface = t
            setDefaultRenderstates()
            setDefaultSettings()

            '#Basic Objects
            LoadEngineObjects()


            '#add Thread Loding of Project
            AddHandler _dxLoading.DoWork, AddressOf loadingProject
            AddHandler _dxLoading.RunWorkerCompleted, AddressOf loadingCompleted

            '#add Thread ReLoding of Project by LostDevice
            AddHandler _dxReLoading.DoWork, AddressOf reloadingProject
            AddHandler _dxReLoading.RunWorkerCompleted, AddressOf reloadingCompleted

            '#Mainloop
            AddHandler Me.Paint, AddressOf PresentToScreen


            '#run thread to load project
            _deviceStatus = DeviceStatus.DeviceIsOK
            _dxLoading.WorkerReportsProgress = True
            _dxLoading.WorkerSupportsCancellation = True
            _dxLoading.RunWorkerAsync()
            Return True
        End Get
    End Property
#End Region
#Region "Create New Device"
    Private Function initDevice(ByVal wHandle As IntPtr, ByVal flags As CreateFlags, ByVal parameters As PresentParameters) As Device
        '# Device erstellen und zurückgeben
        Return New Device(_direct3d, 0, Direct3D9.DeviceType.Hardware, wHandle, flags Or CreateFlags.Multithreaded, parameters)
    End Function
#End Region


#Region "LoadEngineObjects"
    Private Sub LoadEngineObjects()
        _dxCam = New engineCamera(_dxTargetbufferSize.X, _dxTargetbufferSize.Y)
        _dxTransform = New engineTransform
        _dxTexture = New engineTexture
        _dxTimer = New engineTimer
        _dxText2d = New engineText2D

        '#Loading Mesh
        _splashmesh = New sdxMesh
        _splashmesh.MeshFromMemory(My.Resources.logo, 3)
        loadSplashMeshTextures()

        _dxResourceLoader = New engineResourceLoader
        _dxLights = New engineLights

    End Sub
#End Region
#Region "createTextureSurfaces"
    Private Shared Sub LoadTextureSurfaces()
        '#Texture Surfaces
        _dxTexturesurface(0) = New TextureSurface(_dxTargetbufferSize)
        _dxTexturesurface(1) = New TextureSurface(_dxTargetbufferSize)

        _dxTargetSurface(0) = _dxGraphicDevice.GetRenderTarget(0)
        _dxTargetSurface(1) = _dxTexturesurface(0).Surface
        _dxTargetSurface(2) = _dxTexturesurface(1).Surface
        _dxCurrentTarget = _dxTargetSurface(0)
    End Sub
    Private Shared Sub DisposeTextureSurfaces()
        If Not _dxTexturesurface(0) Is Nothing Then _dxTexturesurface(0).Dispose()
        If Not _dxTexturesurface(1) Is Nothing Then _dxTexturesurface(1).Dispose()
        If Not _dxTargetSurface(0) Is Nothing Then _dxTargetSurface(0).Dispose()
        If Not _dxTargetSurface(1) Is Nothing Then _dxTargetSurface(1).Dispose()
        If Not _dxTargetSurface(2) Is Nothing Then _dxTargetSurface(2).Dispose()
    End Sub
#End Region
#Region "createInputControls"
    Private Sub DirectInputMouse()

        Dim _dinput As New DirectInput.DirectInput

        _dxMouse = New DirectInput.Mouse(_dinput)
        If Not _dxMouseCooperativeLevel = Nothing Then _dxMouse.SetCooperativeLevel(_dxWindowHandle, _dxMouseCooperativeLevel)
        _dinput.Dispose()
    End Sub
    Private Sub DirectInputKeyboard()
        Dim _dinput As New DirectInput.DirectInput

        _dxKeyboard = New DirectInput.Keyboard(_dinput)
        If Not _dxKeyboardCooperativeLevel = Nothing Then _dxKeyboard.SetCooperativeLevel(_dxWindowHandle, _dxKeyboardCooperativeLevel)
        _dinput.Dispose()
    End Sub
#End Region

#Region "Default Renderstates"
    Private Sub setDefaultRenderstates()
        _dxGraphicDevice.SetRenderState(RenderState.Lighting, False)
        _dxGraphicDevice.SetRenderState(RenderState.SpecularEnable, False)
        _dxGraphicDevice.SetRenderState(RenderState.ZEnable, False)
        _dxGraphicDevice.SetRenderState(RenderState.AlphaBlendEnable, False)
        _dxGraphicDevice.SetRenderState(RenderState.StencilEnable, False)
        '#Texturfilterfür Stage 0
        _dxGraphicDevice.SetSamplerState(0, SamplerState.MagFilter, TextureFilter.Linear)
        _dxGraphicDevice.SetSamplerState(0, SamplerState.MinFilter, TextureFilter.Linear)
        '#Texturfilterfür Stage 1
        _dxGraphicDevice.SetSamplerState(1, SamplerState.MagFilter, TextureFilter.Linear)
        _dxGraphicDevice.SetSamplerState(1, SamplerState.MinFilter, TextureFilter.Linear)
        '#Texturfilterfür Stage 0,1 MipMap
        _dxGraphicDevice.SetSamplerState(0, SamplerState.MipFilter, TextureFilter.Linear)
        _dxGraphicDevice.SetSamplerState(1, SamplerState.MipFilter, TextureFilter.Linear)
    End Sub
#End Region
#Region "Default Settings"
    Private Sub setDefaultSettings()
        _dxScreenScaling.X = CSng(_dxTargetbufferSize.X / 1024)
        _dxScreenScaling.Y = CSng(_dxTargetbufferSize.X / 1024)
        dxTransformWorld = Matrix.Identity
    End Sub
#End Region

#Region "Input Device - Keyboard"
    Private Shared Sub updateKeyboard()
        With _dxKeyboard
            _dxKeyboardState = .GetCurrentState
            RaiseEvent dxKeyboardState()
        End With
    End Sub
#End Region
#Region "Input Device - Mouse"
    Private Sub updateMouse()
        With _dxMouse
            _dxMouseState = .GetCurrentState
            RaiseEvent dxMouseState()
        End With
        '2D Space
        _dxMousePosition = Me.PointToClient(System.Windows.Forms.Cursor.Position)
        If _dxMousePosition.X < 0 Then _dxMousePosition.X = 0
        If _dxMousePosition.X > _dxTargetbufferSize.X Then _dxMousePosition.X = _dxTargetbufferSize.X
        If _dxMousePosition.Y < 0 Then _dxMousePosition.Y = 0
        If _dxMousePosition.Y > _dxTargetbufferSize.Y Then _dxMousePosition.Y = _dxTargetbufferSize.Y
    End Sub

#End Region
#Region "Input Device - Acquire"
    ''' <summary>
    ''' Erstellt den Zugriff auf den Keyboard- und Mousedevice.
    ''' Der Vorgang wird automatisch ausgeführt.
    ''' </summary>
    Public Sub AquiereInputDevices()
        _dxKeyboard.Acquire()
        _dxMouse.Acquire()
    End Sub
    Public Shared Sub UnAquiereInputDevices()
        Try
            If Not _dxKeyboard Is Nothing Then
                _dxKeyboard.Unacquire()
            End If
        Catch ex As Exception
        End Try

        Try
            If Not _dxMouse Is Nothing Then
                _dxMouse.Unacquire()
            End If
        Catch ex As Exception
        End Try
    End Sub
    Public Shared Sub DisposeInputDevices()
        If Not _dxKeyboard Is Nothing Then
            _dxKeyboard.Dispose()
        End If
        If Not _dxMouse Is Nothing Then
            _dxMouse.Dispose()
        End If
    End Sub
#End Region

#Region "Loading Threadet Project Data"
    Private Sub loadingProject(ByVal sender As Object, ByVal e As DoWorkEventArgs)
        _dxLightFX = New shaderLight
        _dxVBuffer = New enginevbuffers
        LoadTextureSurfaces()
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then RaiseEvent dxLoadDeviceObjects()
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then RaiseEvent dxLoadObjectsSettings()
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then RaiseEvent dxLoadObjectsTextures()
    End Sub
    Private Sub loadingCompleted(ByVal sender As Object, ByVal e As RunWorkerCompletedEventArgs)
        If _deviceStatus = DeviceStatus.DeviceIsOK Then _currentScene = SceneManager.drawProject
    End Sub
#End Region
#Region "ReLoading Threadet Project Data"
    Private Sub reloadingProject(ByVal sender As Object, ByVal e As DoWorkEventArgs)
        RaiseEvent dxDeviceReset()
        RaiseEvent dxLoadObjectsTextures()
    End Sub
    Private Sub reloadingCompleted(ByVal sender As Object, ByVal e As RunWorkerCompletedEventArgs)
        If _deviceStatus = DeviceStatus.DeviceIsOK Then _currentScene = SceneManager.drawProject
    End Sub
#End Region

#Region "## Draw Loading Splash ##"
    Private Sub drawLoadingSplash()
        'Exit Sub 'disable splash
        dxCamera.currentViewMatrix = Matrix.Identity
        With _splashmesh
            dxTransformWorld = Matrix.Translation(New Vector3(0, 0.5, 3))
            .Draw(0, 0)
            dxTransformWorld = Matrix.Translation(New Vector3(0, -0.5, 3))
            .Draw(1, 1)
            dxTexture.AlphaBlending3 = True
            dxTransformWorld = dxMatrix.TimebasedRotationZ(-0.002) * Matrix.Translation(New Vector3(0, 0.5, 3))
            .Draw(0, 2)
            dxTexture.AlphaBlending3 = False
        End With
        Threading.Thread.Sleep(10)
        dx2DText.DrawLoadingInfo(_dxLoadingStatusText)
    End Sub
#End Region

#Region "## Update Loop                            ##"
    Protected Sub PresentToScreen(ByVal sender As Object, ByVal e As EventArgs)


        '#update Timer
        _dxTimer.UpdateGametime()


        If _deviceStatus = DeviceStatus.DeviceIsOK Then
            _dxGraphicDevice.BeginScene()
            _dxGraphicDevice.Clear(_dxClearFlags, dxEngine.TargetBufferColor, 1, 0)


            Select Case _currentScene
                Case SceneManager.drawProject
                    RaiseEvent dxRenderScene()

                    dxCamera.update()
                Case SceneManager.drawSplashScreen
                    drawLoadingSplash()
            End Select
            _dxGraphicDevice.EndScene()

            updateMouse()
            updateKeyboard()

            _deviceStatus = getNewDeviceStatus()
            If _deviceStatus = DeviceStatus.DeviceIsOK Then _dxGraphicDevice.Present()
            Me.Invalidate()
            Exit Sub
        End If


        If _deviceStatus = DeviceStatus.DeviceIsLost Then
            _deviceStatus = getNewDeviceStatus()
            Me.Invalidate()
            Exit Sub
        End If


        If _deviceStatus = DeviceStatus.DeviceNotReset Then
            Try
                DisposeTextureSurfaces()
                UnAquiereInputDevices()
                RaiseEvent dxDeviceLost()
                Threading.Thread.Sleep(50)
                _deviceStatus = getNewDeviceStatus()
                _dxGraphicDevice.Reset(_dxEnum.dxPresentParameters)
                Threading.Thread.Sleep(50)
                _deviceStatus = DeviceStatus.DeviceIsReset
            Catch ex As Exception
                MsgBox(ex.ToString, MsgBoxStyle.Critical)
                CloseBaseEngine()
            End Try
            Me.Invalidate()
            Exit Sub
        End If


        If _deviceStatus = DeviceStatus.DeviceIsReset Then
            _deviceStatus = getNewDeviceStatus()
            setDefaultRenderstates()
            dxCamera.RestoreProjectionMatrix()
            dxCamera.currentViewMatrix = Matrix.Identity
            LoadTextureSurfaces()
            _dxGraphicDevice.SetRenderTarget(0, _dxTargetSurface(0))
            AquiereInputDevices()
            loadSplashMeshTextures()
            _currentScene = SceneManager.drawSplashScreen
            _dxReLoading.RunWorkerAsync()
            Me.Invalidate()
            Exit Sub
        End If

        If _deviceStatus = DeviceStatus.DeviceIsClosing Then
        End If

    End Sub
#End Region
#Region "## Device Status  ##"
    Private Function getNewDeviceStatus() As DeviceStatus
        Dim status As Result = _dxGraphicDevice.TestCooperativeLevel
        Select Case status.Name
            Case Is = "S_OK"
                Return DeviceStatus.DeviceIsOK
            Case Is = "D3DERR_DEVICELOST"
                Return DeviceStatus.DeviceIsLost
            Case Is = "D3DERR_DEVICENOTRESET"
                Return DeviceStatus.DeviceNotReset
        End Select
    End Function



#End Region
#Region "## change Surface                         ##"
    Private Sub changeSurface()
        _dxGraphicDevice.SetRenderTarget(0, _dxTargetSurface(_dxDefaultTargetIndex))
    End Sub
#End Region


#Region "Class - TextureSurface"
    Public Class TextureSurface
        Private _surface As Surface
        Private _texture As Texture

        ''' <summary>
        ''' Erstellt ein neues TextureRenderSurface. Es wird dir Objektgebundene Textur verwendet.
        ''' </summary>
        ''' <param name="TextureSize">Legt die Größe der Textur fest</param>
        Public Sub New(ByVal TextureSize As Point, Optional ByVal Format As Format = Format.A8R8G8B8)
            _texture = New Texture(dxDevice, TextureSize.X, TextureSize.Y, 1, Usage.RenderTarget, Format, Pool.Default)
            _surface = _texture.GetSurfaceLevel(0)
        End Sub
        Public Sub New(ByVal TextureSize As Point, ByVal LevelCount As Integer, Optional ByVal Format As Format = Format.A8R8G8B8)
            _texture = New Texture(dxDevice, TextureSize.X, TextureSize.Y, LevelCount, Usage.RenderTarget, Format, Pool.Default)
            _surface = _texture.GetSurfaceLevel(0)
        End Sub


        ''' <summary>
        ''' Gibt das Surface zurück
        ''' </summary>
        Public ReadOnly Property Surface() As Surface
            Get
                Return _surface
            End Get
        End Property

        ''' <summary>
        ''' Gibt die erstellte Textur zurück oder legt eine fest
        ''' </summary>
        Public ReadOnly Property Texture() As Texture
            Get
                Return _texture
            End Get
        End Property

        ''' <summary>
        ''' Gibt die verwendeten Ressourcen der Klasse wieder frei.
        ''' </summary>
        Public Sub Dispose()
            If Not _surface Is Nothing Then _surface.Dispose()
            If Not _texture Is Nothing Then _texture.Dispose()
        End Sub
    End Class
#End Region
#Region "Class - TextureAnimation"
    Public Class TextureAnimation
        Private _tex() As Texture
        Private _count As Integer
        Private _index As Integer
        Private _update As Single = 50
        Private _time As Single

        Public Sub New()
            AddHandler dxEngineClosing, AddressOf Dispose
            AddHandler dxDeviceLost, AddressOf DeviceLost
        End Sub

        Public Property Textures() As Texture()
            Get
                Return _tex
            End Get
            Set(ByVal value As Texture())
                _tex = value
            End Set
        End Property
        Public ReadOnly Property Texture() As Texture
            Get
                If dxGametime.Gametime - _time > _update Then
                    _index += 1
                    _time = dxGametime.Gametime
                    If _index > _count Then _index = 0
                End If
                Return _tex(_index)
            End Get
        End Property
        ''' <summary>
        ''' Updatetime in milliseconds
        ''' </summary>
        ''' <value></value>
        ''' <remarks></remarks>
        Public WriteOnly Property Update() As Single
            Set(ByVal value As Single)
                _update = value
            End Set
        End Property

        Private Sub Dispose()
            If Not _tex Is Nothing Then
                For i = 0 To _count
                    If Not _tex(i) Is Nothing Then _tex(i).Dispose()
                Next
            End If
        End Sub
        Private Sub DeviceLost()
            If Not _tex Is Nothing Then
                For i = 0 To _count
                    If Not _tex(i) Is Nothing Then _tex(i).Dispose()
                Next
            End If
        End Sub

#Region "TextureAnimation Load"
        Public Sub AnimationFromFile(ByVal src As String, ByVal ext As String, ByVal count As Integer, ByVal colorKey As Color)
            _count = count - 1
            ReDim _tex(_count)
            For i = 0 To _count
                _tex(i) = dxTexture.FromFile(src & (i + 1).ToString & ext, colorKey)
            Next
        End Sub

        Public Sub AnimationFromEngineResource(ByVal src As String, ByVal ext As String, ByVal count As Integer, ByVal colorKey As Color)
            _count = count - 1
            ReDim _tex(_count)
            For i = 0 To _count
                _tex(i) = dxTexture.FromEngineResource(src & (i + 1).ToString & ext, colorKey)
            Next
        End Sub
#End Region
    End Class
#End Region


#Region "Base - Property"

    ''' <summary>
    ''' Ruft die VertexBuffer Classe ab
    ''' </summary>
    Public Shared ReadOnly Property dxVBuffer() As enginevbuffers
        Get
            Return _dxVBuffer
        End Get
    End Property

    '''<summary>
    '''ruft die 2DText-Klasse ab
    '''</summary>
    Public ReadOnly Property dx2DText() As engineText2D
        Get
            Return _dxText2d
        End Get
    End Property

    '''<summary>
    '''ruft den Grafikdevice ab
    '''</summary>
    Public Shared ReadOnly Property dxDevice() As Device
        Get
            Return _dxGraphicDevice
        End Get
    End Property

    '''<summary>
    '''ruft die Gametimer-Klasse ab
    '''</summary>
    Public Shared ReadOnly Property dxGametime() As engineTimer   'dxgametime
        Get
            Return _dxTimer
        End Get
    End Property

    '''<summary>
    '''ruft die Textur-Klasse ab
    '''</summary>
    Public Shared ReadOnly Property dxTexture() As engineTexture
        Get
            Return _dxTexture
        End Get
    End Property

    '''<summary>
    '''ruft  die Kamera-Klasse ab
    '''</summary>
    Public Shared ReadOnly Property dxCamera() As engineCamera
        Get
            Return _dxCam
        End Get
    End Property

    '''<summary>
    '''ruft die Transform-Klasse zur Erstellung von Worldtransformations ab
    '''</summary>
    Public Shared ReadOnly Property dxMatrix() As engineTransform
        Get
            Return _dxTransform
        End Get
    End Property



    '''<summary>
    '''Legt die neue Worldmatrix fest.
    '''</summary>
    Public Shared WriteOnly Property dxTransformWorld() As Matrix
        Set(ByVal value As Matrix)
            _dxCurrentWorldMatrix = value
            _dxGraphicDevice.SetTransform(TransformState.World, _dxCurrentWorldMatrix)
        End Set
    End Property


    ''' <summary>
    ''' ruft die Light-Klasse ab
    ''' </summary>
    Public Shared ReadOnly Property dxLightsFFP() As engineLights
        Get
            Return _dxLights
        End Get
    End Property

    ''' <summary>
    ''' ruft die erweitereten Engine Properties ab
    ''' </summary>
    Public Shared ReadOnly Property dxEngine() As clsEngine
        Get
            Return _dxEngine
        End Get
    End Property

    ''' <summary>
    ''' ruft die ZBuffer-Klasse ab
    ''' </summary>
    Public Shared ReadOnly Property dxZBuffer() As engineZBuffer
        Get
            Return _dxZbuffer
        End Get
    End Property

    ''' <summary>
    ''' ruft den StencilBuffer ab
    ''' </summary>
    Public Shared ReadOnly Property dxStencilBuffer() As engineStencilBuffer
        Get
            Return _dxStencil
        End Get
    End Property


    ''' <summary>
    ''' ruft die World - Klasse ab
    ''' </summary>
    Public Shared ReadOnly Property dxWorld() As clsWorld
        Get
            Return _dxWorld
        End Get
    End Property

    Public Shared ReadOnly Property dxLightsFX() As shaderLight
        Get
            Return _dxLightFX
        End Get
    End Property


#End Region
#Region "Base - Helper Functions"
    Public Shared Function Radiant(ByVal value As Single) As Single
        Return CSng(value * Math.PI / 180)
    End Function
    Public Shared Function ImageToByteBuffer(ByVal Image As Image) As Byte()
        Dim stream As New IO.MemoryStream
        Image.Save(stream, Imaging.ImageFormat.Jpeg)
        ImageToByteBuffer = stream.ToArray
    End Function
    Public Shared Function BitmapToByteBuffer(ByVal Bitmap As Image) As Byte()
        Dim stream As New IO.MemoryStream
        Bitmap.Save(stream, Imaging.ImageFormat.Bmp)
        BitmapToByteBuffer = stream.ToArray
    End Function

    Public Shared Function clamp(ByVal value As Single, ByVal min As Single, ByVal max As Single) As Single
        If value > max Then Return max
        If value < min Then Return min
        Return value
    End Function

    Public Shared Function ImportPositions(ByVal Source As String, Optional ByVal invertZ As Boolean = True) As Vector3()
        Dim vec() As String
        Dim text As String
        Dim file As String = dxEngine.ApplicationPath & Source
        Dim sr As New System.IO.StreamReader(file)
        text = sr.ReadToEnd
        text = Replace(text, ".", ",", 1, -1)
        vec = text.Split(CChar(vbLf))

        Dim pos(vec.Length - 2) As Vector3
        For i = 0 To vec.Length - 2
            Dim v() As String
            v = vec(i).Split(CChar(vbTab))
            pos(i).X = CSng(v(0))
            pos(i).Y = CSng(v(1))
            pos(i).Z = CSng(v(2))
            If invertZ Then pos(i).Z *= -1
        Next
        Return pos
    End Function

    <System.Security.SuppressUnmanagedCodeSecurity()> _
     Private Declare Function PeekMessage Lib "User32.dll" Alias "PeekMessageA" ( _
     ByRef msg As MessageStruct, _
     ByRef hWnd As IntPtr, _
     ByVal messageFilterMin As Integer, _
     ByVal messageFilterMax As Integer, _
     ByVal flags As Integer) As Integer

    Private Structure MessageStruct
        Public HWnd As IntPtr
        Public Msg As Message
        Public WParam As IntPtr
        Public LParam As IntPtr
        Public Time As Integer
        Public P As Point
    End Structure
#End Region
#Region "Base - Close Engine"
    Public Sub CloseBaseEngine()
        If _deviceStatus = DeviceStatus.DeviceIsOK Then
            _deviceStatus = SDX9BaseEngine.DeviceStatus.DeviceIsClosing
            Threading.Thread.Sleep(1000)
            RaiseEvent dxEngineClosing()
        End If

        _dxEnum = Nothing
        _dxCam = Nothing
        _dxTexture = Nothing
        _dxTimer = Nothing
        _dxTransform = Nothing

        DisposeTextureSurfaces()
        UnAquiereInputDevices()
        DisposeInputDevices()

        If Not _dxGraphicDevice Is Nothing Then _dxGraphicDevice.Dispose()
        If Not _direct3d Is Nothing Then _direct3d.Dispose()

        RaiseEvent dxEngineClosed()
    End Sub
#End Region

#Region "Property  Class - dxEngine "

    ''' <summary>
    ''' Erweiterte Engine Properties
    ''' </summary>
    Public Class clsEngine

        Public ReadOnly Property DeviceStatus() As DeviceStatus
            Get
                Return _deviceStatus
            End Get
        End Property

        Public Sub ClearErrorDescription()
            _EngineErrorDescription = ""
            _EngineError = False
        End Sub
        Public Property EngineErrorDescription() As String
            Get
                Return _EngineErrorDescription
            End Get
            Set(ByVal value As String)
                _EngineError = True
                _EngineErrorDescription = _EngineErrorDescription & value & vbCrLf
            End Set
        End Property
        Public ReadOnly Property EngineError() As Boolean
            Get
                Return _EngineError
            End Get
        End Property
        Public ReadOnly Property ResourceLoader() As engineResourceLoader
            Get
                Return _dxResourceLoader
            End Get
        End Property

        Public WriteOnly Property LoadingStatus() As String
            Set(ByVal value As String)
                _dxLoadingStatusText = value
            End Set
        End Property

        ''' <summary>
        ''' Ruft die ClearFlags des Devices ab oder legt diese fest. (Default= Target|ZBuffer)
        ''' </summary>
        Public Property SurfaceClearFlags() As ClearFlags
            Set(ByVal value As ClearFlags)
                _dxClearFlags = value
            End Set
            Get
                Return _dxClearFlags
            End Get
        End Property

        Public ReadOnly Property Adapterinfo() As AdapterInformation
            Get
                Return _dxAdapterinfo
            End Get
        End Property

        ''' <summary>
        ''' Gibt die Textur aus dem TexturSurface0 zurück
        ''' </summary>
        Public ReadOnly Property TextureFromTextureSurface0() As Texture
            Get
                Return _dxTexturesurface(0).Texture
            End Get
        End Property
        ''' <summary>
        ''' Gibt die Textur aus dem TexturSurface1 zurück
        ''' </summary>
        Public ReadOnly Property TextureFromTextureSurface1() As Texture
            Get
                Return _dxTexturesurface(1).Texture
            End Get
        End Property

        ''' <summary>
        ''' Legt das Default Surface fest, das nach jedem Frame gesetzt wird
        ''' </summary>
        Public WriteOnly Property SurfaceDefault() As SDX9BaseEngine.EngineSurfaces
            Set(ByVal value As SDX9BaseEngine.EngineSurfaces)
                _dxDefaultTargetIndex = value
            End Set
        End Property

        ''' <summary>
        ''' legt ein neues Surface fest
        ''' </summary>
        Public WriteOnly Property SurfaceCurrent(ByVal ClearColor As Color, Optional ByVal Clear As Boolean = True) As Surface
            Set(ByVal value As Surface)
                _dxGraphicDevice.EndScene()
                _dxGraphicDevice.SetRenderTarget(0, value)
                If Clear Then _dxGraphicDevice.Clear(_dxClearFlags, ClearColor, 1, 0)
                _dxGraphicDevice.BeginScene()
            End Set
        End Property

        ''' <summary>
        ''' Weechselt im Renderloop auf das ausgewälte Rendertarget
        ''' </summary>
        ''' <param name="Clear">Legt fest, ob der Buffer überschrieben wird</param>
        Public WriteOnly Property SurfaceChange(Optional ByVal Clear As Boolean = True) As SDX9BaseEngine.EngineSurfaces
            Set(ByVal value As SDX9BaseEngine.EngineSurfaces)
                Select Case value
                    Case Is = EngineSurfaces.Backbuffer
                        _dxGraphicDevice.EndScene()
                        _dxGraphicDevice.SetRenderTarget(0, _dxTargetSurface(0))
                        If Clear Then _dxGraphicDevice.Clear(_dxClearFlags, _dxTargetbufferColor, 1, 0)
                        _dxGraphicDevice.BeginScene()
                        _dxcurrentTargetIndex = 0
                    Case Is = EngineSurfaces.Texture0_Full
                        _dxGraphicDevice.EndScene()
                        _dxGraphicDevice.SetRenderTarget(0, _dxTargetSurface(1))
                        If Clear Then _dxGraphicDevice.Clear(_dxClearFlags, _dxTargetbufferColor, 1, 0)
                        _dxGraphicDevice.BeginScene()
                        _dxcurrentTargetIndex = 1
                    Case Is = EngineSurfaces.Texture1_Full
                        _dxGraphicDevice.EndScene()
                        _dxGraphicDevice.SetRenderTarget(0, _dxTargetSurface(2))
                        If Clear Then _dxGraphicDevice.Clear(_dxClearFlags, _dxTargetbufferColor, 1, 0)
                        _dxGraphicDevice.BeginScene()
                        _dxcurrentTargetIndex = 2
                    Case Is = EngineSurfaces.DefaultTarget
                        _dxGraphicDevice.EndScene()
                        _dxGraphicDevice.SetRenderTarget(0, _dxTargetSurface(_dxDefaultTargetIndex))
                        If Clear Then _dxGraphicDevice.Clear(_dxClearFlags, _dxTargetbufferColor, 1, 0)
                        _dxGraphicDevice.BeginScene()
                        _dxcurrentTargetIndex = _dxDefaultTargetIndex
                End Select
            End Set
        End Property

        ''' <summary>
        ''' ruft den Pfad der Anwendung ab
        ''' </summary>
        Public ReadOnly Property ApplicationPath() As String
            Get
                Return Application.StartupPath & "\"
            End Get
        End Property

        Public ReadOnly Property PhysicalMemory() As String
            Get
                Dim mem As Long = My.Application.Info.WorkingSet
                mem = CLng(mem * 0.0009765625)
                mem = CLng(mem * 0.0009765625)
                mem = CLng(Math.Round(mem, 2))
                Return mem.ToString & " MB"
            End Get
        End Property

        ''' <summary>
        ''' ruft den aktuellen Keyboardstate ab
        ''' </summary>
        Public ReadOnly Property KeyboardState() As DirectInput.KeyboardState
            Get
                Return _dxKeyboardState
            End Get
        End Property
        ''' <summary>
        ''' ruft ab, ob der angegebene Key gedrückt wurde.
        ''' Der Key muss zur erneuten Funktion released werden.
        ''' </summary>
        ''' <param name="pressedKEY">
        ''' Legt den gedrückten KEY fest</param>
        Public ReadOnly Property KeyAsSwitch(ByVal pressedKEY As DirectInput.Key) As Boolean
            Get
                With dxEngine.KeyboardState
                    If .IsPressed(pressedKEY) And _isKEYreleased(pressedKEY) Then
                        _isKEYreleased(pressedKEY) = .IsReleased(pressedKEY)
                        Return True
                    Else
                        _isKEYreleased(pressedKEY) = .IsReleased(pressedKEY)
                        Return False
                    End If
                End With
            End Get
        End Property
        ''' <summary>
        ''' ruft ab, ob der angegebene Key gedrückt wurde.
        ''' </summary>
        ''' <param name="pressedKEY">
        ''' Legt den gedrückten KEY fest</param>
        Public ReadOnly Property KeyPressed(ByVal pressedKEY As DirectInput.Key) As Boolean
            Get
                With dxEngine.KeyboardState
                    If .IsPressed(pressedKEY) Then Return True
                End With
            End Get
        End Property

        ''' <summary>
        ''' Ruft den aktuellen Mousestate ab
        ''' </summary>
        Public ReadOnly Property MouseState() As DirectInput.MouseState
            Get
                Return _dxMouseState
            End Get
        End Property

        ''' <summary>
        ''' Ruft die aktuelle Mausbewegung ab
        ''' </summary>
        Public ReadOnly Property MouseMovement() As Vector2
            Get
                With _dxMouseState
                    Return New Vector2(.X, .Y)
                End With
            End Get
        End Property
        ''' <summary>
        ''' Ruft ab, ob der angegebene Button gedrückt wurde.
        ''' </summary>
        ''' <param name="index">legt den Index des Button fest (max=10)</param>
        Public ReadOnly Property MouseButtonPressed(ByVal index As Integer) As Boolean
            Get
                With _dxMouseState
                    If .IsPressed(index) Then Return True
                End With
            End Get
        End Property
        ''' <summary>
        ''' Ruft ab, ob der angegebene Button gedrückt wurde.
        ''' Der Button muss zur erneuten Funktion released werden.
        ''' </summary>
        ''' <param name="index">legt den Index des Button fest (max=10)</param>
        Public ReadOnly Property MouseButtonAsSwitch(ByVal index As Integer) As Boolean
            Get
                With _dxMouseState
                    If .IsPressed(index) And _isBUTTONreleased(index) Then
                        _isBUTTONreleased(index) = .IsReleased(index)
                        Return True
                    Else
                        _isBUTTONreleased(index) = .IsReleased(index)
                        Return False
                    End If
                End With
            End Get
        End Property
        ''' <summary>
        ''' Ruft die aktuelle Cursorposition im Screenspace ab.
        ''' </summary>
        ''' <value></value>
        Public ReadOnly Property MousePosition() As Vector2
            Get
                With _dxMousePosition
                    Return New Vector2(.X, .Y)
                End With
            End Get
        End Property

        ''' <summary>
        ''' ruft das Direct3D Object ab
        ''' </summary>
        ''' <value></value>
        Public ReadOnly Property D3D9() As Direct3D
            Get
                Return _direct3d
            End Get
        End Property

        ''' <summary>
        ''' Legt den Cooperative Level des Directinput Devices fest.
        ''' </summary>
        Public WriteOnly Property MouseSetCooperativeLevel() As DirectInput.CooperativeLevel
            Set(ByVal value As DirectInput.CooperativeLevel)
                _dxMouseCooperativeLevel = value
            End Set
        End Property

        ''' <summary>
        ''' Legt den Cooperative Level des Directinput Devices fest.
        ''' </summary>
        Public WriteOnly Property KeyboardSetCooperativeLevel() As DirectInput.CooperativeLevel
            Set(ByVal value As DirectInput.CooperativeLevel)
                _dxKeyboardCooperativeLevel = value
            End Set
        End Property

        '''<summary>
        '''Legt die Löschfarbe des Backbuffers fest
        '''</summary>
        Public Property TargetBufferColor() As Color
            Set(ByVal value As Color)
                _dxTargetbufferColor = value
            End Set
            Get
                Return _dxTargetbufferColor
            End Get
        End Property
        '''<summary>
        '''Ruft den Faktor des Seitenverhältnisses ab
        '''</summary>
        Public ReadOnly Property BackBufferRatioFactor() As Single
            Get
                Return _dxTargetbufferRatiofactor
            End Get
        End Property

        '''<summary>
        '''Ruft die Backbuffergröße ab
        '''</summary>
        Public ReadOnly Property BackBufferSize() As Point
            Get
                Return _dxTargetbufferSize
            End Get
        End Property
        '''<summary>
        '''ruft den Mittelpunkt des Backbuffers ab
        '''</summary>
        Public ReadOnly Property BackBufferCenter() As Vector2
            Get
                Return _dxTargetbufferCenter
            End Get
        End Property

        Public WriteOnly Property BackBufferDraw() As Boolean
            Set(ByVal value As Boolean)
                If value Then
                    With _dxGraphicDevice
                        .SetRenderState(RenderState.AlphaBlendEnable, False)
                        .SetRenderState(RenderState.SourceBlend, Blend.One)
                        .SetRenderState(RenderState.DestinationBlend, Blend.Zero)
                    End With
                Else
                    With _dxGraphicDevice
                        .SetRenderState(RenderState.AlphaBlendEnable, True)
                        .SetRenderState(RenderState.SourceBlend, Blend.Zero)
                        .SetRenderState(RenderState.DestinationBlend, Blend.One)
                    End With
                End If
            End Set
        End Property


        ''' <summary>
        ''' Legt fest, bei welcher ScreenWidth der SpriteScreenScalingFactor = 1 ist.
        ''' </summary>
        Public WriteOnly Property SpriteScreenScaling() As Integer
            Set(ByVal value As Integer)
                Dim factor As Single = CSng(_dxTargetbufferSize.X / value)
                _dxScreenScaling.X = factor
                _dxScreenScaling.Y = factor
            End Set
        End Property

        ''' <summary>
        ''' ruft den Skalierungsfaktor in Abhängigkeit des SpriteScreenScalings ab 
        ''' </summary>
        ''' <value></value>
        Public ReadOnly Property SpriteScreenScalingFactor() As Vector2
            Get
                Return _dxScreenScaling
            End Get
        End Property



        ''' <summary>
        ''' Erstellt einen Screenshot aus dem Backbuffer.
        ''' </summary>
        Public Sub SaveScreenshot()
            Dim filename As String = dxEngine.ApplicationPath & _ScreenshotCounter & "Screenshot.jpg"
            Surface.ToFile(dxDevice.GetBackBuffer(0, 0), filename, ImageFileFormat.Jpg)
            _ScreenshotCounter += 1
        End Sub
        ''' <summary>
        ''' Erstellt einen Screenshot aus dem Backbuffer.
        ''' </summary>
        ''' <param name="filename">Legt den Dateinamen fest.</param>
        Public Sub SaveScreenshot(ByVal filename As String)
            filename = dxEngine.ApplicationPath & _ScreenshotCounter & filename & ".jpg"
            Surface.ToFile(dxDevice.GetBackBuffer(0, 0), filename, ImageFileFormat.Jpg)
            _ScreenshotCounter += 1
        End Sub
        ''' <summary>
        ''' Erstellt einen Screenshot aus dem Backbuffer.
        ''' </summary>
        ''' <param name="filename">Legt den Dateinamen fest.</param>
        ''' <param name="format">Legt das Bildformat fest.</param>
        ''' <remarks></remarks>
        Public Sub SaveScreenshot(ByVal filename As String, ByVal format As ImageFileFormat)
            filename = dxEngine.ApplicationPath & _ScreenshotCounter & filename
            Surface.ToFile(dxDevice.GetBackBuffer(0, 0), filename, format)
            _ScreenshotCounter += 1
        End Sub
        ''' <summary>
        ''' Setzt den Cursor in die Bildschirmmitte.
        ''' </summary>
        ''' <remarks></remarks>
        Public Sub CenterCursor()
            With Screen.PrimaryScreen.Bounds
                System.Windows.Forms.Cursor.Position = New Point(CInt(.Width / 2), CInt(.Height / 2))
            End With
        End Sub

        ''' <summary>
        ''' Prüft ob eine Datei im Applicationverzeichnis vorhanden ist.
        ''' </summary>
        ''' <param name="filesource">Legt den Dateipfad der Datei im ApplicationPath fest</param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Function FileExists(ByVal filesource As String) As Boolean
            filesource = dxEngine.ApplicationPath & filesource
            Dim file As New IO.FileInfo(filesource)
            Return file.Exists
        End Function
    End Class
#End Region
#Region "Property  Class - dxWorld "
    ''' <summary>
    ''' Gibt die aktuellen Matricen der Engine zurück
    ''' </summary>
    ''' <remarks></remarks>
    Public Class clsWorld

        ''' <summary>
        ''' ruft die aktuelle World - Matrix ab.
        ''' </summary>
        Public ReadOnly Property World() As Matrix
            Get
                Return _dxCurrentWorldMatrix
            End Get
        End Property
        ''' <summary>
        ''' ruft die aktuell invertierte World - Matrix ab.
        ''' </summary>
        Public ReadOnly Property InverseWorld() As Matrix
            Get
                Return Matrix.Invert(_dxCurrentWorldMatrix)
            End Get
        End Property

        ''' <summary>
        ''' ruft die aktuelle Projection - Matrix ab.
        ''' </summary>
        Public ReadOnly Property Projection() As Matrix
            Get
                Return dxCamera.currentProjectionMatrix
            End Get
        End Property
        ''' <summary>
        ''' ruft die aktuelle View - Matrix ab.
        ''' </summary>
        Public ReadOnly Property View() As Matrix
            Get
                Return dxCamera.currentViewMatrix
            End Get
        End Property
        ''' <summary>
        ''' ruft die aktuelle ViewProjection - Matrix ab.
        ''' </summary>
        Public ReadOnly Property ViewProjection() As Matrix
            Get
                With dxCamera
                    Return .currentViewMatrix * .currentProjectionMatrix
                End With
            End Get
        End Property

        ''' <summary>
        ''' ruft die aktuelle WorldViewProjection - Matrix ab.
        ''' </summary>
        Public ReadOnly Property WorldViewProjection() As Matrix
            Get
                With dxCamera
                    Return _dxCurrentWorldMatrix * .currentViewMatrix * .currentProjectionMatrix
                End With
            End Get
        End Property

        ''' <summary>
        ''' ruft die aktuelle ReflectedWorldViewProjection - Matrix ab.
        ''' </summary>
        ''' <param name="Height">Höhe der Reflektionsebene</param>
        Public ReadOnly Property WorldViewProjectionReflectionY(Optional ByVal Height As Single = 0) As Matrix
            Get
                With dxCamera
                    Return _dxCurrentWorldMatrix * .LookAtMatrixLHReflectionY(Height) * .currentProjectionMatrix
                End With
            End Get
        End Property

        ''' <summary>
        ''' ruft die aktuelle InverseWorldViewProjection - Matrix ab.
        ''' </summary>
        Public ReadOnly Property InverseWorldViewProjection() As Matrix
            Get
                Dim m As Matrix
                With dxCamera
                    m = _dxCurrentWorldMatrix * .currentViewMatrix * .currentProjectionMatrix
                End With
                Return Matrix.Invert(m)
            End Get
        End Property

        ''' <summary>
        ''' ruft die aktuelle InverseViewProjection - Matrix ab.
        ''' </summary>
        Public ReadOnly Property InverseViewProjection() As Matrix
            Get
                Dim m As Matrix
                With dxCamera
                    m = .currentViewMatrix * .currentProjectionMatrix
                End With
                Return Matrix.Invert(m)
            End Get
        End Property
        ''' <summary>
        ''' ruft die aktuelle InverseViewProjection - Matrix ab.
        ''' </summary>
        Public ReadOnly Property InverseTransposedView() As Matrix
            Get
                Dim m As Matrix
                With dxCamera
                    m = .currentViewMatrix
                    m = Matrix.Invert(m)
                    m = Matrix.Transpose(m)
                End With
                Return m
            End Get
        End Property

        ''' <summary>
        ''' ruft die aktuelle InverseViewProjection - Matrix ab.
        ''' </summary>
        Public ReadOnly Property InverseTransposedProjection() As Matrix
            Get
                Dim m As Matrix
                With dxCamera
                    m = .currentProjectionMatrix
                    m = Matrix.Invert(m)
                    m = Matrix.Transpose(m)
                End With
                Return m
            End Get
        End Property

        ''' <summary>
        ''' ruft die aktuelle WorldInverseTranspose - Matrix ab.
        ''' </summary>

        Public ReadOnly Property InverseTransposeWorld() As Matrix
            Get
                Dim m As Matrix = Matrix.Invert(_dxCurrentWorldMatrix)
                Return Matrix.Transpose(m)
            End Get
        End Property
    End Class
#End Region

#Region "Custom Vertex Structure Position"
    ''' <summary>
    ''' VertexStructure Position  fest
    ''' </summary>
    ''' <remarks></remarks>
    Public Structure VertexPosition
        ''' <summary>
        ''' Legt die Vertexposition fest
        ''' </summary>
        Public position As Vector3

        ''' <summary>
        ''' Erstellt eine neue VertexStructure Position,Colore
        ''' </summary>
        ''' <param name="pos">Angabe der Vertex-Position</param>
        ''' <remarks></remarks>
        Public Sub New(ByVal pos As Vector3)
            Me.position = pos
        End Sub
        ''' <summary>
        ''' Rückgabe der Bytegröße des Formats
        ''' </summary>
        Public Shared ReadOnly Property SizeOfBytes() As Integer
            Get
                Return System.Runtime.InteropServices.Marshal.SizeOf(GetType(VertexPosition))
            End Get
        End Property
        ''' <summary>
        ''' Rückbage des Vertexformates
        ''' </summary>
        Public Shared ReadOnly Property FVF() As Direct3D9.VertexFormat
            Get
                Return (VertexFormat.Position)
            End Get
        End Property
    End Structure
#End Region
#Region "Custom Vertex Structure PositionColor"
    ''' <summary>
    ''' VertexStructure Position,Colored fest
    ''' </summary>
    ''' <remarks></remarks>
    Public Structure VertexPositionColor
        ''' <summary>
        ''' Legt die Vertexposition fest
        ''' </summary>
        Public position As Vector3
        ''' <summary>
        ''' Legt die Vertexfarbe fest
        ''' </summary>
        Public color As Integer
        ''' <summary>
        ''' Erstellt eine neue VertexStructure Position,Colore
        ''' </summary>
        ''' <param name="pos">Angabe der Vertex-Position</param>
        ''' <param name="col">Angabe der Vertex-Farbe</param>
        ''' <remarks></remarks>
        Public Sub New(ByVal pos As Vector3, ByVal col As Integer)
            Me.position = pos
            Me.color = col
        End Sub
        ''' <summary>
        ''' Rückgabe der Bytegröße des Formats
        ''' </summary>
        Public Shared ReadOnly Property SizeOfBytes() As Integer
            Get
                Return System.Runtime.InteropServices.Marshal.SizeOf(GetType(VertexPositionColor))
            End Get
        End Property
        ''' <summary>
        ''' Rückbage des Vertexformates
        ''' </summary>
        Public Shared ReadOnly Property FVF() As Direct3D9.VertexFormat
            Get
                Return (VertexFormat.Position Or VertexFormat.Diffuse)
            End Get
        End Property
    End Structure
#End Region
#Region "Custom Vertex Structure PositionTexture"
    ''' <summary>
    ''' VertexStructure Position,Texture
    ''' </summary>
    Public Structure VertexPositionTexture
        ''' <summary>
        ''' Legt die Veretexposition fest
        ''' </summary>
        Public position As Vector3
        ''' <summary>
        ''' Legt die Texturkoordianten fest
        ''' </summary>
        Public texcord As Vector2
        ''' <summary>
        ''' Erstellt eine neue VertexStructure Position,Texture
        ''' </summary>
        ''' <param name="pos">Angabe der Vertex-Position</param>
        ''' <param name="tex">Angabe der Vertex-Texturkoordinate</param>
        ''' <remarks></remarks>
        Public Sub New(ByVal pos As Vector3, ByVal tex As Vector2)
            Me.position = pos
            Me.texcord = tex
        End Sub

        ''' <summary>
        ''' Rückgabe der Bytegröße des Formats
        ''' </summary>
        Public Shared ReadOnly Property SizeOfBytes() As Integer
            Get
                Return System.Runtime.InteropServices.Marshal.SizeOf(GetType(VertexPositionTexture))
            End Get
        End Property
        ''' <summary>
        ''' Rückbage des Vertexformates
        ''' </summary>
        Public Shared ReadOnly Property FVF() As Direct3D9.VertexFormat
            Get
                Return (VertexFormat.Position Or VertexFormat.Texture1)
            End Get
        End Property
    End Structure
#End Region
#Region "Custom Vertex Structure PositionColorTexture"
    ''' <summary>
    ''' VertexStructure Position,Normal,Color,Texture
    ''' </summary>
    Public Structure VertexPositionColorTexture
        ''' <summary>
        ''' Legt die Veretexposition fest
        ''' </summary>
        Public position As Vector3
        ''' <summary>
        ''' Legt die Farbe fest
        ''' </summary>
        Public color As Integer
        ''' <summary>
        ''' Legt die Texturkoordianten fest
        ''' </summary>
        Public texcord As Vector2
        ''' <summary>
        ''' Erstellt eine neue VertexStructure Position,Color,Texture
        ''' </summary>
        ''' <param name="pos">Angabe der Vertex-Position</param>
        ''' <param name="col">Angabe der Vertex-Color</param>
        ''' <param name="tex">Angabe der Vertex-Texturkoordinate</param>
        ''' <remarks></remarks>
        Public Sub New(ByVal pos As Vector3, ByVal col As Integer, ByVal tex As Vector2)
            Me.position = pos
            Me.color = col
            Me.texcord = tex
        End Sub

        ''' <summary>
        ''' Rückgabe der Bytegröße des Formats
        ''' </summary>
        Public Shared ReadOnly Property SizeOfBytes() As Integer
            Get
                Return System.Runtime.InteropServices.Marshal.SizeOf(GetType(VertexPositionColorTexture))
            End Get
        End Property
        ''' <summary>
        ''' Rückbage des Vertexformates
        ''' </summary>
        Public Shared ReadOnly Property FVF() As Direct3D9.VertexFormat
            Get
                Return (VertexFormat.Position Or VertexFormat.Diffuse Or VertexFormat.Texture1)
            End Get
        End Property
    End Structure
#End Region
#Region "Custom Vertex Structure PositionNormalColor"
    ''' <summary>
    ''' VertexStructure Position,Normal,Colore fest
    ''' </summary>
    ''' <remarks></remarks>
    Public Structure VertexPositionNormalColor
        ''' <summary>
        ''' Legt die Vertexposition fest
        ''' </summary>
        Public position As Vector3
        ''' <summary>
        ''' Legt die Vertexnormale
        ''' </summary>
        Public normal As Vector3
        ''' <summary>
        ''' Legt die Vertexfarbe fest
        ''' </summary>
        Public color As Integer
        ''' <summary>
        ''' Erstellt eine neue VertexStructure Position,Normal,Color
        ''' </summary>
        ''' <param name="pos">Angabe der Vertex-Position</param>
        ''' <param name="col">Angabe der Vertex-Farbe</param>
        ''' <param name="nrm">Angabe der Vertex-Normalen</param>
        Public Sub New(ByVal pos As Vector3, ByVal nrm As Vector3, ByVal col As Integer)
            Me.position = pos
            Me.color = col
        End Sub
        ''' <summary>
        ''' Rückgabe der Bytegröße des Formats
        ''' </summary>
        Public Shared ReadOnly Property SizeOfBytes() As Integer
            Get
                Return System.Runtime.InteropServices.Marshal.SizeOf(GetType(VertexPositionNormalColor))
            End Get
        End Property
        ''' <summary>
        ''' Rückbage des Vertexformates
        ''' </summary>
        Public Shared ReadOnly Property FVF() As Direct3D9.VertexFormat
            Get
                Return (VertexFormat.Position Or VertexFormat.Normal Or VertexFormat.Diffuse)
            End Get
        End Property
    End Structure
#End Region
#Region "Custom Vertex Structure PositionNormalTexture"
    ''' <summary>
    ''' VertexStructure Position,Normal,Texture
    ''' </summary>
    Public Structure VertexPositionNormalTexture
        ''' <summary>
        ''' Legt die Veretexposition fest
        ''' </summary>
        Public position As Vector3
        ''' <summary>
        '''  Legt den Normalvektor fest
        ''' </summary>
        Public normal As Vector3
        ''' <summary>
        ''' Legt die Texturkoordianten fest
        ''' </summary>
        Public texcord As Vector2
        ''' <summary>
        ''' Erstellt eine neue VertexStructure Position,Normal,Textured
        ''' </summary>
        ''' <param name="pos">Angabe der Vertex-Position</param>
        ''' <param name="nrm">Angabe der Vertex-Normalen</param>
        ''' <param name="tex">Angabe der Vertex-Texturkoordinate</param>
        ''' <remarks></remarks>
        Public Sub New(ByVal pos As Vector3, ByVal nrm As Vector3, ByVal tex As Vector2)
            Me.position = pos
            Me.normal = nrm
            Me.texcord = tex
        End Sub
        ''' <summary>
        ''' Rückgabe der Bytegröße des Formats
        ''' </summary>
        Public Shared ReadOnly Property SizeOfBytes() As Integer
            Get
                Return System.Runtime.InteropServices.Marshal.SizeOf(GetType(VertexPositionNormalTexture))
            End Get
        End Property
        ''' <summary>
        ''' Rückbage des Vertexformates
        ''' </summary>
        Public Shared ReadOnly Property FVF() As Direct3D9.VertexFormat
            Get
                Return (VertexFormat.Position Or VertexFormat.Normal Or VertexFormat.Texture1)
            End Get
        End Property
    End Structure
#End Region
#Region "Custom Vertex Structure PositionNormalColoreTexture"
    ''' <summary>
    ''' VertexStructure Position,Normal,Color,Texture
    ''' </summary>
    Public Structure VertexPositionNormalColorTexture
        ''' <summary>
        ''' Legt die Veretexposition fest
        ''' </summary>
        Public position As Vector3
        ''' <summary>
        '''  Legt den Normalvektor fest
        ''' </summary>
        Public normal As Vector3
        ''' <summary>
        ''' Legt die Texturkoordianten fest
        ''' </summary>
        Public color As Integer
        ''' <summary>
        ''' Legt die Texturkoordianten fest
        ''' </summary>
        Public texcord As Vector2
        ''' <summary>
        ''' Erstellt eine neue VertexStructure Position,Normal,Color,Texture
        ''' </summary>
        ''' <param name="pos">Angabe der Vertex-Position</param>
        ''' <param name="nrm">Angabe der Vertex-Normalen</param>
        ''' <param name="col">Angabe der Vertex-Color</param>
        ''' <param name="tex">Angabe der Vertex-Texturkoordinate</param>
        ''' <remarks></remarks>
        Public Sub New(ByVal pos As Vector3, ByVal nrm As Vector3, ByVal col As Integer, ByVal tex As Vector2)
            Me.position = pos
            Me.normal = nrm
            Me.color = col
            Me.texcord = tex
        End Sub

        ''' <summary>
        ''' Rückgabe der Bytegröße des Formats
        ''' </summary>
        Public Shared ReadOnly Property SizeOfBytes() As Integer
            Get
                Return System.Runtime.InteropServices.Marshal.SizeOf(GetType(VertexPositionNormalColorTexture))
            End Get
        End Property
        ''' <summary>
        ''' Rückbage des Vertexformates
        ''' </summary>
        Public Shared ReadOnly Property FVF() As Direct3D9.VertexFormat
            Get
                Return (VertexFormat.Position Or VertexFormat.Normal Or VertexFormat.Diffuse Or VertexFormat.Texture1)
            End Get
        End Property
    End Structure
#End Region
#Region "Custom Structure Billboard"
    Public Structure Billboard
        Public posX As Single
        Public posY As Single
        Public posZ As Single
        Public sclA As Single
        Public sclB As Single
        Public sclC As Single
        Public Radius As Single
        Public MinAA As Vector3
        Public MaxBB As Vector3
        Public Sub New(ByVal positionX As Single, ByVal PositionY As Single, ByVal PositionZ As Single, _
                       ByVal ScaleA As Single, ByVal ScaleB As Single, _
                       ByVal ScaleC As Single, _
                       ByVal FrustumSphereRadius As Single, _
                       ByVal FrustumMinAA As Vector3, _
                       ByVal FrustumMaxBB As Vector3)
            posX = positionX
            posY = PositionY
            posZ = PositionZ
            sclA = ScaleA
            sclB = ScaleB
            sclC = ScaleC
            Radius = FrustumSphereRadius
            MinAA = FrustumMinAA
            MaxBB = FrustumMaxBB
        End Sub
        Public Property Position() As Vector3
            Get
                Return New Vector3(posX, posY, posZ)
            End Get
            Set(ByVal value As Vector3)
                posX = value.X
                posY = value.Y
                posZ = value.Z
            End Set
        End Property
        '   Public ReadOnly Property Scale() As Vector2
        '     Get
        'Return New Vector2(sclA, sclB)
        '       End Get
        '  End Property
    End Structure
#End Region
#Region "Custom Structure Billboard"
    Public Enum VegetationType As Integer
        Grass = 0
        Plant1 = 1
        Plant2 = 2
        Plant3 = 3
        Plant4 = 4
        Plant5 = 5
        Plant6 = 6
        Tree1 = 7
        Tree2 = 8
        Tree3 = 9
        Tree4 = 10
        Palm1 = 11
        Palm2 = 12
        Palm3 = 13
        Palm4 = 14
        RandomPlant = 15
        RandomTree = 16
        RandomPalm = 17
        RandomJungle = 18
    End Enum

    Public Structure BillboardVegetationVertex
        Public Position As Vector3
        Public TexCoords0 As Vector4
        Public TexCoords1 As Vector2
        Public Scaling As Vector4
    End Structure

    Public Structure BillboardVegetation

        Public VertexStruxture As BillboardVegetationVertex
        Public FrustumCenterBottom As Vector3
        Public FrustumCenterTop As Vector3
        Public InvSclW As Single
        Public RndVal As Single
        Public FrustumMinAA As Vector3
        Public FrustumMaxBB As Vector3
        Public Type As VegetationType
        Public IsBillboard As Boolean

        Public FrustumRad As Single

        Public Sub New(ByVal posX As Single, ByVal posY As Single, ByVal posZ As Single, _
                       ByVal sclX As Single, ByVal sclY As Single, ByVal sclZ As Single, ByVal sclW As Single, _
                       ByVal rndValue As Single, _
                       ByVal vegType As VegetationType)

            VertexStruxture.Position = New Vector3(posX, posY, posZ)
            VertexStruxture.Scaling = New Vector4(sclX, sclY, sclZ, sclW)
            InvSclW = CSng(sclW * -0.9)

            RndVal = rndValue

            FrustumCenterBottom = VertexStruxture.Position
            FrustumCenterTop = FrustumCenterBottom + New Vector3(0, sclY, 0)

            FrustumRad = CSng(Math.Sqrt(2) * 0.5) * sclX
            FrustumMinAA = VertexStruxture.Position + New Vector3(sclX, 0, sclX)
            FrustumMaxBB = VertexStruxture.Position - New Vector3(sclX, 0, sclX) + New Vector3(0, sclY, 0)

            Type = vegType
        End Sub
    End Structure
#End Region

#Region "Custom Structure RGBAcolor"
    Public Structure RGBAcolor
        Public R As Single
        Public G As Single
        Public B As Single
        Public A As Single
        Public Sub New(ByVal rgb As Single)
            R = rgb
            G = rgb
            B = rgb
            A = 1
        End Sub
        Public Sub New(ByVal rgb As Integer)
            R = CSng(rgb / 255)
            G = CSng(rgb / 255)
            B = CSng(rgb / 255)
            A = 1
        End Sub
        Public Sub New(ByVal Red As Single, ByVal Green As Single, ByVal Blue As Single, Optional ByVal Alpha As Single = 1)
            R = Red
            G = Green
            B = Blue
            A = Alpha
        End Sub
        Public Sub New(ByVal Red As Integer, ByVal Green As Integer, ByVal Blue As Integer, Optional ByVal Alpha As Integer = 255)
            R = CSng(Red / 255)
            G = CSng(Green / 255)
            B = CSng(Blue / 255)
            A = CSng(Alpha / 255)
        End Sub
    End Structure
#End Region

#Region "IComparer Class - SortVector3"
    ''' <summary>
    ''' Sortiert eine ListOf Vector3 nach Abstand zur Kameraposition
    ''' </summary>
    ''' <remarks></remarks>
    Public Class SortVector3byDistance
        Implements IComparer(Of Vector3)
        Private t As Vector3 = dxCamera.currentPosition

        ''' <summary>
        ''' Sortiert eine ListOf Vector3 nach Abstand zur Kameraposition
        ''' </summary>
        ''' <param name="Y">Legt fest ob die Y Componente mit berechnet wird</param>

        Public Sub New(Optional ByVal Y As Boolean = False)
            If Y = False Then t.Y = 0
        End Sub
        Public Function Compare(ByVal x As SlimDX.Vector3, ByVal y As SlimDX.Vector3) As Integer Implements System.Collections.Generic.IComparer(Of SlimDX.Vector3).Compare
            Dim lx As Single = (x - t).LengthSquared
            Dim ly As Single = (y - t).LengthSquared
            Select Case lx
                Case Is > ly
                    Return -1
                Case Is < ly
                    Return 1
                Case Is = 0
                    Return 0
            End Select
        End Function
    End Class
#End Region
#Region "IComparer Class - SortBillboard"
    ''' <summary>
    ''' Sortiert eine ListOf Vector3 nach Abstand zur Kameraposition
    ''' </summary>
    ''' <remarks></remarks>
    Public Class SortBillboardByDistance
        Implements IComparer(Of Billboard)
        Private t As Vector3 = dxCamera.currentPosition

        ''' <summary>
        ''' Sortiert eine ListOf Vector3 nach Abstand zur Kameraposition
        ''' </summary>
        ''' <param name="Y">Legt fest ob die Y Componente mit berechnet wird</param>

        Public Sub New(Optional ByVal Y As Boolean = False)
            If Y = False Then t.Y = 0
        End Sub
        Public Function Compare(ByVal x As Billboard, ByVal y As Billboard) As Integer Implements System.Collections.Generic.IComparer(Of Billboard).Compare
            Dim lx As Single = (x.Position - t).LengthSquared
            Dim ly As Single = (y.Position - t).LengthSquared
            Select Case lx
                Case Is > ly
                    Return -1
                Case Is < ly
                    Return 1
                Case Is = 0
                    Return 0
            End Select
        End Function
    End Class
#End Region
#Region "IComparer Class - SortBillboardVegetation"
    ''' <summary>
    ''' Sortiert eine ListOf Vector3 nach Abstand zur Kameraposition
    ''' </summary>
    ''' <remarks></remarks>
    Public Class SortBillboardVegByDistance
        Implements IComparer(Of BillboardVegetation)
        Private t As New Vector3(dxCamera.currentPosition.X, dxCamera.currentPosition.Y, dxCamera.currentPosition.Z)
        Private _lx As Integer
        Private _ly As Integer
        Private lx As Single
        Private ly As Single

        ''' <summary>
        ''' Sortiert eine ListOf Vector3 nach Abstand zur Kameraposition
        ''' </summary>
        ''' <param name="Y">Legt fest ob die Y Componente mit berechnet wird</param>
        ''' <param name="Invert">Invertiert die Sortierung</param>
        Public Sub New(Optional ByVal Invert As Boolean = False, Optional ByVal Y As Boolean = False)
            If Y = False Then t.Y = 0
            If Invert Then
                _lx = 1
                _ly = -1
            Else
                _lx = -1
                _ly = 1
            End If
        End Sub
        Public Function Compare(ByVal x As BillboardVegetation, ByVal y As BillboardVegetation) As Integer Implements System.Collections.Generic.IComparer(Of BillboardVegetation).Compare
            Try

                lx = (x.VertexStruxture.Position - t).LengthSquared
                ly = (y.VertexStruxture.Position - t).LengthSquared

                If lx > ly Then Return _lx
                If lx < ly Then Return _ly

                Return 0

            Catch ex As Exception
                MsgBox(ex)
            End Try
        End Function
    End Class
#End Region

#Region "InitializeComponent for Designer"
    Private Sub InitializeComponent()
        Dim resources As System.ComponentModel.ComponentResourceManager = New System.ComponentModel.ComponentResourceManager(GetType(SDX9BaseEngine))
        Me.SuspendLayout()
        Me.ClientSize = New System.Drawing.Size(256, 256)
        Me.Icon = CType(resources.GetObject("$this.Icon"), System.Drawing.Icon)
        Me.Name = "baseEngine"
        Me.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen
        Me.ResumeLayout(True)
    End Sub
#End Region

End Class