﻿
#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region




Imports SlimDX
Imports SlimDX.Direct3D9
Imports sdxBaseEngine.SDX9BaseEngine

    '''<summary>
'''Camera Klasse zur Erstellung einer Kamera
    '''</summary>


Public Class engineCamera

    Private Structure directions
        Public rotateLeftRight As Single
        Public rotateUpDown As Single
        Public moveForwardBackward As Single
    End Structure


    Public Shared Event dxIntersectionTest(ByVal position As Vector3)

    Private _camType As CamType
    Private _camUp As New Vector3(0, 1, 0)
    Private _camTransformedUp As New Vector3(0, 1, 0)
    Private _camtiming As Single = 1
    Private _camSpeed As Vector2 = New Vector2(1, 1)

    Private _camRotX As Single
    Private _camRotY As Single
    Private _camRotZ As Single
    Private _camRot As Vector3

    Private _camFPersonRot As Vector2
    Private _camFPersonHeight As Single
    Private _camFPersonRotMatrix As Matrix
    Private _camFPersonRefTarget As New Vector3(0, 0, 1)
    Private _camFPersonRotTarget As Vector3
    Private _camFPersonLookClamp As Vector2 = New Vector2(-89, 89)
    Private _camFPersonIntersectTest As Boolean

    Private _camLerpVec3 As Vector3
    Private _camLerpVec2 As Vector2
    Private _camTargetLerp As Vector3
    Private _camSmoothing As Single
    Private _camUseSmoothing As Boolean

    Private _camAngle As Single



    Private _IntersectionTest As sdxMeshIntersection.TriangleIntersection
    Private _camMoveMatrix As Matrix
    Private _camRotationMatrix As Matrix

    Private _camPosition As New Vector3(0)
    Private _camPositionTmp As Vector3
    Private _camTarget As New Vector3(0, 0, 0)

    Private _camTransformedTarget As New Vector3(0)

    Private _camMovement As New Vector3(0, 0, 1)
    Private _camBehind As New Vector3(0, 0, -2)

    Private _camFar As Single = 500
    Private _camFarBillboard As Single = 5
    Private _camNear As Single = 0.5
    Private _camDirections As directions

    '#Autocams
    Private _caminterpolation As Single
    Private _camEndVector As Vector3
    Private _camStartVector As Vector3
    Private _camCurved As Boolean
    Private _camHeight As Single
    Private _camRadius As Single
    Private _camCenter As Vector3
    Private _camMaxHeight As Single
    Private _camCirclePos As Vector2

    '#ViewFrustum
    Private _frustumPlane(6) As Plane
    Private _frustumCorner(7) As Vector3
    Private _ViewProjection As Matrix
    Private _ViewProjectionBillboard As Matrix

    '#Projection
    Private _projectTang As Single
    Private _projectRatio As Single
    Private _curViewMatrix As Matrix
    Private _ViewMatrix As Matrix

    Private _curProjectionMatrix As Matrix
    Private _ProjectionMatrix As Matrix

    '''<summary>
    '''Erstellt einen neue Kameraklasse
    '''</summary>
    ''' <param name="projectionX" >Angabe X - Auflösung zur Erstellung der Aspect ratio</param>
    ''' <param name="projectionY" >Angabe Y - Auflösung zur Erstellung der Aspect ratio</param>
    Public Sub New(ByVal projectionX As Single, ByVal projectionY As Single)
        _projectRatio = projectionX / projectionY
        _projectTang = CSng(Math.PI / 4)
        dxDevice.SetTransform(TransformState.World, Matrix.Identity)
        _camType = CamType.fixed
        RestoreProjectionMatrix()
    End Sub
#Region "set Projection"
    Public Sub RestoreProjectionMatrix()
        _ProjectionMatrix = Matrix.PerspectiveFovLH(_projectTang, _projectRatio, _camNear, _camFar)
        _curProjectionMatrix = _ProjectionMatrix
        setProjectionMatrix()
    End Sub

    ''' <summary>
    ''' Übergibt die ProjektionMatrix an den Device.
    ''' Die Matrix wird in Abhängigkeit der Backbuffergröße erstellt.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub setProjectionMatrix()
        dxDevice.SetTransform(TransformState.Projection, _ProjectionMatrix)
    End Sub
#End Region
#Region "LookAtMatrix"
    ''' <summary>
    ''' Ruft die LookAtMatrix ab (LH)
    ''' </summary>
    Public ReadOnly Property LookAtMatrixLH() As Matrix
        Get
            Return Matrix.LookAtLH(_camPosition, _camTarget, _camUp)
        End Get
    End Property
    ''' <summary>
    ''' Ruft die reflektierte LookAtMatrix ab (LH)
    ''' </summary>
    ''' <param name="Height"></param>
    ''' <value>Spiegelungspunkt auf der Y-Achse</value>
    Public ReadOnly Property LookAtMatrixLHReflectionY(ByVal Height As Single) As Matrix
        Get
            Dim refPosition = _camPosition
            Dim refCamTarget As Vector3 = _camTarget
            refPosition.Y = -_camPosition.Y + Height + Height
            refCamTarget.Y = -_camTarget.Y + Height + Height
            Return Matrix.LookAtLH(refPosition, refCamTarget, _camUp)
        End Get
    End Property

#End Region
#Region "Cam Update"
    '''<summary>
    '''Update der Kameraeinstellungen (interne Nutzung)
    '''</summary>
    Public Sub update()

        Select Case _camType
            Case CamType.fixed
                CamTypeFixed()
            Case CamType.firstPerson
                CamTypeFirstPerson()
            Case CamType.moving
                CamTypeMoving()
            Case CamType.rotate
                CamTypeRotate()
            Case CamType.mouseRotate
                CamTypeMouseRotate()
        End Select

        '#update view
        _ViewMatrix = LookAtMatrixLH
        Me.currentViewMatrix = _ViewMatrix

        Me.createViewFrustum()
    End Sub
#End Region
#Region "CamType: Fixed"
    Private Sub CamTypeFixed()
    End Sub
#End Region
#Region "CamType: moving"
    Private Sub CamTypeMoving()
        _caminterpolation = CSng(dxGametime.TimebasedSinus(_camSpeed.X, 0.01))
        _camEndVector = _camMovement
        If _camCurved Then _camEndVector.Y = CSng(_camStartVector.Y * (1 - _caminterpolation) + (_camMovement.Y * Math.Sign(_caminterpolation)))
        _camPosition = Vector3.Lerp(_camStartVector, _camEndVector, _
                                              _caminterpolation)
    End Sub
#End Region
#Region "Camtype: rotate"
    Private Sub CamTypeRotate()
        _camCirclePos = dxGametime.TimebasedCircle(_camAngle)
        If _camCurved Then
            Dim y As Single = CSng(dxGametime.Timefading(0.02))
            dxCamera.currentPosition = New Vector3(_camCirclePos.X * _camRadius, y * _camMaxHeight + _camHeight, _camCirclePos.Y * _camRadius) + _camCenter
        Else
            dxCamera.currentPosition = New Vector3(_camCirclePos.X * _camRadius, _camHeight, _camCirclePos.Y * _camRadius) + _camCenter
        End If
    End Sub
#End Region
#Region "CamType: First Person"

    Private Sub CamTypeFirstPerson()

        _camtiming = dxGametime.Frametime

        Dim v As Vector2
        v.X = CSng(_camSpeed.X * 0.3)
        v.Y = CSng(_camSpeed.Y * 0.3)

        _camFPersonRot.Y += _camDirections.rotateLeftRight * v.Y
        _camFPersonRot.X += _camDirections.rotateUpDown * v.X
        _camFPersonRot.X = clamp(_camFPersonRot.X, _camFPersonLookClamp.X, _camFPersonLookClamp.Y)

        '#smooth rotations
        _camLerpVec2 = Vector2.Lerp(_camLerpVec2, _camFPersonRot, CSng(0.02 * _camtiming))

        '#create rotation Matrix
        _camFPersonRotMatrix = Matrix.RotationX(Radiant(_camLerpVec2.X)) * Matrix.RotationY(Radiant(_camLerpVec2.Y))
        _camFPersonRotTarget = Vector3.TransformCoordinate(_camFPersonRefTarget, _camFPersonRotMatrix)

        '#rotate movement
        _camMoveMatrix = Matrix.RotationY(Radiant(_camLerpVec2.Y))
        _camMovement = Vector3.TransformCoordinate(New Vector3(0, 0, 1), _camMoveMatrix)

        '#IntersectionTest if enabled
        If _camFPersonIntersectTest Then
            _camPositionTmp = _camPosition + _camMovement * (_camDirections.moveForwardBackward * _camtiming)

            RaiseEvent dxIntersectionTest(_camPositionTmp)

            If _IntersectionTest.Intersected Then
                _camPositionTmp = _IntersectionTest.Vector
                _camPositionTmp.Y += _camFPersonHeight
                _camPosition.X = _camPositionTmp.X
                _camPosition.Z = _camPositionTmp.Z
            End If
            _camPosition.Y = Vector3.Lerp(_camPosition, _camPositionTmp, CSng(0.25)).Y
        Else
            _camPosition += _camMovement * (_camDirections.moveForwardBackward * _camtiming)
        End If
        _camTarget = _camPosition + _camFPersonRotTarget
        _camDirections.moveForwardBackward = 0
    End Sub

    ''' <summary>
    ''' Legt die Kamerabewegung über den KeyboardDevice fest.
    ''' </summary>
    Public WriteOnly Property useKeyboardInput(Optional ByVal Speed As Single = 1) As CamMovements
        Set(ByVal value As CamMovements)
            Select Case value
                Case CamMovements.Left
                    _camDirections.rotateLeftRight = -Speed
                Case CamMovements.Right
                    _camDirections.rotateLeftRight = Speed
                Case CamMovements.Up
                    _camDirections.rotateUpDown = -Speed
                Case CamMovements.Down
                    _camDirections.rotateUpDown = Speed
                Case CamMovements.Forward
                    _camDirections.moveForwardBackward = CSng(Speed * 0.007)
                Case CamMovements.Backward
                    _camDirections.moveForwardBackward = CSng(-Speed * 0.007)
            End Select
        End Set
    End Property
    ''' <summary>
    ''' Legt die Rotation der Kamera über den MouseDevice fest.
    ''' </summary>
    ''' <param name="Speed">Legt die Geschwindigkeitsanpassung fest</param>
    Public WriteOnly Property useMouseRotation(Optional ByVal Speed As Single = 1) As Vector2
        Set(ByVal value As Vector2)
            _camDirections.rotateLeftRight = value.X * Speed
            _camDirections.rotateUpDown = value.Y * Speed
        End Set
    End Property


    ''' <summary>
    ''' Legt den Interpolationswert fest.
    ''' Werte zwischen 0 und 1 sind zulässig.
    ''' </summary>
    Public Property SmoothingSpeed() As Single
        Get
            Return _camSmoothing
        End Get
        Set(ByVal value As Single)
            _camSmoothing = value
        End Set
    End Property

#End Region
#Region "CamType: MouseRotate"
    Private Sub CamTypeMouseRotate()
        _camtiming = dxGametime.Frametime

        Dim v As Vector2
        v.X = CSng(_camSpeed.X * 0.01)
        v.Y = CSng(_camSpeed.Y * 0.01)

        _camFPersonRot.Y += _camDirections.rotateLeftRight * v.Y
        _camFPersonRot.X += _camDirections.rotateUpDown * v.X
        _camFPersonRot.X = clamp(_camFPersonRot.X, _camFPersonLookClamp.X, _camFPersonLookClamp.Y)

        '#smooth rotations
        _camLerpVec2 = Vector2.Lerp(_camLerpVec2, _camFPersonRot, CSng(0.01 * _camtiming))


        '//
        '// New Position
        '// ------------
        _camRot.X = CSng(Math.Cos(Radiant(_camLerpVec2.Y))) * _camRadius
        _camRot.Z = CSng(Math.Sin(Radiant(_camLerpVec2.Y))) * _camRadius

        _camTarget = New Vector3(_camCenter.X, -_camLerpVec2.X, _camCenter.Y)
        _camPosition = New Vector3(_camRot.X, _camHeight, _camRot.Z)

    End Sub
    ''' <summary>
    ''' Legt die Y-Rotation der Kamera über den MouseDevice fest.
    ''' </summary>
    Public WriteOnly Property useMouseUpDown() As Vector2
        Set(ByVal value As Vector2)
            _camDirections.rotateUpDown = value.Y
        End Set
    End Property

    ''' <summary>
    ''' Legt die X-Rotation der Kamera über den MouseDevice fest.
    ''' </summary>
    Public WriteOnly Property useMouseLeftRight() As Vector2
        Set(ByVal value As Vector2)
            _camDirections.rotateLeftRight = value.X
        End Set
    End Property
#End Region

#Region "Properties"
    Public ReadOnly Property rightVector() As Vector3
        Get
            Dim m As Matrix = _ViewMatrix
            Dim v As New Vector3(m.M11, m.M21, m.M31)
            Return Vector3.Normalize(v)
        End Get
    End Property

    Public ReadOnly Property upVector() As Vector3
        Get
            Dim v As New Vector3(_ViewMatrix.M12, _ViewMatrix.M22, _ViewMatrix.M32)
            Return Vector3.Normalize(v)
        End Get
    End Property

    ''' <summary>
    ''' Gibt die inizialisierte ProjektionMatrix invertiert zurück
    ''' </summary>
    Public ReadOnly Property InvertedProjectionMatrix() As Matrix
        Get
            Return Matrix.Invert(_ProjectionMatrix)
        End Get
    End Property
    ''' <summary>
    ''' Rückgabe der ProjektionMatrix
    ''' </summary>
    Public ReadOnly Property ProjectionMatrix() As Matrix
        Get
            Return _ProjectionMatrix
        End Get
    End Property
    ''' <summary>
    ''' Rückgabe der ViewMatrix
    ''' </summary>
    Public ReadOnly Property ViewMatrix() As Matrix
        Get
            Return _ViewMatrix
        End Get
    End Property
    ''' <summary>
    ''' Rückgabe der 8 Ecken des ViewFrustums in der Reihenfolge: 
    ''' ntl, ntr, nbl, nbr - ftl, ftr, fbl, fbr
    ''' </summary>
    Public Function getViewFrustumCorners() As Vector3()
        Return _frustumCorner
    End Function
    ''' <summary>
    ''' Rückgabe der 8 Ecken des ViewFrustums in der Reihenfolge: 
    ''' ntl, ntr, nbl, nbr - ftl, ftr, fbl, fbr
    ''' </summary>
    ''' <param name="M">Legt die TransformCoordinate Matrix fest</param>
    Public Function getViewFrustumCorners(ByVal M As Matrix) As Vector3()
        Dim v(7) As Vector3
        For i = 0 To 7
            v(i) = Vector3.TransformCoordinate(_frustumCorner(i), M)
        Next
        Return v
    End Function

    ''' <summary>
    ''' Ruft die aktuell benutze ProjectionMatrix ab, oder legt diese fest.
    ''' </summary>
    Public Property currentProjectionMatrix() As Matrix
        Set(ByVal value As Matrix)
            _curProjectionMatrix = value
            dxDevice.SetTransform(TransformState.Projection, _ProjectionMatrix)
        End Set
        Get
            Return _curProjectionMatrix
        End Get
    End Property

    ''' <summary>
    ''' Ruft die aktuell gespeicherte ViewMatrix ab, oder legt diese fest.
    ''' </summary>
    Public Property currentViewMatrix() As Matrix
        Set(ByVal value As Matrix)
            _curViewMatrix = value
            dxDevice.SetTransform(TransformState.View, _curViewMatrix)
        End Set
        Get
            Return _curViewMatrix
        End Get
    End Property

    '''<summary>
    ''' Ruft den Wert des IntersectionTest ab oder legt diesen fest.
    ''' Über den Event dxIntersectionTest wird ein sdxMeshIntersection getestet.
    ''' Wird True zurückgegeben, befindet sich die Camerea noch im beweglichen Raum.
    ''' Das sdxIntersectionMesh testet gegen Ground (0,-1,0).
    '''</summary>
    Public Property IntersectionData() As sdxMeshIntersection.TriangleIntersection
        Get
            Return _IntersectionTest
        End Get
        Set(ByVal value As sdxMeshIntersection.TriangleIntersection)
            _IntersectionTest = value
        End Set
    End Property

    ''' <summary>
    ''' Legt fest ob ein IntersectionTest durchgeführt wird oder nicht.
    ''' Wenn "True" werden geladene sdxIntersectionMesh Objekte geprüft
    ''' </summary>
    ''' <value></value>
    Public Property IntersectionTestEnable() As Boolean
        Get
            Return _camFPersonIntersectTest
        End Get
        Set(ByVal value As Boolean)
            _camFPersonIntersectTest = value
        End Set
    End Property
    ''' <summary>
    ''' Ruft den aktuellen Kameratypen ab oder legt diesen fest.
    ''' </summary>
    Public Property currentType() As CamType
        Get
            Return _camType
        End Get
        Set(ByVal value As CamType)
            _camType = value
        End Set
    End Property

    '''<summary>
    '''Ruft die aktuelle Kameraposition ab oder legt diese fest.
    '''</summary>
    Public Property currentPosition() As Vector3
        Get
            Return _camPosition
        End Get
        Set(ByVal value As Vector3)
            _camPosition = value
        End Set
    End Property
    '''<summary>
    '''Ruft die aktuelle Kameraposition ohne Y ab.
    '''</summary>
    Public ReadOnly Property currentPositionXZ() As Vector3
        Get
            Return New Vector3(_camPosition.X, 0, _camPosition.Z)
        End Get
    End Property


    ''' <summary>
    ''' Ruft den Text zur Kamerainformation ab.
    ''' </summary>
    Public ReadOnly Property InfoText() As String
        Get
            Return vbCrLf & _
                   "Cam Type" & vbCrLf & _
                   "Cam Target" & vbCrLf & _
                   "Cam Position" & vbCrLf

        End Get
    End Property

    ''' <summary>
    ''' Ruft die Daten zur Kamerainformat ab.
    ''' </summary>
    Public ReadOnly Property InfoData() As String
        Get
            Return vbCrLf & _
                       ":  " & _camType.ToString & vbCrLf & _
                       ":  " & _camTarget.ToString & vbCrLf & _
                       ":  " & _camPosition.ToString & vbCrLf

        End Get
    End Property

    '''<summary>
    '''Ruft das aktuelle Kameraziel ab oder legt diese fest.
    '''</summary>
    Public Property currentTarget() As Vector3
        Get
            Return _camTarget
        End Get
        Set(ByVal value As Vector3)
            _camTarget = value
        End Set
    End Property

    ''' <summary>
    ''' Ruft die aktuelle Kamerageschwindigkeit ab oder legt diese fest.
    ''' </summary>
    Public Property currentSpeed() As Vector2
        Get
            Return _camSpeed
        End Get
        Set(ByVal value As Vector2)
            _camSpeed = value
        End Set
    End Property

    '''<summary>
    '''Ruft die Sichteweitenbegrenzung der Kamera ab oder legt diese fest (vordere Begrenzung).
    '''</summary>
    Public Property currentPlaneNear() As Single
        Get
            Return _camNear
        End Get
        Set(ByVal value As Single)
            _camNear = value
            RestoreProjectionMatrix()
        End Set
    End Property

    '''<summary>
    '''Ruft die Sichteweitenbegrenzung der Kamera ab oder legt diese fest (hintere Begrenzung).
    '''</summary>
    Public Property currentPlaneFar() As Single
        Get
            Return _camFar
        End Get
        Set(ByVal value As Single)
            _camFar = value
            RestoreProjectionMatrix()
        End Set
    End Property



    ''' <summary>
    ''' Legt ein neues CameraSet fest.
    ''' </summary>
    ''' <param name="CameraSet">
    ''' Legt das zu verwendende CamearSet fest</param>
    Public Sub useCameraSet(ByVal CameraSet As cameraSetFixed)
        With CameraSet
            _camType = .CamType
            _camPosition = .Position
            _camTarget = .Target
        End With
    End Sub
    ''' <summary>
    ''' Legt ein neues CameraSet fest
    ''' </summary>
    ''' <param name="CameraSet">
    ''' Legt das zu verwendende CamearSet fest</param>
    Public Sub useCameraSet(ByVal CameraSet As cameraSetMoving)
        With CameraSet
            _camType = .CamType
            _camStartVector = .StartPoint
            _camTarget = .Target
            _camCurved = .Curved
            _camMovement = .Movement
            _camSpeed = .Speed
        End With
    End Sub
    ''' <summary>
    ''' Legt ein neues CameraSet fest
    ''' </summary>
    ''' <param name="CameraSet">
    ''' Legt das zu verwendende CamearSet fest</param>
    Public Sub useCameraSet(ByVal CameraSet As cameraSetMouseRotate)
        With CameraSet
            _camType = .CamType
            _camHeight = .Height
            _camRadius = .Radius
            _camUseSmoothing = .useSmoothRotations
            _camSmoothing = .SmoothingSpeed
            _camSpeed = .Speed
            _camFPersonLookClamp = .LookClamp
        End With
    End Sub

    ''' <summary>
    ''' Legt ein neues CameraSet fest
    ''' </summary>
    ''' <param name="CameraSet">
    ''' Legt das zu verwendende CamearSet fest</param>
    Public Sub useCameraSet(ByVal CameraSet As cameraSetRotate)
        With CameraSet
            _camType = .CamType
            _camHeight = .Height
            _camTarget = .Target
            _camSpeed = .RotationSpeed
            _camRadius = .Radius
            _camAngle = .Angle
            _camCenter.X = .Center.X
            _camCenter.Z = .Center.Y
            _camCurved = .Curved
            _caminterpolation = .CurvedSpeed
            _camMaxHeight = .maxHeight
        End With
    End Sub
    ''' <summary>
    ''' Legt ein neues CameraSet fest
    ''' </summary>
    ''' <param name="CameraSet">
    ''' Legt das zu verwendende CamearSet fest</param>
    Public Sub useCameraSet(ByVal CameraSet As cameraSetFirstPerson)
        With CameraSet
            _camType = .CamType
            _camFPersonLookClamp = .ClampRotation
            _camPosition = .Position
            _camFPersonHeight = .Position.Y
            _camSpeed = .Speed
            _camSmoothing = .SmoothingSpeed
        End With
    End Sub
#End Region

#Region "Frustum Culling"

    Private Sub createViewFrustum()
        '#new ViewProjection
        _ViewProjection = _ViewMatrix * _ProjectionMatrix

        With _ViewProjection
            '#Left Plane
            _frustumPlane(0) = New Plane(.M14 + .M11, .M24 + .M21, .M34 + .M31, .M44 + .M41)
            '#Right Plane
            _frustumPlane(1) = New Plane(.M14 - .M11, .M24 - .M21, .M34 - .M31, .M44 - .M41)
            '#Near Plane
            _frustumPlane(2) = New Plane(.M13, .M23, .M33, .M43)
            '#Far Plane
            _frustumPlane(3) = New Plane(.M14 - .M13, .M24 - .M23, .M34 - .M33, .M44 - .M43)
            '#Top Plane
            _frustumPlane(4) = New Plane(.M14 - .M12, .M24 - .M22, .M34 - .M32, .M44 - .M42)
            '#Bottom Plane
            _frustumPlane(5) = New Plane(.M14 + .M12, .M24 + .M22, .M34 + .M32, .M44 + .M42)
        End With

        '#Far Plane to switch to Billboard
        _ViewProjectionBillboard = _ViewMatrix * Matrix.PerspectiveFovLH(_projectTang, _projectRatio, 1, 40)
        With _ViewProjectionBillboard

            _frustumPlane(6) = New Plane(.M14 - .M13, .M24 - .M23, .M34 - .M33, .M44 - .M43)
        End With

        '#Normalize all Planes
        For i As Integer = 0 To 6
            _frustumPlane(i).Normalize()
        Next


        '#Corners of ViewFrustum
        Dim nc, fc, X, Y, Z As Vector3

        'compute the Z axis of camera
        'this axis points in the opposite direction from 
        'the looking direction

        Z = _camPosition - _camTarget
        Z.Normalize()

        'X axis of camera with given "up" vector and Z axis
        X = Vector3.Cross(_camUp, Z)
        X.Normalize()

        'the real "up" vector is the cross product of Z and X
        Y = Vector3.Cross(X, Z)

        'compute the centers of the near and far planes
        nc = _camPosition - Z * _camNear
        fc = _camPosition - Z * _camFar

        'compute width and height of the near and far plane sections

        Dim nh As Single = _camNear * _projectTang
        Dim nw As Single = nh * _projectRatio
        Dim fh As Single = _camFar * _projectRatio
        Dim fw As Single = fh * _projectTang

        'compute the 4 corners of the frustum on the near plane
        _frustumCorner(0) = nc + Y * nh - X * nw 'ntl
        _frustumCorner(1) = nc + Y * nh + X * nw 'ntr
        _frustumCorner(2) = nc - Y * nh - X * nw 'nbl
        _frustumCorner(3) = nc - Y * nh + X * nw 'nbr

        'compute the 4 corners of the frustum on the far plane
        _frustumCorner(4) = fc + Y * fh - X * fw 'ftl
        _frustumCorner(5) = fc + Y * fh + X * fw 'ftr
        _frustumCorner(6) = fc - Y * fh - X * fw 'fbl
        _frustumCorner(7) = fc - Y * fh + X * fw 'fbr




    End Sub

    Public Function FrustumTestLeft(ByVal position As Vector3, Optional ByVal Radius As Single = 0) As Boolean
        Dim vec4 As New Vector4(position.X, position.Y, position.Z, 1)
        If (Plane.Dot(_frustumPlane(0), vec4) + Radius) > 0 Then Return True
    End Function
    Public Function FrustumTestRight(ByVal position As Vector3, Optional ByVal Radius As Single = 0) As Boolean
        Dim vec4 As New Vector4(position.X, position.Y, position.Z, 1)
        If (Plane.Dot(_frustumPlane(1), vec4) + Radius) > 0 Then Return True
    End Function
    Public Function FrustumTestNear(ByVal position As Vector3, Optional ByVal Radius As Single = 0) As Boolean
        Dim vec4 As New Vector4(position.X, position.Y, position.Z, 1)
        If (Plane.Dot(_frustumPlane(2), vec4) + Radius) > 0 Then Return True
    End Function
    Public Function FrustumTestFar(ByVal position As Vector3, Optional ByVal Radius As Single = 0) As Boolean
        Dim vec4 As New Vector4(position.X, position.Y, position.Z, 1)
        If (Plane.Dot(_frustumPlane(3), vec4) + Radius) > 0 Then Return True
    End Function
    Public Function FrustumTestTop(ByVal position As Vector3, Optional ByVal Radius As Single = 0) As Boolean
        Dim vec4 As New Vector4(position.X, position.Y, position.Z, 1)
        If (Plane.Dot(_frustumPlane(4), vec4) + Radius) > 0 Then Return True
    End Function
    Public Function FrustumTestBottom(ByVal position As Vector3, Optional ByVal Radius As Single = 0) As Boolean
        Dim vec4 As New Vector4(position.X, position.Y, position.Z, 1)
        If (Plane.Dot(_frustumPlane(5), vec4) + Radius) > 0 Then Return True
    End Function
    Public Function FrustumTestFarBillboard(ByVal position As Vector3, Optional ByVal Radius As Single = 0) As Boolean
        Dim vec4 As New Vector4(position.X, position.Y, position.Z, 1)
        If (Plane.Dot(_frustumPlane(6), vec4) + Radius) > 0 Then Return True
    End Function


    Public Function FrustumTestAll(ByVal position As Vector3) As Boolean
        Dim vec4 As New Vector4(position.X, position.Y, position.Z, 1)
        For i As Integer = 0 To 5
            If (Plane.Dot(_frustumPlane(i), vec4)) > 0 Then Return True
        Next i
    End Function

    ''' <summary>
    ''' Prüft ob sich eine Bounding-Sphere im Sichtfeld der Kamera befindet.
    ''' </summary>
    ''' <param name="position">Angabe der Position der Sphere</param>
    ''' <param name="radius">Angabe des Radius der Sphere</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function FrustumTestSphere(ByVal position As Vector3, ByVal radius As Single) As Boolean
        Dim vec4 As New Vector4(position.X, position.Y, position.Z, 1)
        For i As Integer = 0 To 5
            If (Plane.Dot(_frustumPlane(i), vec4) + radius) > 0 Then Return True
        Next
    End Function
#End Region


#Region "Enum Camtypes"
    ''' <summary>
    ''' Legt den Camera Typ fest
    ''' </summary>
    Public Enum CamType As Integer
        ''' <summary>
        ''' Es werden keine InputDevices aus der Klasse verarbeitet. 
        ''' </summary>
        fixed = 0
        ''' <summary>
        ''' Aktiviert eine FirstPerson Camera.
        ''' </summary>
        firstPerson = 1
        ''' <summary>
        ''' Interpoliert eine beidseitige Kamerabewegung vom Mittelpunkt.
        ''' </summary>
        moving = 2
        ''' <summary>
        ''' Interpoliert eine beidseitige Kamerabewegung vom Mittelpunkt.
        ''' </summary>
        rotate = 3
        ''' <summary>
        ''' Aktiviert eine mausgesteuerte Kamerabewegung um ein Zentrum.
        ''' </summary>
        mouseRotate = 4
    End Enum
#End Region
#Region "Enum CamMovements"
    ''' <summary>
    ''' Legt die Kamerabewegungen über einen Inputdevice fest
    ''' </summary>
    Public Enum CamMovements As Integer
        Left = 0
        Right = 1
        Up = 2
        Down = 3
        Forward = 4
        Backward = 5
    End Enum
#End Region

#Region "Structure CameraSet - Fixed"

    ''' <summary>
    ''' CamSet einer festen Kamera.
    ''' </summary>
    ''' <remarks></remarks>
    Public Structure cameraSetFixed
        Private _position As Vector3
        Private _target As Vector3
        Private _name As String

        ''' <summary>
        ''' Legt den Namen der Kamera fest.
        ''' </summary>
        Public Property Name() As String
            Get
                Return _name
            End Get
            Set(ByVal value As String)
                _name = value
            End Set
        End Property
        ''' <summary>
        ''' Ruft den Kameratyp ab.
        ''' </summary>
        ''' <value></value>
        Public ReadOnly Property CamType() As CamType
            Get
                Return engineCamera.CamType.fixed
            End Get
        End Property

        ''' <summary>
        ''' Legt die Position der Kamera fest oder ruft diese ab.
        ''' </summary>
        Public Property Position() As Vector3
            Set(ByVal value As Vector3)
                _position = value
            End Set
            Get
                Return _position
            End Get
        End Property

        ''' <summary>
        ''' Legt das Kameraziel fest oder ruft dieses ab.
        ''' </summary>
        Public Property Target() As Vector3
            Set(ByVal value As Vector3)
                _target = value
            End Set
            Get
                Return _target
            End Get
        End Property
    End Structure
#End Region
#Region "Structure CameraSet - Moving"

    ''' <summary>
    ''' CamSet einer interpolierten Kamerabewegung.
    ''' </summary>
    ''' <remarks></remarks>
    Public Structure cameraSetMoving
        Private _start As Vector3
        Private _movement As Vector3
        Private _target As Vector3
        Private _speed As Vector2
        Private _curved As Boolean
        Private _name As String

        ''' <summary>
        ''' Legt den Namen der Kamera fest.
        ''' </summary>
        Public Property Name() As String
            Get
                Return _name
            End Get
            Set(ByVal value As String)
                _name = value
            End Set
        End Property

        ''' <summary>
        ''' Ruft den Kameratyp ab.
        ''' </summary>
        ''' <value></value>
        Public ReadOnly Property CamType() As CamType
            Get
                Return engineCamera.CamType.moving
            End Get
        End Property

        ''' <summary>
        ''' Legt den Startpunkt der Kamera fest oder ruft diesen ab.
        ''' </summary>
        Public Property StartPoint() As Vector3
            Set(ByVal value As Vector3)
                _start = value
            End Set
            Get
                Return _start
            End Get
        End Property

        ''' <summary>
        ''' Legt das Kameraziel fest oder ruft dieses ab.
        ''' </summary>
        Public Property Target() As Vector3
            Set(ByVal value As Vector3)
                _target = value
            End Set
            Get
                Return _target
            End Get
        End Property

        ''' <summary>
        ''' Legt die Strecke der Kamera fest oder ruft diese ab.
        ''' Der Differenzvektor zwischen StartPoint und Movement wird beidseitig interpoliert.
        ''' </summary>
        Public Property Movement() As Vector3
            Set(ByVal value As Vector3)
                _movement = value
            End Set
            Get
                Return _movement
            End Get
        End Property

        ''' <summary>
        ''' Legt die Geschwindigkeit der Kamera fest oder ruft diese ab.
        ''' </summary>
        Public Property Speed() As Vector2
            Set(ByVal value As Vector2)
                _speed = value
            End Set
            Get
                Return _speed
            End Get
        End Property

        ''' <summary>
        ''' Legt die interpolation der Y-Achse fest.
        ''' </summary>
        Public Property Curved() As Boolean
            Set(ByVal value As Boolean)
                _curved = value
            End Set
            Get
                Return _curved
            End Get
        End Property
    End Structure
#End Region
#Region "Structure CameraSet - Rotate"

    ''' <summary>
    ''' CamSet einer Rotierenden Kamerabewegung um Y
    ''' </summary>
    ''' <remarks></remarks>
    Public Structure cameraSetRotate
        Private _target As Vector3
        Private _center As Vector2
        Private _speed As Vector2
        Private _radius As Single
        Private _height As Single
        Private _maxheight As Single
        Private _angle As Single
        Private _curved As Boolean
        Private _interpolation As Single
        Private _name As String

        ''' <summary>
        ''' Legt den Namen der Kamera fest.
        ''' </summary>
        Public Property Name() As String
            Get
                Return _name
            End Get
            Set(ByVal value As String)
                _name = value
            End Set
        End Property

        ''' <summary>
        ''' Ruft den Kameratyp ab.
        ''' </summary>
        Public ReadOnly Property CamType() As CamType
            Get
                Return engineCamera.CamType.rotate
            End Get
        End Property
        ''' <summary>
        ''' Legt die maxiemal interpolierte Höhe der Kamera fest oder ruft diese ab.
        ''' </summary>
        Public Property maxHeight() As Single
            Get
                Return _maxheight
            End Get
            Set(ByVal value As Single)
                _maxheight = value
            End Set
        End Property
        ''' <summary>
        ''' Legt die Interpolationsgeschwindigkeit fest oder ruft diese ab.
        ''' </summary>
        Public Property CurvedSpeed() As Single
            Get
                Return _interpolation
            End Get
            Set(ByVal value As Single)
                _interpolation = value
            End Set
        End Property

        ''' <summary>
        ''' Legt fest ob die Y-Achse interpoliert wird.
        ''' </summary>
        Public Property Curved() As Boolean
            Get
                Return _curved
            End Get
            Set(ByVal value As Boolean)
                _curved = value
            End Set
        End Property

        ''' <summary>
        ''' Legt den Rotationswinkel der Kamera fest oder ruft diesen ab.
        ''' </summary>
        Public Property Angle() As Single
            Set(ByVal value As Single)
                _angle = value
            End Set
            Get
                Return _angle
            End Get
        End Property

        ''' <summary>
        ''' Legt den Rotationspunkt der Kamera fest oder ruft diesen ab. (XZ)
        ''' </summary>
        Public Property Center() As Vector2
            Set(ByVal value As Vector2)
                _center = value
            End Set
            Get
                Return _center
            End Get
        End Property

        ''' <summary>
        ''' Legt das Kameraziel fest oder ruft dieses ab.
        ''' </summary>
        Public Property Target() As Vector3
            Set(ByVal value As Vector3)
                _target = value
            End Set
            Get
                Return _target
            End Get
        End Property

        ''' <summary>
        ''' Legt die Geschwindigkeit der Kamera fest oder ruft diese ab.
        ''' </summary>
        Public Property RotationSpeed() As Vector2
            Set(ByVal value As Vector2)
                _speed = value
            End Set
            Get
                Return _speed
            End Get
        End Property
        ''' <summary>
        ''' Legt den Radius um den Rotationspunkt fest oder ruft diesen ab.
        ''' </summary>
        Public Property Radius() As Single
            Get
                Return _radius
            End Get
            Set(ByVal value As Single)
                _radius = value
            End Set
        End Property
        ''' <summary>
        ''' Legt die Höhe auf der Y-Achse fest oder ruft diesen ab.
        ''' </summary>
        Public Property Height() As Single
            Get
                Return _height
            End Get
            Set(ByVal value As Single)
                _height = value
            End Set
        End Property
    End Structure
#End Region
#Region "Structure CameraSet - FirstPerson"

    ''' <summary>
    ''' CamSet einer FirstPerson Kamera.
    ''' </summary>
    ''' <remarks></remarks>
    Public Structure cameraSetFirstPerson
        Private _position As Vector3
        Private Shared _speed As Vector2 = New Vector2(1, 1)
        Private Shared _clamp As New Vector2(-89, 89)
        Private Shared _lerpspeed As Single = 0.3
        Private Shared _name As String = "First Person Cam"


        ''' <summary>
        ''' Legt den Namen der Kamera fest.
        ''' </summary>
        Public Property Name() As String
            Get
                Return _name
            End Get
            Set(ByVal value As String)
                _name = value
            End Set
        End Property

        ''' <summary>
        ''' ruft den Kameratyp ab.
        ''' </summary>
        ''' <value></value>
        Public ReadOnly Property CamType() As CamType
            Get
                Return engineCamera.CamType.firstPerson
            End Get
        End Property

        ''' <summary>
        ''' Legt die Position der Kamera fest oder ruft diese ab.
        ''' </summary>
        Public Property Position() As Vector3
            Set(ByVal value As Vector3)
                _position = value
            End Set
            Get
                Return _position
            End Get
        End Property

        ''' <summary>
        ''' Legt die Geschwindigkeit der Kamerarotationen  fest oder ruft diese ab. (default 1,1)
        ''' </summary>
        Public Property Speed() As Vector2
            Set(ByVal value As Vector2)
                _speed = value
            End Set
            Get
                Return _speed
            End Get
        End Property

        ''' <summary>
        ''' Legt den Rotationsbereich der X-Achse fest oder ruft
        ''' diesen ab. Vector2(max,min).
        ''' (Die Rotation sollte auf +-89.9 Grad eingeschränkt werden. Der Radiantenwert
        ''' beträgt 1.56)
        ''' </summary>
        Public Property ClampRotation() As Vector2
            Set(ByVal value As Vector2)
                _clamp = value
            End Set
            Get
                Return _clamp
            End Get
        End Property
        ''' <summary>
        ''' Legt den Interpolationswert fest.
        ''' Werte zwischen 0 und 1 sind zulässig.
        ''' </summary> 
        Public Property SmoothingSpeed() As Single
            Get
                Return _lerpspeed
            End Get
            Set(ByVal value As Single)
                _lerpspeed = value
            End Set
        End Property
    End Structure
#End Region
#Region "Structure CameraSet - MouseRotate"

    ''' <summary>
    ''' CamSet einer Mouserotation Kamera.
    ''' </summary>
    ''' <remarks></remarks>
    Public Structure cameraSetMouseRotate
        Private _reference As Vector3
        Private _speed As Vector2
        Private _smoothing As Boolean
        Private _amount As Single
        Private _amountXY As Vector2
        Private _radius As Single
        Private _height As Single
        Private _center As Vector2
        Private _name As String
        Private Shared _clamp As Vector2 = New Vector2(-50, 32)

        ''' <summary>
        ''' Legt den Namen der Kamera fest.
        ''' </summary>
        Public Property Name() As String
            Get
                Return _name
            End Get
            Set(ByVal value As String)
                _name = value
            End Set
        End Property

        ''' <summary>
        ''' Legt den Rotationspunkt der Kamera fest oder ruft diesen ab. (XZ)
        ''' </summary>
        Public Property Center() As Vector2
            Set(ByVal value As Vector2)
                _center = value
            End Set
            Get
                Return _center
            End Get
        End Property
        ''' <summary>
        ''' ruft den Kameratyp ab.
        ''' </summary>
        ''' <value></value>
        Public ReadOnly Property CamType() As CamType
            Get
                Return engineCamera.CamType.mouseRotate
            End Get
        End Property

        ''' <summary>
        ''' Legt die Geschwindigkeit der Kamera fest oder ruft diese ab.
        ''' </summary>
        Public Property Speed() As Vector2
            Set(ByVal value As Vector2)
                _speed = value
            End Set
            Get
                Return _speed
            End Get
        End Property

        ''' <summary>
        ''' Legt fest, ob Kamerabewegungen interpoliert wird
        ''' </summary>
        ''' <value></value>

        Public Property useSmoothRotations() As Boolean
            Set(ByVal value As Boolean)
                _smoothing = value
            End Set
            Get
                Return _smoothing
            End Get
        End Property
        ''' <summary>
        ''' Legt den Interpolationswert fest.
        ''' Werte zwischen 0 und 1 sind zulässig.
        ''' </summary> 
        Public Property SmoothingSpeed() As Single
            Get
                Return _amount
            End Get
            Set(ByVal value As Single)
                _amount = value
            End Set
        End Property

        ''' <summary>
        ''' Legt den Interpolationswert fest.
        ''' Werte zwischen 0 und 1 sind zulässig.
        ''' </summary> 
        Public Property SmoothingSpeeds() As Vector2
            Get
                Return _amountXY
            End Get
            Set(ByVal value As Vector2)
                _amountXY = value
            End Set
        End Property

        ''' <summary>
        ''' Legt den Radius um den Rotationspunkt fest oder ruft diesen ab.
        ''' </summary>
        Public Property Radius() As Single
            Get
                Return _radius
            End Get
            Set(ByVal value As Single)
                _radius = value
            End Set
        End Property
        ''' <summary>
        ''' Legt die Höhe auf der Y-Achse fest oder ruft diesen ab.
        ''' </summary>
        Public Property Height() As Single
            Get
                Return _height
            End Get
            Set(ByVal value As Single)
                _height = value
            End Set
        End Property
        ''' <summary>
        ''' Legt die Beschränkung der Rotation um X fest.
        ''' </summary>
        Public Property LookClamp() As Vector2
            Get
                Return _clamp
            End Get
            Set(ByVal value As Vector2)
                _clamp = value
            End Set
        End Property
    End Structure
#End Region


End Class