﻿' Title    : sdxBaseEngine Framework
' Author   : Sascha Bajus
' Created  : 12/2008
' Copyright © 2008 Sascha Bajus

#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region




Imports SlimDX
Imports SlimDX.Direct3D9
Imports Microsoft.VisualBasic
Imports sdxBaseEngine.SDX9BaseEngine


    '''<summary>
    ''' Light Klasse zum erstellen von DirectX Licht. Die Klasse stellt drei einfache Basislichter und ein
    ''' default Material zur Verfügung 
    '''</summary>
    Public Class engineLights
        Private _material As Material
        Private _directional As New Light
        Private _point As New Light

        ''' <summary>
        ''' Erstellt eine neue Light Klasse
        ''' </summary>
        ''' <remarks></remarks>
    Public Sub New()
        dxEngine.LoadingStatus = "Loading Engine: Class - engineLights..."
        defaultmaterial()
        defaultLightsetup()
    End Sub
#Region "Default Material"
        Private Sub defaultmaterial()
            With _material '# default Material
            .Ambient = Color.FromArgb(255, 30, 30, 30)
            .Diffuse = Color.White
            .Emissive = Color.Black
            .Specular = Color.White
            .Power = 2
            End With
        End Sub
#End Region
#Region "Default Lightsetup"
        Private Sub defaultLightsetup()
            With _directional
            .Type = LightType.Directional
            .Ambient = Color.White
            .Diffuse = Color.White
            .Specular = Color.White
            .Direction = New Vector3(1, -1, 0)
            End With

            With _point
                .Type = LightType.Point
                .Ambient = Color.FromArgb(255, 30, 30, 30)
                .Diffuse = Color.White
                .Specular = Color.CornflowerBlue
                .Attenuation1 = 0.05
                .Range = 50
            .Position = New Vector3(-8, 8, 0)
            End With

            dxDevice.SetLight(0, _directional)
            dxDevice.SetLight(1, _point)
        End Sub
#End Region
#Region "Property"
        ''' <summary>
        '''Aktivieren/Deaktivieren des Basic DirectionalLights (Light(0))
        ''' </summary>
        Public WriteOnly Property EnableDirectionalLight() As Boolean
            Set(ByVal value As Boolean)
                dxDevice.EnableLight(0, value)
            End Set
        End Property
        ''' <summary>
        '''Aktivieren/Deaktivieren des Basic PointLights (Light(1))
        ''' </summary>
        Public WriteOnly Property EnablePointLight() As Boolean
            Set(ByVal value As Boolean)
                dxDevice.EnableLight(1, value)
            End Set
        End Property
        ''' <summary>
        ''' Legt ein neues DirectX Licht fest
        ''' </summary>
        ''' <param name="lightIndex">
        ''' Legt den Index des Lichts fest</param>
        Public WriteOnly Property LightAddNew(ByVal lightIndex As Integer) As Light
            Set(ByVal value As Light)
                dxDevice.SetLight(lightIndex, value)
            End Set
        End Property
        ''' <summary>
        ''' Legt fest  welches erstellete Licht aktiviert wird.
        ''' </summary>
        Public WriteOnly Property LightIndexEnable() As Integer
            Set(ByVal value As Integer)
                dxDevice.EnableLight(value, True)
            End Set
        End Property
        ''' <summary>
        ''' Legt fest  welches erstellete Licht deaktiviert wird.
        ''' </summary>
        Public WriteOnly Property LightIndexDisable() As Integer
            Set(ByVal value As Integer)
                dxDevice.EnableLight(value, False)
            End Set
        End Property

        ''' <summary>
        ''' Legt fest ob die DirectX Beleuchtung aktiviert wird oder nicht.
        ''' </summary>
        Public WriteOnly Property EnableLighting() As Boolean
            Set(ByVal value As Boolean)
                dxDevice.SetRenderState(RenderState.Lighting, value)
            End Set
        End Property
        ''' <summary>
        '''Aktivieren/Deaktivieren des spekularen Lichts
        ''' </summary>
        Public WriteOnly Property EnableSpecularLighting() As Boolean
            Set(ByVal value As Boolean)
                dxDevice.SetRenderState(RenderState.SpecularEnable, value)
            End Set
        End Property
        ''' <summary>
        ''' Legt das aktuelle Material fest.
        ''' </summary>
        ''' <value></value>
        ''' <remarks></remarks>
        Public WriteOnly Property Material() As Material
            Set(ByVal value As Material)
                dxDevice.Material = value
            End Set
        End Property
        ''' <summary>
        '''Übergibt das default Material an den Device.
        ''' </summary>
        ''' <remarks >
        ''' Objekte ohne Material reflektieren keine Licht bei aktivierter Beleuchtung.
        ''' Das Defaultmaterial:  # Ambient = Color.White  #  Diffuse = Color.White  #  
        ''' Emissive = Color.Black  #  Specular = Color.White  #  Power = 0 </remarks>
    Public Sub SetDefaultMaterial()
        dxDevice.Material = _material
    End Sub
#End Region
    End Class

