﻿' Title    : sdxBaseEngine Framework
' Name     : engineText2D
' Property : dx2DText
' Author   : Sascha Bajus
' Created  : 12/2008
' Copyright © 2008 Sascha Bajus

#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region


Imports SlimDX
Imports SlimDX.Direct3D9
Imports sdxBaseEngine.SDX9BaseEngine

    '''<summary>
    ''' Text Klasse zur Erstellung von renderbaren 2D Texten. Der Klasse können Textobjekte mit verschiedenen Fonts,
    ''' Farben und 2D-Renderbereiche hinzugefügt werden. Die Textobjekte werden einer Liste hinzugefügt. Beim Renderaufruf  
    ''' wird der Index des Textobjektes und der zu rendernde Text übergeben.
    '''</summary>

Public Class engineText2D

    Private defaultFont As Font
    Private defaultColor As Color = Color.FromArgb(200, 225, 0, 29)
    Private defaultRecText As Rectangle = New Rectangle(2, 1, 300, 300)
    Private defaultRecDataText As Rectangle = New Rectangle(100, 1, 300, 300)


    Private _loadingColor As Color = Color.DimGray
    Private _loadingRecText As Rectangle = New Rectangle(0, dxEngine.BackBufferSize.Y - 100, dxEngine.BackBufferSize.X, 40)


    Private EngineText As String = "Render Device" & vbCrLf & _
                                   "Render Frames"
    Private EngineData As String


    Public Sub New()
        AddHandler dxDeviceLost, AddressOf DeviceLost
        AddHandler dxDeviceReset, AddressOf DeviceReset
        AddHandler dxEngineClosing, AddressOf Dispose

        dxEngine.LoadingStatus = "Loading Engine: Text2D..."
        defaultFont = New Font(dxDevice, New System.Drawing.Font("Verdana", 8, FontStyle.Regular))
    End Sub

#Region "Event Handles"
    Private Sub DeviceLost()
        defaultFont.OnLostDevice()
    End Sub
    Private Sub DeviceReset()
        defaultFont.OnResetDevice()
    End Sub
    Private Sub Dispose()
        defaultFont.Dispose()
    End Sub
#End Region

#Region "##   Render   ##"
    ''' <summary>
    ''' Gibt die Bilder pro Sekunde zurück
    ''' </summary>
    ''' <param name="InfoText">Legt einen weiteren Infotext fest</param>
    ''' <param name="InfoData">Legt den dazugehörigen Datentext fest</param>
    ''' <remarks></remarks>
    Public Sub DrawFPS(Optional ByVal InfoText As String = "", Optional ByVal InfoData As String = "")
        defaultFont.DrawString(Nothing, EngineText & vbCrLf & InfoText, 0, 0, defaultColor)

        EngineData = ":  " & dxEngine.Adapterinfo.Details.Description & vbCrLf & _
                     ":  " & dxEngine.BackBufferSize.X & "x" & dxEngine.BackBufferSize.Y & " @ " & dxGametime.FrameCounter & " Fps " & dxGametime.GameRendertime & " ms"

        defaultFont.DrawString(Nothing, EngineData & vbCrLf & InfoData, defaultRecDataText, DrawTextFormat.Left, defaultColor)
    End Sub

    Public Sub DrawLoadingInfo(ByVal InfoText As String)
        defaultFont.DrawString(Nothing, InfoText, _loadingRecText, DrawTextFormat.Center, _loadingColor)
    End Sub
#End Region


End Class