﻿' Title    : sdxBaseEngine Framework
' Name     : engineTexture
' Property : dxTexture
' Author   : Sascha Bajus
' Created  : 4/2009
' Copyright © 2008 Sascha Bajus

#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region


Imports SlimDX
Imports SlimDX.Direct3D9
Imports sdxBaseEngine.SDX9BaseEngine

'''<summary>
''' Textur Klasse zur Effektbearbeitung, wie Transformationen, Multitexturing, 
''' und Alphablending
'''</summary>
Public Class engineTexture
    Private _material As New Material
    Private _filter As New clsSamplerState
    Private _blendfaktorHalf As Integer = Color.FromArgb(125, 125, 125, 125).ToArgb

    '#to reset the Stages
    Private Structure StageValues
        Dim ColorArgument0 As Integer
        Dim ColorArgument1 As Integer
        Dim ColorArgument2 As Integer
        Dim ColorOperation As Integer
    End Structure



#Region "One Pass DOT3"
    '''<summary>
    '''One Pass DOT3-Bumpmaping (Normalmap=Stage0, Colormap=Stage1)
    '''</summary>
    '''<param name="lightvector ">Angabe der Lichtrichtung als Vector3 </param>
    Public Sub onePassDOT3(ByVal lightvector As Vector3)
        With dxDevice
            .SetRenderState(RenderState.TextureFactor, Vector3toColorRGB(lightvector).ToArgb)

            .SetTextureStageState(0, TextureStage.ColorArg1, TextureArgument.Texture)
            .SetTextureStageState(0, TextureStage.ColorArg2, TextureArgument.TFactor)
            .SetTextureStageState(0, TextureStage.ColorOperation, TextureOperation.DotProduct3)

            .SetTextureStageState(1, TextureStage.TexCoordIndex, 0)
            .SetTextureStageState(1, TextureStage.ColorArg1, TextureArgument.Texture)
            .SetTextureStageState(1, TextureStage.ColorArg2, TextureArgument.Current)
            .SetTextureStageState(1, TextureStage.ColorOperation, TextureOperation.Modulate)

            .SetTextureStageState(2, TextureStage.TexCoordIndex, 0)
            .SetTextureStageState(2, TextureStage.ColorArg1, TextureArgument.Diffuse)
            .SetTextureStageState(2, TextureStage.ColorArg2, TextureArgument.Current)
            .SetTextureStageState(2, TextureStage.ColorOperation, TextureOperation.Modulate)
        End With
    End Sub
#End Region
#Region "One Pass Colorblending 1X"
    '''<summary>
    '''One Pass Textureblending (modulate)
    '''</summary>
    Public Sub onePassColorBlending_1X()
        With dxDevice
            .SetTextureStageState(0, TextureStage.ColorArg1, TextureArgument.Texture)
            .SetTextureStageState(0, TextureStage.ColorArg2, TextureArgument.Diffuse)
            .SetTextureStageState(0, TextureStage.ColorOperation, TextureOperation.Modulate)

            .SetTextureStageState(1, TextureStage.TexCoordIndex, 0)
            .SetTextureStageState(1, TextureStage.ColorArg1, TextureArgument.Texture)
            .SetTextureStageState(1, TextureStage.ColorArg2, TextureArgument.Current)
            .SetTextureStageState(1, TextureStage.ColorOperation, TextureOperation.Modulate)
        End With
    End Sub
#End Region
#Region "One Pass Colorblending 2X"
    '''<summary>
    '''One Pass Textureblending (modulate2x)
    '''</summary>
    Public Sub onePassColorBlending_2X()
        With dxDevice
            .SetTextureStageState(0, TextureStage.ColorArg1, TextureArgument.Texture)
            .SetTextureStageState(0, TextureStage.ColorArg2, TextureArgument.Diffuse)
            .SetTextureStageState(0, TextureStage.ColorOperation, TextureOperation.Modulate)

            .SetTextureStageState(1, TextureStage.TexCoordIndex, 0)
            .SetTextureStageState(1, TextureStage.ColorArg1, TextureArgument.Texture)
            .SetTextureStageState(1, TextureStage.ColorArg2, TextureArgument.Current)
            .SetTextureStageState(1, TextureStage.ColorOperation, TextureOperation.Modulate2X)
        End With
    End Sub
#End Region
#Region "One Pass Colorblending 4X"
    '''<summary>
    '''One Pass Textureblending (modulate4x)
    '''</summary>
    Public Sub onePassColorBlending_4X()
        With dxDevice
            .SetTextureStageState(0, TextureStage.ColorArg1, TextureArgument.Texture)
            .SetTextureStageState(0, TextureStage.ColorArg2, TextureArgument.Diffuse)
            .SetTextureStageState(0, TextureStage.ColorOperation, TextureOperation.Modulate)

            .SetTextureStageState(1, TextureStage.TexCoordIndex, 0)
            .SetTextureStageState(1, TextureStage.ColorArg1, TextureArgument.Texture)
            .SetTextureStageState(1, TextureStage.ColorArg2, TextureArgument.Current)
            .SetTextureStageState(1, TextureStage.ColorOperation, TextureOperation.Modulate4X)
        End With
    End Sub
#End Region
#Region "TFactor Modulation"
    ''' <summary>
    ''' Moduliert eine Textur mit einer Farbe
    ''' </summary>
    ''' <param name="modulationColor">Angabe der Modulationsfarbe</param>
    ''' <remarks></remarks>
    Public Sub tfactorModulation(ByVal modulationColor As Color)
        dxDevice.SetRenderState(RenderState.TextureFactor, modulationColor.ToArgb)
        dxDevice.SetTextureStageState(0, TextureStage.ColorArg2, TextureArgument.TFactor)
        dxDevice.SetTextureStageState(0, TextureStage.ColorOperation, TextureOperation.Modulate)
    End Sub
#End Region
#Region "Texture Transformation"
    '''<summary>
    '''Matrixanwendug zur Erstellung einer 2D Texturtransformation
    '''</summary>
    '''<param name="TextureStage">Legt das zu verwendende Texturestage fest</param>
    '''<param name="scaleUV ">Skalierung der UV Koordianten der Textur</param>
    '''<param name="posUV">Position der UV Koordianten der Textur</param>


    Public Sub transformTexture2D(ByVal TextureStage As Integer, ByVal scaleUV As Vector2, ByVal posUV As Vector2)
        Dim _matrix As Matrix
        With _matrix
            .M11 = scaleUV.X
            .M22 = scaleUV.Y

            .M31 = posUV.X
            .M32 = posUV.Y
        End With

        'Transform Texture
        dxDevice.SetTextureStageState(TextureStage, Direct3D9.TextureStage.TextureTransformFlags, TextureTransform.Count2)
        Select Case TextureStage
            Case Is = 0
                dxDevice.SetTransform(TransformState.Texture0, _matrix)
            Case Is = 1
                dxDevice.SetTransform(TransformState.Texture1, _matrix)
            Case Is = 2
                dxDevice.SetTransform(TransformState.Texture2, _matrix)
            Case Is = 3
                dxDevice.SetTransform(TransformState.Texture3, _matrix)
            Case Is = 4
                dxDevice.SetTransform(TransformState.Texture4, _matrix)
            Case Is = 5
                dxDevice.SetTransform(TransformState.Texture5, _matrix)
            Case Is = 6
                dxDevice.SetTransform(TransformState.Texture6, _matrix)
            Case Is = 7
                dxDevice.SetTransform(TransformState.Texture7, _matrix)
        End Select
    End Sub
#End Region
#Region "Reset Transformation"
    '''<summary>
    '''Deaktivierung einer vorhergehenden Texturtransformation
    '''</summary>
    '''<param name="TextureStage">Legt das zu deaktivierende Texturestage fest</param>
    Public Sub transformTextureReset(ByVal TextureStage As Integer)
        dxDevice.SetTextureStageState(TextureStage, Direct3D9.TextureStage.TextureTransformFlags, TextureTransform.Disable)
    End Sub
#End Region
#Region "Reset Stages"
    '''<summary>
    '''Rücksetzen einer Texturoperation. Stage0 erhält die default Einstellungen
    '''</summary>
    '''<param name="stagenumber ">Legt das zu verwendende Texturestage fest</param>
    Public Sub resetStage(ByVal stagenumber As Integer)
        With dxDevice
            .SetTextureStageState(stagenumber, TextureStage.ColorOperation, TextureOperation.Disable)
            .SetTextureStageState(0, TextureStage.ColorArg0, TextureArgument.Current)
            .SetTextureStageState(0, TextureStage.ColorArg1, TextureArgument.Texture)
            .SetTextureStageState(0, TextureStage.ColorArg2, TextureArgument.Current)
            .SetTextureStageState(0, TextureStage.ColorOperation, TextureOperation.Modulate)
        End With
    End Sub

#End Region
#Region "Reset Texture"
    '''<summary>
    '''Rücksetzen einer Textur.
    '''</summary>
    '''<param name="stagenumber ">Legt das zu verwendende Texturestage fest</param>
    Public Sub resetTexture(ByVal stagenumber As Integer)
        With dxDevice
            .SetTexture(stagenumber, Nothing)
        End With
    End Sub

#End Region

#Region "DOT3 ColorVector"
    '''<summary>
    '''Vektortransformation in den RGB Farbraum
    '''</summary>
    '''<param name="LightVector">Richtung des Lichtes</param>

    Public Function Vector3toColorRGB(ByVal LightVector As Vector3) As Color
        Dim r, g, b As Integer
        LightVector.Normalize()
        r = CInt(LightVector.X * 127 + 128)
        g = CInt(LightVector.Y * 127 + 128)
        b = CInt(LightVector.Z * 127 + 128)

        Return Color.FromArgb(255, r, g, b)
    End Function
#End Region
#Region "Texture Load"
    '''<summary>
    '''Läd eine Textur aus einer Datei
    '''</summary>
    ''' <param name="colorKey" >Legt den Alpha Colorkey fest</param>
    ''' <param name="src" >Legt den Dateipfad der zu ladenen Textur fest</param>
    Public Function FromFile(ByVal src As String, ByVal colorKey As Color, Optional ByVal Format As Format = Format.A8R8G8B8) As Texture
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then
            Try
                dxEngine.LoadingStatus = "Loading Texture: " & src & "..."
                Return Texture.FromFile(dxDevice, src, 0, 0, 0, 0, Format.Unknown, Pool.Default, Filter.None, Filter.Linear, colorKey.ToArgb)
            Catch ex As Exception
                dxEngine.EngineErrorDescription = "Error loading Textur from File '" & dxEngine.ApplicationPath & src & "'" & vbCrLf
                Return Nothing
            End Try
        Else
            Return Nothing
        End If
    End Function

    ''' <summary>
    '''Läd eine Textur aus einem ResourceObject
    ''' </summary>
    ''' <param name="src">Legt das ResourceObject fest</param>
    ''' <param name="colorKey">Legt den Alpha Colorkey fest</param>
    Public Function FromEngineResource(ByVal src As String, ByVal colorKey As Color, Optional ByVal Format As Format = Format.A8R8G8B8) As Texture
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then
            With dxEngine.ResourceLoader
                Try
                    FromEngineResource = Texture.FromMemory(dxDevice, .GetResourceObject(src), 0, 0, 0, 0, Format.Unknown, Pool.Default, Filter.None, Filter.Linear, colorKey.ToArgb)
                    dxEngine.LoadingStatus = "Loading Texture: " & .currentObjectName & "..."
                Catch ex As Exception
                    dxEngine.EngineErrorDescription = "Error loading Textur from Resource '" & .currentResourceFile & "\" & src & "'"
                    Return Nothing
                End Try
            End With
        Else
            Return Nothing
        End If
    End Function
    Public Function FromImage(ByVal src As Bitmap, ByVal colorKey As Color, Optional ByVal ResName As String = "") As Texture
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then
            Return Texture.FromMemory(dxDevice, ImageToByteBuffer(src), 0, 0, 0, 0, Format.Unknown, Pool.Default, Filter.None, Filter.Linear, colorKey.ToArgb)
        Else
            Return Nothing
        End If
    End Function

    Public Function FromMemory(ByVal src As Byte(), ByVal colorKey As Color, Optional ByVal ResName As String = "") As Texture
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then
            Return Texture.FromMemory(dxDevice, src, 0, 0, 0, 0, Format.Unknown, Pool.Default, Filter.None, Filter.Linear, colorKey.ToArgb)
        Else
            Return Nothing
        End If
    End Function
#End Region


#Region "Public Property"
    ''' <summary>
    ''' Legt die zu rendernde Textur im Sampler fest
    ''' </summary>
    ''' <param name="sampler">Index des Samplers</param>
    Public WriteOnly Property SetTexture(Optional ByVal sampler As Integer = 0) As Texture
        Set(ByVal value As Texture)
            dxDevice.SetTexture(sampler, value)
        End Set
    End Property


    ''' <summary>
    ''' SourceBlend=SourceAlpha |
    ''' DestinationBlend=InverseSourceAlpha 
    ''' </summary>
    ''' <param name="Cull">Auswahl der Cullmodes. Beim Rendern von Sprites muss Cull.Clockwise verwendet werden. </param>
    Public WriteOnly Property AlphaBlending1(Optional ByVal Cull As Cull = Cull.Counterclockwise) As Boolean
        Set(ByVal value As Boolean)
            If value Then
                dxDevice.SetRenderState(RenderState.AlphaBlendEnable, True)
                dxDevice.SetRenderState(RenderState.SourceBlend, Blend.SourceAlpha)
                dxDevice.SetRenderState(RenderState.DestinationBlend, Blend.InverseSourceAlpha)
                dxDevice.SetRenderState(RenderState.BlendOperation, BlendOperation.Add)
                dxDevice.SetRenderState(RenderState.CullMode, Cull)
            Else
                dxDevice.SetRenderState(RenderState.AlphaBlendEnable, False)
                dxDevice.SetRenderState(RenderState.CullMode, Cull)
            End If
        End Set
    End Property

    ''' <summary>
    ''' SourceBlend=SourceAlpha |
    ''' DestinationBlend=DestinationAlpha 
    ''' </summary>
    ''' <param name="Cull">Auswahl der Cullmodes. Beim Rendern von Sprites muss Cull.Clockwise verwendet werden. </param>
    Public WriteOnly Property AlphaBlending2(Optional ByVal Cull As Cull = Cull.Counterclockwise) As Boolean
        Set(ByVal value As Boolean)
            If value Then
                dxDevice.SetRenderState(RenderState.AlphaBlendEnable, True)
                dxDevice.SetRenderState(RenderState.SourceBlend, Blend.SourceAlpha)
                dxDevice.SetRenderState(RenderState.DestinationBlend, Blend.DestinationAlpha)
                dxDevice.SetRenderState(RenderState.CullMode, Cull)
            Else
                dxDevice.SetRenderState(RenderState.AlphaBlendEnable, False)
                dxDevice.SetRenderState(RenderState.CullMode, Cull)
            End If
        End Set
    End Property

    ''' <summary>
    ''' SourceBlend=One |
    ''' DestinationBlend=One 
    ''' </summary>
    ''' <param name="Cull">Auswahl der Cullmodes. Beim Rendern von Sprites muss Cull.Clockwise verwendet werden. </param>
    ''' <remarks></remarks>
    Public WriteOnly Property AlphaBlending3(Optional ByVal Cull As Cull = Cull.Counterclockwise) As Boolean
        Set(ByVal value As Boolean)
            If value Then
                dxDevice.SetRenderState(RenderState.AlphaBlendEnable, True)
                dxDevice.SetRenderState(RenderState.SourceBlend, Blend.One)
                dxDevice.SetRenderState(RenderState.DestinationBlend, Blend.One)
                dxDevice.SetRenderState(RenderState.CullMode, Cull)
            Else
                dxDevice.SetRenderState(RenderState.AlphaBlendEnable, False)
                dxDevice.SetRenderState(RenderState.CullMode, Cull)
            End If
        End Set
    End Property

    ''' <summary>
    ''' SourceBlend=SourceColor |
    ''' DestinationBlend=DestinationColor 
    ''' </summary>
    ''' <param name="Cull">Auswahl der Cullmodes. Beim Rendern von Sprites muss Cull.Clockwise verwendet werden. </param>
    Public WriteOnly Property ColorBlending1(Optional ByVal Cull As Cull = Cull.Counterclockwise) As Boolean
        Set(ByVal value As Boolean)
            If value Then
                dxDevice.SetRenderState(RenderState.AlphaBlendEnable, True)
                dxDevice.SetRenderState(RenderState.SourceBlend, Blend.SourceColor)
                dxDevice.SetRenderState(RenderState.DestinationBlend, Blend.DestinationColor)
                dxDevice.SetRenderState(RenderState.CullMode, Cull)
            Else
                dxDevice.SetRenderState(RenderState.AlphaBlendEnable, False)
                dxDevice.SetRenderState(RenderState.CullMode, Cull)
            End If
        End Set
    End Property

    ''' <summary>
    ''' SourceBlend=SourceColor |
    ''' DestinationBlend=BlendFaktor 0.5
    ''' </summary>
    ''' <param name="Cull">Auswahl der Cullmodes. Beim Rendern von Sprites muss Cull.Clockwise verwendet werden. </param>

    Public WriteOnly Property ColorBlending2(Optional ByVal Cull As Cull = Cull.Counterclockwise) As Boolean
        Set(ByVal value As Boolean)

            If value Then
                dxDevice.SetRenderState(RenderState.AlphaBlendEnable, True)
                dxDevice.SetRenderState(RenderState.SourceBlend, Blend.SourceColor)
                dxDevice.SetRenderState(RenderState.DestinationBlend, Blend.BlendFactor)
                dxDevice.SetRenderState(RenderState.BlendFactor, _blendfaktorHalf)
                dxDevice.SetRenderState(RenderState.CullMode, Cull)
            Else
                dxDevice.SetRenderState(RenderState.AlphaBlendEnable, False)
                dxDevice.SetRenderState(RenderState.CullMode, Cull)
            End If
        End Set
    End Property

    ''' <summary>
    ''' Aktiviert\Deaktiviert default Renderstates um Pointsprites zu rendern.
    ''' </summary>
    ''' <param name="size">Angabe der Punktgröße</param>
    ''' <value></value>
    ''' <remarks></remarks>
    Public WriteOnly Property PointSprite(ByVal size As Single) As Boolean
        Set(ByVal value As Boolean)
            If value Then
                With dxDevice
                    .SetRenderState(RenderState.EmissiveMaterialSource, ColorSource.Color1)
                    .SetRenderState(RenderState.PointSpriteEnable, True)
                    .SetRenderState(RenderState.PointScaleEnable, True)
                    .SetRenderState(RenderState.PointSize, size)
                End With
            Else
                dxDevice.SetRenderState(RenderState.PointSpriteEnable, False)
            End If
        End Set
    End Property
    ''' <summary>
    ''' Aktiviert\Deaktiviert default Renderstates um Pointsprites zu rendern.
    ''' </summary>
    ''' 
    Public WriteOnly Property PointSprite() As Boolean
        Set(ByVal value As Boolean)
            If value Then
                With dxDevice
                    .SetRenderState(RenderState.EmissiveMaterialSource, ColorSource.Color1)
                    .SetRenderState(RenderState.PointSpriteEnable, True)
                    .SetRenderState(RenderState.PointScaleEnable, True)
                    .SetRenderState(RenderState.PointSize, 1)
                End With
            Else
                dxDevice.SetRenderState(RenderState.PointSpriteEnable, False)
            End If
        End Set
    End Property

    Public ReadOnly Property setSamplerState() As clsSamplerState
        Get
            Return _filter
        End Get
    End Property
#End Region
#Region "Property Class - clsFilter"
    ''' <summary>
    ''' Ruft die Texturfilter - Einstellungen ab.
    ''' </summary>
    Public Class clsSamplerState
        ''' <summary>
        ''' Legt den MipMap's Filter fest.
        ''' </summary>
        ''' <param name="State">Legt das Samplerstate fest.</param>
        Public WriteOnly Property MipFilter(Optional ByVal State As Integer = 0) As TextureFilter
            Set(ByVal value As TextureFilter)
                dxDevice.SetSamplerState(State, SamplerState.MipFilter, value)
            End Set
        End Property
        ''' <summary>
        ''' Legt den MagFilter fest. (Filter für weit entfernte Pixel)
        ''' </summary>
        ''' <param name="State">Legt das Samplerstate fest.</param>
        Public WriteOnly Property MagFilter(Optional ByVal State As Integer = 0) As TextureFilter
            Set(ByVal value As TextureFilter)
                dxDevice.SetSamplerState(State, SamplerState.MagFilter, value)
                dxDevice.SetSamplerState(0, SamplerState.MinFilter, TextureFilter.Linear)
            End Set
        End Property
        ''' <summary>
        ''' Legt den MinFilter fest. (Filter für nahe Pixel)
        ''' </summary>
        ''' <param name="State">Legt das Samplerstate fest.</param>
        Public WriteOnly Property MinFilter(Optional ByVal State As Integer = 0) As TextureFilter
            Set(ByVal value As TextureFilter)
                dxDevice.SetSamplerState(State, SamplerState.MinFilter, value)
            End Set
        End Property
        ''' <summary>
        ''' Legt die MaxAnisotropy fest.
        ''' </summary>
        ''' <param name="State">Legt das Samplerstate fest.</param>
        Public WriteOnly Property MaxAnisotropy(Optional ByVal State As Integer = 0) As Integer
            Set(ByVal value As Integer)
                dxDevice.SetSamplerState(State, SamplerState.MaxAnisotropy, value)
            End Set
        End Property
        ''' <summary>
        ''' Legt den MipMapLodBias fest. (negative Werte erhöhen die Qualität)
        ''' </summary>
        ''' <param name="State">Legt das Samplerstate fest.</param>
        Public WriteOnly Property MipMapLodBias(Optional ByVal State As Integer = 0) As Single
            Set(ByVal value As Single)
                dxDevice.SetSamplerState(State, SamplerState.MipMapLodBias, value)
            End Set
        End Property

        Public WriteOnly Property AdressU(Optional ByVal State As Integer = 0) As TextureAddress
            Set(ByVal value As TextureAddress)
                dxDevice.SetSamplerState(State, SamplerState.AddressU, value)
            End Set
        End Property
        Public WriteOnly Property AdressV(Optional ByVal State As Integer = 0) As TextureAddress
            Set(ByVal value As TextureAddress)
                dxDevice.SetSamplerState(State, SamplerState.AddressV, value)
            End Set
        End Property
    End Class
#End Region

End Class




