﻿' Title    : sdxBaseEngine Framework
' Name     : engineTimer
' Property : dxTimer
' Author   : Sascha Bajus
' Created  : 12/2008
' Copyright © 2008 Sascha Bajus

#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region



Imports System
Imports System.Runtime.InteropServices
Imports SlimDX

    '''<summary>
    '''High Performance Timer Klasse zur genauen Zeitmessung in zehntel Millisekunden
    '''</summary>
Public Class engineTimer

    Private _fps As String
    Private _frametime As Single
    Private _fpsTick As Single
    Private _startTime As Long
    Private _frequency As Long
    Private _Gametime As Single
    Private _Runningtime As Single
    Private _Runningtimeold As Single
    Private _Gamespeed As Single
    Private Declare Ansi Function QueryPerformanceCounter Lib "kernel32.dll" (ByRef x As Long) As Integer
    Private Declare Ansi Function QueryPerformanceFrequency Lib "kernel32.dll" (ByRef x As Long) As Integer

    '''<summary>
    '''Erstellt einen neuen Gametimer
    '''</summary>
    Public Sub New()
        Reset()
    End Sub

#Region "Timer Reset"
    '''<summary>
    '''reset des Timers
    '''</summary>
    Public Sub Reset()
        _startTime = GetValue()
        _frequency = GetFrequency()
        _fps = "0"
        _fpsTick = 0
        _Gametime = 0
        _Runningtime = 0
        _Runningtimeold = 0
        _Gamespeed = 0
    End Sub
#End Region
#Region "Timer Update"
    '''<summary>
    '''Aktuallisiert den Gametimer
    '''</summary>
    Public Sub UpdateGametime()
        _Runningtime = CType((((GetValue() - StartTime) / CType(Frequency, Double)) * 1000), Long)
        _Gamespeed = _Runningtime - _Runningtimeold
        If _Runningtime - _fpsTick >= 1000 Then
            _fps = CInt((_Runningtime - _fpsTick) / _Gamespeed).ToString
            _fpsTick = _Runningtime
        End If
        _Runningtimeold = _Runningtime
        _Gametime = CSng(_Gametime + _Gamespeed)
        _frametime = 1000 / _Gamespeed
    End Sub
#End Region
#Region "Timer Elapsed Runningtime"
    '''<summary>
    '''Rückgabe der verstrichene Zeit seit dem Timerstart (zehntel Millisekunden)
    '''</summary>
    Public ReadOnly Property Runningtime() As Double
        Get
            Return _Runningtime
        End Get
    End Property
#End Region
#Region "Timer Elapsed Gametime"
    '''<summary>
    ''' Rückgabe der Zeit zur Multiplikation von renderzeitabhängigen Werten (zehntel Millisekunden)
    ''' Der Zeitfaktor addiert sich bei jedem Durchlauf.
    '''</summary>
    Public ReadOnly Property Gametime() As Single
        Get
            Return _Gametime
        End Get
    End Property
#End Region
#Region "Timer Gamespeed"
    '''<summary>
    '''Rückgabe der verstrichene Zeit, seit dem letzten Rendervorgang (zehntel Millisekunden)
    '''</summary>
    Public ReadOnly Property GameRendertime() As Single
        Get
            Return _Gamespeed
        End Get
    End Property
    ''' <summary>
    ''' Rückgabe des Zeitfaktors zur Multiplikation von renderzeitabhängigen Werten
    ''' </summary>
    ''' <param name="timeUpdate"></param>
    Public ReadOnly Property GameRendertime(ByVal timeUpdate As Single) As Single
        Get
            Return timeUpdate / _Gamespeed
        End Get
    End Property
    ''' <summary>
    ''' Rückgabe des Framefaktors zur Multiplikation von frameabhängigen Werten
    ''' </summary>
    ''' <value></value>
    Public ReadOnly Property Frametime() As Single
        Get
            Return 1000 / _frametime
        End Get
    End Property
#End Region

#Region "Timebased Fading"
    ''' <summary>
    ''' Rückgabe eines pendelnden Wertes zwischen 0 und 1 in Abhängigkeit von Gametime und updateTime
    ''' </summary>
    ''' <param name="speed">Angabe der Geschwindigkeit</param>

    Public Function Timefading(ByVal speed As Single) As Single
        speed = CSng(speed * Gametime)
        Timefading = CSng(Math.Sin(speed))
        Timefading *= Math.Sign(Timefading)
    End Function

    ''' <summary>
    ''' Rückgabe eines pendelnden Wertes zwischen 0 und 1 in Abhängigkeit von Gametime und updateTime
    ''' </summary>
    ''' <param name="speed">Angabe der Geschwindigkeit</param>
    ''' <param name="min">Angabe des Minimalwertes (Wert muss größer, gleich 0 sein)</param>
    ''' <param name="max">Angabe des Maximalwertes (Wert muss kleiner, gleich 1 sein</param>

    Public Function Timefading(ByVal speed As Single, ByVal min As Single, ByVal max As Single) As Double
        speed = CSng(speed * Gametime)
        Timefading = Math.Sin(speed)
        Timefading *= Math.Sign(Timefading)
        If Timefading < min Then Return min
        If Timefading > max Then Return max
    End Function
#End Region
#Region "Timebased Sin\Cos"
    Public Function TimebasedCircle(ByVal speed As Single) As Vector2
        speed = speed * _Gametime
        TimebasedCircle.X = CSng(Math.Cos(speed))
        TimebasedCircle.Y = CSng(Math.Sin(speed))
    End Function
#End Region
#Region "Timebased Sin"
    Public Function TimebasedSinus(ByVal updateTime As Double, ByVal angle As Single) As Single
        angle = CSng(angle * (_Runningtime / updateTime))
        TimebasedSinus = CSng(Math.Sin(angle))
    End Function

    Public Function TimebasedSinus(ByVal updateTime As Double, ByVal angle As Single, ByVal min As Single, ByVal max As Single) As Double
        angle = CSng(angle * (_Runningtime / updateTime))
        TimebasedSinus = Math.Sin(angle)
        If TimebasedSinus < min Then Return min
        If TimebasedSinus > max Then Return max
    End Function
#End Region
#Region "Timebased Cos"
    Public Function TimebasedCosinus(ByVal updateTime As Double, ByVal angle As Single) As Double
        angle = CSng(angle * (_Runningtime / updateTime))
        TimebasedCosinus = Math.Cos(angle)
    End Function

    Public Function TimebasedCosinus(ByVal updateTime As Double, ByVal angle As Single, ByVal min As Single, ByVal max As Single) As Double
        angle = CSng(angle * (_Runningtime / updateTime))
        TimebasedCosinus = Math.Cos(angle)
        If TimebasedCosinus < min Then Return min
        If TimebasedCosinus > max Then Return max
    End Function
#End Region

#Region "Return FPS"
    '''<summary>
    '''Rückgabe der "Frames pro Sekunde" als String
    '''</summary>
    Public ReadOnly Property FrameCounter() As String
        Get
            Return _fps
        End Get
    End Property
#End Region
#Region "Privates"
    Private Property StartTime() As Long
        Get
            Return _startTime
        End Get
        Set(ByVal Value As Long)
            _startTime = Value
        End Set
    End Property
    Private Property Frequency() As Long
        Get
            Return _frequency
        End Get
        Set(ByVal Value As Long)
            _frequency = Value
        End Set
    End Property
    Private Function GetValue() As Long
        Dim ret As Long = 0
        If QueryPerformanceCounter(ret) = 0 Then Throw New NotSupportedException("QueryPerformanceCounter not Supported.")
        Return ret
    End Function
    Private Function GetFrequency() As Long
        Dim ret As Long = 0
        If QueryPerformanceFrequency(ret) = 0 Then Throw New NotSupportedException("QueryPerformanceCounter not Supported.")
        Return ret
    End Function
#End Region

End Class
