﻿' Title    : sdxBaseEngine Framework
' Name     : engineTransform
' Property : dxTransform
' Author   : Sascha Bajus
' Created  : 12/2008
' Copyright © 2008 Sascha Bajus

#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region




Imports SlimDX
Imports SlimDX.Direct3D9
Imports sdxBaseEngine.SDX9BaseEngine


'''<summary>
'''Matrix Klasse zur Transformation von Objekten. Objekte können auch Zeitabhänging rotiert, skaliert, und transformiert werden.
'''</summary>
Public Class engineTransform
    Private _matrix As Matrix
    Private _billboardMatrix As Matrix

    '''<summary>
    '''Erstellt einen neue Transformklasse
    '''</summary>
    Public Sub New()
    End Sub

#Region "Rotate-X"
    '''<summary>
    '''Zeitabhänginge rotation um die X-Achse
    '''</summary>
    ''' <param name="speed">Angabe der Geschwindigkeit</param>
    Public Function TimebasedRotationX(ByVal speed As Single) As Matrix
        speed = speed * dxGametime.Gametime
        Return Matrix.RotationX(speed)
    End Function
#End Region
#Region "Rotate-Y"
    '''<summary>
    '''Zeitabhänginge rotation um die Y-Achse
    '''</summary>
    ''' <param name="speed">Angabe der Geschwindigkeit</param>
    Public Function TimebasedRotationY(ByVal speed As Single) As Matrix
        speed = speed * dxGametime.Gametime
        Return Matrix.RotationY(speed)
    End Function
#End Region
#Region "Rotate-Z"
    '''<summary>
    '''Zeitabhänginge rotation um die Z-Achse
    '''</summary>
    ''' <param name="speed">Angabe der Geschwindigkeit</param>
    Public Function TimebasedRotationZ(ByVal speed As Single) As Matrix
        speed = speed * dxGametime.Gametime
        Return Matrix.RotationZ(speed)
    End Function
#End Region
#Region "Rotate-XYZ"
    ''' <summary>
    ''' Zeitabhänginge rotation um die XYZ-Achse
    ''' </summary>
    ''' <param name="speed">Angabe der Geschwindigkeit</param>
    Public Function TimebasedRotationXYZ(ByVal speed As Vector3) As Matrix
        speed *= dxGametime.Gametime
        _matrix = Matrix.RotationX(speed.X) * _
                  Matrix.RotationY(speed.Y) * _
                  Matrix.RotationZ(speed.Z)
        Return _matrix
    End Function

    ''' <summary>
    ''' Rotation um die XYZ-Achse
    ''' </summary>
    ''' <param name="speed">Angabe der Geschwindigkeit</param>
    ''' <param name="translate">Angabe der Positionierung als Vector3 (Nothing positioniert im Vector3(0,0,0))</param>
    ''' <returns></returns>
    ''' <remarks></remarks>

    Public Function TimebasedRotationXYZ(ByVal speed As Vector3, ByVal translate As Vector3) As Matrix
        speed *= dxGametime.Gametime
        _matrix = Matrix.RotationX(speed.X) * _
                  Matrix.RotationY(speed.Y) * _
                  Matrix.RotationZ(speed.Z) * _
                  Matrix.Translation(translate)
        Return _matrix
    End Function
#End Region

#Region "Timebased ScaleTranslateRotate transform"
    '''<summary>
    ''' Erstellt eine WorldMatrix zur zeitbasierten Skalierung, Rotation, Position und wendet diese an.
    '''</summary >
    '''<param name="speed">Angabe des Geschwindigkeit</param>
    '''<param name="scale">Angabe der Skalierung als Single, XYZ gleiche Werte</param>
    '''<param name="translate ">Angabe der Positionierung als Vector3 (Nothing positioniert im Vector3(0,0,0))</param>
    '''<param name="Billboarding ">Transformiert das Objekt mit der Frontseite zur Kamera</param>
    Public Function TimebasedScaleTranslateRotate(ByVal speed As Vector3, _
                          ByVal translate As Vector3, Optional ByVal scale As Single = 1, Optional ByVal Billboarding As Boolean = False) As Matrix

        If Not speed = Nothing Then speed *= CSng(dxGametime.Gametime)


        _matrix = Matrix.Scaling(New Vector3(scale, scale, scale)) * _
                  Matrix.RotationX(speed.X) * _
                  Matrix.RotationY(speed.Y) * _
                  Matrix.RotationZ(speed.Z) * _
                  Matrix.Translation(translate)

        If Billboarding Then
            _billboardMatrix = dxDevice.GetTransform(TransformState.View)
            _billboardMatrix.Invert()

            Dim posX, posY, posZ As Single
            posX = _matrix.M41
            posY = _matrix.M42
            posZ = _matrix.M43

            _matrix *= _billboardMatrix
            _matrix.M41 = posX
            _matrix.M42 = posY
            _matrix.M43 = posZ
            _matrix.M44 = 1.0
        End If
        Return _matrix
    End Function

    
#End Region
#Region "ScaleTranslateRotate Matrix"
    '''<summary>
    ''' Erstellt eine WorldMatrix zur Skalierung, Rotation und Position und gibt diese zurück. 
    ''' Optional kann Billboarding verwendet werden. 
    ''' Billboarding benutzt eine invertierte ViewMatrix mit relativer Position des Objekts.
    '''</summary>
    '''<param name="angle">Angabe des Rotationswinkels im Bogenmaß als Vector3 (Nothing um keine Rotation zu verwenden)</param>
    '''<param name="scale">Angabe der Skalierung als Vector3 (Nothing um keine Skalierung zu verwenden)</param>
    '''<param name="translate ">Angabe der Positionierung als Vector3 (Nothing positioniert im Vector3(0,0,0))</param>
    '''<param name="Billboarding">Transformiert das Objekt mit der Frontseite zur Kamera</param>
    ''' <remarks >
    ''' Winkelrotationen werden im Bogenmaß angegeben. Eine Rotation der Y-Achse um 45° entspricht als Bogenmaß PI/4 = 0.785. 
    ''' Der enstprechende Rotationsvector ist Vector3(0,0.785,0).
    '''</remarks>
    Public Function ScaleTranslateRotate(ByVal scale As Vector3, ByVal angle As Vector3, ByVal translate As Vector3, ByVal Billboarding As Boolean) As Matrix
        _matrix = Matrix.Scaling(scale) * _
                Matrix.RotationX(angle.X) * _
                Matrix.RotationY(angle.Y) * _
                Matrix.RotationZ(angle.Z) * _
                Matrix.Translation(translate)

        If Billboarding Then
            _billboardMatrix = dxDevice.GetTransform(TransformState.View)
            _billboardMatrix.Invert()

            Dim posX, posY, posZ As Single
            posX = _matrix.M41
            posY = _matrix.M42
            posZ = _matrix.M43

            _matrix *= _billboardMatrix
            _matrix.M41 = posX
            _matrix.M42 = posY
            _matrix.M43 = posZ
            _matrix.M44 = 1.0#
        End If
        Return _matrix
    End Function
#End Region
#Region "Billboard"
    Public Function Billboard(ByVal Position As Vector3, ByVal Scale As Vector3) As Matrix
        _billboardMatrix = dxDevice.GetTransform(TransformState.View)
        _billboardMatrix.Invert()
        _billboardMatrix *= Matrix.Scaling(Scale)
        _billboardMatrix.M41 = Position.X
        _billboardMatrix.M42 = Position.Y
        _billboardMatrix.M43 = Position.Z
        _billboardMatrix.M44 = 1.0
        Return _billboardMatrix
    End Function
#End Region
#Region "Orbitalpath"
    ''' <summary>
    ''' Erstellt eine WorldMatrix zur zeitbasierten Rotation um ein Zentrum.
    ''' </summary>
    ''' <param name="center">Angabe des Rotationsmittelpunkts</param>
    ''' <param name="scale">Angabe des Radius</param>
    ''' <param name="speed">Angabe der Geschwindigkeit als Vector3</param>
    ''' <param name="updateTime">Angabe der Zeit bis zur nächsten Rotation (zehntel Millisekunden)</param>
    ''' <param name="sinX">Vertauscht Sin und Cos für XY</param>
    ''' 
    Public Function orbitalpathTimebased(ByVal center As Vector3, ByVal scale As Vector3, ByVal speed As Single, ByVal updateTime As Double, Optional ByVal sinX As Boolean = False) As Matrix
        Dim rad As Double = dxGametime.Gametime * speed
        If sinX Then
            _matrix.M41 = CSng(Math.Sin(rad)) * scale.X
            _matrix.M42 = CSng(Math.Cos(rad)) * scale.Y
            _matrix.M43 = CSng(Math.Cos(rad)) * scale.Z
            _matrix *= Matrix.Translation(center)
        Else
            _matrix.M41 = CSng(Math.Cos(rad)) * scale.X
            _matrix.M42 = CSng(Math.Sin(rad)) * scale.Y
            _matrix.M43 = CSng(Math.Cos(rad)) * scale.Z
            _matrix *= Matrix.Translation(center)
        End If
        Return _matrix
    End Function
#End Region

End Class



