﻿' Title    : sdxBaseEngine Framework
' Author   : Sascha Bajus
' Created  : 01/2009
' Copyright © 2008 Sascha Bajus

#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region




Imports SlimDX
Imports SlimDX.Direct3D9
Imports sdxBaseEngine.SDX9BaseEngine


'''<summary>
'''Geometrie Klasse zur Erstellung einer FVF Billboardfläche. Das Vertexformat ist Position\Normal\Textured
'''</summary>
Public Class sdxBillboard
    Private _vbuffer As VertexBuffer            '#VertexBuffer
    Private _vertexdec As VertexDeclaration     '#VertexDeclaration
    Private _texList As New List(Of Texture)    '#List of Textures for the Object
    Private _material As New Material           '#Material of the Object
    Private _enabled As Boolean = True          '#Enable\Disable Rendering
    Private _nrm As Vector3                     '#NormalVector 
    Private _PrimitiveCount As Integer = 2      '#Triangle Count

    Private _pos(5) As Vector3
    Private _tex(5) As Vector2

    ''' <summary>
    ''' Erstellt eine Rechteck-Geometrie für ein texturiertes Billboard
    ''' Die Fläche wird default zentriert im Nullpunkt erstellt.
    ''' </summary>
    ''' <param name="scale">Legt die Größe und Richtung der Skalierung fest</param>
    Public Sub New(ByVal scale As Vector3)

        AddHandler dxEngineClosing, AddressOf dispose
        AddHandler dxDeviceLost, AddressOf DeviceLost
        AddHandler dxDeviceReset, AddressOf DeviceReset

        Dim posX, posY, posZ, scaX, scaY, scaZ As Single

        posX = scale.X / 2 * -1
        posY = scale.Y / 2
        posZ = scale.Z / 2

        scaX = scale.X
        scaY = scale.Y
        scaZ = scale.Z

        createBillboard(posX, posY, posZ, scaX, scaY, scaZ)
        defaultmaterial()
    End Sub

    ''' <summary>
    ''' Erstellt eine Rechteck-Geometrie für ein texturiertes Billboard
    ''' </summary>
    ''' <param name="scale">Legt die Größe und Richtung der Skalierung fest</param>
    ''' <param name="center">Legt Zetrierungsachsen fest</param>
    Public Sub New(ByVal scale As Vector3, ByVal center As Vector3)
        AddHandler dxEngineClosing, AddressOf dispose
        AddHandler dxDeviceLost, AddressOf DeviceLost
        AddHandler dxDeviceReset, AddressOf DeviceReset

        Dim posX, posY, posZ, scaX, scaY, scaZ As Single

        posX = (scale.X - (scale.X / 2) * center.X) * -1
        posY = scale.Y - (scale.Y / 2) * center.Y
        posZ = scale.Z - (scale.Z / 2) * center.Z
        scaX = scale.X
        scaY = scale.Y
        scaZ = scale.Z

        createBillboard(posX, posY, posZ, scaX, scaY, scaZ)
        defaultmaterial()
    End Sub



#Region "New Billboard"
    Private Sub createBillboard(ByVal posX As Single, _
                                 ByVal posY As Single, _
                                 ByVal posZ As Single, _
                                 ByVal scaX As Single, _
                                 ByVal scaY As Single, _
                                 ByVal scaZ As Single)


        '#get Positions,Normal and Texcords


        '#Triangle 1 
        _pos(0) = (New Vector3(posX, posY, posZ))               'Top left  (texture point 0,0)
        _pos(1) = (New Vector3(posX + scaX, posY, posZ))        'Top right (texture point 1,0)
        _pos(2) = (New Vector3(posX, posY - scaY, posZ - scaZ)) 'Bot left  (texture point 0,1)

        '#TextureCords
        _tex(0) = (New Vector2(0, 0))
        _tex(1) = (New Vector2(1, 0))
        _tex(2) = (New Vector2(0, 1))

        'get NormalVector
        _nrm = ComputeNormalVector(_pos(0), _pos(1), _pos(2))

        '--------------------------------------------------------------------------------

        '#Triangle 2
        _pos(3) = (New Vector3(posX + scaX, posY, posZ))               'Top right  (texture point 1,0)
        _pos(4) = (New Vector3(posX + scaX, posY - scaY, posZ - scaZ)) 'Bot right  (texture point 1,1)
        _pos(5) = (New Vector3(posX, posY - scaY, posZ - scaZ))        'Bot left   (texture point 0,1)

        '#TextureCords
        _tex(3) = (New Vector2(1, 0))
        _tex(4) = (New Vector2(1, 1))
        _tex(5) = (New Vector2(0, 1))

        createVbuffer()
    End Sub


    Private Sub createVbuffer()
        '#create vertexstructure
        Dim vertex(5) As VertexPositionNormalTexture
        For i As Integer = 0 To 5
            vertex(i) = New VertexPositionNormalTexture(_pos(i), _nrm, _tex(i))
        Next

        '#create vbuffer
        _vbuffer = New VertexBuffer(dxDevice, VertexPositionNormalTexture.SizeOfBytes * 6, Usage.None, VertexFormat.None, Pool.Managed)
        Dim stream As DataStream = _vbuffer.Lock(0, 0, LockFlags.None)
        stream.WriteRange(vertex, 0, 0)
        stream.Close()
        stream.Dispose()
        _vbuffer.Unlock()

        '#create vertexdeclaration
        Dim elements(3) As VertexElement
        elements(0) = New VertexElement(0, 0, DeclarationType.Float3, DeclarationMethod.Default, DeclarationUsage.Position, 0)
        elements(1) = New VertexElement(0, 12, DeclarationType.Float3, DeclarationMethod.Default, DeclarationUsage.Normal, 0)
        elements(2) = New VertexElement(0, 24, DeclarationType.Float2, DeclarationMethod.Default, DeclarationUsage.TextureCoordinate, 0)
        elements(3) = VertexElement.VertexDeclarationEnd
        _vertexdec = New VertexDeclaration(dxDevice, elements)
    End Sub
#End Region
#Region "Default Material"
    Private Sub defaultmaterial()
        With _material '# default Material
            .Ambient = Color.White
            .Diffuse = Color.White
            .Emissive = Color.White
            .Specular = Color.White
            .Power = 2
        End With
    End Sub
#End Region
#Region "## Render ##"
    '''<summary>
    '''Rendert das erstellte Billboard
    '''</summary>
    ''' <param name="useDefaultMaterial">Legt fest, ob das emissive default Material an den Device übergeben wird</param>
    Public Sub draw(Optional ByVal useDefaultMaterial As Boolean = True)
        If useDefaultMaterial Then setMaterial()
        dxDevice.VertexDeclaration = _vertexdec
        dxDevice.SetStreamSource(0, _vbuffer, 0, VertexPositionNormalTexture.SizeOfBytes)
        dxDevice.DrawPrimitives(PrimitiveType.TriangleList, 0, _PrimitiveCount)
    End Sub
#End Region

#Region "Create Normal"
    '#Berechnung des Normalvectors
    Private Function ComputeNormalVector(ByVal v1 As Vector3, _
                                ByVal v2 As Vector3, _
                                ByVal v3 As Vector3) As Vector3
        Dim retval As Vector3
        retval = Vector3.Cross(Vector3.Subtract(v2, v1), Vector3.Subtract(v3, v1))
        Return Vector3.Normalize(retval)
    End Function
#End Region
#Region "Set Texture for Billboard"
    '''<summary>
    '''Legt die zu rendernde Textur fest (TextureStage,TexturIndex)
    '''</summary>
    '''<param name="TextureStage">legt das zu verwendende Texturestage fest</param>
    '''<param name="TexturIndex">legt die zu verwendende Textur aus der Liste fest</param>

    Public Sub setTexture(ByVal TextureStage As Integer, ByVal TexturIndex As Integer)
        dxDevice.SetTexture(TextureStage, _texList(TexturIndex))
    End Sub
#End Region
#Region "Add Texture to List"

    ''' <summary>
    ''' Fügt dem Billboard Texturen hinzu. Die Texturen müssen vor dem Rendern mit setTexture zugewiesen werden.
    ''' </summary>
    ''' <param name="colormap">Gibt die Texture an</param>
    Public Sub addColormap(ByVal colormap As Texture)
        _texList.Add(colormap)
    End Sub

#End Region
#Region "Property"
    ''' <summary>
    ''' ruft den objektgebundenen Abfragewert ab oder legt diesen fest. (Default = True)
    ''' </summary>
    ''' 
    Public Property Enabled() As Boolean
        Get
            Return _enabled
        End Get
        Set(ByVal value As Boolean)
            _enabled = value
        End Set
    End Property

    ''' <summary>
    ''' Legt den Ambientwert des Materials fest
    ''' </summary>
    Public WriteOnly Property materialAmbient() As Color
        Set(ByVal value As Color)
            _material.Ambient = value
        End Set
    End Property

    ''' <summary>
    ''' Legt den Diffusewert des Materials fest
    ''' </summary>
    Public WriteOnly Property materialDiffuse() As Color
        Set(ByVal value As Color)
            _material.Diffuse = value
        End Set
    End Property

    ''' <summary>
    ''' Legt den Emissivewert des Materials fest
    ''' </summary>
    Public WriteOnly Property materialEmissive() As Color
        Set(ByVal value As Color)
            _material.Emissive = value
        End Set
    End Property
    ''' <summary>
    ''' ruft den untransformierten NormalVector ab.
    ''' </summary>
    Public ReadOnly Property NormalVector() As Vector3
        Get
            Return _nrm
        End Get
    End Property
#End Region

#Region "Event Dispose"
    Private Sub Dispose()
        For i As Integer = 0 To _texList.Count - 1
            If Not _texList(i) Is Nothing Then _texList(i).Dispose()
        Next
        _texList.Clear()
        If Not _vertexdec Is Nothing Then _vertexdec.Dispose()
        If Not _vbuffer Is Nothing Then _vbuffer.Dispose()
    End Sub
#End Region

#Region "Event DeviceLost"
    Private Sub DeviceLost()
        Dispose()
    End Sub
#End Region

#Region "Event DeviceReset"
    Private Sub DeviceReset()
        createVbuffer()
    End Sub
#End Region

#Region "Privates"
    Private Sub setMaterial()
        dxDevice.Material = _material
    End Sub

    '#Load Texture from file
    Private Function createTextureFromFile(ByVal src As String, ByVal colorKey As Color) As Texture
        Return Texture.FromFile(dxDevice, src, 0, 0, 0, 0, Format.Unknown, Pool.Managed, Filter.Linear, Filter.Linear, colorKey.ToArgb)
    End Function
    Private Function createTextureFromMemory(ByVal memory As Byte(), ByVal colorKey As Color) As Texture
        Return Texture.FromMemory(dxDevice, memory, 0, 0, 0, 0, Format.Unknown, Pool.Managed, Filter.Linear, Filter.Linear, colorKey.ToArgb)
    End Function
#End Region
End Class
