﻿' Title    : sdxBaseEngine Class
' Name     : sdxClipPlane
' Author   : Sascha Bajus
' Created  : 01/2009
' Copyright © 2009 Sascha Bajus

#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region




Imports SlimDX
Imports SlimDX.Direct3D9
Imports sdxBaseEngine.SDX9BaseEngine

''' <summary>
''' ClipPlane-Klasse zur Erstellung einer rechteckigen Clipping Fläche
''' </summary>
''' <remarks></remarks>
Public Class sdxClipPlane

    Private a As Vector3 '#UpperLeft
    Private b As Vector3 '#UpperRight
    Private c As Vector3 '#DownLeft
    Private d As Vector3 '#DownRight

    Private _a As Vector3 '#transformedUpperLeft
    Private _b As Vector3 '#transformedUpperRight
    Private _c As Vector3 '#transformedDownLeft
    Private _d As Vector3 '#transformedDownRight

    Private _plane As Plane
    Private _matrix As Matrix


  
    ''' <summary>
    ''' Erstellt eine neue ClippingPlane.
    ''' </summary>
    ''' <param name="scale">Legt die Größe und Richtung der Skalierung fest.</param>
    Public Sub New(ByVal scale As Vector3)
        Dim invnormal As Vector3
        If scale.X = 0 Then invnormal = New Vector3(0, 1, 1)
        If scale.Y = 0 Then invnormal = New Vector3(1, 0, 1)
        If scale.Z = 0 Then invnormal = New Vector3(1, 1, 0)


        a = New Vector3((scale.X - (scale.X / 2 * invnormal.X)) * -1, _
                        scale.Y - (scale.Y / 2 * invnormal.Y), _
                        scale.Z - (scale.Z / 2 * invnormal.Z))

        b = New Vector3(a.X + scale.X, a.Y, a.Z)
        c = New Vector3(a.X, a.Y - scale.Y, a.Z - scale.Z)
        d = New Vector3(a.X + scale.X, a.Y - scale.Y, a.Z - scale.Z)

        _a = a
        _b = b
        _c = c
        _d = d

        _plane = New Plane(_a, _c, _b)
    End Sub
    ''' <summary>
    ''' Erstellt eine neue ClippingPlane.
    ''' </summary>
    ''' <param name="scale">Legt die Größe und Richtung der Skalierung fest.</param>
    ''' <param name="trim">Legt die Justierung des Startpunktes fest</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal scale As Vector3, ByVal trim As Vector3)
        Dim invnormal As Vector3
        If scale.X = 0 Then invnormal = New Vector3(0, 1, 1)
        If scale.Y = 0 Then invnormal = New Vector3(1, 0, 1)
        If scale.Z = 0 Then invnormal = New Vector3(1, 1, 0)

        a = New Vector3((scale.X - (scale.X / 2 * invnormal.X)) * -1, _
                        scale.Y - (scale.Y / 2 * invnormal.Y), _
                        scale.Z - (scale.Z / 2 * invnormal.Z))
        a += trim
        b = New Vector3(a.X + scale.X, a.Y, a.Z)
        c = New Vector3(a.X, a.Y - scale.Y, a.Z - scale.Z)
        d = New Vector3(a.X + scale.X, a.Y - scale.Y, a.Z - scale.Z)

        _a = a
        _b = b
        _c = c
        _d = d

        _plane = New Plane(_a, _c, _b)
    End Sub
#Region "Properties"
    ''' <summary>
    ''' Legt die Transformation des Planes fest.
    ''' </summary>
    Public WriteOnly Property TransformCoordinate() As Matrix
        Set(ByVal value As Matrix)
            _a = Vector3.TransformCoordinate(a, value)
            _b = Vector3.TransformCoordinate(b, value)
            _c = Vector3.TransformCoordinate(c, value)
            _d = Vector3.TransformCoordinate(d, value)
            _plane = New Plane(_a, _c, _b)
        End Set
    End Property

    ''' <summary>
    ''' ruft die erstellte Plane ab.
    ''' </summary>
    Public Property ClipPlane() As Plane
        Get
            Return _plane
        End Get
        Set(ByVal value As Plane)
            _plane = value
        End Set
    End Property

    ''' <summary>
    ''' ruft die Reflektions-Matrix des Planes ab.
    ''' </summary>
    Public ReadOnly Property ReflectionMatrix() As Matrix
        Get
            Return Matrix.Reflection(_plane)
        End Get
    End Property
    ''' <summary>
    ''' Aktiviert ClipPlanes um Ausserhalb der Plane zu Clippen oder deaktiviert diese.
    ''' </summary>
    Public WriteOnly Property ClippingEnabled() As Boolean
        Set(ByVal value As Boolean)
            If value = True Then
                With dxDevice
                    '#Mirror Cullmode
                    .SetRenderState(RenderState.CullMode, Cull.Clockwise)
                    '#setup Planes
                    .SetClipPlane(0, New Plane(_a, _c, dxCamera.CurrentPosition))
                    .SetClipPlane(1, New Plane(_c, _d, dxCamera.CurrentPosition))
                    .SetClipPlane(2, New Plane(_d, _b, dxCamera.CurrentPosition))
                    .SetClipPlane(3, New Plane(_b, _a, dxCamera.CurrentPosition))
                    '#enable Planes
                    .SetRenderState(RenderState.ClipPlaneEnable, ClipFlags.All)
                    '#enable Clipping
                    .SetRenderState(RenderState.Clipping, True)
                End With
            Else
                '#disable clipping
                dxDevice.SetRenderState(RenderState.ClipPlaneEnable, False)
            End If
        End Set
    End Property
#End Region
End Class






