﻿' Title    : sdxBaseEngine Class
' Name     : sdxXfile
' Author   : Sascha Bajus
' Created  : 12/2008
' Copyright © 2008 Sascha Bajus

#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region




Imports SlimDX
Imports SlimDX.Direct3D9
Imports System.IO
Imports sdxBaseEngine.SDX9BaseEngine


    '''<summary>
''' Mesh Klasse zum Import von 3D Objekten im Xfile Format. 
''' Alle Materialeinstellungen der Subsets, können aus dem Xfile geladen
''' und in einer Materialliste gespeichert werden. 
''' Dem Meshobjekt können zusätzliche Texturen und Materialien hinzugefügt werden.
''' Das Material und die Texturen können von der FFP benutzt werden.
''' Für die Shadernutzung muss ein Shadermaterial verwendet werden, dass über das Meshobjekt
''' eingestellt werden kann.
''' Dem Meshobjekt werden automatisch Tangente und Bitangente hinzugefügt um ein shaderbasiertes
''' Normalmapping zu realisieren.
'''</summary>

Public Class sdxMesh
    Private _Model As Mesh                                   '#Mesh Objekt
    Private _subsets As Integer                              '#SubsetCount
    Private _enabled As Boolean = True
    Private _shaderMaterial() As shaderLight.MaterialSet = Nothing   '#ShaderMateriallist

#Region "Event Handles"
    Public Sub New()
        AddHandler dxEngineClosing, AddressOf Dispose
    End Sub

    Private Sub Dispose()
        If Not _Model Is Nothing Then _Model.Dispose()
    End Sub
#End Region
#Region "load Mesh"
    '''<summary>
    '''Erstellt ein neues Mesh Objekt aus einem Xfile mit import des Materials
    '''</summary>
    '''<param name="src">Angabe des Dateipfades der Xfile Datei im ApplicationPath</param>

    Public Sub MeshFromFile(ByVal src As String)
        dxEngine.LoadingStatus = "Loading Mesh: " & src & " ..."
        src = dxEngine.ApplicationPath & src
        _Model = Mesh.FromFile(dxDevice, src, MeshFlags.Managed)
        setupModelData()
    End Sub
    Public Sub MeshFromFile(ByVal src As String, ByVal CountShaderMaterial As Integer)
        dxEngine.LoadingStatus = "Loading Mesh: " & src & " ..."
        src = dxEngine.ApplicationPath & src
        _Model = Mesh.FromFile(dxDevice, src, MeshFlags.Managed)
        setupModelData(CountShaderMaterial - 1)
    End Sub

    '#
    Public Sub MeshFromEngineResource(ByVal src As String)
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then
            With dxEngine.ResourceLoader
                _Model = Mesh.FromMemory(dxDevice, .GetResourceObject(src), MeshFlags.Managed)
                dxEngine.LoadingStatus = "Loading Mesh: " & .currentResourceFile & " - " & .currentObjectName & " ..."
            End With
            setupModelData()
        End If
    End Sub

    Public Sub MeshFromEngineResource(ByVal src As String, ByVal CountShaderMaterial As Integer)
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then
            With dxEngine.ResourceLoader
                _Model = Mesh.FromMemory(dxDevice, .GetResourceObject(src), MeshFlags.Managed)
                dxEngine.LoadingStatus = "Loading Mesh: " & .currentResourceFile & " - " & .currentObjectName & " ..."
            End With
            setupModelData(CountShaderMaterial - 1)
        End If
    End Sub


    '#as byte
    Public Sub MeshFromMemory(ByVal src As Byte())
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then
            _Model = Mesh.FromMemory(dxDevice, src, MeshFlags.Managed)
            setupModelData()
        End If
    End Sub
    Public Sub MeshFromMemory(ByVal src As Byte(), ByVal CountShaderMaterial As Integer)
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then
            _Model = Mesh.FromMemory(dxDevice, src, MeshFlags.SystemMemory)
            setupModelData(CountShaderMaterial - 1)
        End If
    End Sub
    Private Sub setupModelData(ByVal ShaderMaterial As Integer)
        Dim _materialfromFile() As ExtendedMaterial = Nothing
        _materialfromFile = _Model.GetMaterials
        _subsets = _materialfromFile.Length - 1
        _shaderMaterial = Nothing
        If ShaderMaterial > 0 Then
            ReDim _shaderMaterial(ShaderMaterial)
        End If
        computeTangent()
    End Sub
    Private Sub setupModelData()
        Dim _materialfromFile() As ExtendedMaterial = Nothing
        _materialfromFile = _Model.GetMaterials
        _subsets = _materialfromFile.Length - 1
        ReDim _shaderMaterial(_subsets)
        computeTangent()
    End Sub
#End Region
#Region "Compute tangents of the mesh"
    Private Sub computeTangent()

        '#create new vertexdeclaration
        Dim elements(5) As VertexElement
        elements(0) = New VertexElement(0, 0, DeclarationType.Float3, DeclarationMethod.Default, DeclarationUsage.Position, 0)
        elements(1) = New VertexElement(0, 12, DeclarationType.Float3, DeclarationMethod.Default, DeclarationUsage.Normal, 0)
        elements(2) = New VertexElement(0, 24, DeclarationType.Float2, DeclarationMethod.Default, DeclarationUsage.TextureCoordinate, 0)
        elements(3) = New VertexElement(0, 32, DeclarationType.Float3, DeclarationMethod.Default, DeclarationUsage.Tangent, 0)
        elements(4) = New VertexElement(0, 44, DeclarationType.Float3, DeclarationMethod.Default, DeclarationUsage.Binormal, 0)
        elements(5) = VertexElement.VertexDeclarationEnd

        Dim tmpmesh As Mesh = _Model.Clone(dxDevice, MeshFlags.Managed, elements)
        _Model.Dispose()
        Try
            _Model = tmpmesh.ComputeTangentFrame(DeclarationUsage.TextureCoordinate, _
                                         0, _
                                         DeclarationUsage.Tangent, _
                                         0, _
                                         DeclarationUsage.Binormal, _
                                         0, _
                                         DeclarationUsage.Normal, _
                                         0, _
                                         TangentOptions.None, _
                                         0, _
                                         0, _
                                         0)

        Catch ex As Exception
            MsgBox(ex.ToString, MsgBoxStyle.Critical)
        End Try
        tmpmesh.Dispose()
    End Sub
#End Region

#Region "Draw"
    '''<summary>
    '''Rendert ein Subset des Models
    '''</summary>
    Public Sub Draw(Optional ByVal useShaderMaterial As Boolean = True)
        For i As Integer = 0 To _subsets
            If useShaderMaterial Then dxDevice.SetTexture(0, _shaderMaterial(i).MapColor)
            _Model.DrawSubset(i)
        Next
    End Sub
    Public Sub Draw(ByVal Subset As Integer)
        dxDevice.SetTexture(0, _shaderMaterial(Subset).MapColor)
        _Model.DrawSubset(Subset)
    End Sub
    Public Sub Draw(ByVal Subset As Integer, ByVal TextureIndex As Integer)
        dxDevice.SetTexture(0, _shaderMaterial(TextureIndex).MapColor)
        _Model.DrawSubset(Subset)
    End Sub

    Public Sub DrawFX(ByVal light As shaderLight.LightSet)
        With dxLightsFX
            .Light = light
            For i As Integer = 0 To _subsets
                .Material = _shaderMaterial(i)
                .Lighting = shaderLight.LightShading.LightBeginn
                .Lighting = shaderLight.LightShading.DrawingBeginn
                _Model.DrawSubset(i)
                .Lighting = shaderLight.LightShading.DrawingEnd
                .Lighting = shaderLight.LightShading.LightEnd
            Next
        End With
    End Sub
    Public Sub DrawFX(ByVal Subset As Integer, ByVal Light As shaderLight.LightSet)
        With dxLightsFX
            .Light = Light
            .Material = _shaderMaterial(Subset)
            .Lighting = shaderLight.LightShading.LightBeginn
            .Lighting = shaderLight.LightShading.DrawingBeginn
            _Model.DrawSubset(Subset)
            .Lighting = shaderLight.LightShading.DrawingEnd
            .Lighting = shaderLight.LightShading.LightEnd
        End With
    End Sub
    Public Sub DrawFX(ByVal Subset As Integer, ByVal Light As shaderLight.LightSet, ByVal ShaderMaterial As Integer)
        With dxLightsFX
            .Light = Light
            .Material = _shaderMaterial(ShaderMaterial)
            .Lighting = shaderLight.LightShading.LightBeginn
            .Lighting = shaderLight.LightShading.DrawingBeginn
            _Model.DrawSubset(Subset)
            .Lighting = shaderLight.LightShading.DrawingEnd
            .Lighting = shaderLight.LightShading.LightEnd
        End With
    End Sub
#End Region
#Region "Property"


    Public ReadOnly Property Model() As Mesh
        Get
            Return _Model
        End Get
    End Property

    ''' <summary>
    ''' Gibt die Anzahl der Subsets im Model zurück
    ''' </summary>
    Public ReadOnly Property Subsets() As Integer
        Get
            Return _subsets
        End Get
    End Property
    Public Property MaterialFX(ByVal Subset As Integer) As shaderLight.MaterialSet
        Get
            Return _shaderMaterial(Subset)
        End Get
        Set(ByVal value As shaderLight.MaterialSet)
            _shaderMaterial(Subset) = value
        End Set
    End Property

    Public Property Enabled() As Boolean
        Get
            Return _enabled
        End Get
        Set(ByVal value As Boolean)
            _enabled = value
        End Set
    End Property
#End Region

End Class
