﻿
' Title    : sdxBaseEngine Class
' Name     : sdxXfile
' Author   : Sascha Bajus
' Created  : 12/2008
' Copyright © 2008 Sascha Bajus

#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region




Imports SlimDX
Imports SlimDX.Direct3D9
Imports System.IO
Imports sdxBaseEngine.SDX9BaseEngine
Imports sdxBaseEngine.engineCamera

'''<summary>
'''</summary>

Public Class sdxMeshIntersection
    Private _Model As Mesh                                   '#Mesh Objekt
    Private _testUp As New Vector3(0, 1, 0)
    Private _testDown As New Vector3(0, -1, 0)
    Private _distance As Single
    Private _indexbuffer() As ibuffer
    Private _vertexbuffer() As Vector3
    Private _intersection As TriangleIntersection

    Private Structure Ibuffer
        Public index0 As UInt16
        Public index1 As UInt16
        Public index2 As UInt16
    End Structure
    Public Structure Triangle
        Public v0 As Vector3
        Public v1 As Vector3
        Public v2 As Vector3
        Public Face As Integer
    End Structure
    Public Structure TriangleIntersection
        Public Tri As Triangle
        Public Vector As Vector3
        Public Origin As Vector3
        Public Direction As Vector3
        Public Distance As Single
        Public Intersected As Boolean
    End Structure

#Region "Event Handles"
    Public Sub New()
        AddHandler dxEngineClosing, AddressOf Dispose
        AddHandler dxIntersectionTest, AddressOf IntersectionTest
    End Sub

    Private Sub Dispose()
        If Not _Model Is Nothing Then _Model.Dispose()
    End Sub
#End Region
#Region "load Mesh"
    '''<summary>
    '''Erstellt ein neues Mesh Objekt aus einem Xfile mit import des Materials
    '''</summary>
    '''<param name="src">Angabe des Dateipfades der Xfile Datei im ApplicationPath</param>

    Public Sub MeshFromFile(ByVal src As String)
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then
            dxEngine.LoadingStatus = "Loading Mesh: " & src & " ..."
            src = dxEngine.ApplicationPath & src
            _Model = Mesh.FromFile(dxDevice, src, MeshFlags.SystemMemory)
            getBuffers()
        End If
    End Sub
    Public Sub MeshFromFile(ByVal src As String, ByVal CountShaderMaterial As Integer)
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then
            dxEngine.LoadingStatus = "Loading Mesh: " & src & " ..."
            src = dxEngine.ApplicationPath & src
            _Model = Mesh.FromFile(dxDevice, src, MeshFlags.SystemMemory)
            getBuffers()
        End If
    End Sub

    '#
    Public Sub MeshFromResource(ByVal src As String)
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then
            With dxEngine.ResourceLoader
                _Model = Mesh.FromMemory(dxDevice, .GetResourceObject(src), MeshFlags.SystemMemory)
                dxEngine.LoadingStatus = "Loading Mesh: " & .currentResourceFile & " - " & .currentObjectName & " ..."
                getBuffers()
            End With
        End If
    End Sub
    Public Sub MeshFromResource(ByVal index As Integer)
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then
            With dxEngine.ResourceLoader
                _Model = Mesh.FromMemory(dxDevice, .GetResourceObject(index), MeshFlags.SystemMemory)
                dxEngine.LoadingStatus = "Loading Mesh: " & .currentResourceFile & " - " & .currentObjectName & " ..."
                getBuffers()
            End With
        End If
    End Sub
    Public Sub MeshFromResource(ByVal src As String, ByVal CountShaderMaterial As Integer)
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then
            With dxEngine.ResourceLoader
                _Model = Mesh.FromMemory(dxDevice, .GetResourceObject(src), MeshFlags.SystemMemory)
                dxEngine.LoadingStatus = "Loading Mesh: " & .currentResourceFile & " - " & .currentObjectName & " ..."
                getBuffers()
            End With
        End If
    End Sub
    Public Sub MeshFromResource(ByVal index As Integer, ByVal CountShaderMaterial As Integer)
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then
            With dxEngine.ResourceLoader
                _Model = Mesh.FromMemory(dxDevice, .GetResourceObject(index), MeshFlags.SystemMemory)
                dxEngine.LoadingStatus = "Loading Mesh: " & .currentResourceFile & " - " & .currentObjectName & " ..."
                getBuffers()
            End With
        End If
    End Sub

    '#as byte
    Public Sub MeshFromMemory(ByVal src As Byte())
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then
            _Model = Mesh.FromMemory(dxDevice, src, MeshFlags.SystemMemory)
            getBuffers()
        End If
    End Sub
    Public Sub MeshFromMemory(ByVal src As Byte(), ByVal CountShaderMaterial As Integer)
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then
            _Model = Mesh.FromMemory(dxDevice, src, MeshFlags.SystemMemory)
            getBuffers()
        End If
    End Sub

    Public Sub MeshFromEngineResource(ByVal src As String)
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then
            With dxEngine.ResourceLoader
                _Model = Mesh.FromMemory(dxDevice, .GetResourceObject(src), MeshFlags.SystemMemory)
                dxEngine.LoadingStatus = "Loading Mesh: " & .currentResourceFile & " - " & .currentObjectName & " ..."
            End With
            getBuffers()
        End If
    End Sub
#End Region

    Private Sub getBuffers()
        Dim dstream As DataStream
        Dim indexlength As Integer = _Model.FaceCount
        Dim vertexcount As Integer = _Model.VertexCount
        ReDim _vertexbuffer(vertexcount - 1)
        ReDim _indexbuffer(indexlength - 1)

        dstream = _Model.LockIndexBuffer(LockFlags.ReadOnly)
        dstream.ReadRange(_indexbuffer, 0, indexlength)
        dstream.Close()
        _Model.UnlockIndexBuffer()

        dstream = _Model.LockVertexBuffer(LockFlags.ReadOnly)
        dstream.ReadRange(_vertexbuffer, 0, vertexcount)
        dstream.Close()
        _Model.UnlockVertexBuffer()

        _Model.Dispose()
    End Sub


#Region "RayTest"
    Private Sub IntersectionTest(ByVal pos As Vector3)
        Dim direction = New Vector3(0, -1, 0)
        Dim tri As Triangle
        Dim intersection As TriangleIntersection

        For i = 0 To _indexbuffer.Length - 1

            tri.v0 = _vertexbuffer(_indexbuffer(i).index0)
            tri.v1 = _vertexbuffer(_indexbuffer(i).index1)
            tri.v2 = _vertexbuffer(_indexbuffer(i).index2)
            tri.Face = i + 1

            '#test
            intersection = IntersectTriangle(pos, direction, tri)
            If intersection.Intersected Then
                dxCamera.IntersectionData = intersection
                Exit Sub
            End If
        Next
        dxCamera.IntersectionData = intersection
    End Sub




    Private Function IntersectTriangle(ByVal origin As Vector3, ByVal direction As Vector3, ByVal tri As Triangle) As TriangleIntersection
        Dim e1, e2, tv, pv, qv As Vector3
        Dim u, v, t, rezdis, dis As Single
        Dim intersection As TriangleIntersection

        e1 = tri.v1 - tri.v0
        e2 = tri.v2 - tri.v0
        tv = origin - tri.v0

        pv = Vector3.Cross(direction, e2)

        dis = Vector3.Dot(pv, e1)
        If dis = 0 Then Return intersection
        rezdis = 1 / dis

        u = rezdis * Vector3.Dot(pv, tv)
        If u < 0 Or u > 1 Then Return intersection

        qv = Vector3.Cross(tv, e1)
        v = rezdis * Vector3.Dot(qv, direction)
        If v < 0 Or (v + u) > 1 Then Return intersection


        t = Vector3.Dot(qv, e2) * rezdis

        intersection.Vector = origin + direction * t
        intersection.Direction = direction
        intersection.Tri = tri
        intersection.Intersected = True
        intersection.Origin = origin
        intersection.Distance = t

        Return intersection
    End Function



#End Region
End Class
