﻿Imports SlimDX
Imports SlimDX.Direct3D9
Imports sdxBaseEngine.SDX9BaseEngine

Public Class sdxVBFS
    Private _vbuffer As VertexBuffer
    Private _vertexdec As VertexDeclaration
    Private _vertexdecSizeOfBytes As Integer

    Public Sub New()
        createVertexDeclaration()
        createVertexbuffer()
        fillVertexbuffer()
    End Sub

    Private Sub createVertexDeclaration()
        Dim elements(1) As VertexElement
        elements(0) = New VertexElement(0, 0, DeclarationType.Float4, DeclarationMethod.Default, DeclarationUsage.Position, 0)
        elements(1) = VertexElement.VertexDeclarationEnd
        _vertexdec = New VertexDeclaration(dxDevice, elements)
        _vertexdecSizeOfBytes = 16
    End Sub
    Private Sub createVertexbuffer()
        _vbuffer = New VertexBuffer(dxDevice, _vertexdecSizeOfBytes * 4, Usage.None, VertexFormat.None, Pool.Default)
    End Sub
    Private Sub fillVertexbuffer()

        Dim stream As DataStream = _vbuffer.Lock(0, 0, LockFlags.None)
        Dim offsetX As Single = CSng(0.5 / dxEngine.BackBufferSize.X)
        Dim offsetY As Single = CSng(0.5 / dxEngine.BackBufferSize.Y)

        Dim vertex(3) As Vector4

        vertex(0) = New Vector4(-1, -1, 0 + offsetX, 1 + offsetY)
        vertex(1) = New Vector4(-1, 1, 0 + offsetX, 0 + offsetY)
        vertex(2) = New Vector4(1, -1, 1 + offsetX, 1 + offsetY)
        vertex(3) = New Vector4(1, 1, 1 + offsetX, 0 + offsetY)

        stream.WriteRange(vertex)
        stream.Close()
        stream.Dispose()
        _vbuffer.Unlock()
    End Sub

    ''' <summary>
    ''' Gibt den VertexBuffer zurück
    ''' </summary>
    Public ReadOnly Property Buffer() As VertexBuffer
        Get
            Return _vbuffer
        End Get
    End Property
    ''' <summary>
    ''' Gibt die VertexDeclaration zurück
    ''' </summary>
    Public ReadOnly Property Declaration() As VertexDeclaration
        Get
            Return _vertexdec
        End Get
    End Property
    ''' <summary>
    ''' Gibt die Bytegröße eines Vertex zurück
    ''' </summary>
    Public ReadOnly Property SizeOfBytes() As Integer
        Get
            Return _vertexdecSizeOfBytes
        End Get
    End Property

    ''' <summary>
    ''' Rendert den Buffer
    ''' </summary>
    Public Sub Draw()
        dxDevice.VertexDeclaration = _vertexdec
        dxDevice.SetStreamSource(0, _vbuffer, 0, _vertexdecSizeOfBytes)
        dxDevice.DrawPrimitives(PrimitiveType.TriangleStrip, 0, 2)
    End Sub
    Public Sub Dispose()
        If Not _vbuffer Is Nothing Then _vbuffer.Dispose()
        If Not _vertexdec Is Nothing Then _vertexdec.Dispose()
    End Sub
End Class
