﻿Imports SlimDX
Imports SlimDX.Direct3D9
Imports sdxBaseEngine.SDX9BaseEngine

Public Class sdxVBQuad
    Private _vbuffer As VertexBuffer
    Private _vertexdec As VertexDeclaration
    Private _vertexdecSizeOfBytes As Integer
    Private Structure _VertexStructure
        Public Position As Vector3
        Public TexCoords As Vector2
    End Structure
    Public Enum Offset As Integer
        XY_Orientation = 0
        XZ_Orientation = 80
    End Enum
    Public Sub New()
        createVertexDeclaration()
        createVertexbuffer()
        fillVertexbuffer()
    End Sub
    Private Sub createVertexbuffer()
        _vbuffer = New VertexBuffer(dxDevice, _vertexdecSizeOfBytes * 8, Usage.None, VertexFormat.None, Pool.Default)
    End Sub
    Private Sub fillVertexbuffer()

        Dim stream As DataStream = _vbuffer.Lock(0, 0, LockFlags.None)

        Dim pos As New List(Of Vector3)
        Dim tex As New List(Of Vector2)
        Dim x As Single
        Dim y As Single
        Dim z As Single


        x = 1
        y = 1
        z = 0


        pos.Add(New Vector3(-x, -y, z))
        pos.Add(New Vector3(-x, y, z))
        pos.Add(New Vector3(x, -y, z))
        pos.Add(New Vector3(x, y, z))


        tex.Add(New Vector2(0, 1))
        tex.Add(New Vector2(0, 0))
        tex.Add(New Vector2(1, 1))
        tex.Add(New Vector2(1, 0))


        x = 1
        y = 0
        z = 1


        pos.Add(New Vector3(-x, y, -z))
        pos.Add(New Vector3(-x, y, z))
        pos.Add(New Vector3(x, y, -z))
        pos.Add(New Vector3(x, y, z))


        tex.Add(New Vector2(0, 1))
        tex.Add(New Vector2(0, 0))
        tex.Add(New Vector2(1, 1))
        tex.Add(New Vector2(1, 0))


        Dim length As Integer = 7
        Dim vertex(length) As _VertexStructure

        For i = 0 To length
            vertex(i).Position = pos(i)
            vertex(i).TexCoords = tex(i)
        Next


        stream.WriteRange(vertex)
        stream.Close()
        stream.Dispose()
        _vbuffer.Unlock()

    End Sub
    Private Sub createVertexDeclaration()
        Dim elements(2) As VertexElement
        elements(0) = New VertexElement(0, 0, DeclarationType.Float3, DeclarationMethod.Default, DeclarationUsage.Position, 0)
        elements(1) = New VertexElement(0, 12, DeclarationType.Float2, DeclarationMethod.Default, DeclarationUsage.TextureCoordinate, 0)
        elements(2) = VertexElement.VertexDeclarationEnd
        _vertexdec = New VertexDeclaration(dxDevice, elements)
        _vertexdecSizeOfBytes = 12 + 8
    End Sub

    ''' <summary>
    ''' Gibt den VertexBuffer zurück
    ''' </summary>
    Public ReadOnly Property Buffer() As VertexBuffer
        Get
            Return _vbuffer
        End Get
    End Property
    ''' <summary>
    ''' Gibt die VertexDeclaration zurück
    ''' </summary>
    Public ReadOnly Property Declaration() As VertexDeclaration
        Get
            Return _vertexdec
        End Get
    End Property
    ''' <summary>
    ''' Gibt die Bytegröße eines Vertex zurück
    ''' </summary>
    Public ReadOnly Property SizeOfBytes() As Integer
        Get
            Return _vertexdecSizeOfBytes
        End Get
    End Property
    ''' <summary>
    ''' Gibt den Buffer an den Device
    ''' </summary>
    Public WriteOnly Property SetBuffer() As Offset
        Set(ByVal value As Offset)
            dxDevice.VertexDeclaration = _vertexdec
            dxDevice.SetStreamSource(0, _vbuffer, CInt(value), _vertexdecSizeOfBytes)
        End Set
    End Property
    ''' <summary>
    ''' Rendert den Buffer
    ''' </summary>
    Public Sub Draw()
        dxDevice.DrawPrimitives(PrimitiveType.TriangleStrip, 0, 2)
    End Sub
    Public Sub Dispose()
        If Not _vbuffer Is Nothing Then _vbuffer.Dispose()
        If Not _vertexdec Is Nothing Then _vertexdec.Dispose()
    End Sub
End Class
