﻿' Title    : sdxBaseEngine Framework
' Author   : Sascha Bajus
' Created  : 12/2008
' Copyright © 2008 Sascha Bajus

#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region



Imports SlimDX
Imports SlimDX.Direct3D9
Imports SlimDX.DXGI
Imports sdxBaseEngine.SDX9BaseEngine
Imports System.Reflection
Imports Microsoft.VisualBasic




''' <summary>
''' Erstellt eine Form zur Enumeration des Devices. Die Enumeration umfasst die Auswahl der
''' Auflösung, die Festlegung der Hardwareflags, Benutzung des V-Sync, Benutzung des Triplebuffers, Auswahl des
''' Multisamplings, Auswahl des Backbuffer und Depth-Format. 
''' </summary>
''' <remarks></remarks>

Public Class sdxEnumeration
    ''' <summary>
    ''' Gibt die Structure mit Inhalt der CreateFlags und Presentparameters an
    ''' </summary>
    Public Structure deviceEnumeration
        ''' <summary>
        ''' Legt die CreateFlags zur Erstellung des Devices fest
        ''' </summary>
        Public dxDeviceFlags As CreateFlags
        ''' <summary>
        ''' Legt die Presentparameters zur Erstellung des Devices fest
        ''' </summary>
        Public dxPresentParameters As PresentParameters
    End Structure


    Private _dxAdapterinfo As AdapterInformation     '#to read Adapter informations
    Private _dxDeviceCaps As Direct3D9.Capabilities  '#to check Device Flags
    Private _dxDeviceFlags As CreateFlags            '#to set DeviceFlags
    Private _dxEnumerationResult As deviceEnumeration

    Private supportedDisplaymode As New Collection
    Private supportedAFormat As New List(Of Direct3D9.Format)
    Private supportedBFormat As New List(Of Direct3D9.Format)
    Private supportedDFormat As New List(Of Direct3D9.Format)
    Private supportedMFormat As New List(Of MultisampleType)

    Private _ini As New clsINI(My.Application.Info.DirectoryPath & "\settings.ini") '#to read\save settings


    'Public Overloads Function showdialog() As deviceEnumeration
    '    Return _dxEnumerationResult
    'End Function


    ''' <summary>
    ''' Erstellt ein EnumerationWindow um die Device Settings zu wählen (nur DirectX 9)
    ''' </summary>
    Public Sub New(ByVal ShaderVersion As String, ByVal forceStencil As Boolean)
        InitializeComponent() '#for Designer only

        Dim ass() As Assembly = AppDomain.CurrentDomain.GetAssemblies()
        Dim text() As String
        For i As Integer = 0 To ass.Length - 1
            text = Split(ass(i).FullName, ",", -1, CompareMethod.Text)
            If text(0) Like "SDX9Base*" Then
                Me.Text = "SDX9BaseEngine" & Replace(text(1), "Version=", " ", 1, -1)
                Exit For
            End If
        Next

        '#get Adapterinformation
        _dxAdapterinfo = dxEngine.D3D9.Adapters.DefaultAdapter
        '#get Device Caps for Flag creation
        _dxDeviceCaps = dxEngine.D3D9.GetDeviceCaps(0, Direct3D9.DeviceType.Hardware)

        '#Enumeration for default adapter and D3D9 only
        checkDeviceName()        '#get the DeviceName
        checkHardwareForFlags(ShaderVersion)  '#get Hardware Support
        checkAdapterFormats()    '#get supported Displaymodes, Adapterformat, BackbufferFormat (Fullsreen only)
        checkDepthFormat()       '#get supporrted DepthFormat
        checkMultisample()       '#get supported Multisampling

        readINI()                '#read saved settings

        '#Force StencilFormat
        If forceStencil Then
            ddlDepth.Text = Direct3D9.Format.D24S8.ToString
            ddlDepth.Enabled = False
        End If
    End Sub
#Region "Check supports in Hardware for Flags creation"
    Private Sub checkHardwareForFlags(ByVal ShaderVersion As String)
        '#test Hardware T&L - PureDevice
        cbPureDevice.Enabled = False
        If CBool(_dxDeviceCaps.DeviceCaps And DeviceCaps.HWTransformAndLight) Then
            TransformAndLightToolStripMenuItem.Checked = True
            If CBool(_dxDeviceCaps.DeviceCaps And DeviceCaps.PureDevice) Then
                PurDeviceToolStripMenuItem.Checked = True
                cbPureDevice.Enabled = True
            End If
        End If
        '#get Pixel und Vertexshader Version
        If _dxDeviceCaps.PixelShaderVersion.ToString < ShaderVersion Then
            MsgBox("Shader Enumeration failed. No Hardewaresupport for Pixelshader " & ShaderVersion & " !", MsgBoxStyle.Critical, "ShaderVersion")
            Me.btRun.Enabled = False
        End If

        PixelshaderToolStripMenuItem.Text = "Pixelshader: " & _dxDeviceCaps.PixelShaderVersion.ToString
        If Not _dxDeviceCaps.PixelShaderVersion.ToString = "0.0" Then PixelshaderToolStripMenuItem.Checked = True
        VertexshaderToolStripMenuItem.Text = "Vertexshader: " & _dxDeviceCaps.VertexShaderVersion.ToString
        If Not _dxDeviceCaps.VertexShaderVersion.ToString = "0.0" Then VertexshaderToolStripMenuItem.Checked = True
    End Sub
#End Region
#Region "Check Devicename"
    Private Sub checkDeviceName()
        '#get Devicename
        lbdevicename.Text = " " & _dxAdapterinfo.Details.Description
    End Sub
#End Region
#Region "Check Adapter Formats"
    Private Sub checkAdapterFormats()
        On Error Resume Next

        For Each aFormat As Direct3D9.Format In _AdapterFormats
            Dim modes As DisplayModeCollection = dxEngine.D3D9.Adapters.DefaultAdapter.GetDisplayModes(aFormat)
            Dim colAFormat As New Collection
            Dim colBFormat As New Collection

            For Each dMode As DisplayMode In modes
                For Each bFormat As Direct3D9.Format In _dxBackBufferFormats
                    If dxEngine.D3D9.CheckDeviceType(dxEngine.D3D9.Adapters.DefaultAdapter.Adapter, DeviceType.Hardware, aFormat, bFormat, False) Then
                        'add Supported Displaymode
                        If Not supportedDisplaymode.Contains(dMode.Width & dMode.Height & dMode.RefreshRate) Then
                            supportedDisplaymode.Add(New Vector2(dMode.Width, dMode.Height), dMode.Width & dMode.Height & dMode.RefreshRate)
                            ddlSreenRes.Items.Add(dMode.Width & " x " & dMode.Height & "   -   " & dMode.RefreshRate & " Hz")
                        End If

                        'add Supported Adapter Format
                        If Not colAFormat.Contains(CStr(aFormat)) Then
                            colAFormat.Add(bFormat, CStr(aFormat))
                            supportedAFormat.Add(aFormat)
                        End If

                        'add Supported Backbuffer Format
                        If Not colBFormat.Contains(CStr(bFormat)) Then
                            colBFormat.Add(bFormat, CStr(bFormat))
                            supportedBFormat.Add(bFormat)
                            ddlBackbuffer.Items.Add(bFormat.ToString)
                        End If

                    End If
                Next
            Next
        Next

        'Default Settings
        ddlSreenRes.Text = CStr(ddlSreenRes.Items(0))
        ddlBackbuffer.Text = CStr(ddlBackbuffer.Items(0))
    End Sub
#End Region
#Region "Check DephtFormat"
    Private Sub checkDepthFormat()
        Dim colDFormat As New Collection
        For Each dFormat As Direct3D9.Format In _DepthSencilFormats
            For Each aFormat As Direct3D9.Format In supportedAFormat
                For Each bFormat As Direct3D9.Format In supportedBFormat
                    If dxEngine.D3D9.CheckDepthStencilMatch(dxEngine.D3D9.Adapters.DefaultAdapter.Adapter, DeviceType.Hardware, aFormat, bFormat, dFormat) Then
                        If Not colDFormat.Contains(CStr(dFormat)) Then
                            colDFormat.Add(dFormat, CStr(dFormat))
                            supportedDFormat.Add(dFormat)
                            ddlDepth.Items.Add(dFormat.ToString)
                        End If
                    End If
                Next
            Next
        Next
        ddlDepth.Text = CStr(ddlDepth.Items(0))
    End Sub
#End Region
#Region "Check Multisample"
    Private Sub checkMultisample()
        Dim colMST As New Collection
        For Each mst As MultisampleType In _MultisampleType
            For Each bFormat As Direct3D9.Format In supportedBFormat
                If dxEngine.D3D9.CheckDeviceMultisampleType(dxEngine.D3D9.Adapters.DefaultAdapter.Adapter, DeviceType.Hardware, bFormat, False, mst) Then
                    If Not colMST.Contains(CStr(mst)) Then
                        colMST.Add(mst, CStr(mst))
                        supportedMFormat.Add(mst)
                        ddlMultisample.Items.Add(mst.ToString)
                    End If
                End If
            Next
        Next
        ddlMultisample.Text = CStr(ddlMultisample.Items(0))
    End Sub
#End Region


#Region "Create Presentationparameter"
    Private Function DevicePresentParameters() As PresentParameters
        DevicePresentParameters = New PresentParameters
        With DevicePresentParameters
            '# Windowed or Fullscreen
            If cbFullscreen.Checked Then .Windowed = False

            Dim backbuffersize As New Vector2

            backbuffersize = CType(supportedDisplaymode.Item(ddlSreenRes.SelectedIndex + 1), Vector2)
            .BackBufferWidth = CInt(backbuffersize.X)   '# Backbuffersize
            .BackBufferHeight = CInt(backbuffersize.Y)  '# Backbuffersize
            .SwapEffect = Direct3D9.SwapEffect.Discard

            .BackBufferCount = 1                                '# Doublebuffer
            If cbTriplebuffer.Checked Then .BackBufferCount = 2 '# Triplebuffer 
            .BackBufferFormat = supportedBFormat.Item(ddlBackbuffer.SelectedIndex)

            '#BackBufferFormat
            .AutoDepthStencilFormat = supportedDFormat.Item(ddlDepth.SelectedIndex)
            .EnableAutoDepthStencil = True


            '#Multisampling
            .Multisample = supportedMFormat.Item(ddlMultisample.SelectedIndex)
            .MultisampleQuality = 0

            .PresentationInterval = PresentInterval.Immediate                   '# V-Sync off
            If cbVsync.Checked Then .PresentationInterval = PresentInterval.One '# V-Sync on
        End With
        Return DevicePresentParameters
    End Function
#End Region
#Region "read INI"
    Private Sub readINI()
        With _ini
            ddlSreenRes.Text = .readValue("D3D9-Screen", "Resolution")
            Try
                cbFullscreen.Checked = CBool(.readValue("D3D9-Screen", "Fullscreen", "False"))
            Catch ex As Exception
            End Try
            Try
                cbVsync.Checked = CBool(.readValue("D3D9-Screen", "Vsync", "False"))
            Catch ex As Exception
            End Try
            Try
                cbTriplebuffer.Checked = CBool(.readValue("D3D9-Screen", "Triplebuffer", "False"))
            Catch ex As Exception
            End Try
            Try
                cbPureDevice.Checked = CBool(.readValue("D3D9-Screen", "PureDevice", "False"))
            Catch ex As Exception
            End Try
            ddlBackbuffer.Text = .readValue("D3D9-Format", "Backbuffer")
            ddlDepth.Text = .readValue("D3D9-Format", "Depth")
            ddlMultisample.Text = .readValue("D3D9-Filter", "MultisampleType")
        End With
    End Sub
#End Region


#Region "## save settings ##"
    Private Sub btRun_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btRun.Click
        Me.Visible = False
        Me.Refresh()
        With _ini
            .writeValue("D3D9-Device", "PureDevice") = cbPureDevice.Checked.ToString
            .writeValue("D3D9-Screen", "Resolution") = ddlSreenRes.Text
            .writeValue("D3D9-Screen", "Fullscreen") = cbFullscreen.Checked.ToString
            .writeValue("D3D9-Screen", "Vsync") = cbVsync.Checked.ToString
            .writeValue("D3D9-Screen", "Triplebuffer") = cbTriplebuffer.Checked.ToString
            .writeValue("D3D9-Format", "Backbuffer") = ddlBackbuffer.Text
            .writeValue("D3D9-Format", "Depth") = ddlDepth.Text
            .writeValue("D3D9-Filter", "MultisampleType") = ddlMultisample.Text
        End With

        'Flags
        If TransformAndLightToolStripMenuItem.Checked = True Then
            _dxDeviceFlags = CreateFlags.HardwareVertexProcessing
            If PurDeviceToolStripMenuItem.Checked = True _
            AndAlso cbPureDevice.Checked = True Then
                _dxDeviceFlags = _dxDeviceFlags Or CreateFlags.PureDevice
            End If
        Else
            _dxDeviceFlags = CreateFlags.SoftwareVertexProcessing
        End If

        _dxEnumerationResult.dxDeviceFlags = _dxDeviceFlags
        _dxEnumerationResult.dxPresentParameters = DevicePresentParameters()
        Me.DialogResult = System.Windows.Forms.DialogResult.OK
        Me.Close()

    End Sub
#End Region
#Region "Exit"
    Private Sub btExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btExit.Click
        Me.DialogResult = System.Windows.Forms.DialogResult.Cancel
        Me.Dispose()
    End Sub
    Private Sub sdxEnumerationClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If e.Cancel Then Me.DialogResult = System.Windows.Forms.DialogResult.Cancel
        Me.Dispose()
    End Sub
#End Region

#Region "Properties"
    ''' <summary>
    ''' Rückgabe der Hardwareflags und Presentparameters 
    ''' </summary>
    Public ReadOnly Property EnumerationResult() As deviceEnumeration
        Get
            Return _dxEnumerationResult
        End Get
    End Property
    ''' <summary>
    ''' Gibt die Adapterinformationen zurück
    ''' </summary>
    ''' <value></value>
    Public ReadOnly Property Adapterinfo() As AdapterInformation
        Get
            Return _dxAdapterinfo
        End Get
    End Property
#End Region

#Region "AdapterFormats"
    Private _AdapterFormats() As Integer = {Direct3D9.Format.X8R8G8B8, _
                                           Direct3D9.Format.X1R5G5B5, _
                                           Direct3D9.Format.R5G6B5, _
                                           Direct3D9.Format.A2R10G10B10}
#End Region
#Region "BackbufferFormats"
    Private _dxBackBufferFormats() As Integer = {Direct3D9.Format.A8R8G8B8, _
                                              Direct3D9.Format.X8R8G8B8, _
                                              Direct3D9.Format.R8G8B8, _
                                              Direct3D9.Format.A2R10G10B10, _
                                              Direct3D9.Format.A8R3G3B2, _
                                              Direct3D9.Format.A4R4G4B4, _
                                              Direct3D9.Format.A1R5G5B5, _
                                              Direct3D9.Format.X4R4G4B4, _
                                              Direct3D9.Format.X1R5G5B5, _
                                              Direct3D9.Format.R5G6B5, _
                                              Direct3D9.Format.R3G3B2}
#End Region
#Region "DepthFormat"
    Private _DepthSencilFormats() As Integer = {Direct3D9.Format.D32, _
                                               Direct3D9.Format.D24X4S4, _
                                               Direct3D9.Format.D24X8, _
                                               Direct3D9.Format.D24S8, _
                                               Direct3D9.Format.D16, _
                                               Direct3D9.Format.D15S1}
#End Region
#Region "MultisampleType"
    Private _MultisampleType() As Integer = {MultisampleType.None, _
                                             MultisampleType.NonMaskable, _
                                             MultisampleType.TwoSamples, _
                                             MultisampleType.ThreeSamples, _
                                             MultisampleType.FourSamples, _
                                             MultisampleType.FiveSamples, _
                                             MultisampleType.SixSamples, _
                                             MultisampleType.SevenSamples, _
                                             MultisampleType.EightSamples, _
                                             MultisampleType.NineSamples, _
                                             MultisampleType.TenSamples, _
                                             MultisampleType.ElevenSamples, _
                                             MultisampleType.ThirteenSamples, _
                                             MultisampleType.TwelveSamples, _
                                             MultisampleType.FourteenSamples, _
                                             MultisampleType.FifteenSamples, _
                                             MultisampleType.SixteenSamples}
#End Region
#Region "INI Class"
    Private Class clsINI

        Private Declare Function GetPrivateProfileStringA Lib "kernel32.dll" (ByVal lpSection As String, ByVal lpSetting As String, ByVal lpDefault As String, ByVal lpReturnedString As String, ByVal nSize As Integer, ByVal lpFileName As String) As Integer
        Private Declare Function WritePrivateProfileStringA Lib "kernel32.dll" (ByVal lpSection As String, ByVal lpSetting As String, ByVal lpValue As String, ByVal lpFileName As String) As Integer
        Private Declare Function DeletePrivateProfileSection Lib "kernel32" Alias "WritePrivateProfileStringA" (ByVal Section As String, ByVal NoKey As Integer, ByVal NoSetting As Integer, ByVal FileName As String) As Integer
        Private Declare Function DeletePrivateProfileKey Lib "kernel32" Alias "WritePrivateProfileStringA" (ByVal Section As String, ByVal Key As String, ByVal Setting As Integer, ByVal FileName As String) As Integer
        Private defaultiniFile As String

        ''' <summary>
        ''' Erstellt eine neue INI-Klasse
        ''' </summary>
        ''' <param name="filesource">Legt den default Dateipfad fest</param>
        Public Sub New(Optional ByVal filesource As String = "")
            defaultiniFile = filesource
        End Sub

        ''' <summary>
        ''' Rückgabe des Wertes aus der INI-Datei
        ''' </summary>
        ''' <param name="Section">Legt die Section fest</param>
        ''' <param name="Key">Legt den Schlüssel fest</param>
        ''' <param name="Default_Renamed">Legt den Returnwert fest, wenn der Schlüssel nicht vorhanden ist </param>
        ''' <param name="IniFile">Legt fest, ob ein anderer Dateipfad verwendet werden soll</param>
        ''' <returns>Rückgabe des Wertes als String</returns>
        Public ReadOnly Property readValue(ByVal Section As String, ByVal Key As String, Optional ByVal Default_Renamed As String = "", Optional ByVal IniFile As String = "") As String
            Get
                If IniFile = Nothing Then IniFile = defaultiniFile
                Dim s As String
                s = Space(1000)
                Dim n As Integer
                n = GetPrivateProfileStringA(Section, Key, Default_Renamed, s, 1000, IniFile)
                Return Microsoft.VisualBasic.Left(s.ToString, n)
            End Get
        End Property
        ''' <summary>
        ''' Schreibt einen Wert als String
        ''' </summary>
        ''' <param name="Section">Legt die Section fest</param>
        ''' <param name="Key">Legt den Schlüssel fest</param>
        ''' <param name="IniFile">Legt fest, ob ein anderer Dateipfad verwendet werden soll</param>
        ''' <value>Angabe des Wertes</value>
        Public WriteOnly Property writeValue(ByVal Section As String, ByVal Key As String, Optional ByVal IniFile As String = "") As String
            Set(ByVal value As String)
                If IniFile = Nothing Then IniFile = defaultiniFile
                Dim n As Integer
                n = WritePrivateProfileStringA(Section, Key, value, IniFile)
                If n = 0 Then MsgBox("Es ist eine Zugriffsverletzung aufgetreten. Die Daten konnten nicht geschrieben werden.", MsgBoxStyle.Critical, "Zugriffsfehler")
            End Set
        End Property
        ''' <summary>
        ''' Löschen eines Bereiches
        ''' </summary>
        ''' <param name="IniFile">Legt fest, ob ein anderer Dateipfad verwendet werden soll</param>
        ''' <value>Angabe des Bereichnamens</value>
        Public WriteOnly Property eraseSection(Optional ByVal IniFile As String = "") As String
            Set(ByVal value As String)
                If IniFile = Nothing Then IniFile = defaultiniFile
                Dim n As Integer
                n = DeletePrivateProfileSection(value, 0, 0, IniFile)
                If n = 0 Then Err.Raise(5) 'Invalid procedure call or argument
            End Set
        End Property
        ''' <summary>
        ''' Löschen eines Schlüssels in einem Bereich
        ''' </summary>
        ''' <param name="Section">Legt die Section fest</param>
        ''' <param name="IniFile">Legt fest, ob ein anderer Dateipfad verwendet werden soll</param>
        ''' <value>Anagbe des Schlüssels</value>
        Public WriteOnly Property eraseKey(ByVal Section As String, Optional ByVal IniFile As String = "") As String
            Set(ByVal value As String)
                If IniFile = Nothing Then IniFile = defaultiniFile
                Dim n As Integer
                n = DeletePrivateProfileSection(Section, CInt(value), 0, IniFile)
                If n = 0 Then Err.Raise(5) 'Invalid procedure call or argument
            End Set
        End Property
    End Class
#End Region


End Class