// -----------------------------------------------------------------------------------------------------------------
// Function to check if it is shadowed or not (one Sample)
// -----------------------------------------------------------------------------------------------------------------

float DepthBias = 0.001f;

float HardShadow(float4 lpos)
{
   	//Projected Coords
   	float2 ptex   = 0.5 * lpos.xy/lpos.w + float2(0.5,0.5);
   		   ptex.y = 1 - ptex.y;
   		   
   		   
   	float shadowdepth = tex2D(ShadowMapSampler, ptex); // Get the current depth stored in the shadow map 
	return shadowdepth;
}


// -----------------------------------------------------------------------------------------------------------------
// Function to check if it is shadowed or not (PCF Sample)
// -----------------------------------------------------------------------------------------------------------------


float PCFShadow(float4 texZ)
{
// Generate the 9 texture co-ordinates for a 3x3 PCF kernel
	float4 TexCoords[5];
	float  ShadowTerms[5];
    float  ShadowTerm = 0;
	// Texel size
	float2 fTexelSize = float2(1/640,1/480);

	TexCoords[0] = texZ;

	TexCoords[1] = texZ + float4( -fTexelSize.x, 0,0,0);
	TexCoords[2] = texZ + float4(  fTexelSize.x, 0,0,0);
	TexCoords[3] = texZ + float4( 0, -fTexelSize.y,0,0);
	TexCoords[4] = texZ + float4( 0,  fTexelSize.y,0,0);

	// Sample each of them checking whether the pixel under test is shadowed or not
	

	for( int i = 0; i < 5; i++ )
	{
		float A = tex2Dproj(ShadowMapSampler, TexCoords[i]).x; 
		float B = (texZ.z/texZ.w - 0.001f);
		
		// Texel is shadowed
		if (A<B)
		{ShadowTerms[i] =0;}
		else
		{ShadowTerms[i]  =1;}

		ShadowTerm += ShadowTerms[i];
	}

	// Get the average
	return ShadowTerm/5;
}





// -----------------------------------------------------------------------------------------------------------------
// VertexShader 
// -----------------------------------------------------------------------------------------------------------------


// VertexShader generate ShadowDepthMap
// ------------------------------------
vs_OutShadow vsoutShadowDepthMap(float4 pos: POSITION)
{
    vs_OutShadow Out;
    Out.pos   = mul(pos,WorldLightViewProjection); 
    Out.depth = Out.pos.z;   
    return Out; 
}

// -----------------------------------------------------------------------------------------------------------------
//  PixelShader Section ShadowMap
// -----------------------------------------------------------------------------------------------------------------

// PixelShader ShadowDepthMap
// --------------------------
float4 psoutShadowDepthMap(float z : TEXCOORD0) : COLOR
{ 
    return saturate(z/80);
}


// ---------------------------------------------------------------------
//  Technique Section Shadow
// ---------------------------------------------------------------------
technique generateShadowDepthMap
{
   pass pass0
   {
      VertexShader = compile vs_2_0 vsoutShadowDepthMap();
      PixelShader  = compile ps_2_0 psoutShadowDepthMap();
   }
}
