//-----------------------------------------------------------------------------
//
// Basic Billboard.fx
//
//-----------------------------------------------------------------------------


//Matrices
matrix World;
matrix ViewProjection;


//Colormap
Texture ColorMap0;
sampler MapSampler0=sampler_state { texture   = <ColorMap0>;
                                          minfilter = LINEAR;
                                          magfilter = LINEAR;
                                          mipfilter = LINEAR;
                                          AddressU  = Clamp;
                                          AddressV  = Clamp;};


//Vertex Output Structure
struct VS_OUTPUT
{
    float4 Pos    : POSITION0;
    float2 Tex    : TEXCOORD0;
};


// -----------------------------------------------------------------------------------------------------------------
// VertexShader Section
// -----------------------------------------------------------------------------------------------------------------

VS_OUTPUT vsoutXY(float3 pos :POSITION0, float2 tex :TEXCOORD0, float4 value :TEXCOORD1)
{
	VS_OUTPUT Out;									

	float3 center =  mul(pos, World);					//Die Center-Position des Billboards
	float3 position = center;

    position += (tex.x - 0.5)* float3(value.x,0,0);		//Berechnung der 6 Vertices anhand der Texturecoords
    position += (0.5 - tex.y)* float3(0,value.y,0);     //und scales

	float4 pos4 = float4(position,1);					//Umrechung der Position in Vector4
    Out.Pos  = mul(pos4, ViewProjection);				//Transformation in ViewProjection
    Out.Tex = tex;
    return Out;
}

VS_OUTPUT vsoutXZ(float3 pos :POSITION0, float2 tex :TEXCOORD0, float4 value :TEXCOORD1)
{

	VS_OUTPUT Out;									

	float3 center =  mul(pos, World);					//Die Center-Position des Billboards
	float3 position = center;

    position += (tex.x - 0.5)* float3(value.x,0,0);		//Berechnung der 6 Vertices anhand der Texturecoords
    position += (0.5 - tex.y)* float3(0,0,value.y);     //und scales

	float4 pos4 = float4(position,1);					//Umrechung der Position in Vector4
    Out.Pos  = mul(pos4, ViewProjection);				//Transformation in ViewProjection
    Out.Tex = tex;
    return Out;
}

// -----------------------------------------------------------------------------------------------------------------
//  PixelShader Section
// -----------------------------------------------------------------------------------------------------------------
float4 psout(float2 tex : TEXCOORD0) : COLOR0
{
	return tex2D(MapSampler0, tex);
}



technique BillboardsXY
{
    pass Render
    {
      VertexShader = compile vs_2_0 vsoutXY();
      PixelShader  = compile ps_2_0 psout();
    }
}
technique BillboardsXZ
{
    pass Render
    {
      VertexShader = compile vs_2_0 vsoutXZ();
      PixelShader  = compile ps_2_0 psout();
    }
}
