//-----------------------------------------------------------------------------
// 
//-----------------------------------------------------------------------------

matrix World;
matrix WorldViewProjection;

float2 	 Level;
float4   Repeat;
float3   CamPos;                    //Camera Position

float4	 Ambient;					//Light - Ambient Color
float    AIntensity;				//Light - Ambient Intensity
float4   Diffuse;					//Light - Diffuse Color
float    DIntensity; 				//Light - Diffuse Intensity
float3   LightDirection;            //Light - Light Direction

float4   Specular;					//Material - Specular Color
float    SIntensity;				//Material - Specular Intensity
float    Sharpen;					//Material - Specular Power
float    Brightness;				//Material - Light Power


//ColorMap
Texture ColorMap0;
sampler MapSampler0=sampler_state { texture   = <ColorMap0>;
                                    minfilter = LINEAR;
                                    magfilter = LINEAR;
                                    mipfilter = LINEAR;
                                    AddressU  = Wrap;
                                    AddressV  = Wrap;};
Texture ColorMap1;
sampler MapSampler1=sampler_state { texture   = <ColorMap1>;
                                    minfilter = LINEAR;
                                    magfilter = LINEAR;
                                    mipfilter = LINEAR;
                                    AddressU  = Wrap;
                                    AddressV  = Wrap;};
Texture ColorMap2;
sampler MapSampler2=sampler_state { texture   = <ColorMap2>;
                                    minfilter = LINEAR;
                                    magfilter = LINEAR;
                                    mipfilter = LINEAR;
                                    AddressU  = Wrap;
                                    AddressV  = Wrap;};   

Texture ColorMap3;
sampler MapSampler3=sampler_state { texture   = <ColorMap3>;
                                    minfilter = LINEAR;
                                    magfilter = LINEAR;
                                    mipfilter = LINEAR;
                                    AddressU  = Wrap;
                                    AddressV  = Wrap;};                                                                                                                                         
                                    
//NormalMap
Texture NormalMap0;
sampler NMapSampler0=sampler_state { texture   = <NormalMap0>;
                                    minfilter = LINEAR;
                                    magfilter = LINEAR;
                                    mipfilter = LINEAR;
                                    AddressU  = Wrap;
                                    AddressV  = Wrap;};
Texture NormalMap1;
sampler NMapSampler1=sampler_state { texture   = <NormalMap1>;
                                    minfilter = LINEAR;
                                    magfilter = LINEAR;
                                    mipfilter = LINEAR;
                                    AddressU  = Wrap;
                                    AddressV  = Wrap;};
Texture NormalMap2;
sampler NMapSampler2=sampler_state { texture   = <NormalMap2>;
                                    minfilter = LINEAR;
                                    magfilter = LINEAR;
                                    mipfilter = LINEAR;
                                    AddressU  = Wrap;
                                    AddressV  = Wrap;};   

Texture NormalMap3;
sampler NMapSampler3=sampler_state { texture   = <NormalMap3>;
                                    minfilter = LINEAR;
                                    magfilter = LINEAR;
                                    mipfilter = LINEAR;
                                    AddressU  = Wrap;
                                    AddressV  = Wrap;};                                      

//Vertex Output Structure
struct VSOUT
{
    float4 pos    : POSITION0;
    float2 tex    : TEXCOORD0;
    float  posY   : TEXCOORD1;
    float3 l 	  : TEXCOORD2;
    float3 h 	  : TEXCOORD3;
    float3 n      : TEXCOORD4;
};


// -----------------------------------------------------------------------------------------------------------------
// VertexShader Section
// -----------------------------------------------------------------------------------------------------------------
VSOUT vsout(float4 pos :POSITION0, float2 tex :TEXCOORD0, float3 nrm :NORMAL)
{
	VSOUT Out;
	float4 posworld = mul(pos,World);
	float3 viewvec;
								 				
    Out.pos = mul(pos, WorldViewProjection);	
    Out.posY= posworld.y;		
	Out.tex = tex;
	viewvec = CamPos - posworld.xyz ;		
    Out.l	= normalize(LightDirection);   
    Out.h   = normalize(LightDirection + viewvec) ; 		
	Out.n   = nrm;
    return Out;
}
// -----------------------------------------------------------------------------------------------------------------
// VertexShader TBN Section
// -----------------------------------------------------------------------------------------------------------------
VSOUT vsoutTBN(float4 pos :POSITION, float4 N :NORMAL, float2 tex :TEXCOORD0, float4 T :TANGENT , float4 B :BINORMAL)
{
	VSOUT Out;
	
  	float3x3 WorldToTangentSpace;				//World to Tangent Matrix
  	float3   lightvec;							//Normalized Lightvector in Tangentspace
  	float3   posworld;							//Position in Worldspace
  	float3   viewvec;							//Normalized ViewVector in Tangentspace
  	float3   halfvec;

 	float3 b = normalize(mul(T,World));			//B - Tangent
  	float3 t = normalize(mul(B,World));	        //T - Binormal
  	float3 n = normalize(mul(N,World));			//N - Normal

  	WorldToTangentSpace[0]=t;					//T - Tangent  to TangentSpace
  	WorldToTangentSpace[1]=b;	        		//B - Binormal to TangentSpace
  	WorldToTangentSpace[2]=n;					//N - Normal   to TangentSpace

  	posworld      = mul(pos,World).xyz;						   		//Position in WorldSpace
  	viewvec       = mul(WorldToTangentSpace,(CamPos - posworld));	//ViewVector
  	lightvec      = mul(WorldToTangentSpace,LightDirection);     	//LightVector
  	halfvec       = lightvec + viewvec;
										 				
    Out.pos = mul(pos, WorldViewProjection);	
    Out.posY= posworld.y;		
	Out.tex = tex;
    Out.l	= normalize(lightvec);               //LightVector
    Out.h   = normalize(halfvec);
    Out.n   = N; 		
    return Out;
}
// - End -




// -----------------------------------------------------------------------------------------------------------------
//  PixelShader Section TEXTURE
// -----------------------------------------------------------------------------------------------------------------
float4 psoutTex0(float2 tex :TEXCOORD0, float  y :TEXCOORD1, 
                float3   l :TEXCOORD2,  
                float3   h :TEXCOORD3, float3 n :TEXCOORD4) : COLOR0
{
	float4  c =0;
	float4	ca1=0;
	float4	ca2=0;
	float4	ca3=0;

	float4	c1=0;
	float4	c2=0;
	float4	c3=0;
	
	float   w0=0;
	float   w1=0;
	
	float4 fc1=0; 
	float4 fc2=0;
	
	//ca1 = tex2D(MapSampler0, tex);  //sand
	//ca2 = tex2D(MapSampler1, tex);  //grass
	//ca3 = tex2D(MapSampler2, tex);  //rock
	//ca4 = tex2D(MapSampler3, tex);  //snow

	w0=saturate(y - Level.x+1); //weight by height
	w1=saturate((n.y-0.66)*3) ;	//weight by Normal
	
	if (y<Level.x){ca1 = tex2D (MapSampler0, tex*Repeat.x);}  //sand
	
	ca2 = tex2D (MapSampler1, tex*Repeat.y);   				 //grass
	ca3 = tex2D (MapSampler2, tex*Repeat.z);  				 //rock
	
	c1  = float4(ca1.rgb * ca1.a,0);
	c2  = float4(ca2.rgb * ca2.a,0);
	c3  = float4(ca3.rgb * ca3.a,0);
		
	fc1=lerp(c1,c2,w0);
	c  =lerp(c3,fc1,w1);
	
	//Light
   	float  nDotL = saturate(dot(n, l));									 //Dot(Normal,Light) for Diffuse
  	float  nDotH = saturate(dot(n, h));									 //Dot(Normal,Half) for Specular
  	float  power = (nDotL == 0) ? 0 : pow(nDotH,Sharpen);                //Calculate SpecularPower

 	float4 lAmbient  = Ambient * AIntensity;					  		 //New Color from Ambient
  	float4 lDiffuse  = Diffuse * DIntensity * nDotL; 		   	  		 //New Color from Diffuse
  	float4 lSpecular = saturate(Specular * SIntensity  * power);         //New Color from Specular
  	float4 light     = saturate(lAmbient + lDiffuse);
  	return saturate (light * c + lSpecular)* Brightness;			     //Return Final Color	
}
// -----------------------------------------------------------------------------------------------------------------
//  PixelShader Section TEXTURE no Spec
// -----------------------------------------------------------------------------------------------------------------
float4 psoutTex1(float2 tex :TEXCOORD0, float  y :TEXCOORD1, 
                float3   l :TEXCOORD2,
                float3   h :TEXCOORD3, float3 n :TEXCOORD4) : COLOR0
{
	float4  c =0;
	float4	ca1=0;
	float4	ca2=0;
	float4	ca3=0;

	float4	c1=0;
	float4	c2=0;
	float4	c3=0;
	
	float   w0=0;
	float   w1=0;
	
	float4 fc1=0; 
	float4 fc2=0;
	
	w0=saturate(y - Level.x+1); //weight by height
	w1=saturate((n.y-0.66)*3) ;	//weight by Normal
	
	if (y<Level.x){ca1 = tex2D (MapSampler0, tex*Repeat.x);}  //sand
	
	ca2 = tex2D (MapSampler1, tex*Repeat.y);   				 //grass
	ca3 = tex2D (MapSampler2, tex*Repeat.z);  				 //rock
	
	c1  = float4(ca1.rgb * ca1.a,0);
	c2  = float4(ca2.rgb * ca2.a,0);
	c3  = float4(ca3.rgb * ca3.a,0);
		
	fc1=lerp(c1,c2,w0);
	c  =lerp(c3,fc1,w1);
	
	//Light
   	float  nDotL = saturate(dot(n, l));		

 	float4 lAmbient  = Ambient * AIntensity;					  		
  	float4 lDiffuse  = Diffuse * DIntensity * nDotL; 		   	  		
  	float4 light     = saturate(lAmbient + lDiffuse);
  	return saturate (light * c)* Brightness;			
}






// -----------------------------------------------------------------------------------------------------------------
//  PixelShader Section TEXTURE + Normal
// -----------------------------------------------------------------------------------------------------------------
float4 psoutTexTBN0(float2 tex :TEXCOORD0, float  y :TEXCOORD1, 
                   float3   l :TEXCOORD2, 
                   float3   h :TEXCOORD3, float3 nrm :TEXCOORD4) : COLOR0
{
	
	float4  c =0;
	float4	ca1=0;
	float4	ca2=0;
	float4	ca3=0;
	float4	ca4=0;
	
	float4	c1=0;
	float4	c2=0;
	float4	c3=0;
	float4	c4=0;
	
    float3	n1=0;
	float3	n2=0;
	float3	n3=0;
	float3	n4=0;
	
	float w0;
	float w1;
	
	float4 fc1; 
	float4 fc2;
	
	float3 fn1; 
	float3 fn2;
	float3 n;
	

	w0=saturate(y - Level.x+1); //weight by height
	w1=saturate((nrm.y-0.66)*3) ;	//weight by Normal
	
	if (y<Level.x){ca1 = tex2D(MapSampler0, tex*Repeat.x);n1 =0.2 * (2* tex2D(NMapSampler0, tex*Repeat.x) -1);} //sand
	
	ca2 = tex2D(MapSampler1, tex*Repeat.y);n2 = (2* tex2D(NMapSampler1, tex*Repeat.y) -1);  			   //grass
	ca3 = tex2D(MapSampler2, tex*Repeat.z);n3 = (2* tex2D(NMapSampler2, tex*Repeat.z) -1);			   //rock
	
	c1  = float4(ca1.rgb * ca1.a,0);
	c2  = float4(ca2.rgb * ca2.a,0);
	c3  = float4(ca3.rgb * ca3.a,0);
		
	fc1=lerp(c1,c2,w0);
	c  =lerp(c3,fc1,w1);

	fn1=lerp(n1,n2,w0);
	n  =lerp(n3,fn1,w1);
	
	//light  
  	float  nDotL = saturate(dot(n, l));									 //Dot(Normal,Light) for Diffuse
  	float  nDotH = saturate(dot(n, h));									 //Dot(Normal,Half) for Specular
  	float  power = (nDotL == 0) ? 0 : pow(nDotH,Sharpen);                //Calculate SpecularPower

 	float4 lAmbient  = Ambient * AIntensity;					  		 //New Color from Ambient
  	float4 lDiffuse  = Diffuse * DIntensity * nDotL; 		   	  		 //New Color from Diffuse
  	float4 lSpecular = saturate(Specular * SIntensity  * power);         //New Color from Specular
  	float4 light     = saturate(lAmbient + lDiffuse);
  	return saturate (light * c + lSpecular)* Brightness;			     //Return Final Color	
}
// -----------------------------------------------------------------------------------------------------------------
//  PixelShader Section TEXTURE + Normal no Spec
// -----------------------------------------------------------------------------------------------------------------
float4 psoutTexTBN1(float2 tex :TEXCOORD0, float  y :TEXCOORD1, 
                   float3   l :TEXCOORD2, 
                   float3   h :TEXCOORD3, float3 nrm :TEXCOORD4) : COLOR0
{
	
	float4  c =0;
	float4	ca1=0;
	float4	ca2=0;
	float4	ca3=0;
	float4	ca4=0;
	
	float4	c1=0;
	float4	c2=0;
	float4	c3=0;
	float4	c4=0;
	
    float3	n1=0;
	float3	n2=0;
	float3	n3=0;
	float3	n4=0;
	
	float w0;
	float w1;
	
	float4 fc1; 
	float4 fc2;
	
	float3 fn1; 
	float3 fn2;
	float3 n;
	

	w0=saturate(y - Level.x+1); 	//weight by height
	w1=saturate((nrm.y-0.66)*3) ;	//weight by Normal
	
	if (y<Level.x){ca1 = tex2D(MapSampler0, tex*Repeat.x);n1 =0.2 * (2* tex2D(NMapSampler0, tex*Repeat.x) -1);} //sand
	
	ca2 = tex2D(MapSampler1, tex*Repeat.y);n2 = (2* tex2D(NMapSampler1, tex*Repeat.y) -1); //grass
	ca3 = tex2D(MapSampler2, tex*Repeat.z);n3 = (2* tex2D(NMapSampler2, tex*Repeat.z) -1); //rock
	
	c1  = float4(ca1.rgb * ca1.a,0);
	c2  = float4(ca2.rgb * ca2.a,0);
	c3  = float4(ca3.rgb * ca3.a,0);
		
	fc1=lerp(c1,c2,w0);
	c  =lerp(c3,fc1,w1);

	fn1=lerp(n1,n2,w0);
	n  =lerp(n3,fn1,w1);
	
	//light  
  	float  nDotL = saturate(dot(n, l));					

 	float4 lAmbient  = Ambient * AIntensity;					  		
  	float4 lDiffuse  = Diffuse * DIntensity * nDotL; 		   	  		
  	float4 light     = saturate(lAmbient + lDiffuse);
  	return saturate (light * c)* Brightness;			    
}


technique Textured
{
    pass pass0
    {
 	ZEnable = true;
    ZWriteEnable = true;
    VertexShader = compile vs_3_0 vsout();
    PixelShader  = compile ps_3_0 psoutTex0();
    }
    pass pass1
    {
 	ZEnable = true;
    ZWriteEnable = true;
    VertexShader = compile vs_3_0 vsout();
    PixelShader  = compile ps_3_0 psoutTex1();
    }
}
technique TexturedNormal
{
    pass pass0
    {
 	ZEnable = true;
    ZWriteEnable = true;
    VertexShader = compile vs_3_0 vsoutTBN();
    PixelShader  = compile ps_3_0 psoutTexTBN0();
    }
    pass pass1
    {
 	ZEnable = true;
    ZWriteEnable = true;
    VertexShader = compile vs_3_0 vsoutTBN();
    PixelShader  = compile ps_3_0 psoutTexTBN1();
    }
}
