#include ".\include\glow.fx"

//Matrices
matrix  World;
matrix  WorldViewProjection;
matrix  ReflectedWorldViewProjection;

//ReflectionMap
Texture ReflectionMap;
sampler ReflectionSampler=sampler_state { texture   = <ReflectionMap>;
                                          minfilter = LINEAR;
                                          magfilter = LINEAR;
                                          mipfilter = NONE;
                                          AddressU  = Clamp;
                                          AddressV  = Clamp;};

//RefractionMap
Texture RefractionMap;
sampler RefractionSampler=sampler_state { texture= <RefractionMap>;
                                          minfilter = LINEAR;
                                          magfilter = LINEAR;
                                          mipfilter = NONE;
                                          AddressU  = Clamp;
                                          AddressV  = Clamp;};
//DepthMap
Texture DepthMap;
sampler DepthMapSampler=sampler_state { texture= <DepthMap>;
                                          minfilter = LINEAR;
                                          magfilter = LINEAR;
                                          mipfilter = NONE;
                                          AddressU  = Clamp;
                                          AddressV  = Clamp;};
                                          
//WaveNormalMap
Texture WaveNormalMap;
sampler WaveNormalSampler=sampler_state { texture= <WaveNormalMap>;
                                          minfilter = LINEAR;
                                          magfilter = LINEAR;
                                          mipfilter = LINEAR;
                                          AddressU  = Wrap;
                                          AddressV  = Wrap;};

//WaveNormalMap
Texture CousticMap;
sampler CousticMapSampler=sampler_state { texture= <CousticMap>;
                                          minfilter = LINEAR;
                                          magfilter = LINEAR;
                                          mipfilter = LINEAR;
                                          AddressU  = Wrap;
                                          AddressV  = Wrap;};                                          

                                        
//Waterbump
float   WaveLength;		//to change Wavelenght
float   WaveHeight;		//to change Waveheight
float   WindForce;		//to change Scrollingspeed of Bumpmap
float2  WindDirection;	//to change Scrolling direction of Bumpmap

//Globals
float4  DullColor;		//Dull Watercolor
float3  CamPos;			//CameraPosition
float3  SunPos;			//SunPosition
float   GameTime;		//Renderspeed Value
float   FresnelAdjust;	//Adjust Refraction and Reflection
float   SPower;   	    //Specular Power
float4  SColor;			//Specular Color
float   WaterHeight;


// ---------------------------------------------------------------------
//  Structures
// ---------------------------------------------------------------------

//Vertex Output Structure
struct VS_WATER
{
  float4 pos        : POSITION;		//Vertexposition
  float4 reflected  : TEXCOORD0;	//Texcoords Reflectionmap
  float4 refracted  : TEXCOORD1;	//Texcoords Refractionmap
  float2 bump       : TEXCOORD2;	//Texcoords NormalMap
  float3 p          : TEXCOORD3;    //PosWorld
};

struct VS_UWATER
{
  float4 pos        : POSITION;		//Vertexposition
  float4 reflected  : TEXCOORD0;	//Texcoords Reflectionmap
  float2 bump       : TEXCOORD1;	//Texcoords NormalMap
  float3 p          : TEXCOORD2;    //PosWorld
};

//Vertex Output Structure
struct VS_OUTPUTTEX
{
  float4 pos        : POSITION;		//Vertexposition
  float2 tex        : TEXCOORD0;	//Texcoords Reflectionmap
  float3 p			: TEXCOORD1;
};

//Vertex Output Structure
struct VS_OUTPUTTEXNRM
{
  float4 pos        : POSITION;		//Vertexposition
  float2 tex        : TEXCOORD0;	//Texcoords Reflectionmap
  float3 p			: TEXCOORD1;
  float3 n          : TEXCOORD2;
};

//Vertex Output Structure
struct VS_OUTPUTDEPTH
{
  float4 pos        : POSITION;		//Vertexposition
  float3 p			: TEXCOORD0;
};


// ---------------------------------------------------------------------
//  VertexShader Section
// ---------------------------------------------------------------------


VS_WATER vsOutW       (float4 pos :POSITION,
                       float2 tex :TEXCOORD0)
{
  VS_WATER Out;

  float3 posworld = mul(pos,World);						   //Position in Worldspace

  Out.pos	      = mul(pos,WorldViewProjection);		   //Default Positions
  Out.reflected   = mul(pos,ReflectedWorldViewProjection); //Reflected Positions
  Out.refracted   = mul(pos,WorldViewProjection);		   //Refrected Positions
  Out.bump        = tex*WaveLength + WindDirection *
                    GameTime * WindForce * 0.001;           //Calc Position to move the NormalMap
  Out.p           = posworld;		           			   //Position in Worldspace
  return Out;
}
// ---------------------------------------------------------------------

VS_UWATER vsOutU      (float4 pos :POSITION,
                       float2 tex :TEXCOORD0)
{
  VS_UWATER Out;

  float3 posworld = mul(pos,World);						   //Position in Worldspace

  Out.pos	      = mul(pos,WorldViewProjection);		   //Default Positions
  Out.reflected   = mul(pos,WorldViewProjection);          //Reflected Positions
  Out.bump        = tex*WaveLength + WindDirection *
                    GameTime * WindForce * 0.001;           //Calc Position to move the NormalMap
  Out.p           = posworld;		           			   //Position in Worldspace
  return Out;
}
// ---------------------------------------------------------------------
//  VertexShader Section Spec\GlowMap
// ---------------------------------------------------------------------

VS_OUTPUTTEX vsOutSpec   (float4 pos :POSITION,
                          float2 tex :TEXCOORD0)
{
  VS_OUTPUTTEX Out;

  Out.pos	      = mul(pos,WorldViewProjection);
  Out.p           = mul(pos,World); 
  Out.tex         = tex*WaveLength  + WindDirection *
                    GameTime * WindForce * 0.001;            
  return Out;
}

// ---------------------------------------------------------------------
//  VertexShader Section DepthMap
// ---------------------------------------------------------------------

VS_OUTPUTDEPTH vsOutDepth   (float4 pos :POSITION)
{
  VS_OUTPUTDEPTH Out;

  Out.pos	      = mul(pos,WorldViewProjection);
  Out.p           = mul(pos,World);          
  return Out;
}


// ---------------------------------------------------------------------
//  VertexShader Section Coustic
// ---------------------------------------------------------------------

VS_OUTPUTTEXNRM vsOutCoust  (float4 pos :POSITION,
                             float2 tex :TEXCOORD0,
                             float3 nrm :NORMAL)
{
  VS_OUTPUTTEXNRM Out;

  Out.pos	      = mul(pos,WorldViewProjection);
  Out.p           = mul(pos,World); 
  Out.tex         = tex;
  Out.n           = nrm;         
  return Out;
}

VS_OUTPUTTEX vsOutCoustFog  (float4 pos :POSITION,
                             float2 tex :TEXCOORD0)
{
  VS_OUTPUTTEX Out;

  Out.pos	      = mul(pos,WorldViewProjection);
  Out.p           = mul(pos,World); 
  Out.tex         = tex;         
  return Out;
}



// ---------------------------------------------------------------------
//  PS
// ---------------------------------------------------------------------
float4 psWater  (float4 reflected     :TEXCOORD0,
                 float4 refracted     :TEXCOORD1,
                 float2 bump          :TEXCOORD2,
                 float3 p             :TEXCOORD3) : COLOR
{
//calc projected Texcoords
  reflected.x =  reflected.x/reflected.w/2 +0.5;
  reflected.y = -reflected.y/reflected.w/2 +0.5;
  refracted.x =  refracted.x/refracted.w/2 +0.5;
  refracted.y = -refracted.y/refracted.w/2 +0.5;	     
  
  float3 n =2*(tex2D(WaveNormalSampler,bump))-1;
  float2 pert= WaveHeight * n.rg;
         
//Sample  with pertubated TexCoords
  float4 reflectedColor = tex2D(ReflectionSampler,reflected + pert);
  float4 refractedColor = tex2D(RefractionSampler,refracted + pert * 0.25);
  
//Sample DepthMap for Fog and Edges
  float  depth = tex2D(DepthMapSampler,refracted + pert * 0.25);
  float  fog   = saturate(depth*2-0.25);
  float  edge  = 1-saturate(depth*12);
  float4 FogRefractedColor = lerp(refractedColor, DullColor, fog);

//Fresnel & Edges
  float3 up = float3(0,1,0);
  float3 v = normalize(CamPos - p);
  float fresnelDot   = saturate(1-dot(v,up));
  float fresnelTerm  = clamp(pow(fresnelDot,FresnelAdjust)*FresnelAdjust-edge,0.008,1);

//Mix Reflected and Refracted Map by FresnelTermnDotL
  float4 mixedColorMap = lerp(FogRefractedColor,reflectedColor,fresnelTerm);
//Final Color
return mixedColorMap;
}



// ---------------------------------------------------------------------
//  PS
// ---------------------------------------------------------------------
float4 psUnderWater  (float4 reflected     :TEXCOORD0,
                      float2 bump          :TEXCOORD1,
                      float3 p             :TEXCOORD2) : COLOR
{
//calc projected Texcoords
  reflected.x =  reflected.x/reflected.w/2 +0.5;
  reflected.y = -reflected.y/reflected.w/2 +0.5;
  
  float3 n =2*(tex2D(WaveNormalSampler,bump))-1;
  float2 pert= WaveHeight * n.rg;
         
//Sample  with pertubated TexCoords
  float4 reflectedColor = tex2D(ReflectionSampler,reflected + pert);
  
//Final Color
return reflectedColor;
}



// ---------------------------------------------------------------------
//  PixelShader Section FresnelMap
// ---------------------------------------------------------------------


float4 psFresnel   (float4 reflected     :TEXCOORD0,
                    float4 refracted     :TEXCOORD1,
                    float2 bump          :TEXCOORD2,
                    float3 p             :TEXCOORD3) : COLOR
{
//Calc Lightvec, Viewvec, Halfvec, and fresnel
  float3 v = normalize(CamPos - p);
  float3 l = normalize(SunPos);
  float3 h = -v+l;
  
  refracted.x  =  refracted.x/refracted.w/2 +0.5;
  refracted.y  = -refracted.y/refracted.w/2 +0.5;
  float  depth = tex2D(DepthMapSampler,refracted);
  float  edge  = 1-saturate(depth*4.5);

  float3 up = float3(0,1,0);
  float  fresnelDot  = saturate(1-dot(v,up));
  float  fresnelTerm = saturate(pow(fresnelDot,FresnelAdjust)*FresnelAdjust - edge);
  return fresnelTerm;
}

// ---------------------------------------------------------------------
//  PixelShader Section SpecularMap (generate)
// ---------------------------------------------------------------------
float4 psSpecular(float2 tex :TEXCOORD0,float3 p :TEXCOORD1) : COLOR
{

//Sample NormalMap
  float3 n = 2 * tex2D(WaveNormalSampler,tex *2 )-1;
  float3 v = normalize(CamPos - p);
  float3 l = normalize(SunPos);
  float3 h = -v+l;
  
  //Sample DepthMap for Edges

//reflected Specular Light
  return saturate(SColor * pow(dot(normalize(h),n),SPower));
}

// ---------------------------------------------------------------------
//  PixelShader Section DepthMap (generate) FogAbove
// ---------------------------------------------------------------------
float4 psDepth(float3 p :TEXCOORD0) : COLOR
{
	float d=(p.y-WaterHeight)/(-4.6-WaterHeight);
	return d;
}


// ---------------------------------------------------------------------
//  PixelShader Section CousticMap (generate)
// ---------------------------------------------------------------------
float4 psCoustic(float2 tex :TEXCOORD0, float3 p :TEXCOORD1, float3 n:TEXCOORD2) : COLOR
{
	float  fogalpha=clamp(length(CamPos-p)*0.25,0,0.8);
	float  foglerp=saturate((WaterHeight - p.y)*0.5);
	
	float  cousticlen= 1- saturate(length(CamPos.xz-p.xz)*0.02);
	float4 coustic=tex2D(CousticMapSampler,tex*6)*foglerp * cousticlen * n.y;
	float4 fogcolor=lerp(float4(0.68,0.81,1,1),DullColor,foglerp);
	float4 color = saturate(coustic+fogcolor);
	return float4(color.rgb,fogalpha);
}
float4 psCousticF(float2 tex :TEXCOORD0, float3 p :TEXCOORD1) : COLOR
{
	float  fogalpha=clamp(length(CamPos-p)*0.25,0,0.8);
	float  foglerp=saturate((WaterHeight - p.y)*0.5);
	float4 fogcolor=lerp(float4(0.68,0.81,1,1),DullColor,foglerp);
	return float4(fogcolor.rgb,fogalpha);
}


// ---------------------------------------------------------------------
//  Technique Section
// ---------------------------------------------------------------------

technique LakeWaterMap
{
  pass p0
  {
	ZEnable = true;
	ZWriteEnable =true;
	CullMode = none; 
    VertexShader = compile vs_2_0  vsOutW();
    PixelShader  = compile ps_2_0  psWater();
  }
}

technique UnderWaterMap
{
  pass p0
  {
	ZEnable = true;
	ZWriteEnable =true;
	CullMode = none; 
    VertexShader = compile vs_2_0  vsOutU();
    PixelShader  = compile ps_2_0  psUnderWater();
  }
}

technique Fresnel
{
  pass p0
  {
	ZEnable = true;
	ZWriteEnable =true;
    VertexShader = compile vs_2_0  vsOutW();
    PixelShader  = compile ps_2_0  psFresnel();
  }
} 
technique GenerateSpecular
{
  pass p0
  {
    ZEnable = true;
	ZWriteEnable =false;
    VertexShader = compile vs_2_0  vsOutSpec();
    PixelShader  = compile ps_2_0  psSpecular();
  }
}
technique GenerateGlow
{
  pass p0
  {
  	ZEnable = true;
	ZWriteEnable =false;
    VertexShader = compile vs_2_0  vsGlow();
    PixelShader  = compile ps_2_0  psGlow(float2(1,0));
  }
    pass p1
  {
    VertexShader = compile vs_2_0  vsGlow();
    PixelShader  = compile ps_2_0  psGlow(float2(0,1));
  }
}
technique GenerateDepth
{
  pass p0
  {
  	ZEnable = true;
	ZWriteEnable =true;
	CullMode=NONE;
    VertexShader = compile vs_2_0  vsOutDepth();
    PixelShader  = compile ps_2_0  psDepth();
  }
}  

technique GenerateCoustic
{
  pass p0
  {
  	ZEnable = true;
	ZWriteEnable =true;
	CullMode=NONE;
    VertexShader = compile vs_2_0  vsOutCoust();
    PixelShader  = compile ps_2_0  psCoustic();
  }
    pass p1
  {
  	ZEnable = true;
	ZWriteEnable =true;
	CullMode=NONE;
    VertexShader = compile vs_2_0  vsOutCoustFog();
    PixelShader  = compile ps_2_0  psCousticF();
  }
}

