//-----------------------------------------------------------------------------
// skydome.fx
//-----------------------------------------------------------------------------

matrix World;
matrix WorldViewProjection;

float3 CamPos;						//current Cam position
float  GameTime;					//current gametime

//Clouds
float  Speed;						//Cloud speed
float2 Direction;					//Cloud moving direction (x,z)
float  CloudColorR;					//CloudColor
float  CloudColorG;					//CloudColor
float  CloudColorB;					//CloudColor
float  CloudBrightness;
float  CloudDensity;
float  CloudThickness;
float  CloudShadow;
float  CloudDistance;

//Sky
float4 TopColor;					//TopColor of Dome
float4 GroundColor;					//BottomColor of Dome
float  GroundBlendPoint;            //Blendposition
float  SkyBlendPoint;  

Texture NoiseTexture;				//Noise Texture to generate Clouds
sampler NoiseMap=sampler_state { texture   = <NoiseTexture>;
                                    minfilter = LINEAR;
                                    magfilter = LINEAR;
                                    mipfilter = None;
                                    AddressU  = Wrap;
                                    AddressV  = Wrap;};
                                                       

//Vertex Output Structure
struct VS_OUTPUT
{
    float4 Pos    : POSITION0;
    float2 Tex    : TEXCOORD0;
    float2 Value  : TEXCOORD1;
};






// -----------------------------------------------------------------------------------------------------------------
// VertexShader Section
// -----------------------------------------------------------------------------------------------------------------

VS_OUTPUT vsout(float4 pos :POSITION0, float3 nrm :NORMAL, float2 tex :TEXCOORD0 )
{
	VS_OUTPUT Out;
	
	float3 posv = normalize(pos);
	
    Out.Pos  = mul(pos, WorldViewProjection);	//transformed Positions
    Out.Tex  = tex;								//Texcoords
    Out.Value.y = posv.y;					//untransformed Positions
    Out.Value.x = CloudDistance-length(float3(posv.x,0,posv.z));
    return Out;
}


// -----------------------------------------------------------------------------------------------------------------
//  PixelShader Section
// -----------------------------------------------------------------------------------------------------------------

float4 psout(float2 tex : TEXCOORD0, float2 val :TEXCOORD1) : COLOR0
{

   float3  c = float3(CloudColorR,CloudColorG,CloudColorB);	  //Basic CloudColor
   float   cloudspeed  = GameTime * Speed; //CloudSpeed
   
   
   //generate cloudlayer
   float4 cloudlayer  = tex2D(NoiseMap,tex     + cloudspeed *        Direction ) /2; 
   		  cloudlayer += tex2D(NoiseMap,tex *2  + cloudspeed *  1.5 * Direction ) /4;
    	  cloudlayer += tex2D(NoiseMap,tex *4  + cloudspeed *  3.5 * Direction ) /8;
    	  cloudlayer += tex2D(NoiseMap,tex *6  + cloudspeed *  7.5 * Direction ) /16;
          cloudlayer += tex2D(NoiseMap,tex *16 + cloudspeed * 14.5 * Direction ) /32;
          cloudlayer += tex2D(NoiseMap,tex *32 + cloudspeed *  7.5 * Direction ) /32;


   //get Colors
   float a =cloudlayer.a;					//Color for CloudMaps
   float r =cloudlayer.r*CloudBrightness;	//Color for Cloudlayer1
   float g =cloudlayer.g*CloudShadow;		//Color for Cloudlayer2
   float b =cloudlayer.b*CloudShadow*0.25;	//Color for Cloudlayer3
   
   
	//generate CloudMaps
   float CD =CloudDensity*saturate(pow(val.x,2)*4);
   float d1 =saturate(CD-a+0.08); //Density1
   float d2 =saturate(CD-a+0.05); //Density2
   float d3 =saturate(CD-a);	  //Density3

   float map1  = saturate(pow(d1, 1.6-CloudThickness));//Thickness1
   float map2  = saturate(pow(d2, 1.2-CloudThickness));//Thickness2
   float map3  = saturate(pow(d3, 1.1-CloudThickness));//Thickness3


   //interpolate Skycolor by Maps
   float3 skycolor1 = lerp(TopColor,c*r,map1);
   float3 skycolor2 = lerp(skycolor1,1.2-g,map2);
   float3 skycolor3 = lerp(skycolor2,1-b*CloudBrightness,map3);

   //interpolate Skycolor to white by blendpoint
   float3 skycolor4 = lerp(1, skycolor3,saturate(pow(val.y,SkyBlendPoint)*5));
  
   //interpolate GroundColor to Sky
   float3 skycolor5 = lerp(GroundColor, skycolor4,saturate(pow(val.y,GroundBlendPoint)*85));

   //return final color
   return float4(skycolor5,1);
}
 
technique Skydome
{
    pass Pass0
    {
    	//CullMode = None;
    	ZEnable  = true;
    	ZWriteEnable =false;
        VertexShader = compile vs_2_0 vsout();
        PixelShader  = compile ps_2_0 psout();
    }
}
