//-----------------------------------------------------------------------------
// Sun.fx
//-----------------------------------------------------------------------------



//Camera parameters.
matrix WorldViewProjection;

//ColorMap
Texture ColorMap0;
sampler MapSampler0=sampler_state { texture   = <ColorMap0>;
                                    minfilter = LINEAR;
                                    magfilter = LINEAR;
                                    mipfilter = LINEAR;
                                    AddressU  = Clamp;
                                    AddressV  = Clamp;};
Texture ColorMap1;
sampler MapSampler1=sampler_state { texture   = <ColorMap1>;
                                    minfilter = LINEAR;
                                    magfilter = LINEAR;
                                    mipfilter = LINEAR;
                                    AddressU  = Clamp;
                                    AddressV  = Clamp;};                                                                    

//Variables
float3 FlareVec;
float3 FlareSize;
float  FlareOcclusion;
float  FlareIntensity;
float  ScreenFactor;


const float2 size[13]  ={2.20,1,
						 2.00,0,
						 0.33,0,
						 0.31,0,
						 0.26,0,
						 0.10,0,
						 0.16,0,
						 0.31,0,
						 0.26,0,
						 0.31,0,
						 0.33,0,
						 0.50,0,
						 0.75,0,};

const float scale[13] = {1.0,
						 2.1,
						 0.5,
						 0.4,
						 0.3,
						 0.1,
					    -0.1,
						-0.2,
						-0.3,
						-0.4,
						-0.5,
						-0.6,
						-0.85,};

//Vertex Output Structure
struct VS_OUTPUT
{
    float4 Pos    : POSITION0;
    float2 Tex    : TEXCOORD0;
};


// -----------------------------------------------------------------------------------------------------------------
// VertexShader Section
// -----------------------------------------------------------------------------------------------------------------

VS_OUTPUT vsoutSun(float4 pos :POSITION0, float2 tex :TEXCOORD0)
{
	VS_OUTPUT Out;										 				
    Out.Pos =  mul(pos, WorldViewProjection);			
	Out.Tex = tex;
    return Out;
}
// -----------------------------------------------------------------------------------------------------------------

VS_OUTPUT vsoutFlare(float3 pos :POSITION0, float2 tex :TEXCOORD0)
{
	VS_OUTPUT Out;
	float i = pos.z;
	
	if (size[i].y == 1)
	   {pos *= FlareSize * size[i].x * ScreenFactor * pow(FlareOcclusion,2);}
	   else
	   {pos *= FlareSize * size[i].x * ScreenFactor;}
	

	pos += FlareVec  * scale[i];
	pos.z=1;
	
	Out.Pos = float4(pos,1);
	Out.Tex = tex;
	return Out;

}

// -----------------------------------------------------------------------------------------------------------------
//  PixelShader Section
// -----------------------------------------------------------------------------------------------------------------
float4 psoutSun(float2 tex : TEXCOORD0) : COLOR0
{
	float  a = tex2D(MapSampler0, tex);
	float4 color= float4(1,0.8,0.8,1);
	 
    return saturate(color*a*2);
}
// -----------------------------------------------------------------------------------------------------------------

float4 psoutOcc(float2 tex : TEXCOORD0) : COLOR0
{
	float  a = tex2D(MapSampler0, tex);
	clip(a-0.3);
    return 0;
}
// -----------------------------------------------------------------------------------------------------------------

float4 psoutFlare(float2 tex : TEXCOORD0) : COLOR0
{
	float4 color = tex2D(MapSampler0, tex);
	color *=pow(FlareOcclusion,2)*1.3;
    return saturate(color*FlareIntensity);
}
// -----------------------------------------------------------------------------------------------------------------

technique Sun
{
    pass Render0
    {

    AlphaBlendEnable =true;
    SrcBlend = one;
    DestBlend = invsrccolor;
 	ZEnable = true;
    ZWriteEnable = false;
    VertexShader = compile vs_2_0 vsoutSun();
    PixelShader  = compile ps_2_0 psoutSun();
    }
}

technique Occlusion
{
    pass Render0
    {
 	ZEnable = true;
    ZWriteEnable = false;
    ColorWriteEnable=0;
    VertexShader = compile vs_2_0 vsoutSun();
    PixelShader  = compile ps_2_0 psoutOcc();
    }
}
technique Flares
{
    pass Render0
    {
    AlphaBlendEnable = true;
    SrcBlend = one;
    DestBlend = invsrccolor;
 	ZEnable = false;
    ZWriteEnable = false;
        
    VertexShader = compile vs_2_0 vsoutFlare();
    PixelShader  = compile ps_2_0 psoutFlare();
    }
}
