// -----------------------------------------------------------------------------------------------------------------
//  VertexShader Section
//
//	pos		= Position des unteren Center des BillBoards, jeweils fr 24 Vertices gleich
//  tex3	= 	.x  - U-Coord wird geviertelt, Textur besteht aus 4 Teilen
//				.y	- V-Coord
//				.z  - Wert zur Ausrichtung des Billboards
//					  legt die Achsen-Orientierung des Billboards fest auf X,Z,ZX,XZ
//
//	value	=   .x  - Skalierung von X
//				.y  - Skalierung von Y
//				.z  - Texurposition
//				.w  - Differenzwert zur Verschiebung der Top-Vertices
// -----------------------------------------------------------------------------------------------------------------

// 
matrix World;
matrix ViewProjection;

//
float  BlendStart =40;
float  WindForce;	
float  GameTime;		
float  Angle=0.002;			
float3 CamPos;
float3 LightDirection;

float3 nrm[4]={ 0,0,-1, 
	           -1,0, 0,
				1,0,-1,
				1,0, 1};
				
float3 rot[4]={ 1,0,0,
	            0,0,1,
				1,0,1,
	           -1,0,1};
	                 

//Map
Texture ColorMap;
sampler MapSampler=sampler_state { texture   = <ColorMap>;
                                          minfilter = LINEAR;
                                          magfilter = LINEAR;
                                          mipfilter = LINEAR;
                                          AddressU  = MIRROR;
                                          AddressV  = MIRROR;};
                                                 

//Vertex Output Structure

struct VS_OUTPUT0
{
    float4 Pos    : POSITION0;
    float2 Tex    : TEXCOORD0;
    float3 Posw   : TEXCOORD1;
    float3 N      : TEXCOORD2;
}; 

struct VS_OUTPUT1
{
    float4 Pos    : POSITION0;
    float  TexY   : TEXCOORD0;
    float2 Tex    : TEXCOORD1;
    float3 Posw   : TEXCOORD2;
    float  V      : TEXCOORD3;
    float  D      : TEXCOORD4;
}; 

struct VS_OUTPUT2
{
    float4 Pos    : POSITION0;
    float2 Tex    : TEXCOORD0;
};

// -----------------------------------------------------------------------------------------------------------------
// Function to get Alpha value by distance
// -----------------------------------------------------------------------------------------------------------------

//float BlendByDistance(float3 pos)
//{
//	return 1- saturate((distance(CamPos,pos)-BlendEnd)*1/BlendDistance);
//}
  float FogByDistance(float3 pos)
  {
 	return saturate(distance(CamPos,pos)-BlendStart);
  }


// -----------------------------------------------------------------------------------------------------------------
// Function to get Alpha by normal to position
// -----------------------------------------------------------------------------------------------------------------

float BlendByOrthogonal(float3 pos, float3 normal)
{
	float  o = dot(normalize(CamPos-pos),normal);
	if (o<0){o*=-1;}
	return o;
}

// -----------------------------------------------------------------------------------------------------------------
// Function to add Wind to Top Vertices
// -----------------------------------------------------------------------------------------------------------------


float3 addWind(float3 pos, float speed, float amount)
{
	float  sinus;
	float  wind;

 	sinus 	   = sin(Angle * GameTime * amount);
	wind       = sinus * WindForce * speed;
	return float3(1,0,1) * wind;
}


// -----------------------------------------------------------------------------------------------------------------
// Vegetation Drawing Starboards for Grass
// -----------------------------------------------------------------------------------------------------------------
VS_OUTPUT0 vsout0(float3 pos :POSITION0, float4 tex0 :TEXCOORD0,float2 tex1 :TEXCOORD1, float4 value :TEXCOORD2)
{
	VS_OUTPUT0 Out;
	float  i=tex0.z;
	
    //Berechnung der VertexPosition anhand der Texturecoords
    pos += (tex0.x - 0.5)* rot[i] * value.x;
    pos += (1 - tex0.y)* float3(0,value.y,0); 
	
	//Hinzufgen einer Position abhngig vom Wind (nur Top Verticen) (position,scaleY,move)
    if (tex0.y <0.5) 
    	{
    	 pos += addWind(pos,value.z,value.w);
    	}

	float4 pos4 = float4(pos,1);						
    Out.Pos  = mul(pos4, ViewProjection);
    Out.Posw = pos;
    Out.Tex  = tex1;						
    Out.N =nrm[i];
    return Out;
}

// -----------------------------------------------------------------------------------------------------------------

float4 psout0(float2 tex : TEXCOORD0, float3 posw :TEXCOORD1,float3 n :TEXCOORD2) : COLOR0
{
float4 mapcolor = tex2D(MapSampler, tex);
float  b =clamp(BlendByOrthogonal(posw,n)*3,0.2,1);
       mapcolor.a *= b;
return mapcolor;
}



// -----------------------------------------------------------------------------------------------------------------
// Vegetation Drawing Billboards,Crossboards
// -----------------------------------------------------------------------------------------------------------------
VS_OUTPUT1 vsout1(float3 pos :POSITION0, float4 tex0 :TEXCOORD0,float2 tex1 :TEXCOORD1, float4 value :TEXCOORD2)
{
	VS_OUTPUT1 Out;
	float  i=tex0.z;
	float  v=1;
	float3 center = float3(pos.x,0,pos.z);
	
	
    //Berechnung der VertexPosition anhand der Texturecoords
    //Abfrage aus VBuffer ob CrossBoards oder CamrotatedBillBoards
    if (tex0.w >0.5)
    	{
		float3 v=cross(normalize(center-CamPos),float3(0,1,0));
		pos += (tex0.x - 0.5)* v * value.x;}
    	else
    	{
        pos += (tex0.x - 0.5)* rot[i] * value.x;
        v=saturate(BlendByOrthogonal(pos,nrm[i])*3);
        }
    pos += (1 - tex0.y)* float3(0,value.y,0); 
	
	//Hinzufgen einer Position abhngig vom Wind (nur Top Verticen) (position,scaleY,move)
    if (tex0.y <0.5) {pos += addWind(pos,value.z,value.w);}

	float4 pos4 = float4(pos,1);						
    Out.Pos  = mul(pos4, ViewProjection);
    Out.Posw = pos;						
    Out.TexY  = tex0.y;
    Out.Tex  = tex1;
    Out.V =v;
    Out.D =FogByDistance(center);
    return Out;
}


// -----------------------------------------------------------------------------------------------------------------

float4 psout1(float2 tex : TEXCOORD1, float3 posw :TEXCOORD2,float v :TEXCOORD3) : COLOR0
{
if (v<0.05){clip(-1);}
float4 mapcolor = tex2D(MapSampler, tex);
if (mapcolor.a<0.55){clip(-1);}
return mapcolor;
}


// -----------------------------------------------------------------------------------------------------------------

float4 psout2(float texY : TEXCOORD0, float2 tex : TEXCOORD1, float3 posw :TEXCOORD2,float v :TEXCOORD3,float d :TEXCOORD4) : COLOR0
{
if (v<0.05){clip(-1);}
float4 mapcolor = tex2D(MapSampler, tex);
float  a=mapcolor.a;
if (d>0.1)
	{if (a<0.1)
		{float fog=saturate(texY*4 - 2);
	 	mapcolor=float4(0.06,0.1,0.0,1)*fog * d;
	}}
return mapcolor;
}
// END -----------------------------------------------------------------------------------------------------------------



// -----------------------------------------------------------------------------------------------------------------
// Vegetation Drawing Billboards (Low Quality)
// -----------------------------------------------------------------------------------------------------------------
VS_OUTPUT2 vsout2(float3 pos :POSITION0, float4 tex0 :TEXCOORD0,float2 tex1 :TEXCOORD1, float4 value :TEXCOORD2)
{
	VS_OUTPUT2 Out;
	float3 center = float3(pos.x,0,pos.z);
	
    //Berechnung der VertexPosition anhand der Texturecoords
    //Abfrage aus VBuffer ob CrossBoards oder CamrotatedBillBoards


	float3 v=cross(normalize(center-CamPos),float3(0,1,0));
	pos += (tex0.x - 0.5)* v * value.x;
    pos += (1 - tex0.y)* float3(0,value.y,0); 
	
	//Hinzufgen einer Position abhngig vom Wind (nur Top Verticen) (position,scaleY,move)
    //if (tex0.y <0.5) {pos += addWind(pos,value.z,value.w);}

	float4 pos4 = float4(pos,1);						
    Out.Pos  = mul(pos4, ViewProjection);					
    Out.Tex  = tex1;
    return Out;
}

// -----------------------------------------------------------------------------------------------------------------

float4 psout3(float2 tex : TEXCOORD0) : COLOR0
{
float4 color = tex2D(MapSampler, tex);
return color;
}
// END -----------------------------------------------------------------------------------------------------------------

// -----------------------------------------------------------------------------------------------------------------
technique Grass
// -----------------------------------------------------------------------------------------------------------------
{
    pass AlphaBlended
    {
     	AlphaBlendEnable =true;
        SrcBlend = SrcAlpha;
        DestBlend = InvSrcAlpha;
 		ZEnable = true;
        ZWriteEnable =false;
        CullMode = none; 

        VertexShader = compile vs_2_0 vsout0();
        PixelShader  = compile ps_2_0 psout0();
    }
}
// -----------------------------------------------------------------------------------------------------------------
technique Vegetation
// -----------------------------------------------------------------------------------------------------------------
{
    pass ZBuffered
    {
     	AlphaBlendEnable =false;
        SrcBlend = SrcAlpha;
        DestBlend = InvSrcAlpha;
 		ZEnable = true;
        ZWriteEnable =true;
        CullMode = none; 

        VertexShader = compile vs_2_0 vsout1();
        PixelShader  = compile ps_2_0 psout1();
    }
        pass AlphaBlended
    {
     	AlphaBlendEnable =true;
        SrcBlend = SrcAlpha;
        DestBlend = InvSrcAlpha;
 		ZEnable = true;
        ZWriteEnable =false;
        CullMode = none; 

        VertexShader = compile vs_2_0 vsout1();
        PixelShader  = compile ps_2_0 psout2();
    }
}
// -----------------------------------------------------------------------------------------------------------------
technique LowQuality
// -----------------------------------------------------------------------------------------------------------------   
{
        pass AlphaBlended
    {
     	AlphaBlendEnable =true;
        SrcBlend = SrcAlpha;
        DestBlend = InvSrcAlpha;
 		ZEnable = true;
        ZWriteEnable =false;
        CullMode = none; 

        VertexShader = compile vs_2_0 vsout2();
        PixelShader  = compile ps_2_0 psout3();
    }
}


