﻿
Imports SlimDX
Imports SlimDX.Direct3D9
Imports sdxBaseEngine.SDX9BaseEngine

Public Class shaderAlphaBox

    Private Structure _VertexStructure
        Public Position As Vector3
        Public TexCoords As Vector2
    End Structure

    Private _texture(3) As Texture
    Private _position As Vector3

    Private _fx As Effect
    Private _fxHandle(1) As EffectHandle
    Private _fxTechnique(0) As EffectHandle
    Private _vertexdecSizeOfBytes As Integer
    Private _vertexdec As VertexDeclaration
    Private _mesh As Mesh
    Private _boxposition As Vector3
    Private _boxscaling As Vector3 = New Vector3(4, 4, 4)
    Private _posY As Single


    Public Sub New()
        AddHandler dxLoadDeviceObjects, AddressOf LoadDeviceObjects
        AddHandler dxDeviceLost, AddressOf DeviceLost
        AddHandler dxDeviceReset, AddressOf DeviceReset
        AddHandler dxEngineClosing, AddressOf Dispose
    End Sub
#Region "Event Handles"
    Private Sub LoadDeviceObjects()
        dxEngine.LoadingStatus = "Loading Engine: Shader - shaderAlphaBox..."
        createShaderHandles()
        createMesh()
    End Sub
    Private Sub DeviceLost()
        _mesh.Dispose()
        _fx.OnLostDevice()
        For i = 0 To _texture.Length - 1
            If Not _texture(i) Is Nothing Then _texture(i).Dispose()
        Next
    End Sub
    Private Sub DeviceReset()
        createMesh()
        _fx.OnResetDevice()
    End Sub

    Private Sub Dispose()
        If Not _fx Is Nothing Then _fx.Dispose()
        For i = 0 To _fxHandle.Length - 1
            If Not _fxHandle(i) = Nothing Then _fxHandle(i).Dispose()
        Next
        For i = 0 To _fxTechnique.Length - 1
            If Not _fxTechnique(i) = Nothing Then _fxTechnique(i).Dispose()
        Next
        For i = 0 To _texture.Length - 1
            If Not _texture(i) Is Nothing Then _texture(i).Dispose()
        Next
        If Not _mesh Is Nothing Then _mesh.Dispose()
    End Sub
#End Region
#Region "Create ShaderHandles"
    Private Sub createShaderHandles()
        '//Effect
        _fx = Effect.FromMemory(dxDevice, My.Resources.shaderAlphabox, ShaderFlags.PartialPrecision)

        '#Shader Parameter Handles
        _fxHandle(0) = _fx.GetParameter(Nothing, "WorldViewProjection")
        _fxHandle(1) = _fx.GetParameter(Nothing, "ColorMap")

        '//EffectTechnique
        _fxTechnique(0) = _fx.GetTechnique("AlphaBox")

    End Sub
#End Region
#Region "Create Mesh"
    Private Sub createMesh()
        _mesh = Mesh.FromMemory(dxDevice, My.Resources.alphabox, MeshFlags.Managed)
    End Sub

#End Region

#Region "Property"
    Public WriteOnly Property PostionY() As Single
        Set(ByVal value As Single)
            _posY = value
        End Set
    End Property
    Public WriteOnly Property Scaling() As Vector3
        Set(ByVal value As Vector3)
            _boxscaling = value
        End Set
    End Property

    Public WriteOnly Property Texture90() As Texture
        Set(ByVal value As Texture)
            _texture(0) = value
        End Set
    End Property
    Public WriteOnly Property Texture270() As Texture
        Set(ByVal value As Texture)
            _texture(1) = value
        End Set
    End Property
    Public WriteOnly Property Texture180() As Texture
        Set(ByVal value As Texture)
            _texture(2) = value
        End Set
    End Property
    Public WriteOnly Property Texture360() As Texture
        Set(ByVal value As Texture)
            _texture(3) = value
        End Set
    End Property
#End Region

#Region "# Render #"
    Public Sub Draw()

        _boxposition = dxCamera.currentPosition
        _boxposition.Y = _posY

        '#Shader settigs
        dxTransformWorld = Matrix.Scaling(_boxscaling) * Matrix.Translation(_boxposition)

        _fx.SetValue(_fxHandle(0), dxWorld.WorldViewProjection)
        _fx.SetTexture(_fxHandle(1), _texture(0))
        _fx.Technique = _fxTechnique(0)
        _fx.Begin()
        For i = 0 To 3
            _fx.SetTexture(_fxHandle(1), _texture(i))
            _fx.BeginPass(0)
            _mesh.DrawSubset(i)
            _fx.EndPass()
        Next
        _fx.End()
    End Sub
#End Region

End Class
