﻿' Title    : sdxBaseEngine Class
' Name     : shaderBillboard
' Author   : Sascha Bajus
' Created  : 11/2009
' Copyright © 2008 Sascha Bajus

#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region

Imports SlimDX
Imports SlimDX.Direct3D9
Imports sdxBaseEngine.SDX9BaseEngine

Public Class shaderBillboard
    Private _vbuffer As VertexBuffer            '#VertexBuffer
    Private _vertexdecSizeOfBytes As Integer
    Private _billboard As New List(Of Billboard)
    Private _render As New List(Of Billboard)
    Private _vertexdec As VertexDeclaration     '#VertexDeclaration
    Private _texture As Texture                 '#List of Textures for the Object

    Private _fx As Effect
    Private _fxHandle(2) As EffectHandle
    Private _fxTechnique(1) As EffectHandle
    Private _orientation As Orientation

    Private _camclip As Boolean
    Private _camcliprange As Vector2


    Public Enum Orientation As Integer
        BillboardXY = 0
        BillboardXZ = 1
    End Enum


    Private Structure _VertexStructure
        Public Position As Vector3
        Public TexCoords As Vector2
        Public Values As Vector4
    End Structure
    ''' <summary>
    ''' Erstellt einen neuen Buffer für Billboards
    ''' </summary>
    ''' <param name="BillboardData">Daten der Billboards</param>

    Public Sub New(ByVal BillboardData As List(Of Billboard))
        _billboard.AddRange(BillboardData)
        createShaderHandles()
        createVertexdeclaration()
        createVertexbuffer()
        FillVertexbuffer()
    End Sub



#Region "Create ShaderHandles"
    Private Sub createShaderHandles()
        '//Effect
        _fx = Effect.FromMemory(dxDevice, My.Resources.shaderBillboard, ShaderFlags.PartialPrecision)

        '#Shader Parameter Handles
        _fxHandle(0) = _fx.GetParameter(Nothing, "World")
        _fxHandle(1) = _fx.GetParameter(Nothing, "ViewProjection")
        _fxHandle(2) = _fx.GetParameter(Nothing, "ColorMap0")

        '//EffectTechnique
        _fxTechnique(0) = _fx.GetTechnique("BillboardsXY")
        _fxTechnique(1) = _fx.GetTechnique("BillboardsXZ")
    End Sub
#End Region
#Region "Create Vertexdeclaration"
    Private Sub createVertexdeclaration()
        Dim elements(3) As VertexElement
        elements(0) = New VertexElement(0, 0, DeclarationType.Float3, DeclarationMethod.Default, DeclarationUsage.Position, 0)
        elements(1) = New VertexElement(0, 12, DeclarationType.Float2, DeclarationMethod.Default, DeclarationUsage.TextureCoordinate, 0)
        elements(2) = New VertexElement(0, 20, DeclarationType.Float4, DeclarationMethod.Default, DeclarationUsage.TextureCoordinate, 1)
        elements(3) = VertexElement.VertexDeclarationEnd
        _vertexdec = New VertexDeclaration(dxDevice, elements)

        '#size of FloatValues = 4 bytes
        _vertexdecSizeOfBytes = 12 + 8 + 16
    End Sub
#End Region
#Region "Create VertexBuffer"
    Private Sub createVertexbuffer()
        '#create vbuffer 
        _vbuffer = New VertexBuffer(dxDevice, _vertexdecSizeOfBytes * 6 * _billboard.Count, Usage.None, VertexFormat.None, Pool.Default)
    End Sub
#End Region

#Region "FrustumCulling"
    Private Sub FrustumCullingXZ(ByVal ListIndex As Integer)
        With dxCamera
            If .FrustumTestNear(_billboard(ListIndex).Position, 0) = False Then Exit Sub
            If .FrustumTestLeft(_billboard(ListIndex).Position, _billboard(ListIndex).Radius) = False Then Exit Sub
            If .FrustumTestRight(_billboard(ListIndex).Position, _billboard(ListIndex).Radius) = False Then Exit Sub
            If .FrustumTestTop(_billboard(ListIndex).Position, _billboard(ListIndex).Radius) = False Then Exit Sub
            If .FrustumTestBottom(_billboard(ListIndex).Position, _billboard(ListIndex).Radius) = False Then Exit Sub
        End With
        _render.Add(_billboard(ListIndex))
    End Sub
    Private Sub FrustumCullingXY(ByVal ListIndex As Integer)
        With dxCamera

        End With
        _render.Add(_billboard(ListIndex))
    End Sub
#End Region
#Region "Fill Vertexbuffer"
    Private Function FillVertexbuffer() As Boolean

        '#Frustum Culling
        If _orientation = Orientation.BillboardXZ Then
            For i = 0 To _billboard.Count - 1
                FrustumCullingXZ(i)
            Next
        Else
            For i = 0 To _billboard.Count - 1
                FrustumCullingXY(i)
            Next
        End If


        'Der vertexbuffer hat folgenden Aufbau:
        'Position  = xyz - unterer Center des Billboards
        'TexCoords = uv  - Texturkoordinaten
        'Values    = Vector4
        '               x = Skalierung des Billboards X
        '               y = Skalierung des Billboards Y
        '               z = RND Value * 4 zur Texturauswahl im shader
        '               w = RND Value +- 50 zur ungleichmäßigen Vertexverschiebung

        'Fill Vertexbuffer

        If _render.Count = 0 Then Return False

        Dim stream As DataStream = _vbuffer.Lock(0, 0, LockFlags.Discard)
        Dim vertex(_render.Count * 6 - 1) As _VertexStructure
        Dim ii As Integer = 0


            For i = 0 To _render.Count - 1

                '00
                vertex(ii).Position = _render(i).Position
                vertex(ii).TexCoords = New Vector2(0, 0)
            vertex(ii).Values = New Vector4(_render(i).sclA, _
                                            _render(i).sclB, _
                                            _render(i).sclC * 4, _
                                            0)
                '10
                ii += 1
                vertex(ii).Position = _render(i).Position
                vertex(ii).TexCoords = New Vector2(1, 0)
            vertex(ii).Values = New Vector4(_render(i).sclA, _
                                            _render(i).sclB, _
                                            _render(i).sclC * 4, _
                                            0)
                '01
                ii += 1
                vertex(ii).Position = _render(i).Position
                vertex(ii).TexCoords = New Vector2(0, 1)
            vertex(ii).Values = New Vector4(_render(i).sclA, _
                                            _render(i).sclB, _
                                            _render(i).sclC * 4, _
                                            0)

                '10
                ii += 1
                vertex(ii).Position = _render(i).Position
                vertex(ii).TexCoords = New Vector2(1, 0)
            vertex(ii).Values = New Vector4(_render(i).sclA, _
                                            _render(i).sclB, _
                                            _render(i).sclC * 4, _
                                            0)

                '11
                ii += 1
                vertex(ii).Position = _render(i).Position
                vertex(ii).TexCoords = New Vector2(1, 1)
            vertex(ii).Values = New Vector4(_render(i).sclA, _
                                            _render(i).sclB, _
                                            _render(i).sclC * 4, _
                                            0)
                '01
                ii += 1
                vertex(ii).Position = _render(i).Position
                vertex(ii).TexCoords = New Vector2(0, 1)
            vertex(ii).Values = New Vector4(_render(i).sclA, _
                                            _render(i).sclB, _
                                            _render(i).sclC * 4, _
                                            0)

                ii += 1
            Next
            stream.WriteRange(vertex)

        stream.Close()
        stream.Dispose()
        _vbuffer.Unlock()
        Return True
    End Function
#End Region
#Region "Property"
    ''' <summary>
    ''' Ruft die Liste der Billboards ab, oder legt diese fest
    ''' </summary>
    Public Property ListOfBillboards() As List(Of Billboard)
        Set(ByVal value As List(Of Billboard))
            _billboard.Clear()
            _billboard.AddRange(value)
        End Set
        Get
            Return _billboard
        End Get
    End Property

    ''' <summary>
    ''' Legt die Texture fest
    ''' </summary>
    Public WriteOnly Property Colormap0() As Texture
        Set(ByVal value As Texture)
            _texture = value
        End Set
    End Property
    Public ReadOnly Property Info() As String
        Get
            Return "Render Billboards " & _render.Count.ToString & "/" & _billboard.Count.ToString
        End Get
    End Property

    Public WriteOnly Property CamClip() As Boolean
        Set(ByVal value As Boolean)
            _camclip = value
        End Set
    End Property

    Public WriteOnly Property CamClipRange() As Vector2
        Set(ByVal value As Vector2)
            _camcliprange = value
        End Set
    End Property

    Public WriteOnly Property Technique() As Orientation
        Set(ByVal value As Orientation)
            _orientation = value
            _fx.Technique = _fxTechnique(value)
        End Set
    End Property

#End Region
#Region "## Render ##"
    '''<summary>
    '''Rendert den Vertexbuffer
    '''</summary>
    Public Sub Draw()
        '#clear renderlist
        _render.Clear()
        If _camclip Then
            Dim y As Single = dxCamera.currentTarget.Y
            If y > _camcliprange.Y Or y < _camcliprange.X Then Exit Sub
        End If

        '#Update VertexBuffer
        If FillVertexbuffer() Then

            '#Shader
            _fx.SetValue(_fxHandle(0), dxWorld.World)
            _fx.SetValue(_fxHandle(1), dxWorld.ViewProjection)
            _fx.SetTexture(_fxHandle(2), _texture)

            _fx.Begin()
            _fx.BeginPass(0)
            dxDevice.VertexDeclaration = _vertexdec
            dxDevice.SetStreamSource(0, _vbuffer, 0, _vertexdecSizeOfBytes)
            dxDevice.DrawPrimitives(PrimitiveType.TriangleList, 0, _render.Count * 2)
            _fx.EndPass()
            _fx.End()
        End If
    End Sub
#End Region
#Region "Dispose"
    ''' <summary>
    ''' Gibt die verwendeten Ressourcen der Klasse wieder frei.
    ''' </summary>
    Public Sub Dispose()
        _texture.Dispose()
        _vertexdec.Dispose()
        _vbuffer.Dispose()
        _fx.Dispose()
        For i As Integer = 0 To _fxHandle.Length - 1
            _fxHandle(i).Dispose()
        Next
        _fxTechnique(0).Dispose()
        _fxTechnique(1).Dispose()
    End Sub
#End Region


End Class
