﻿Imports SlimDX
Imports SlimDX.Direct3D9
Imports sdxBaseEngine.SDX9BaseEngine


Public Class shaderLakeWater


    Private _mapNormal As Texture
    Private _mapCoustic As TextureAnimation
    Private _mapRefraction As TextureSurface
    Private _mapReflection As TextureSurface
    Private _mapDepth As TextureSurface
    Private _Surface1 As TextureSurface
    Private _Surface2 As TextureSurface
    Private _Surface3 As TextureSurface


    '#Clippingplane
    Private _plane As sdxClipPlaneH

    Private _fx As Effect
    Private _fxHandle(21) As EffectHandle
    Private _fxTechnique(6) As EffectHandle
    Private _height As Single
    Private _Matrix As Matrix
    Private _FogObjectMatrix As Matrix

    '#Set Shader Defaults
    Private _WaveHeight As Single = 0.05
    Private _WaveLength As Single = 2
    Private _WindDirection As New Vector2(0.5, 0.5)
    Private _SunPosition As New Vector3(0, 1, 0)
    Private _SpecularPower As Single = 250
    Private _SColor As New RGBAcolor(255, 125, 95)
    Private _DullColor As New RGBAcolor(60, 120, 30)
    Private _Size As New Vector3(5)
    Private _WindForce As Single = 0.03
    Private _Position As Vector3
    Private _FresnelSharpness As Single = 1
    Private _offsetsize As Vector2
    Private _renderPos As Vector3

    Public Event DrawReflectedObjects()
    Public Event DrawRefractedObjects()
    Public Event DrawFogObjects()

    '#for Frustum Culling
    Private _frustumRad As Single
    Private _status As Status

    Public Enum ClipPlane As Integer
        ClipSpace = 0
        WorldSpace = 1
    End Enum

    Public Enum Status As Integer
        NotVisible = 0
        Visible = 1
        UnderWater = 2
    End Enum
    Private Enum fxTechnique As Integer
        LakeWaterMap = 0
        LakeUWaterMap = 1
        FresnelMap = 2
        GenerateSpecular = 3
        GenerateGlow = 4
        GenerateDepthMap = 5
        GenerateCoustic = 6
    End Enum

    Public Sub New()
        AddHandler dxLoadDeviceObjects, AddressOf Load
        AddHandler dxDeviceLost, AddressOf DeviceLost
        AddHandler dxDeviceReset, AddressOf DeviceReset
        AddHandler dxEngineClosing, AddressOf Dispose
        _plane = New sdxClipPlaneH(0)
    End Sub
#Region "Event Handles"
    Private Sub Load()
        dxEngine.LoadingStatus = "Loading Engine: Shader - shaderLakeWater"
        createShaderHandles()
        _mapNormal = dxTexture.FromMemory(My.Resources.ddsLakeWater_NORM, Color.Black)

        If dxEngine.BackBufferSize.Y < 800 Then
            _Surface1 = New TextureSurface(dxEngine.BackBufferSize)
        Else
            _Surface1 = New TextureSurface(New Point(800, 800))
        End If

        If dxEngine.BackBufferSize.Y < 512 Then
            _mapRefraction = New TextureSurface(dxEngine.BackBufferSize, Format.A8R8G8B8)
            _mapReflection = New TextureSurface(dxEngine.BackBufferSize, Format.A8R8G8B8)
            _mapDepth = New TextureSurface(dxEngine.BackBufferSize, 1, Format.R32F)
            _Surface2 = New TextureSurface(dxEngine.BackBufferSize, Format.A8R8G8B8)
            _Surface3 = New TextureSurface(dxEngine.BackBufferSize, Format.A8R8G8B8)
            _offsetsize = New Vector2(CSng(1 / dxEngine.BackBufferSize.X), CSng(1 / dxEngine.BackBufferSize.Y))
        Else
            _mapRefraction = New TextureSurface(New Point(512, 512), Format.A8R8G8B8)
            _mapReflection = New TextureSurface(New Point(512, 512), Format.A8R8G8B8)
            _mapDepth = New TextureSurface(New Point(512, 512), 1, Format.R32F)
            _Surface2 = New TextureSurface(New Point(512, 512), Format.A8R8G8B8)
            _Surface3 = New TextureSurface(New Point(512, 512), Format.A8R8G8B8)
            _offsetsize = New Vector2(CSng(1 / 512), CSng(1 / 512))
        End If
    End Sub

    Private Sub DeviceLost()
        _fx.OnLostDevice()
        _mapReflection.Dispose()
        _mapRefraction.Dispose()
        _mapDepth.Dispose()
        _mapNormal.Dispose()
        _Surface1.Dispose()
        _Surface2.Dispose()
        _Surface3.Dispose()
    End Sub
    Private Sub DeviceReset()
        _fx.OnResetDevice()
        Load()
    End Sub
    Private Sub Dispose()
        If Not _mapReflection Is Nothing Then _mapReflection.Dispose()
        If Not _mapRefraction Is Nothing Then _mapRefraction.Dispose()
        If Not _mapDepth Is Nothing Then _mapDepth.Dispose()
        If Not _mapNormal Is Nothing Then _mapNormal.Dispose()
        If Not _Surface1 Is Nothing Then _Surface1.Dispose()
        If Not _Surface2 Is Nothing Then _Surface2.Dispose()
        If Not _Surface3 Is Nothing Then _Surface3.Dispose()

        For i = 0 To _fxHandle.Length - 1
            On Error Resume Next
            If Not _fxHandle(i) Is Nothing Then _fxHandle(i).Dispose()
        Next
        _fx.Dispose()

        For i = 0 To _fxTechnique.Length - 1
            _fxTechnique(i).Dispose()
        Next
    End Sub
#End Region
#Region "Create ShaderHandles"
    Private Sub createShaderHandles()
        '#load Shader fx-file
        If _fx Is Nothing Then

            _fx = Effect.FromMemory(dxDevice, My.Resources.shaderLakeWater, ShaderFlags.PartialPrecision)

            '#Shader Parameter Handles
            _fxHandle(0) = _fx.GetParameter(Nothing, "World")
            _fxHandle(1) = _fx.GetParameter(Nothing, "WorldViewProjection")
            _fxHandle(2) = _fx.GetParameter(Nothing, "ReflectedWorldViewProjection")
            _fxHandle(3) = _fx.GetParameter(Nothing, "ReflectionMap")
            _fxHandle(4) = _fx.GetParameter(Nothing, "RefractionMap")
            _fxHandle(5) = _fx.GetParameter(Nothing, "WaveNormalMap")
            _fxHandle(6) = _fx.GetParameter(Nothing, "GlowMap")
            _fxHandle(7) = _fx.GetParameter(Nothing, "WaveLength")
            _fxHandle(8) = _fx.GetParameter(Nothing, "WaveHeight")
            _fxHandle(9) = _fx.GetParameter(Nothing, "WindForce")
            _fxHandle(10) = _fx.GetParameter(Nothing, "WindDirection")
            _fxHandle(11) = _fx.GetParameter(Nothing, "CamPos")
            _fxHandle(12) = _fx.GetParameter(Nothing, "GameTime")
            _fxHandle(13) = _fx.GetParameter(Nothing, "SunPos")
            _fxHandle(14) = _fx.GetParameter(Nothing, "SPower")
            _fxHandle(15) = _fx.GetParameter(Nothing, "DullColor")
            _fxHandle(16) = _fx.GetParameter(Nothing, "FresnelAdjust")
            _fxHandle(17) = _fx.GetParameter(Nothing, "OffsetSize")
            _fxHandle(18) = _fx.GetParameter(Nothing, "SColor")
            _fxHandle(19) = _fx.GetParameter(Nothing, "WaterHeight")
            _fxHandle(20) = _fx.GetParameter(Nothing, "DepthMap")
            _fxHandle(21) = _fx.GetParameter(Nothing, "CousticMap")

            '#Shader Technique Handles
            _fxTechnique(0) = _fx.GetTechnique("LakeWaterMap")
            _fxTechnique(1) = _fx.GetTechnique("UnderWaterMap")
            _fxTechnique(2) = _fx.GetTechnique("Fresnel")
            _fxTechnique(3) = _fx.GetTechnique("GenerateSpecular")
            _fxTechnique(4) = _fx.GetTechnique("GenerateGlow")
            _fxTechnique(5) = _fx.GetTechnique("GenerateDepth")
            _fxTechnique(6) = _fx.GetTechnique("GenerateCoustic")
        End If
    End Sub
#End Region


#Region "Generate Maps"
    Private Sub GenerateWaterMaps()

        '#save setting
        _Matrix = dxCamera.currentViewMatrix


        _renderPos = _Position
        _renderPos.Y = CSng(_renderPos.Y - 0.05)
        _plane.Distance = _Position.Y
        generateGlowMap()
        _plane.ClippingEnabled = True
        generateDepthMap()
        generateRefractionMap()
        generateReflectionMap()
        _plane.ClippingEnabled = False


        '#Save Maps to File (Debug only)
        'Texture.ToFile(_mapRefraction.Texture, dxEngine.ApplicationPath & "_mapRefraction.jpg", ImageFileFormat.Jpg)
        'Texture.ToFile(_mapReflection.Texture, dxEngine.ApplicationPath & "_mapReflection.jpg", ImageFileFormat.Jpg)
        'Texture.ToFile(_mapDepth.Texture, dxEngine.ApplicationPath & "_mapdepth.jpg", ImageFileFormat.Jpg)

        'Texture.ToFile(_Surface1.Texture, dxEngine.ApplicationPath & "Surface1_Spec.jpg", ImageFileFormat.Jpg)
        'Texture.ToFile(_Surface2.Texture, dxEngine.ApplicationPath & "Surface2_GlowX.jpg", ImageFileFormat.Jpg)
        'Texture.ToFile(_Surface3.Texture, dxEngine.ApplicationPath & "Surface3_GlowY.jpg", ImageFileFormat.Jpg)

        '#switch to defaultsurface
        dxEngine.SurfaceChange() = EngineSurfaces.DefaultTarget
        dxCamera.currentViewMatrix = _Matrix
    End Sub

    Private Sub GenerateUnderWaterMaps()

        '#save setting
        _Matrix = dxCamera.currentViewMatrix

        _renderPos = _Position
        _renderPos.Y = CSng(_renderPos.Y + 0.05)

        generateUnderWaterMap()
        generateUnderWaterCousticMap()

        '#Save Maps to File (Debug only)
        'Texture.ToFile(_mapRefraction.Texture, dxEngine.ApplicationPath & "_mapRefraction.jpg", ImageFileFormat.Jpg)
        'Texture.ToFile(_mapReflection.Texture, dxEngine.ApplicationPath & "_mapReflection.jpg", ImageFileFormat.Jpg)
        'Texture.ToFile(_mapDepth.Texture, dxEngine.ApplicationPath & "_mapdepth.jpg", ImageFileFormat.Jpg)

        'Texture.ToFile(_Surface1.Texture, dxEngine.ApplicationPath & "Surface1_Spec.jpg", ImageFileFormat.Jpg)
        'Texture.ToFile(_Surface2.Texture, dxEngine.ApplicationPath & "Surface2_GlowX.jpg", ImageFileFormat.Jpg)
        'Texture.ToFile(_Surface3.Texture, dxEngine.ApplicationPath & "Surface3_GlowY.jpg", ImageFileFormat.Jpg)

        '#switch to defaultsurface
        dxEngine.SurfaceChange() = EngineSurfaces.DefaultTarget
        dxCamera.currentViewMatrix = _Matrix
    End Sub
#End Region

#Region "MAP - Refraction"
    Private Sub generateRefractionMap()
        '#Settings for RefractionMap
        dxEngine.SurfaceCurrent(Color.Black) = _mapRefraction.Surface
        dxCamera.currentViewMatrix = dxCamera.LookAtMatrixLH
        _plane.PlaneType = sdxClipPlaneH.type.refraction

        '#Draw RefrectionMap
        RaiseEvent DrawRefractedObjects()
    End Sub
#End Region
#Region "MAP - Reflection"
    Private Sub generateReflectionMap()
        '#Settings for ReflectionMap
        dxEngine.SurfaceCurrent(Color.Black) = _mapReflection.Surface
        dxCamera.currentViewMatrix = dxCamera.LookAtMatrixLHReflectionY(_renderPos.Y)
        _plane.PlaneType = sdxClipPlaneH.type.reflection
        '#Draw ReflectionMap
        RaiseEvent DrawReflectedObjects()
    End Sub
#End Region
#Region "MAP - DepthMap"
    Private Sub generateDepthMap()
        '#Settings for DepthMap
        dxCamera.currentViewMatrix = dxCamera.LookAtMatrixLH
        dxEngine.SurfaceCurrent(Color.Black) = _mapDepth.Surface
        _plane.PlaneType = sdxClipPlaneH.type.refraction

        '#ShaderTechnique
        _fx.Technique = _fxTechnique(fxTechnique.GenerateDepthMap)


        '#ShaderHandles
        dxTransformWorld = Matrix.Identity
        _fx.SetValue(_fxHandle(0), dxWorld.World)
        _fx.SetValue(_fxHandle(1), dxWorld.WorldViewProjection)
        _fx.SetValue(_fxHandle(19), _plane.Distance)

        _fx.Begin()
        _fx.BeginPass(0)
        RaiseEvent DrawFogObjects()
        _fx.EndPass()
        _fx.End()
    End Sub
#End Region
#Region "MAP - CausticFog"

    Private Sub generateUnderWaterCousticMap()
        '#Settings for DepthMap
        dxCamera.currentViewMatrix = dxCamera.LookAtMatrixLH
        dxEngine.SurfaceCurrent(Color.Black) = _Surface2.Surface

        _plane.PlaneType = sdxClipPlaneH.type.refraction
        _plane.ClippingEnabled = True

        '#Draw Coustics
        _fx.Technique = _fxTechnique(fxTechnique.GenerateCoustic)
        '#ShaderHandles
        dxTransformWorld = _FogObjectMatrix
        _fx.SetValue(_fxHandle(0), dxWorld.World)
        _fx.SetValue(_fxHandle(1), dxWorld.WorldViewProjection)
        _fx.SetValue(_fxHandle(11), dxCamera.currentPosition)
        _fx.SetTexture(_fxHandle(21), _mapCoustic.Texture)
        _fx.Begin()
        _fx.BeginPass(0)
        RaiseEvent DrawFogObjects()
        _fx.EndPass()
        _fx.End()
        _plane.ClippingEnabled = False

        '#ShaderHandles
        _fx.Technique = _fxTechnique(fxTechnique.GenerateCoustic)
        dxTransformWorld = Matrix.Translation(_Position) * Matrix.Scaling(_Size)
        _fx.SetValue(_fxHandle(0), dxWorld.World)
        _fx.SetValue(_fxHandle(1), dxWorld.WorldViewProjection)
        _fx.SetValue(_fxHandle(11), dxCamera.currentPosition)
        _fx.SetTexture(_fxHandle(21), _mapCoustic.Texture)
        dxVBuffer.Quad.SetBuffer = sdxVBQuad.Offset.XZ_Orientation
        _fx.Begin()
        _fx.BeginPass(1)
        '#Draw WaterQuad
        dxVBuffer.Quad.Draw()
        _fx.EndPass()
        _fx.End()

    End Sub
#End Region
#Region "MAP - UnderWater"
    Private Sub generateUnderWaterMap()
        '#Settings for ReflectionMap
        dxEngine.SurfaceCurrent(Color.Black) = _mapReflection.Surface
        dxCamera.currentViewMatrix = dxCamera.LookAtMatrixLH

        '#Draw ReflectionMap
        RaiseEvent DrawReflectedObjects()
    End Sub
#End Region
#Region "MAP - SpecularGlow"
    Private Sub generateGlowMap()
        '#Switch to surface1
        dxEngine.SurfaceCurrent(Color.Black) = _Surface1.Surface

        dxCamera.currentViewMatrix = dxCamera.LookAtMatrixLH
        dxTransformWorld = Matrix.Translation(_renderPos) * Matrix.Scaling(_Size)
        '#ShaderTechnique
        _fx.Technique = _fxTechnique(fxTechnique.GenerateSpecular)

        '#ShaderHandles
        _fx.SetValue(_fxHandle(0), dxWorld.World)
        _fx.SetValue(_fxHandle(1), dxWorld.WorldViewProjection)
        _fx.SetTexture(_fxHandle(5), _mapNormal)
        _fx.SetValue(_fxHandle(7), _WaveLength)
        _fx.SetValue(_fxHandle(8), _WaveHeight)
        _fx.SetValue(_fxHandle(9), _WindForce)
        _fx.SetValue(_fxHandle(10), _WindDirection)
        _fx.SetValue(_fxHandle(11), dxCamera.currentPosition)
        _fx.SetValue(_fxHandle(12), dxGametime.Gametime)
        _fx.SetValue(_fxHandle(13), _SunPosition)
        _fx.SetValue(_fxHandle(14), _SpecularPower)
        _fx.SetValue(_fxHandle(15), _DullColor)
        _fx.SetValue(_fxHandle(16), _FresnelSharpness)
        _fx.SetValue(_fxHandle(17), _offsetsize)
        _fx.SetValue(_fxHandle(18), _SColor)
        _fx.SetTexture(_fxHandle(20), _mapDepth.Texture)



        '#Draw WaterQuad SpecularMap
        _fx.Begin()
        _fx.BeginPass(0)
        dxVBuffer.Quad.SetBuffer = sdxVBQuad.Offset.XZ_Orientation
        dxVBuffer.Quad.Draw()
        _fx.EndPass()
        _fx.End()



        '#Switch to Surface2
        dxEngine.SurfaceCurrent(Color.Black) = _Surface2.Surface

        '#ShaderHandles
        _fx.Technique = _fxTechnique(fxTechnique.GenerateGlow)
        _fx.SetTexture(_fxHandle(6), _Surface1.Texture)

        '#Draw FSQ Bloom Pass0
        _fx.Begin()
        _fx.BeginPass(0)
        dxVBuffer.FullScreen.Draw()
        _fx.EndPass()
        _fx.End()


        '#Switch to Surface3
        dxEngine.SurfaceCurrent(Color.Black) = _Surface3.Surface

        '#ShaderHandles
        _fx.Technique = _fxTechnique(fxTechnique.GenerateGlow)
        _fx.SetTexture(_fxHandle(6), _Surface2.Texture)

        '#Draw FSQ Bloom Pass1
        _fx.Begin()
        _fx.BeginPass(1)
        dxVBuffer.FullScreen.Draw()
        _fx.EndPass()
        _fx.End()

    End Sub
#End Region

#Region "FrustumCulling"
    Private Function FrustumCulling() As Boolean
        With dxCamera
            If .FrustumTestNear(_Position, _frustumRad) = False Then Return False
            If .FrustumTestLeft(_Position, _frustumRad) = False Then Return False
            If .FrustumTestRight(_Position, _frustumRad) = False Then Return False
        End With
        Return True
    End Function

#End Region
#Region "Refresh Status"
    Public Sub Refresh()
        'frustum test
        If Not FrustumCulling() Then
            _status = Status.NotVisible
            Exit Sub
        End If

        '#Generate Maps 
        If dxCamera.currentPosition.Y < (_Position.Y + 0.05) Then _status = Status.UnderWater
        If dxCamera.currentPosition.Y > (_Position.Y - 0.05) Then _status = Status.Visible
    End Sub
#End Region


#Region "##  Draw  ##"
    Public Sub Draw()
        '#ShaderTechnique
        Select Case _status
            Case Status.NotVisible
                Exit Sub
            Case Status.Visible
                GenerateWaterMaps()
                _fx.Technique = _fxTechnique(fxTechnique.LakeWaterMap)
            Case Status.UnderWater
                GenerateUnderWaterMaps()
                _fx.Technique = _fxTechnique(fxTechnique.LakeUWaterMap)
        End Select


        dxTransformWorld = Matrix.Translation(_renderPos) * Matrix.Scaling(_Size)

        '#ShaderHandles
        _fx.SetValue(_fxHandle(0), dxWorld.World)
        _fx.SetValue(_fxHandle(1), dxWorld.WorldViewProjection)
        _fx.SetValue(_fxHandle(2), dxWorld.WorldViewProjectionReflectionY(_renderPos.Y))
        _fx.SetTexture(_fxHandle(3), _mapReflection.Texture)
        _fx.SetTexture(_fxHandle(4), _mapRefraction.Texture)
        _fx.SetTexture(_fxHandle(5), _mapNormal)

        _fx.SetValue(_fxHandle(7), _WaveLength)
        _fx.SetValue(_fxHandle(8), _WaveHeight)
        _fx.SetValue(_fxHandle(9), _WindForce)
        _fx.SetValue(_fxHandle(10), _WindDirection)
        _fx.SetValue(_fxHandle(11), dxCamera.currentPosition)
        _fx.SetValue(_fxHandle(12), dxGametime.Gametime)
        _fx.SetValue(_fxHandle(13), _SunPosition)
        _fx.SetValue(_fxHandle(14), _SpecularPower)

        _fx.SetValue(_fxHandle(15), _DullColor)
        _fx.SetValue(_fxHandle(16), _FresnelSharpness)
        _fx.SetTexture(_fxHandle(20), _mapDepth.Texture)

        '#set VBuffer
        dxVBuffer.Quad.SetBuffer = sdxVBQuad.Offset.XZ_Orientation

        '#Draw WaterQuad
        _fx.Begin()
        _fx.BeginPass(0)

        dxVBuffer.Quad.Draw()
        _fx.EndPass()
        _fx.End()

    End Sub
#End Region



#Region "Property"
    Public WriteOnly Property CousticMap() As TextureAnimation
        Set(ByVal value As TextureAnimation)
            _mapCoustic = value
        End Set
    End Property

    Public WriteOnly Property FogObjectMatrix() As Matrix
        Set(ByVal value As Matrix)
            _FogObjectMatrix = value
        End Set
    End Property
    Public ReadOnly Property DrawStatus() As Status
        Get
            Return _status
        End Get
    End Property
    Public ReadOnly Property PostEffectSpecularGlow() As Texture
        Get
            Return _Surface3.Texture
        End Get
    End Property
    Public ReadOnly Property PostEffectUnderWaterFog() As Texture
        Get
            Return _Surface2.Texture
        End Get
    End Property

    Public WriteOnly Property SetClipPlane() As ClipPlane
        Set(ByVal value As ClipPlane)
            If value = ClipPlane.WorldSpace Then
                _plane.SetClipplane()
            Else
                _plane.SetClipplane(True)
            End If
        End Set
    End Property

    Public WriteOnly Property WaveHeight() As Single
        Set(ByVal value As Single)
            _WaveHeight = value
        End Set
    End Property
    Public WriteOnly Property WaveLength() As Single
        Set(ByVal value As Single)
            _WaveLength = value
        End Set
    End Property

    Public WriteOnly Property WindDirection() As Vector2
        Set(ByVal value As Vector2)
            _WindDirection = value
        End Set
    End Property
    Public WriteOnly Property WindForce() As Single
        Set(ByVal value As Single)
            _WindForce = value
        End Set
    End Property
    Public WriteOnly Property SpecularPower() As Single
        Set(ByVal value As Single)
            _SpecularPower = value
        End Set
    End Property
    Public WriteOnly Property SpecularColor() As RGBAcolor
        Set(ByVal value As RGBAcolor)
            _SColor = value
        End Set
    End Property
    Public WriteOnly Property SunPosition() As Vector3
        Set(ByVal value As Vector3)
            _SunPosition = Vector3.Normalize(value)
        End Set
    End Property
    Public WriteOnly Property FresnelSharpness() As Single
        Set(ByVal value As Single)
            _FresnelSharpness = value
        End Set
    End Property
    Public Property WaterDullColor() As RGBAcolor
        Set(ByVal value As RGBAcolor)
            _DullColor = value
        End Set
        Get
            Return _DullColor
        End Get
    End Property
    Public WriteOnly Property WaterQuadSize() As Single
        Set(ByVal value As Single)
            _Size = New Vector3(value, 1, value)
            _frustumRad = CSng(Math.Sqrt(2) * value * 0.6)
        End Set
    End Property
    Public Property WaterQuadPosition() As Vector3
        Set(ByVal value As Vector3)
            _Position = value
        End Set
        Get
            Return _Position
        End Get
    End Property
#End Region
#Region "Clipplane Class"
    Private Class sdxClipPlaneH
        ''' <summary>
        ''' ClipPlane-Klasse zur Erstellung einer horizontalen Clipping-Ebene
        ''' </summary>
        Public Enum type As Integer
            refraction = 1
            reflection = -1
        End Enum
        Private _plane As Plane
        Private _planetype As Integer

        ''' <summary>
        ''' Erstellt eine neue horizontale Clipping-Ebene
        ''' </summary>
        ''' <param name="distance">Legt die entfernung zum Nullpunkt fest</param>
        Public Sub New(ByVal distance As Single)
            _plane = New Plane(New Vector3(0, -1, 0), distance)
        End Sub

#Region "Properties"
        ''' <summary>
        ''' Aktiviert ClipPlanes um Ausserhalb der Plane zu Clippen oder deaktiviert diese.
        ''' </summary>
        Public WriteOnly Property ClippingEnabled() As Boolean
            Set(ByVal value As Boolean)
                If value = True Then
                    '#enable Plane0
                    dxDevice.SetRenderState(RenderState.ClipPlaneEnable, 1)
                Else
                    '#disable Plane0
                    dxDevice.SetRenderState(RenderState.ClipPlaneEnable, 0)
                End If
            End Set
        End Property
        ''' <summary>
        ''' Legt den Clipplane Typ fest.
        ''' </summary>
        Public WriteOnly Property PlaneType() As type
            Set(ByVal value As type)
                _planetype = value
            End Set
        End Property
        ''' <summary>
        ''' Übergibt die Plane an den Device
        ''' </summary>
        ''' <param name="Transform">Legt fest ob die plane Transformiert wird (Shader --> True)</param>
        Public Sub SetClipplane(Optional ByVal Transform As Boolean = False)

            'These bits could be ORed together to use with D3DRS_CLIPPLANEENABLE
            '
            'D3DCLIPPLANE0 = 1;   // (1 << 0)
            'D3DCLIPPLANE1 = 2;   // (1 << 1)
            'D3DCLIPPLANE2 = 4;   // (1 << 2)
            'D3DCLIPPLANE3 = 8;   // (1 << 3)
            'D3DCLIPPLANE4 = 16;  // (1 << 4)
            'D3DCLIPPLANE5 = 32;  // (1 << 5)


            If Transform Then
                dxDevice.SetClipPlane(0, Plane.Transform(_plane * _planetype, dxWorld.ViewProjection))
            Else
                dxDevice.SetClipPlane(0, _plane * _planetype)
            End If
        End Sub

        ''' <summary>
        ''' Gibt die Plane zurück
        ''' </summary>
        Public Property Distance() As Single
            Set(ByVal value As Single)
                _plane.D = value
            End Set
            Get
                Return _plane.D
            End Get
        End Property
#End Region
    End Class
#End Region

End Class
