﻿Imports SlimDX
Imports SlimDX.Direct3D9
Imports sdxBaseEngine.SDX9BaseEngine

''' <summary>
''' Class zur Erstellung von shaderbasierten Lightning
''' </summary>

Public Class shaderLight
    '#Shader Technique Enum
    Public Enum Technique As Integer
        Colored = 0
        Texture = 1
        TextureAmbSpec = 2
        TextureNormal = 3
        TextureNormalAmbSpec = 4
        TextureNormalDisAmbSpec = 5
    End Enum

    Public Enum LightShading As Integer
        LightBeginn = 0
        DrawingBeginn = 1
        DrawingEnd = 2
        LightEnd = 3
    End Enum
    Public Enum ShadowShading As Integer
        ShadowBeginn = 0
        DrawingBeginn = 1
        DrawingEnd = 2
        ShadowEnd = 3
    End Enum

    Public Event DrawShadowedObjects()
    

    Private _fx As Effect
    Private _fxHandle(18) As EffectHandle
    Private _fxTechnique(6) As EffectHandle
    Private _light As New shaderLight.LightSet
    Private _material As shaderLight.MaterialSet

    'Private _shadowDepthMap As New TextureSurface(New Point(512, 512), Format.R16F)
    'Private _shadowPCFMap As New TextureSurface(New Point(1024, 1024), Format.R16F)

    Private _shadowDepthMap As New TextureSurface(dxEngine.BackBufferSize)
    Private _projectiveTexMatrix As Matrix
    Private _lightview As Matrix
    Private _lightprojection As Matrix
    Private _lightViewProjection As Matrix


#Region "Event Handles"
    Public Sub New()
        AddHandler dxLoadDeviceObjects, AddressOf LoadDeviceObjects
        AddHandler dxDeviceLost, AddressOf DeviceLost
        AddHandler dxDeviceReset, AddressOf DeviceReset
        AddHandler dxEngineClosing, AddressOf Dispose

        Dim TexOffsetX As Single = CSng(0.5 + (0.5 / dxEngine.BackBufferSize.X))
        Dim TexOffsetY As Single = CSng(0.5 + (0.5 / dxEngine.BackBufferSize.Y))

        _projectiveTexMatrix.M11 = 0.5
        _projectiveTexMatrix.M12 = 0
        _projectiveTexMatrix.M13 = 0
        _projectiveTexMatrix.M14 = 0

        _projectiveTexMatrix.M21 = 0
        _projectiveTexMatrix.M22 = -0.5
        _projectiveTexMatrix.M23 = 0
        _projectiveTexMatrix.M24 = 0

        _projectiveTexMatrix.M31 = 0
        _projectiveTexMatrix.M32 = 0
        _projectiveTexMatrix.M33 = 1
        _projectiveTexMatrix.M34 = 0

        _projectiveTexMatrix.M41 = TexOffsetX
        _projectiveTexMatrix.M42 = TexOffsetY
        _projectiveTexMatrix.M43 = 0
        _projectiveTexMatrix.M44 = 1
    End Sub

    Private Sub LoadDeviceObjects()
        dxEngine.LoadingStatus = "Loading Engine: Shader - shaderLight..."
        createShaderHandles()
    End Sub
    Private Sub DeviceLost()
        If Not _fx Is Nothing Then _fx.OnLostDevice()
    End Sub
    Private Sub DeviceReset()
        If Not _fx Is Nothing Then _fx.OnResetDevice()
    End Sub
    Private Sub Dispose()
        If Not _fx Is Nothing Then _fx.Dispose()
        If Not _shadowDepthMap Is Nothing Then _shadowDepthMap.Dispose()
    End Sub
#End Region
#Region "Create ShaderHandles"
    Private Sub createShaderHandles()
        dxEngine.LoadingStatus = "Loading Engine: Shader - shaderLight..."
        _fx = Effect.FromMemory(dxDevice, My.Resources.shaderDirLightShadow, ShaderFlags.PartialPrecision)

        '#Shader Parameter Handles
        _fxHandle(0) = _fx.GetParameter(Nothing, "World")
        _fxHandle(1) = _fx.GetParameter(Nothing, "WorldViewProjection")
        _fxHandle(2) = _fx.GetParameter(Nothing, "Ambient")
        _fxHandle(3) = _fx.GetParameter(Nothing, "Diffuse")
        _fxHandle(4) = _fx.GetParameter(Nothing, "Specular")
        _fxHandle(5) = _fx.GetParameter(Nothing, "AIntensity")
        _fxHandle(6) = _fx.GetParameter(Nothing, "DIntensity")
        _fxHandle(7) = _fx.GetParameter(Nothing, "SIntensity")
        _fxHandle(8) = _fx.GetParameter(Nothing, "Sharpen")
        _fxHandle(9) = _fx.GetParameter(Nothing, "Brightness")
        _fxHandle(10) = _fx.GetParameter(Nothing, "LightDirection")
        _fxHandle(11) = _fx.GetParameter(Nothing, "ColorMap")
        _fxHandle(12) = _fx.GetParameter(Nothing, "NormalMap")
        _fxHandle(13) = _fx.GetParameter(Nothing, "AmbDisSpecMap")
        _fxHandle(14) = _fx.GetParameter(Nothing, "HeightMapScale")
        _fxHandle(15) = _fx.GetParameter(Nothing, "CamPos")

        _fxHandle(16) = _fx.GetParameter(Nothing, "ShadowMap")
        _fxHandle(17) = _fx.GetParameter(Nothing, "WorldLightViewProjection")
        _fxHandle(18) = _fx.GetParameter(Nothing, "ShadowMatrix")

        '#Shader Technique Handles
        _fxTechnique(0) = _fx.GetTechnique("DirLight")
        _fxTechnique(1) = _fx.GetTechnique("DirLightTex")
        _fxTechnique(2) = _fx.GetTechnique("DirLightTexAmbSpec")
        _fxTechnique(3) = _fx.GetTechnique("DirLightTexNrm")
        _fxTechnique(4) = _fx.GetTechnique("DirLightTexNrmAmbSpec")
        _fxTechnique(5) = _fx.GetTechnique("DirLightTexNrmAmbDisSpec")
        _fxTechnique(6) = _fx.GetTechnique("generateShadowDepthMap")

    End Sub
#End Region


    Private Sub ShadowMapBegin()
        '#Settings for Shadowmap
        dxEngine.SurfaceCurrent(Color.White) = _shadowDepthMap.Surface

        _lightview = Matrix.LookAtLH(_light.LightScaledPosition, New Vector3(0, -1, 0), New Vector3(0, 1, 0))
        '_lightprojection = Matrix.OrthoLH(30, 30, -10, 30)

        _lightViewProjection = _lightview * dxCamera.currentProjectionMatrix
        dxCamera.currentViewMatrix = _lightview
        'dxCamera.currentProjectionMatrix = _lightprojection
    End Sub

    Private Sub ShadowDrawBegin()
        _fx.SetValue(_fxHandle(17), dxWorld.World * _lightViewProjection)
        _fx.Technique = _fxTechnique(6)
        _fx.Begin()
        _fx.BeginPass(0)
    End Sub
    Private Sub ShadowDrawEnd()
        _fx.EndPass()
        _fx.End()
    End Sub
    Private Sub ShadowMapEnd()
        '#Debug only
        'Texture.ToFile(_shadowDepthMap.Texture, dxEngine.ApplicationPath & "depthmap.jpg", ImageFileFormat.Jpg)

        dxCamera.currentViewMatrix = dxCamera.ViewMatrix
        dxCamera.currentProjectionMatrix = dxCamera.ProjectionMatrix
        dxEngine.SurfaceChange = EngineSurfaces.DefaultTarget
    End Sub


#Region "Property"
    Public Property Light() As LightSet
        Set(ByVal value As LightSet)
            _light = value
        End Set
        Get
            Return _light
        End Get
    End Property
    Public WriteOnly Property Material() As MaterialSet
        Set(ByVal value As MaterialSet)
            _material = value
        End Set
    End Property
    Public WriteOnly Property Shadowing() As ShadowShading
        Set(ByVal value As ShadowShading)
            Select Case value
                Case Is = ShadowShading.ShadowBeginn
                    ShadowMapBegin()
                Case Is = ShadowShading.ShadowEnd
                    ShadowMapEnd()
                Case Is = ShadowShading.DrawingBeginn
                    ShadowDrawBegin()
                Case Is = ShadowShading.DrawingEnd
                    ShadowDrawEnd()
            End Select
        End Set
    End Property

    Public WriteOnly Property Lighting() As LightShading
        Set(ByVal value As LightShading)
            Select Case value
                Case Is = LightShading.LightBeginn
                    LightBeginn()
                Case Is = LightShading.LightEnd
                    _fx.End()
                Case Is = LightShading.DrawingBeginn
                    If _material.SpecularIntensity > 0 Then
                        _fx.BeginPass(0)
                    Else
                        _fx.BeginPass(1)
                    End If
                Case Is = LightShading.DrawingEnd
                    _fx.EndPass()
            End Select
        End Set
    End Property


#End Region
#Region "LightBeginn"
    Private Sub LightBeginn()
        _fx.SetValue(_fxHandle(0), dxWorld.World)
        _fx.SetValue(_fxHandle(1), dxWorld.WorldViewProjection)
        _fx.SetValue(_fxHandle(2), _light.Ambient)
        _fx.SetValue(_fxHandle(3), _light.Diffuse)
        _fx.SetValue(_fxHandle(4), _material.Specular)
        _fx.SetValue(_fxHandle(5), _light.AmbientIntensity)
        _fx.SetValue(_fxHandle(6), _light.DiffuseIntensity)
        _fx.SetValue(_fxHandle(7), _material.SpecularIntensity)
        _fx.SetValue(_fxHandle(8), _material.SpecularSharpen)
        _fx.SetValue(_fxHandle(9), _material.Brightness)
        _fx.SetValue(_fxHandle(10), _light.LightDirection)
        _fx.SetTexture(_fxHandle(11), _material.MapColor)
        _fx.SetTexture(_fxHandle(12), _material.MapNormal)
        _fx.SetTexture(_fxHandle(13), _material.MapAmbDisSpec)
        _fx.SetValue(_fxHandle(14), _material.DisplacementHeight)
        _fx.SetValue(_fxHandle(15), dxCamera.currentPosition)
        _fx.SetTexture(_fxHandle(16), _shadowDepthMap.Texture)
        _fx.SetValue(_fxHandle(17), dxWorld.World * _lightViewProjection)
        _fx.SetValue(_fxHandle(18), dxWorld.World * _lightViewProjection * _projectiveTexMatrix)

        _fx.Technique = _fxTechnique(_material.TextureTechnique)
        _fx.Begin()
    End Sub
#End Region

#Region "Class LightSet"
    ''' <summary>
    ''' Legt die Lichteigenschaften des Shaderlights fest.
    ''' </summary>
    Public Class LightSet
        Private _ambient As New RGBAcolor(1.0)
        Private _diffuse As New RGBAcolor(1.0)
        Private _aIntensity As Single = 0.2
        Private _dIntensity As Single = 1
        Private _lightdirection As Vector3
        Private _lightposition As Vector3

        ''' <summary>
        ''' Legt die Richtung des Lichtes fest.
        ''' Eigenschaft für Directionallight.
        ''' </summary>
        Public Property LightDirection() As Vector3
            Set(ByVal value As Vector3)
                _lightdirection = value * -1
            End Set
            Get
                Return _lightdirection
            End Get
        End Property
        ''' <summary>
        ''' Legt die skalierte Position des Lichtes für Shadowmapping fest
        ''' </summary>
        Public Property LightScaledPosition() As Vector3
            Set(ByVal value As Vector3)
                _lightposition = value
            End Set
            Get
                Return _lightposition
            End Get
        End Property
        ''' <summary>
        '''  Legt die Ambientfarbe fest.
        ''' </summary>
        Public Property Ambient() As RGBAcolor
            Set(ByVal value As RGBAcolor)
                _ambient = value
            End Set
            Get
                Return _ambient
            End Get
        End Property
        ''' <summary>
        ''' Legt die Diffusefarbe fest.
        ''' </summary>
        ''' <value></value>
        ''' <remarks></remarks>
        Public Property Diffuse() As RGBAcolor
            Set(ByVal value As RGBAcolor)
                _diffuse = value
            End Set
            Get
                Return _diffuse
            End Get
        End Property


        ''' <summary>
        ''' Legt die Intensität der Ambientfarbe fest
        ''' </summary>
        Public Property AmbientIntensity() As Single
            Set(ByVal value As Single)
                _aIntensity = value
            End Set
            Get
                Return _aIntensity
            End Get
        End Property
        ''' <summary>
        ''' Legt die Intensität Diffusefarbe fest
        ''' </summary>
        Public Property DiffuseIntensity() As Single
            Set(ByVal value As Single)
                _dIntensity = value
            End Set
            Get
                Return _dIntensity
            End Get
        End Property
    End Class
#End Region
#Region "Class Material"
    ''' <summary>
    ''' Legt die Materialeigenschaft eines Objekts fest
    ''' </summary>
    ''' <remarks></remarks>
    Public Class MaterialSet

        Private _colormap As Texture
        Private _normalmap As Texture
        Private _ambdisspecmap As Texture
        Private _heightmapscale As Single = 0
        Private _specular As RGBAcolor = New RGBAcolor(1.0)
        Private _sIntensity As Single = 0
        Private _spower As Single = 50
        Private _lightpower As Single = 1
        Private _lighttechnique As Technique = 0


#Region "Event Handles"
        Public Sub New()
            AddHandler dxDeviceLost, AddressOf DeviceLost
            AddHandler dxEngineClosing, AddressOf Dispose
        End Sub

        Private Sub DeviceLost()
            If Not _colormap Is Nothing Then _colormap.Dispose()
            If Not _normalmap Is Nothing Then _normalmap.Dispose()
            If Not _ambdisspecmap Is Nothing Then _ambdisspecmap.Dispose()
        End Sub
        Private Sub Dispose()
            If Not _colormap Is Nothing Then _colormap.Dispose()
            If Not _normalmap Is Nothing Then _normalmap.Dispose()
            If Not _ambdisspecmap Is Nothing Then _ambdisspecmap.Dispose()
        End Sub
#End Region



        Public Property MapColor() As Texture
            Get
                Return _colormap
            End Get
            Set(ByVal value As Texture)
                _colormap = value
            End Set
        End Property

        Public Property MapNormal() As Texture
            Get
                Return _normalmap
            End Get
            Set(ByVal value As Texture)
                _normalmap = value
            End Set
        End Property

        ''' <summary>
        ''' R = AmbientOclusion
        ''' G = Specular
        ''' </summary>
        Public Property MapAmbDisSpec() As Texture
            Get
                Return _ambdisspecmap
            End Get
            Set(ByVal value As Texture)
                _ambdisspecmap = value
            End Set
        End Property

        Public Property DisplacementHeight() As Single
            Get
                Return _heightmapscale
            End Get
            Set(ByVal value As Single)
                _heightmapscale = value
            End Set
        End Property

        Public Property Specular() As RGBAcolor
            Get
                Return _specular
            End Get
            Set(ByVal value As RGBAcolor)
                _specular = value
            End Set
        End Property
        ''' <summary>
        ''' Legt die Stärke der specularen Reflection fest (default = 0)
        ''' </summary>
        ''' <value></value>
        Public Property SpecularIntensity() As Single
            Get
                Return _sIntensity
            End Get
            Set(ByVal value As Single)
                _sIntensity = value
            End Set
        End Property

        Public Property SpecularSharpen() As Single
            Get
                Return _spower
            End Get
            Set(ByVal value As Single)
                _spower = value
            End Set
        End Property

        Public Property Brightness() As Single
            Get
                Return _lightpower
            End Get
            Set(ByVal value As Single)
                _lightpower = value
            End Set
        End Property
        Public Property TextureTechnique() As Technique
            Get
                Return _lighttechnique
            End Get
            Set(ByVal value As Technique)
                _lighttechnique = value
            End Set
        End Property
    End Class
#End Region
End Class
