﻿Imports SlimDX
Imports SlimDX.Direct3D9
Imports sdxBaseEngine.SDX9BaseEngine

Public Class shaderPostEffect
    Private _fx As Effect
    Private _fxHandle(0) As EffectHandle
    Private _fxTechnique(2) As EffectHandle
    Private _tex As Texture
    Private _technique As Integer


    Public Enum EffectType As Integer
        ''' <summary>
        ''' Textur wird ohne Effekt gerendert.
        ''' </summary>
        ''' <remarks></remarks>
        Bypass = 0
        ''' <summary>
        ''' Die Textur wird per Alpha addiert
        ''' </summary>
        ''' <remarks></remarks>
        AlphaAdd = 1
        ''' <summary>
        ''' Die Textur wird per Alpha addiert.
        ''' Eine Fogfarbe kann festgelegt werden.
        ''' </summary>
        ''' <remarks></remarks>
        AlphaFog = 2
    End Enum

#Region "Event Handles"
    Public Sub New()
        AddHandler dxLoadDeviceObjects, AddressOf LoadDeviceObjects
        AddHandler dxEngineClosing, AddressOf Dispose
        AddHandler dxDeviceLost, AddressOf DeviceLost
        AddHandler dxDeviceReset, AddressOf DeviceReset
    End Sub
    Private Sub DeviceLost()
        _fx.OnLostDevice()
    End Sub
    Private Sub DeviceReset()
        _fx.OnResetDevice()
    End Sub
    Private Sub Dispose()
        _fx.Dispose()
        For i As Integer = 0 To _fxHandle.Length - 1
            _fxHandle(i).Dispose()
        Next
        For i As Integer = 0 To _fxTechnique.Length - 1
            _fxTechnique(i).Dispose()
        Next
        If Not _tex Is Nothing Then _tex.Dispose()
    End Sub

    Public Sub LoadDeviceObjects()
        createShaderHandles()
    End Sub
#End Region
#Region "Create ShaderHandles"
    Private Sub createShaderHandles()
        '//Effect
        _fx = Effect.FromMemory(dxDevice, My.Resources.shaderPosteffect, ShaderFlags.PartialPrecision)

        '#Shader Parameter Handles
        _fxHandle(0) = _fx.GetParameter(Nothing, "ColorMap0")

        '//EffectTechnique
        _fxTechnique(0) = _fx.GetTechnique("Bypass")
        _fxTechnique(1) = _fx.GetTechnique("AlphaAdd")
        _fxTechnique(2) = _fx.GetTechnique("AlphaFog")
    End Sub
#End Region

#Region "Property"
    Public WriteOnly Property SetEffect() As EffectType
        Set(ByVal value As EffectType)
            _technique = value
        End Set
    End Property
    ''' <summary>
    ''' Legt die PostEffect Textur fest.
    ''' </summary>
    Public WriteOnly Property ColorMap0() As Texture
        Set(ByVal value As Texture)
            _tex = value
        End Set
    End Property
#End Region
#Region "## Render ##"
    Public Sub Draw()
        _fx.Technique = _fxTechnique(_technique)
        _fx.SetTexture(_fxHandle(0), _tex)
        _fx.Begin()
        _fx.BeginPass(0)
        dxVBuffer.FullScreen.Draw()
        _fx.EndPass()
        _fx.End()
    End Sub
#End Region


End Class
