﻿


Imports SlimDX
Imports SlimDX.Direct3D9
Imports sdxBaseEngine
Imports sdxBaseEngine.SDX9BaseEngine


Public Class shaderSkydome

    '#Dome
    Private _tex1 As TextureSurface
    Private _skydome As Mesh
    Private _posY As Single
    Private _skydomeposition As Vector3
    Private _skydomescaling As Vector3 = New Vector3(4, 2, 4)
    Private _topcolor As RGBAcolor = New RGBAcolor(153, 179, 235)
    Private _groundcolor As RGBAcolor = New RGBAcolor(1.0)
    Private _skyblendpoint As Single = 1
    Private _groundblendpoint As Single = 1

    '#Clouds
    Private _noisemap As Texture
    Private _speed As Single = 0.00015
    Private _direction As Vector2 = New Vector2(0.5, 0.5)
    Private _colorR As Single = 1
    Private _colorG As Single = 0.8
    Private _colorB As Single = 1
    Private _shadow As Single = 0.8
    Private _density As Single = 0.329
    Private _thickness As Single = 0.392
    Private _brightness As Single = 7.7
    Private _distance As Single = 1.334

    '#noisemap
    Private bmp As Bitmap

    'Camera 
    Private _camclip As Boolean
    Private _camclipheight As Single

    'Shader
    Private _fx As Effect
    Private _fxHandle(18) As EffectHandle
    Private _fxTechnique(0) As EffectHandle

    ''' <summary>
    ''' Erstellt eine neue Skydome-Classe
    ''' </summary>
    ''' <param name="size">Legt die Pixelgröße der NoiseMap fest</param>

#Region "Event Handles"
    Public Sub New(Optional ByVal size As Integer = 32)
        AddHandler dxLoadDeviceObjects, AddressOf LoadDeviceObjects
        AddHandler dxEngineClosing, AddressOf Dispose
        AddHandler dxDeviceLost, AddressOf DeviceLost
        AddHandler dxDeviceReset, AddressOf DeviceReset
        createnoisemap(size)
    End Sub
    Private Sub DeviceLost()
        _noisemap.Dispose()
        _fx.OnLostDevice()
    End Sub
    Private Sub DeviceReset()
        If _skydome Is Nothing Then _skydome = Mesh.FromMemory(dxDevice, My.Resources.skydome, MeshFlags.Managed)
        If _noisemap Is Nothing Then _noisemap = Texture.FromMemory(dxDevice, BitmapToByteBuffer(bmp))
        _fx.OnResetDevice()
    End Sub
    Private Sub Dispose()
        _skydome.Dispose()
        _noisemap.Dispose()
        _fx.Dispose()
        For i As Integer = 0 To _fxHandle.Length - 1
            _fxHandle(i).Dispose()
        Next
        For i As Integer = 0 To _fxTechnique.Length - 1
            _fxTechnique(i).Dispose()
        Next
    End Sub

    Private Sub LoadDeviceObjects()
        '//Skydome
        dxEngine.LoadingStatus = "Loading Engine: Shader - shaderSkydome"
        createShaderHandles()
        _skydome = Mesh.FromMemory(dxDevice, My.Resources.skydome, MeshFlags.Managed)
        _noisemap = Texture.FromMemory(dxDevice, BitmapToByteBuffer(bmp))
    End Sub
#End Region


#Region "Create Noisemap"
    Private Sub createnoisemap(ByVal size As Integer)
        Dim rand As New Random
        bmp = New Bitmap(size, size)
        Dim c As Color
        Dim r(3) As Integer
        For x = 0 To size - 1
            For y = 0 To size - 1
                r(0) = rand.Next(0, 255)
                r(1) = rand.Next(90, 170)
                r(2) = rand.Next(50, 150)
                r(3) = rand.Next(20, 100)
                c = Color.FromArgb(r(0), r(1), r(2), r(3))
                bmp.SetPixel(x, y, c)
            Next
        Next
    End Sub
#End Region
#Region "Create ShaderHandles"
    Private Sub createShaderHandles()
        '//Effect
        If _fx Is Nothing Then

            _fx = Effect.FromMemory(dxDevice, My.Resources.shaderSkydome, ShaderFlags.PartialPrecision)
            '#Shader Parameter Handles
            _fxHandle(0) = _fx.GetParameter(Nothing, "World")
            _fxHandle(1) = _fx.GetParameter(Nothing, "WorldViewProjection")
            _fxHandle(2) = _fx.GetParameter(Nothing, "NoiseTexture")
            _fxHandle(3) = _fx.GetParameter(Nothing, "CamPos")
            _fxHandle(4) = _fx.GetParameter(Nothing, "GameTime")
            _fxHandle(5) = _fx.GetParameter(Nothing, "TopColor")
            _fxHandle(6) = _fx.GetParameter(Nothing, "GroundColor")
            _fxHandle(7) = _fx.GetParameter(Nothing, "SkyBlendPoint")
            _fxHandle(8) = _fx.GetParameter(Nothing, "GroundBlendPoint")
            _fxHandle(9) = _fx.GetParameter(Nothing, "Speed")
            _fxHandle(10) = _fx.GetParameter(Nothing, "Direction")
            _fxHandle(11) = _fx.GetParameter(Nothing, "CloudColorR")
            _fxHandle(12) = _fx.GetParameter(Nothing, "CloudColorG")
            _fxHandle(13) = _fx.GetParameter(Nothing, "CloudColorB")
            _fxHandle(14) = _fx.GetParameter(Nothing, "CloudBrightness")
            _fxHandle(15) = _fx.GetParameter(Nothing, "CloudDensity")
            _fxHandle(16) = _fx.GetParameter(Nothing, "CloudThickness")
            _fxHandle(17) = _fx.GetParameter(Nothing, "CloudShadow")
            _fxHandle(18) = _fx.GetParameter(Nothing, "CloudDistance")

            '//EffectTechnique
            _fxTechnique(0) = _fx.GetTechnique("Skydome")
        End If
    End Sub
#End Region

#Region "Draw"
    ''' <summary>
    ''' Rendert das Skydome
    ''' </summary>
    Public Sub Draw()

        '#Clips all
        If _camclip Then
            If dxCamera.currentTarget.Y < _camclipheight Then Exit Sub
        End If

        '#Skydome Position
        _skydomeposition = dxCamera.currentPosition

        '#Shader settigs
        dxTransformWorld = Matrix.Scaling(_skydomescaling) * Matrix.Translation(_skydomeposition)
        _fx.SetValue(_fxHandle(0), dxWorld.World)
        _fx.SetValue(_fxHandle(1), dxWorld.WorldViewProjection)
        _fx.SetTexture(_fxHandle(2), _noisemap)
        _fx.SetValue(_fxHandle(3), dxCamera.currentPosition)
        _fx.SetValue(_fxHandle(4), dxGametime.Gametime)
        _fx.SetValue(_fxHandle(5), _topcolor)
        _fx.SetValue(_fxHandle(6), _groundcolor)
        _fx.SetValue(_fxHandle(7), _skyblendpoint)
        _fx.SetValue(_fxHandle(8), _groundblendpoint)
        _fx.SetValue(_fxHandle(9), _speed)
        _fx.SetValue(_fxHandle(10), _direction)
        _fx.SetValue(_fxHandle(11), _colorR)
        _fx.SetValue(_fxHandle(12), _colorG)
        _fx.SetValue(_fxHandle(13), _colorB)
        _fx.SetValue(_fxHandle(14), _brightness)
        _fx.SetValue(_fxHandle(15), _density)
        _fx.SetValue(_fxHandle(16), _thickness)
        _fx.SetValue(_fxHandle(17), _shadow)
        _fx.SetValue(_fxHandle(18), _distance)

        '#Draw
        _fx.Begin()
        _fx.BeginPass(0)
        _skydome.DrawSubset(0)
        _fx.EndPass()
        _fx.End()

    End Sub
   
#End Region

#Region "Property"


    ''' <summary>
    ''' Legt die Skalierung des Domes fest
    '''  (default 4) 
    ''' </summary>
    ''' <remarks></remarks>
    Public WriteOnly Property Scaling() As Vector3
        Set(ByVal value As Vector3)
            _skydomescaling = value
        End Set
    End Property

    ''' <summary>
    ''' Legt die Wolkengeschwindigkeit fest.
    '''  (default 1)
    ''' </summary>
    Public WriteOnly Property CloudSpeed() As Single
        Set(ByVal value As Single)
            _speed = CSng(value * 0.0001)
        End Set
    End Property
    ''' <summary>
    ''' Legt die Wolkenrichtung fest (xz default 1,0).
    ''' </summary>
    Public WriteOnly Property CloudDirection() As Vector2
        Set(ByVal value As Vector2)
            _direction = value
        End Set
    End Property
    ''' <summary>
    ''' Legt die Wolkenfarbe fest R-Kanal.
    ''' </summary>
    Public Property CloudColorR() As Single
        Set(ByVal value As Single)
            _colorR = value
        End Set
        Get
            Return _colorR
        End Get
    End Property
    ''' <summary>
    ''' Legt die Wolkenfarbe fest G-Kanal.
    ''' </summary>
    Public Property CloudColorG() As Single
        Set(ByVal value As Single)
            _colorG = clamp(value, 0, 1)
        End Set
        Get
            Return _colorG
        End Get
    End Property
    ''' <summary>
    ''' Legt die Wolkenfarbe fest B-Kanal.
    ''' </summary>
    Public Property CloudColorB() As Single
        Set(ByVal value As Single)
            _colorB = clamp(value, 0, 1)
        End Set
        Get
            Return _colorB
        End Get
    End Property
    ''' <summary>
    ''' Legt die TopColor des Himmels fest.
    ''' </summary>
    Public WriteOnly Property SkyTopColor() As RGBAcolor
        Set(ByVal value As RGBAcolor)
            _topcolor = value
        End Set
    End Property
    ''' <summary>
    ''' Legt die Bodenfarbe fest.
    ''' </summary>
    Public WriteOnly Property GroundColor() As RGBAcolor
        Set(ByVal value As RGBAcolor)
            _groundcolor = value
        End Set
    End Property
    ''' <summary>
    ''' Legt die Höhe des Farbverlaufes des Himmels fest (0-2).
    ''' </summary>
    Public WriteOnly Property SkyBlendPoint() As Single
        Set(ByVal value As Single)
            _skyblendpoint = clamp(value, 0, 2)
        End Set
    End Property
    ''' <summary>
    ''' Legt die Höhe des Farbverlaufes des Bodens fest (0-1).
    ''' </summary>
    Public WriteOnly Property GroundBlendPoint() As Single
        Set(ByVal value As Single)
            _groundblendpoint = clamp(value, 0, 1)
        End Set
    End Property
    ''' <summary>
    ''' Legt die Y Position des Domes fest
    ''' </summary>
    Public WriteOnly Property PositionY() As Single
        Set(ByVal value As Single)
            _posY = value
        End Set
    End Property
    ''' <summary>
    ''' Legt fest ob CamClipping aktiviert wird
    ''' </summary>
    Public WriteOnly Property CamClip() As Boolean
        Set(ByVal value As Boolean)
            _camclip = value
        End Set
    End Property
    ''' <summary>
    ''' Legt die Clippinghöhe fest
    ''' </summary>
    Public WriteOnly Property CamClipHeight() As Single
        Set(ByVal value As Single)
            _camclipheight = value
        End Set
    End Property
    ''' <summary>
    ''' Legt die Leuchtdichte der Wolken fest (1-15)
    ''' </summary>
    Public Property CloudBrightness() As Single
        Get
            Return _brightness
        End Get
        Set(ByVal value As Single)
            _brightness = clamp(value, 1, 15)
        End Set
    End Property
    ''' <summary>
    ''' Legt die Dicke der Wolken fest (0-3)
    ''' </summary>
    Public Property CloudThickness() As Single
        Get
            Return _thickness
        End Get
        Set(ByVal value As Single)
            _thickness = clamp(value, 0, 1)
        End Set
    End Property
    ''' <summary>
    ''' Legt die Schattenstärke der Wolken fest (0-5)
    ''' </summary>
    Public Property CloudShadow() As Single
        Get
            Return _shadow
        End Get
        Set(ByVal value As Single)
            _shadow = clamp(value, 0, 5)
        End Set
    End Property
    ''' <summary>
    ''' Legt die Dichte der Wolkendecke fest (0 - 1)
    ''' </summary>
    Public Property CloudDensity() As Single
        Get
            Return _density
        End Get
        Set(ByVal value As Single)
            _density = clamp(value, 0, 1)
        End Set
    End Property
    ''' <summary>
    ''' Legt die Dichte der Wolkendecke fest (1 - 2)
    ''' </summary>
    Public Property CloudDistance() As Single
        Get
            Return _distance
        End Get
        Set(ByVal value As Single)
            _distance = clamp(value, 1, 2)
        End Set
    End Property

    Public ReadOnly Property InfoText() As String
        Get
            Return vbCrLf & _
                   "Cloud Shadow" & vbCrLf & _
                   "Cloud Densitiy" & vbCrLf & _
                   "Cloud Distance" & vbCrLf & _
                   "Cloud Thickness" & vbCrLf & _
                   "Cloud Color RGB" & vbCrLf & _
                   "Cloud Brightness"
        End Get
    End Property

    Public ReadOnly Property InfoData() As String
        Get
            Return vbCrLf & _
                   ":  " & _shadow.ToString & vbCrLf & _
                   ":  " & _density.ToString & vbCrLf & _
                   ":  " & _distance.ToString & vbCrLf & _
                   ":  " & _thickness.ToString & vbCrLf & _
                   ":  " & _colorR.ToString & " - " & _colorG.ToString & " - " & _colorB.ToString & vbCrLf & _
                   ":  " & _brightness.ToString
        End Get
    End Property
#End Region

End Class
