﻿' Title    : sdxBaseEngine Class
' Name     : shaderSun
' Author   : Sascha Bajus
' Created  : 01/2010
' Copyright © 2008 Sascha Bajus

#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region

Imports SlimDX
Imports SlimDX.Direct3D9
Imports sdxBaseEngine.SDX9BaseEngine

Public Class shaderSun
    Private _vbuffer As VertexBuffer            '#VertexBuffer
    Private _vertexdecSizeOfBytes As Integer
    Private _vertexdec As VertexDeclaration     '#VertexDeclaration
    Private _texture(1) As Texture              '#Texture
    Private _position As Vector3


    Private _fx As Effect
    Private _fxHandle(6) As EffectHandle
    Private _fxTechnique(2) As EffectHandle
    Private _Occlusion As Query
    Private _OcclusionPixel As Integer
    Private _OcclusionValue As Single
    Private _maxPixel As Integer = 1
    Private _valPixel As Integer
    Private _SunPos As Vector3
    Private _sunScale As Vector3 = New Vector3(1.4, 1.4, 1.4)
    Private _FlareIntensity As Single = 1
    Private _FlareVec As Vector3
    Private _FlareScale As Vector3
    Private _ScreenScale As Single
    Private _testHighestValue As Boolean = True

    Private Structure _VertexStructure
        Public Position As Vector3
        Public TexCoords As Vector2
    End Structure

#Region "Event Handles"
    Public Sub New()
        AddHandler dxLoadDeviceObjects, AddressOf LoadDeviceObjects
        AddHandler dxEngineClosing, AddressOf Dispose
        AddHandler dxDeviceLost, AddressOf DeviceLost
        AddHandler dxDeviceReset, AddressOf DeviceReset
    End Sub

    Private Sub LoadDeviceObjects()
        dxEngine.LoadingStatus = "Loading Engine: Shader - shaderSun"

        '# *ScreenScale*
        '# um die Größe der Flares an der Bildschirmauflösung
        '# anzupassen.
        '# Die Flares sind ausgerichtet bei 640x480

        _ScreenScale = CSng(640 / dxEngine.BackBufferSize.X) * 4 / 3
        _ScreenScale = clamp(_ScreenScale, 0, 1)

        '# *MaxValue*
        '# Der maximle Pixelwert fürs Occlusion kann berechnet werden.
        '# Den PixelValue ablesen wenn im Center gerendert wird.
        '# Dies ist der maximal zulässige Pixelwert der prozentual zur 
        '# Auflösung umgerechnet werden muss.
        '# 
        '# Auflösung:   640x480 = 307.020 Pixel
        '# MaxValue:    3900
        '# Prozentwert: 1,27
        '#
        '# der Prozentwert kann beim rendern der Info abgelesen werden.

        Me.FlareScale = 0.4
        createShaderHandles()
        createVertexDeclaration()
        createVertexbuffer()
        fillVertexbuffer()

        _Occlusion = New Query(dxDevice, QueryType.Occlusion)
        _texture(0) = dxTexture.FromImage(My.Resources.Sun, Color.Black)
        _texture(1) = dxTexture.FromImage(My.Resources.Flares, Color.Black)
    End Sub

    Private Sub DeviceLost()
        _fx.OnLostDevice()
        _Occlusion.Dispose()
        _vbuffer.Dispose()
        _texture(0).Dispose()
        _texture(1).Dispose()
    End Sub
    Private Sub DeviceReset()
        _fx.OnResetDevice()
        _Occlusion = New Query(dxDevice, QueryType.Occlusion)
        createVertexbuffer()
        fillVertexbuffer()
        _texture(0) = dxTexture.FromImage(My.Resources.Sun, Color.Black)
        _texture(1) = dxTexture.FromImage(My.Resources.Flares, Color.Black)
    End Sub
    Private Sub Dispose()

        For i As Integer = 0 To _fxHandle.Length - 1
            _fxHandle(i).Dispose()
        Next
        For i As Integer = 0 To _fxTechnique.Length - 1
            _fxTechnique(i).Dispose()
        Next

        If Not _fx Is Nothing Then _fx.Dispose()
        If Not _texture(0) Is Nothing Then _texture(0).Dispose()
        If Not _texture(1) Is Nothing Then _texture(1).Dispose()
        If Not _Occlusion Is Nothing Then _Occlusion.Dispose()
        If Not _vertexdec Is Nothing Then _vertexdec.Dispose()
        If Not _vbuffer Is Nothing Then _vbuffer.Dispose()
    End Sub

    
#End Region


#Region "Create ShaderHandles"
    Private Sub createShaderHandles()
        '//Effect
        _fx = Effect.FromMemory(dxDevice, My.Resources.shaderSun, ShaderFlags.PartialPrecision)

        '#Shader Parameter Handles
        _fxHandle(0) = _fx.GetParameter(Nothing, "WorldViewProjection")
        _fxHandle(1) = _fx.GetParameter(Nothing, "FlareVec")
        _fxHandle(2) = _fx.GetParameter(Nothing, "FlareOcclusion")
        _fxHandle(3) = _fx.GetParameter(Nothing, "FlareIntensity")
        _fxHandle(4) = _fx.GetParameter(Nothing, "FlareSize")
        _fxHandle(5) = _fx.GetParameter(Nothing, "ScreenFactor")
        _fxHandle(6) = _fx.GetParameter(Nothing, "ColorMap0")

        '//EffectTechnique
        _fxTechnique(0) = _fx.GetTechnique("Sun")
        _fxTechnique(1) = _fx.GetTechnique("Occlusion")
        _fxTechnique(2) = _fx.GetTechnique("Flares")

    End Sub
#End Region
#Region "Create Vertexdeclaration"
    Private Sub createVertexDeclaration()
        Dim elements(2) As VertexElement
        elements(0) = New VertexElement(0, 0, DeclarationType.Float3, DeclarationMethod.Default, DeclarationUsage.Position, 0)
        elements(1) = New VertexElement(0, 12, DeclarationType.Float2, DeclarationMethod.Default, DeclarationUsage.TextureCoordinate, 0)
        elements(2) = VertexElement.VertexDeclarationEnd
        _vertexdec = New VertexDeclaration(dxDevice, elements)
        _vertexdecSizeOfBytes = 12 + 8
    End Sub
#End Region
#Region "Create VertexBuffer"
    Private Sub createVertexbuffer()
        '#create vbuffer 
        _vbuffer = New VertexBuffer(dxDevice, _vertexdecSizeOfBytes * 6 * 14, Usage.None, VertexFormat.None, Pool.Default)
    End Sub
#End Region
#Region "Fill Vertexbuffer"
    Private Sub fillVertexbuffer()

        Dim pos As New List(Of Vector3)
        Dim tex As New List(Of Vector2)
        Dim x As Single
        Dim y As Single
        Dim z As Single

        Dim texy As Single = 1 / 8

        x = 1
        y = 1
        z = 0

        For i = 0 To 13
            pos.Add(New Vector3(-x, y, z))
            pos.Add(New Vector3(x, y, z))
            pos.Add(New Vector3(-x, -y, z))
            pos.Add(New Vector3(x, y, z))
            pos.Add(New Vector3(x, -y, z))
            pos.Add(New Vector3(-x, -y, z))

            tex.Add(New Vector2(0, 0))
            tex.Add(New Vector2(1, 0))
            tex.Add(New Vector2(0, 1))
            tex.Add(New Vector2(1, 0))
            tex.Add(New Vector2(1, 1))
            tex.Add(New Vector2(0, 1))
        Next


        Dim length As Integer = 6 * 14 - 1
        Dim vertex(length) As _VertexStructure

        For i = 0 To 5
            vertex(i).Position = pos(i)
            vertex(i).TexCoords = tex(i)
        Next
        For i = 6 To 11
            vertex(i).Position = pos(i)
            vertex(i).TexCoords = tex(i)
            vertex(i).TexCoords.Y = vertex(i).TexCoords.Y * 4 * texy + 2 * texy
        Next
        For i = 12 To 17
            vertex(i).Position = pos(i)
            vertex(i).TexCoords = tex(i)
            vertex(i).Position.Z = 1
            vertex(i).TexCoords.X = vertex(i).TexCoords.X * CSng(0.25)
            vertex(i).TexCoords.Y = vertex(i).TexCoords.Y * texy
        Next
        For i = 18 To 23
            vertex(i).Position = pos(i)
            vertex(i).TexCoords = tex(i)
            vertex(i).Position.Z = 2
            vertex(i).TexCoords.X = vertex(i).TexCoords.X * CSng(0.25) + CSng(0.25)
            vertex(i).TexCoords.Y = vertex(i).TexCoords.Y * texy
        Next
        For i = 24 To 29
            vertex(i).Position = pos(i)
            vertex(i).TexCoords = tex(i)
            vertex(i).Position.Z = 3
            vertex(i).TexCoords.X = vertex(i).TexCoords.X * CSng(0.25) + CSng(0.5)
            vertex(i).TexCoords.Y = vertex(i).TexCoords.Y * texy
        Next
        For i = 30 To 35
            vertex(i).Position = pos(i)
            vertex(i).TexCoords = tex(i)
            vertex(i).Position.Z = 4
            vertex(i).TexCoords.X = vertex(i).TexCoords.X * CSng(0.25) + CSng(0.75)
            vertex(i).TexCoords.Y = vertex(i).TexCoords.Y * texy
        Next
        For i = 36 To 41
            vertex(i).Position = pos(i)
            vertex(i).TexCoords = tex(i)
            vertex(i).Position.Z = 5
            vertex(i).TexCoords.X = vertex(i).TexCoords.X * CSng(0.25)
            vertex(i).TexCoords.Y = vertex(i).TexCoords.Y * texy + texy
        Next
        For i = 42 To 47
            vertex(i).Position = pos(i)
            vertex(i).TexCoords = tex(i)
            vertex(i).Position.Z = 6
            vertex(i).TexCoords.X = vertex(i).TexCoords.X * CSng(0.25)
            vertex(i).TexCoords.Y = vertex(i).TexCoords.Y * texy + texy
        Next
        For i = 48 To 53
            vertex(i).Position = pos(i)
            vertex(i).TexCoords = tex(i)
            vertex(i).Position.Z = 7
            vertex(i).TexCoords.X = vertex(i).TexCoords.X * CSng(0.25) + CSng(0.25)
            vertex(i).TexCoords.Y = vertex(i).TexCoords.Y * texy + texy
        Next
        For i = 54 To 59
            vertex(i).Position = pos(i)
            vertex(i).TexCoords = tex(i)
            vertex(i).Position.Z = 8
            vertex(i).TexCoords.X = vertex(i).TexCoords.X * CSng(0.25) + CSng(0.25)
            vertex(i).TexCoords.Y = vertex(i).TexCoords.Y * texy + texy
        Next
        For i = 60 To 65
            vertex(i).Position = pos(i)
            vertex(i).TexCoords = tex(i)
            vertex(i).Position.Z = 9
            vertex(i).TexCoords.X = vertex(i).TexCoords.X * CSng(0.25) + CSng(0.75)
            vertex(i).TexCoords.Y = vertex(i).TexCoords.Y * texy
        Next
        For i = 66 To 71
            vertex(i).Position = pos(i)
            vertex(i).TexCoords = tex(i)
            vertex(i).Position.Z = 10
            vertex(i).TexCoords.X = vertex(i).TexCoords.X * CSng(0.25) + CSng(0.5)
            vertex(i).TexCoords.Y = vertex(i).TexCoords.Y * texy + texy
        Next
        For i = 72 To 77
            vertex(i).Position = pos(i)
            vertex(i).TexCoords = tex(i)
            vertex(i).Position.Z = 11
            vertex(i).TexCoords.X = vertex(i).TexCoords.X * CSng(0.25) + CSng(0.75)
            vertex(i).TexCoords.Y = vertex(i).TexCoords.Y * texy + texy
        Next
        For i = 78 To 83
            vertex(i).Position = pos(i)
            vertex(i).TexCoords = tex(i)
            vertex(i).Position.Z = 12
            vertex(i).TexCoords.X = vertex(i).TexCoords.X * CSng(0.25) + CSng(0.75)
            vertex(i).TexCoords.Y = vertex(i).TexCoords.Y * texy + texy
        Next

        Dim stream As DataStream = _vbuffer.Lock(0, 0, LockFlags.None)
        stream.WriteRange(vertex)
        _vbuffer.Unlock()
        stream.Close()
        stream.Dispose()

    End Sub
#End Region


#Region "Property"
    ''' <summary>
    ''' Legt die Position der Sonne fest
    ''' </summary>
    ''' <value></value>
    Public WriteOnly Property SunPosition() As Vector3
        Set(ByVal value As Vector3)
            _position = value
        End Set
    End Property
    ''' <summary>
    ''' Legt die Skalierung der Sonne im Worldspace fest
    ''' </summary>
    ''' <value></value>
    Public WriteOnly Property SunScale() As Single
        Set(ByVal value As Single)
            _sunScale = New Vector3(value)
        End Set
    End Property
    Public WriteOnly Property FlareIntensity() As Single
        Set(ByVal value As Single)
            _FlareIntensity = value
        End Set
    End Property
    ''' <summary>
    ''' Legt die Größe der Flares im Screenspace fest
    ''' </summary>
    Public WriteOnly Property FlareScale() As Single
        Set(ByVal value As Single)
            _FlareScale = New Vector3(value, CSng(value * dxEngine.BackBufferRatioFactor), 0)
        End Set
    End Property
    ''' <summary>
    ''' Legt die maxiemale Pixelanzahl im Verhältnis zur Bildschirmauflösung in % fest.
    ''' Der Wert kann beim Rendern der Info abgelesen werden. Dazu muss sich die Sonne im Center befinden.
    ''' </summary>
    Public WriteOnly Property OcclusionTestPixel() As Single
        Set(ByVal value As Single)
            _maxPixel = CInt(dxEngine.BackBufferSize.X * dxEngine.BackBufferSize.Y * value * 0.01)
        End Set
    End Property

    Public ReadOnly Property InfoText() As String
        Get
            Return vbCrLf & _
                   "Sun FlareVc" & vbCrLf & _
                   "Sun Occl.Pix" & vbCrLf & _
                   "Sun Occl.Test" & vbCrLf & _
                   "Sun Occl.Visible" & vbCrLf
        End Get
    End Property

    Public ReadOnly Property InfoData() As String
        Get
            Dim screenpix As Integer = dxEngine.BackBufferSize.X * dxEngine.BackBufferSize.Y
            Dim pro As Single = CSng(Math.Round(CSng((100 / screenpix) * _OcclusionPixel), 2))

            Return vbCrLf & _
                   ":  " & _FlareVec.ToString & vbCrLf & _
                   ":  " & _valPixel & " Pixel  from " & _maxPixel & " Pixel" & vbCrLf & _
                   ":  " & _OcclusionPixel & " Pixel  " & pro & "% from Screen" & vbCrLf & _
                   ":  " & _valPixel & " Pixel  " & Math.Round(_OcclusionValue * 100, 2) & "%" & vbCrLf
        End Get
    End Property
#End Region
#Region "FlareVector"
    Private Function FlareVector(ByVal position As Vector3) As Vector3
        Dim v4 As Vector4
        v4.X = position.X
        v4.Y = position.Y
        v4.Z = position.Z
        v4.W = 1

        v4 = Vector4.Transform(v4, dxWorld.ViewProjection)

        FlareVector.X = v4.X / v4.W
        FlareVector.Y = v4.Y / v4.W
    End Function
#End Region
#Region "Occlusion PixelCount"
    Private Sub updateOcclusionTestData()
        '#Get LightOcclusion
        Try
            Do While _Occlusion.CheckStatus(True) = False
            Loop
            _OcclusionPixel = _Occlusion.GetData(Of Integer)(False)
            _valPixel = CInt(clamp(_OcclusionPixel, 0, _maxPixel))

            '#calc OcclusionValue
            _OcclusionValue = clamp(CSng(_valPixel * 1 / _maxPixel), 0, 1)
        Catch ex As Exception
        End Try
    End Sub

#End Region
#Region "## Render ##"
    '''<summary>
    '''Rendert die Sonne als alphablend Quad und berechnet einen Occlusionwert für den FlareEffect 
    '''</summary>
    Public Sub DrawSun(Optional ByVal OcclusionCulling As Boolean = True)

        _SunPos = Vector3.Add(_position, dxCamera.currentPosition)
        _FlareVec = FlareVector(_SunPos)

        dxTransformWorld = dxMatrix.Billboard(_SunPos, _sunScale)
        dxDevice.VertexDeclaration = _vertexdec
        dxDevice.SetStreamSource(0, _vbuffer, 0, _vertexdecSizeOfBytes)

        '#Shader Draw Sun
        _fx.SetValue(_fxHandle(0), dxWorld.WorldViewProjection)
        _fx.SetTexture(_fxHandle(6), _texture(0))
        _fx.Technique = _fxTechnique(0)
        _fx.Begin()
        _fx.BeginPass(0)
        dxDevice.DrawPrimitives(PrimitiveType.TriangleList, 0, 1 * 2)
        _fx.EndPass()
        _fx.End()

        If OcclusionCulling Then
            updateOcclusionTestData()

            '#Shader Draw Occlusiontest
            _fx.Technique = _fxTechnique(1)
            _fx.Begin()
            _fx.BeginPass(0)
            _Occlusion.Issue(Issue.Begin)
            dxDevice.DrawPrimitives(PrimitiveType.TriangleList, 0, 1 * 2)
            _Occlusion.Issue(Issue.End)
            _fx.EndPass()
            _fx.End()
        Else
            _OcclusionValue = 1
        End If
    End Sub

    ''' <summary>
    ''' Rendert die Flares wenn der Occlusionwert größer 1 ist
    ''' </summary>
    Public Sub DrawFlaires()
        If _OcclusionValue > 0 Then
            dxDevice.VertexDeclaration = _vertexdec
            dxDevice.SetStreamSource(0, _vbuffer, 6 * _vertexdecSizeOfBytes, _vertexdecSizeOfBytes)

            '#Shader
            _fx.SetValue(_fxHandle(1), _FlareVec)
            _fx.SetValue(_fxHandle(2), _OcclusionValue)
            _fx.SetValue(_fxHandle(3), _FlareIntensity)
            _fx.SetValue(_fxHandle(4), _FlareScale)
            _fx.SetValue(_fxHandle(5), _ScreenScale)
            _fx.SetTexture(_fxHandle(6), _texture(1))
            _fx.Technique = _fxTechnique(2)

            _fx.Begin()
            _fx.BeginPass(0)
            dxDevice.DrawPrimitives(PrimitiveType.TriangleList, 0, 13 * 2)
            _fx.EndPass()
            _fx.End()
        End If
    End Sub

#End Region



End Class

