﻿
' Title    : sdxBaseEngine Class
' Name     : shaderTerrain
' Author   : Sascha Bajus
' Created  : 04/2010
' Copyright © 2010 Sascha Bajus

#Region "Copyright© Information (German)"
' Diese Software wird bereitgestellt "wie sie ist", ohne jegliche ausdrückliche oder 
' stillschweigende Gewährleistung. In keinem Fall kann der Author haftbar gemacht werden
' für Schäden, die aus der Nutzung dieser Software entstanden sind.
'
' Es ist erlaubt, diese Software für jeden Zweck zu nutzen, einschließlich der kommerziellen 
' Anwendungen, und sie zu verändern, vorbehaltlich der folgenden Einschränkungen: 
'
' 1. Der Ursprung dieser Software darf nicht falsch dargestellt werden; Sie dürfen nicht behaupten, 
'    dass Sie der Urheber der Original-Software sind. Wenn Sie diese Software in einem Produkt 

' 2. Altered-Source-Versionen müssen klar als solche gekennzeichnet, und dürfen nicht als 
'    Original-Software dargestellt werden.
'
' 3. Diese Informationen dürfen nicht entfernt oder geändert werden.
#End Region
#Region "Copyright© Information (English)"
' This software is provided 'as-is', without any express or implied warranty. In no event will 
' the authors be held liable for any damages arising from the use of this software.
'
' Permission is granted to anyone to use this software for any purpose, including commercial 
' applications, and to alter it and redistribute it freely, subject to the following restrictions:
'
' 1. The origin of this software must not be misrepresented; you must not claim that you wrote 
'    the original software. If you use this software in a product, an acknowledgment in the 
'    product documentation would be appreciated but is not required.
'
' 2. Altered source versions must be plainly marked as such, and must not be misrepresented 
'    as being the original software.
'
' 3. This notice may not be removed or altered from any source distribution.
'
#End Region




Imports SlimDX
Imports SlimDX.Direct3D9
Imports System.IO
Imports sdxBaseEngine.SDX9BaseEngine


'''<summary>
'''</summary>

Public Class shaderTerrainMesh

    Private _Model As Mesh
    Private _subsets As Integer
    Private _textureset As TextureSet

    Private _specular As New RGBAcolor(1.0)
    Private _sIntensity As Single = 0
    Private _spower As Single = 50
    Private _lightpower As Single = 1
    Private _lighttechnique As Technique = 0
    Private _level As New Vector2
    Private _repeat As New Vector4(1)

    Private _fx As Effect
    Private _fxHandle(21) As EffectHandle
    Private _fxTechnique(1) As EffectHandle


    '#Shader Technique Enum
    Public Enum Technique As Integer
        Texture = 0
        TextureNormal = 1
    End Enum

#Region "Event Handles"
    Public Sub New()
        AddHandler dxEngineClosing, AddressOf Dispose
        AddHandler dxDeviceLost, AddressOf DeviceLost
        AddHandler dxDeviceReset, AddressOf DeviceReset
        AddHandler dxLoadDeviceObjects, AddressOf Load
    End Sub
    Private Sub Load()
        createShaderHandles()
    End Sub
    Private Sub DeviceLost()
        _fx.OnLostDevice()
    End Sub
    Private Sub DeviceReset()
        _fx.OnResetDevice()
    End Sub

    Private Sub Dispose()
        If Not _Model Is Nothing Then _Model.Dispose()

        For i As Integer = 0 To _fxHandle.Length - 1
            _fxHandle(i).Dispose()
        Next
        For i As Integer = 0 To _fxTechnique.Length - 1
            _fxTechnique(i).Dispose()
        Next

        If Not _fx Is Nothing Then _fx.Dispose()
    End Sub
#End Region
#Region "load Mesh"
    '''<summary>
    '''Erstellt ein neues Mesh Objekt aus einem Xfile mit import des Materials
    '''</summary>
    '''<param name="src">Angabe des Dateipfades der Xfile Datei im ApplicationPath</param>

    Public Sub MeshFromFile(ByVal src As String)
        dxEngine.LoadingStatus = "Loading Mesh: " & src & " ..."
        src = dxEngine.ApplicationPath & src
        _Model = Mesh.FromFile(dxDevice, src, MeshFlags.SystemMemory)
        setupModelData()
    End Sub

    '#
    Public Sub MeshFromEngineResource(ByVal src As String)
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then
            With dxEngine.ResourceLoader
                _Model = Mesh.FromMemory(dxDevice, .GetResourceObject(src), MeshFlags.SystemMemory)
                dxEngine.LoadingStatus = "Loading Mesh: " & .currentResourceFile & " - " & .currentObjectName & " ..."
            End With
            setupModelData()
        End If
    End Sub


    '#as byte
    Public Sub MeshFromMemory(ByVal src As Byte())
        If dxEngine.DeviceStatus = DeviceStatus.DeviceIsOK Then
            _Model = Mesh.FromMemory(dxDevice, src, MeshFlags.SystemMemory)
            setupModelData()
        End If
    End Sub

    Private Sub setupModelData()
        Dim _materialfromFile() As ExtendedMaterial = Nothing
        _materialfromFile = _Model.GetMaterials
        _subsets = _materialfromFile.Length - 1
        computeTangent()
    End Sub
#End Region
#Region "Compute tangents of the mesh"
    Private Sub computeTangent()

        '#create new vertexdeclaration
        Dim elements(5) As VertexElement
        elements(0) = New VertexElement(0, 0, DeclarationType.Float3, DeclarationMethod.Default, DeclarationUsage.Position, 0)
        elements(1) = New VertexElement(0, 12, DeclarationType.Float3, DeclarationMethod.Default, DeclarationUsage.Normal, 0)
        elements(2) = New VertexElement(0, 24, DeclarationType.Float2, DeclarationMethod.Default, DeclarationUsage.TextureCoordinate, 0)
        elements(3) = New VertexElement(0, 32, DeclarationType.Float3, DeclarationMethod.Default, DeclarationUsage.Tangent, 0)
        elements(4) = New VertexElement(0, 44, DeclarationType.Float3, DeclarationMethod.Default, DeclarationUsage.Binormal, 0)
        elements(5) = VertexElement.VertexDeclarationEnd

        Dim tmpmesh As Mesh = _Model.Clone(dxDevice, MeshFlags.Managed, elements)
        _Model.Dispose()
        Try
            _Model = tmpmesh.ComputeTangentFrame(DeclarationUsage.TextureCoordinate, _
            0, _
            DeclarationUsage.Tangent, _
            0, _
            DeclarationUsage.Binormal, _
            0, _
            DeclarationUsage.Normal, _
             0, _
             TangentOptions.None, _
             0, _
            0, _
            0)

        Catch ex As Exception
            MsgBox(ex.ToString, MsgBoxStyle.Critical)
        End Try
        tmpmesh.Dispose()
    End Sub
#End Region
#Region "Create ShaderHandles"
    Private Sub createShaderHandles()
        '//Effect
        _fx = Effect.FromMemory(dxDevice, My.Resources.shaderDirLightTerrain, ShaderFlags.PartialPrecision)

        '#Shader Parameter Handles
        _fxHandle(0) = _fx.GetParameter(Nothing, "World")
        _fxHandle(1) = _fx.GetParameter(Nothing, "WorldViewProjection")
        _fxHandle(2) = _fx.GetParameter(Nothing, "Ambient")
        _fxHandle(3) = _fx.GetParameter(Nothing, "Diffuse")
        _fxHandle(4) = _fx.GetParameter(Nothing, "Specular")
        _fxHandle(5) = _fx.GetParameter(Nothing, "AIntensity")
        _fxHandle(6) = _fx.GetParameter(Nothing, "DIntensity")
        _fxHandle(7) = _fx.GetParameter(Nothing, "SIntensity")
        _fxHandle(8) = _fx.GetParameter(Nothing, "Sharpen")
        _fxHandle(9) = _fx.GetParameter(Nothing, "Brightness")
        _fxHandle(10) = _fx.GetParameter(Nothing, "LightDirection")
        _fxHandle(11) = _fx.GetParameter(Nothing, "CamPos")
        _fxHandle(12) = _fx.GetParameter(Nothing, "ColorMap0")
        _fxHandle(13) = _fx.GetParameter(Nothing, "ColorMap1")
        _fxHandle(14) = _fx.GetParameter(Nothing, "ColorMap2")
        _fxHandle(15) = _fx.GetParameter(Nothing, "ColorMap3")
        _fxHandle(16) = _fx.GetParameter(Nothing, "NormalMap0")
        _fxHandle(17) = _fx.GetParameter(Nothing, "NormalMap1")
        _fxHandle(18) = _fx.GetParameter(Nothing, "NormalMap2")
        _fxHandle(19) = _fx.GetParameter(Nothing, "NormalMap3")
        _fxHandle(20) = _fx.GetParameter(Nothing, "Level")
        _fxHandle(21) = _fx.GetParameter(Nothing, "Repeat")

        '//EffectTechnique
        _fxTechnique(0) = _fx.GetTechnique("Textured")
        _fxTechnique(1) = _fx.GetTechnique("TexturedNormal")

    End Sub
#End Region

#Region "Draw"
    '''<summary>
    '''Rendert das Terrainmodel mit einem LightSet
    '''</summary>

    Public Sub DrawFX(ByVal light As shaderLight.LightSet)


        _fx.SetValue(_fxHandle(0), dxWorld.World)
        _fx.SetValue(_fxHandle(1), dxWorld.WorldViewProjection)
        _fx.SetValue(_fxHandle(2), light.Ambient)
        _fx.SetValue(_fxHandle(3), light.Diffuse)
        _fx.SetValue(_fxHandle(4), _specular)
        _fx.SetValue(_fxHandle(5), light.AmbientIntensity)
        _fx.SetValue(_fxHandle(6), light.DiffuseIntensity)
        _fx.SetValue(_fxHandle(7), _sIntensity)
        _fx.SetValue(_fxHandle(8), _spower)
        _fx.SetValue(_fxHandle(9), _lightpower)
        _fx.SetValue(_fxHandle(10), light.LightDirection)
        _fx.SetValue(_fxHandle(11), dxCamera.currentPosition)
        _fx.SetTexture(_fxHandle(12), _textureset.ColorMap0)
        _fx.SetTexture(_fxHandle(13), _textureset.ColorMap1)
        _fx.SetTexture(_fxHandle(14), _textureset.ColorMap2)
        _fx.SetTexture(_fxHandle(15), _textureset.ColorMap3)
        _fx.SetTexture(_fxHandle(16), _textureset.NormalMap0)
        _fx.SetTexture(_fxHandle(17), _textureset.NormalMap1)
        _fx.SetTexture(_fxHandle(18), _textureset.NormalMap2)
        _fx.SetTexture(_fxHandle(19), _textureset.NormalMap3)
        _fx.SetValue(_fxHandle(20), _level)
        _fx.SetValue(_fxHandle(21), _repeat)

        _fx.Technique = _fxTechnique(_lighttechnique)
        _fx.Begin()

        If _sIntensity > 0 Then '#Specular or not
            _fx.BeginPass(0)
        Else
            _fx.BeginPass(1)
        End If
        For i = 0 To _subsets
            _Model.DrawSubset(i)
        Next
        _fx.EndPass()
        _fx.End()
    End Sub
    ''' <summary>
    ''' Rendert das Model per FFP
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub Draw()
        For i = 0 To _subsets
            _Model.DrawSubset(i)
        Next
    End Sub
#End Region
#Region "Property"

    Public WriteOnly Property TextureRepeatOffset() As Vector4
        Set(ByVal value As Vector4)
            _repeat = value
        End Set
    End Property

    ''' <summary>
    ''' Leght die Texturtechnik fest
    ''' </summary>
    ''' <value></value>
    Public Property TextureTechnique() As Technique
        Get
            Return _lighttechnique
        End Get
        Set(ByVal value As Technique)
            _lighttechnique = value
        End Set
    End Property

    Public Property Specular() As RGBAcolor
        Get
            Return _specular
        End Get
        Set(ByVal value As RGBAcolor)
            _specular = value
        End Set
    End Property

    Public Property SpecularIntensity() As Single
        Get
            Return _sIntensity
        End Get
        Set(ByVal value As Single)
            _sIntensity = value
        End Set
    End Property

    Public Property SpecularSharpen() As Single
        Get
            Return _spower
        End Get
        Set(ByVal value As Single)
            _spower = value
        End Set
    End Property

    Public Property Brightness() As Single
        Get
            Return _lightpower
        End Get
        Set(ByVal value As Single)
            _lightpower = value
        End Set
    End Property
    Public ReadOnly Property Subsets() As Integer
        Get
            Return _subsets
        End Get
    End Property
    ''' <summary>
    ''' Legt das Textureset fest
    ''' </summary>
    Public Property Textures() As TextureSet
        Get
            Return _textureset
        End Get
        Set(ByVal value As TextureSet)
            _textureset = value
        End Set
    End Property

    ''' <summary>
    ''' Legt das Höhenlevel fest.
    ''' Wechsel zwischen Level0 und Level1 und Wechsel zwischen Level1 und Level2
    ''' </summary>
    Public WriteOnly Property Level() As Vector2
        Set(ByVal value As Vector2)
            _level = value
        End Set
    End Property
#End Region



#Region "Class TextureSet"
    ''' <summary>
    ''' </summary>
    ''' <remarks></remarks>
    Public Class TextureSet


        Private _colormap(3) As Texture
        Private _normalmap(3) As Texture

        Public Sub New()
            AddHandler dxEngineClosing, AddressOf Dispose
            AddHandler dxDeviceLost, AddressOf DeviceLost
        End Sub

        Public Property ColorMap0() As Texture
            Get
                Return _colormap(0)
            End Get
            Set(ByVal value As Texture)
                _colormap(0) = value
            End Set
        End Property
        Public Property ColorMap1() As Texture
            Get
                Return _colormap(1)
            End Get
            Set(ByVal value As Texture)
                _colormap(1) = value
            End Set
        End Property
        Public Property ColorMap2() As Texture
            Get
                Return _colormap(2)
            End Get
            Set(ByVal value As Texture)
                _colormap(2) = value
            End Set
        End Property
        Public Property ColorMap3() As Texture
            Get
                Return _colormap(3)
            End Get
            Set(ByVal value As Texture)
                _colormap(3) = value
            End Set
        End Property

        Public Property NormalMap0() As Texture
            Get
                Return _normalmap(0)
            End Get
            Set(ByVal value As Texture)
                _normalmap(0) = value
            End Set
        End Property
        Public Property NormalMap1() As Texture
            Get
                Return _normalmap(1)
            End Get
            Set(ByVal value As Texture)
                _normalmap(1) = value
            End Set
        End Property
        Public Property NormalMap2() As Texture
            Get
                Return _normalmap(2)
            End Get
            Set(ByVal value As Texture)
                _normalmap(2) = value
            End Set
        End Property
        Public Property NormalMap3() As Texture
            Get
                Return _normalmap(3)
            End Get
            Set(ByVal value As Texture)
                _normalmap(3) = value
            End Set
        End Property

        Private Sub Dispose()
            If Not _colormap(0) Is Nothing Then _colormap(0).Dispose()
            If Not _colormap(1) Is Nothing Then _colormap(1).Dispose()
            If Not _colormap(2) Is Nothing Then _colormap(2).Dispose()
            If Not _colormap(3) Is Nothing Then _colormap(3).Dispose()

            If Not _normalmap(0) Is Nothing Then _normalmap(0).Dispose()
            If Not _normalmap(1) Is Nothing Then _normalmap(1).Dispose()
            If Not _normalmap(2) Is Nothing Then _normalmap(2).Dispose()
            If Not _normalmap(3) Is Nothing Then _normalmap(3).Dispose()
        End Sub
        Private Sub DeviceLost()
            If Not _colormap(0) Is Nothing Then _colormap(0).Dispose()
            If Not _colormap(1) Is Nothing Then _colormap(1).Dispose()
            If Not _colormap(2) Is Nothing Then _colormap(2).Dispose()
            If Not _colormap(3) Is Nothing Then _colormap(3).Dispose()

            If Not _normalmap(0) Is Nothing Then _normalmap(0).Dispose()
            If Not _normalmap(1) Is Nothing Then _normalmap(1).Dispose()
            If Not _normalmap(2) Is Nothing Then _normalmap(2).Dispose()
            If Not _normalmap(3) Is Nothing Then _normalmap(3).Dispose()
        End Sub
    End Class
#End Region


End Class





