﻿
Imports SlimDX
Imports SlimDX.Direct3D9
Imports sdxBaseEngine
Imports sdxBaseEngine.SDX9BaseEngine



Public Class shaderVegetation
    Private Structure Geometry
        Public TriangleVeg As Integer
        Public TriangleLowQuality As Integer
        Public TriangleGrass As Integer
    End Structure




    Private _vbVegetation As VertexBuffer
    Private _vbGrass As VertexBuffer
    Private _vbLowQuality As VertexBuffer
    Private _vertex As New List(Of BillboardVegetationVertex)
    Private _stream As DataStream = Nothing

    Private _bufferVegetation As Boolean
    Private _bufferGrass As Boolean
    Private _bufferLowQuality As Boolean

    Private _vertexdecSizeOfBytes As Integer
    Private _vertexdec As VertexDeclaration     '#VertexDeclaration

    Private _fx As Effect
    Private _fxHandle(5) As EffectHandle
    Private _fxTechnique(2) As EffectHandle
    Private _windforce As Single = 0.3
    Private _colormap As Texture
    Private _lightdirection As Vector3
    Private _billboardDistance As Single = 30

    Private _bbGrass As New List(Of BillboardVegetation)
    Private _bbVegetation As New List(Of BillboardVegetation)
    Private _bbRenderVegetation As New List(Of BillboardVegetation)
    Private _bbRenderGrass As New List(Of BillboardVegetation)
    Private _bbRenderLowQuality As New List(Of BillboardVegetation)

    Private _geometry As Geometry
    Private _tcGrass0(23) As Vector2
    Private _tcGrass1(23) As Vector2
    Private _tcGrass2(23) As Vector2
    Private _tcGrass3(23) As Vector2
    Private _tcPlant1(11) As Vector2
    Private _tcPlant2(11) As Vector2
    Private _tcPlant3(11) As Vector2
    Private _tcPlant4(11) As Vector2
    Private _tcPlant5(11) As Vector2
    Private _tcPlant6(11) As Vector2
    Private _tcTree1(11) As Vector2
    Private _tcTree2(11) As Vector2
    Private _tcTree3(11) As Vector2
    Private _tcTree4(11) As Vector2
    Private _tcPalm1(11) As Vector2
    Private _tcPalm2(11) As Vector2
    Private _tcPalm3(11) As Vector2
    Private _tcPalm4(11) As Vector2

    Private _tcQuad23(23) As Vector4
    Private _tcQuad11(11) As Vector4
    Private _tcQuad5(5) As Vector4

    Private _rad As Single = CSng(Math.Sqrt(2) * 0.5)      '#Radius for FrustumCulling
    Private text As String

    Private _camOldTarget As New Vector3(0, 20000, 0)
    Private _camDistance As Single

    Private vd5(5) As BillboardVegetationVertex
    Private vd11(11) As BillboardVegetationVertex
    Private vd23(23) As BillboardVegetationVertex


#Region "Event Handles"
    Public Sub New()
        AddHandler dxLoadDeviceObjects, AddressOf LoadDeviceObjects
        AddHandler dxEngineClosing, AddressOf Dispose
        AddHandler dxDeviceLost, AddressOf DeviceLost
        AddHandler dxDeviceReset, AddressOf DeviceReset
    End Sub

    Private Sub LoadDeviceObjects()
        Randomize()
        dxEngine.LoadingStatus = "Loading Shader: shaderVegetation - TexCoords..."
        createTexcoords()
        dxEngine.LoadingStatus = "Loading Shader: shaderVegetation - Vertexdeclaration..."
        createVertexdeclaration()
        dxEngine.LoadingStatus = "Loading Shader: shaderVegetation - ShaderHandles..."
        createShaderHandles()
    End Sub

    Private Sub DeviceLost()
        _fx.OnLostDevice()
        If Not _colormap Is Nothing Then _colormap.Dispose()
        If Not _vbGrass Is Nothing Then _vbGrass.Dispose()
        If Not _vbLowQuality Is Nothing Then _vbLowQuality.Dispose()
        If Not _vbVegetation Is Nothing Then _vbVegetation.Dispose()
    End Sub
    Private Sub DeviceReset()
        _fx.OnResetDevice()
        _camOldTarget = New Vector3(0, 20000, 0)
        VBuffersCreate()
    End Sub
    Private Sub Dispose()
        For i As Integer = 0 To _fxHandle.Length - 1
            If Not _fxHandle(i) Is Nothing Then _fxHandle(i).Dispose()
        Next
        For i As Integer = 0 To _fxTechnique.Length - 1
            If Not _fxTechnique(i) Is Nothing Then _fxTechnique(i).Dispose()
        Next

        If Not _colormap Is Nothing Then _colormap.Dispose()
        If Not _vertexdec Is Nothing Then _vertexdec.Dispose()
        If Not _vbGrass Is Nothing Then _vbGrass.Dispose()
        If Not _vbLowQuality Is Nothing Then _vbLowQuality.Dispose()
        If Not _vbVegetation Is Nothing Then _vbVegetation.Dispose()
        If Not _stream Is Nothing Then _stream.Dispose()
        If Not _fx Is Nothing Then _fx.Dispose()
    End Sub
#End Region
#Region "createTexcoords"
    Private Sub createTexcoords()
        tcGrass0()
        tcGrass1()
        tcGrass2()
        tcGrass3()
        tcPlant1()
        tcPlant2()
        tcPlant3()
        tcPlant4()
        tcPlant5()
        tcPlant6()
        tcPalm1()
        tcPalm2()
        tcPalm3()
        tcPalm4()
        tcTree1()
        tcTree2()
        tcTree3()
        tcTree4()
        tcQuad23()
        tcQuad11()
        tcQuad5()
    End Sub
    Private Sub tcGrass0()
        _tcGrass0(0) = New Vector2(0, 0)
        _tcGrass0(1) = New Vector2(0.25, 0)
        _tcGrass0(2) = New Vector2(0, 0.125)
        _tcGrass0(3) = _tcGrass0(1)
        _tcGrass0(4) = New Vector2(0.25, 0.125)
        _tcGrass0(5) = _tcGrass0(2)

        _tcGrass0(6) = New Vector2(0.25, 0)
        _tcGrass0(7) = New Vector2(0.5, 0)
        _tcGrass0(8) = New Vector2(0.25, 0.125)
        _tcGrass0(9) = _tcGrass0(7)
        _tcGrass0(10) = New Vector2(0.5, 0.125)
        _tcGrass0(11) = _tcGrass0(8)

        _tcGrass0(12) = New Vector2(0, 0.125)
        _tcGrass0(13) = New Vector2(0.25, 0.125)
        _tcGrass0(14) = New Vector2(0, 0.25)
        _tcGrass0(15) = _tcGrass0(13)
        _tcGrass0(16) = New Vector2(0.25, 0.25)
        _tcGrass0(17) = _tcGrass0(14)

        _tcGrass0(18) = New Vector2(0.25, 0.125)
        _tcGrass0(19) = New Vector2(0.5, 0.125)
        _tcGrass0(20) = New Vector2(0.25, 0.25)
        _tcGrass0(21) = _tcGrass0(19)
        _tcGrass0(22) = New Vector2(0.5, 0.25)
        _tcGrass0(23) = _tcGrass0(20)
    End Sub
    Private Sub tcGrass1()
        _tcGrass1(0) = New Vector2(0.25, 0)
        _tcGrass1(1) = New Vector2(0.5, 0)
        _tcGrass1(2) = New Vector2(0.25, 0.125)
        _tcGrass1(3) = _tcGrass1(1)
        _tcGrass1(4) = New Vector2(0.5, 0.125)
        _tcGrass1(5) = _tcGrass1(2)

        _tcGrass1(6) = New Vector2(0, 0.125)
        _tcGrass1(7) = New Vector2(0.25, 0.125)
        _tcGrass1(8) = New Vector2(0, 0.25)
        _tcGrass1(9) = _tcGrass1(7)
        _tcGrass1(10) = New Vector2(0.25, 0.25)
        _tcGrass1(11) = _tcGrass1(8)

        _tcGrass1(12) = New Vector2(0.25, 0.125)
        _tcGrass1(13) = New Vector2(0.5, 0.125)
        _tcGrass1(14) = New Vector2(0.25, 0.25)
        _tcGrass1(15) = _tcGrass1(13)
        _tcGrass1(16) = New Vector2(0.5, 0.25)
        _tcGrass1(17) = _tcGrass1(14)

        _tcGrass1(18) = New Vector2(0, 0)
        _tcGrass1(19) = New Vector2(0.25, 0)
        _tcGrass1(20) = New Vector2(0, 0.125)
        _tcGrass1(21) = _tcGrass1(19)
        _tcGrass1(22) = New Vector2(0.25, 0.125)
        _tcGrass1(23) = _tcGrass1(20)
    End Sub
    Private Sub tcGrass2()
        _tcGrass2(0) = New Vector2(0, 0.125)
        _tcGrass2(1) = New Vector2(0.25, 0.125)
        _tcGrass2(2) = New Vector2(0, 0.25)
        _tcGrass2(3) = _tcGrass2(1)
        _tcGrass2(4) = New Vector2(0.25, 0.25)
        _tcGrass2(5) = _tcGrass2(2)

        _tcGrass2(6) = New Vector2(0.25, 0.125)
        _tcGrass2(7) = New Vector2(0.5, 0.125)
        _tcGrass2(8) = New Vector2(0.25, 0.25)
        _tcGrass2(9) = _tcGrass2(7)
        _tcGrass2(10) = New Vector2(0.5, 0.25)
        _tcGrass2(11) = _tcGrass2(8)

        _tcGrass2(12) = New Vector2(0, 0)
        _tcGrass2(13) = New Vector2(0.25, 0)
        _tcGrass2(14) = New Vector2(0, 0.125)
        _tcGrass2(15) = _tcGrass2(13)
        _tcGrass2(16) = New Vector2(0.25, 0.125)
        _tcGrass2(17) = _tcGrass2(14)

        _tcGrass2(18) = New Vector2(0.25, 0)
        _tcGrass2(19) = New Vector2(0.5, 0)
        _tcGrass2(20) = New Vector2(0.25, 0.125)
        _tcGrass2(21) = _tcGrass2(19)
        _tcGrass2(22) = New Vector2(0.5, 0.125)
        _tcGrass2(23) = _tcGrass2(20)
    End Sub
    Private Sub tcGrass3()
        _tcGrass3(0) = New Vector2(0.25, 0.125)
        _tcGrass3(1) = New Vector2(0.5, 0.125)
        _tcGrass3(2) = New Vector2(0.25, 0.25)
        _tcGrass3(3) = _tcGrass3(1)
        _tcGrass3(4) = New Vector2(0.5, 0.25)
        _tcGrass3(5) = _tcGrass3(2)

        _tcGrass3(6) = New Vector2(0, 0)
        _tcGrass3(7) = New Vector2(0.25, 0)
        _tcGrass3(8) = New Vector2(0, 0.125)
        _tcGrass3(9) = _tcGrass3(7)
        _tcGrass3(10) = New Vector2(0.25, 0.125)
        _tcGrass3(11) = _tcGrass3(8)

        _tcGrass3(12) = New Vector2(0.25, 0)
        _tcGrass3(13) = New Vector2(0.5, 0)
        _tcGrass3(14) = New Vector2(0.25, 0.125)
        _tcGrass3(15) = _tcGrass3(13)
        _tcGrass3(16) = New Vector2(0.5, 0.125)
        _tcGrass3(17) = _tcGrass3(14)

        _tcGrass3(18) = New Vector2(0, 0.125)
        _tcGrass3(19) = New Vector2(0.25, 0.125)
        _tcGrass3(20) = New Vector2(0, 0.25)
        _tcGrass3(21) = _tcGrass3(19)
        _tcGrass3(22) = New Vector2(0.25, 0.25)
        _tcGrass3(23) = _tcGrass3(20)
    End Sub
    Private Sub tcPlant1()
        _tcPlant1(0) = New Vector2(0.5, 0)
        _tcPlant1(1) = New Vector2(0.75, 0)
        _tcPlant1(2) = New Vector2(0.5, 0.25)
        _tcPlant1(3) = _tcPlant1(1)
        _tcPlant1(4) = New Vector2(0.75, 0.25)
        _tcPlant1(5) = _tcPlant1(2)

        _tcPlant1(6) = _tcPlant1(0)
        _tcPlant1(7) = _tcPlant1(1)
        _tcPlant1(8) = _tcPlant1(2)
        _tcPlant1(9) = _tcPlant1(3)
        _tcPlant1(10) = _tcPlant1(4)
        _tcPlant1(11) = _tcPlant1(5)
    End Sub
    Private Sub tcPlant2()
        _tcPlant2(0) = New Vector2(0.75, 0)
        _tcPlant2(1) = New Vector2(1, 0)
        _tcPlant2(2) = New Vector2(0.75, 0.25)
        _tcPlant2(3) = _tcPlant2(1)
        _tcPlant2(4) = New Vector2(1, 0.25)
        _tcPlant2(5) = _tcPlant2(2)

        _tcPlant2(6) = _tcPlant2(0)
        _tcPlant2(7) = _tcPlant2(1)
        _tcPlant2(8) = _tcPlant2(2)
        _tcPlant2(9) = _tcPlant2(3)
        _tcPlant2(10) = _tcPlant2(4)
        _tcPlant2(11) = _tcPlant2(5)
    End Sub
    Private Sub tcPlant3()
        _tcPlant3(0) = New Vector2(0, 0.25)
        _tcPlant3(1) = New Vector2(0.25, 0.25)
        _tcPlant3(2) = New Vector2(0, 0.5)
        _tcPlant3(3) = _tcPlant3(1)
        _tcPlant3(4) = New Vector2(0.25, 0.5)
        _tcPlant3(5) = _tcPlant3(2)

        _tcPlant3(6) = _tcPlant3(0)
        _tcPlant3(7) = _tcPlant3(1)
        _tcPlant3(8) = _tcPlant3(2)
        _tcPlant3(9) = _tcPlant3(3)
        _tcPlant3(10) = _tcPlant3(4)
        _tcPlant3(11) = _tcPlant3(5)
    End Sub
    Private Sub tcPlant4()
        _tcPlant4(0) = New Vector2(0.25, 0.25)
        _tcPlant4(1) = New Vector2(0.5, 0.25)
        _tcPlant4(2) = New Vector2(0.25, 0.5)
        _tcPlant4(3) = _tcPlant4(1)
        _tcPlant4(4) = New Vector2(0.5, 0.5)
        _tcPlant4(5) = _tcPlant4(2)

        _tcPlant4(6) = _tcPlant4(0)
        _tcPlant4(7) = _tcPlant4(1)
        _tcPlant4(8) = _tcPlant4(2)
        _tcPlant4(9) = _tcPlant4(3)
        _tcPlant4(10) = _tcPlant4(4)
        _tcPlant4(11) = _tcPlant4(5)
    End Sub
    Private Sub tcPlant5()
        _tcPlant5(0) = New Vector2(0.5, 0.25)
        _tcPlant5(1) = New Vector2(0.75, 0.25)
        _tcPlant5(2) = New Vector2(0.5, 0.5)
        _tcPlant5(3) = _tcPlant5(1)
        _tcPlant5(4) = New Vector2(0.75, 0.5)
        _tcPlant5(5) = _tcPlant5(2)

        _tcPlant5(6) = _tcPlant5(0)
        _tcPlant5(7) = _tcPlant5(1)
        _tcPlant5(8) = _tcPlant5(2)
        _tcPlant5(9) = _tcPlant5(3)
        _tcPlant5(10) = _tcPlant5(4)
        _tcPlant5(11) = _tcPlant5(5)
    End Sub
    Private Sub tcPlant6()
        _tcPlant6(0) = New Vector2(0.75, 0.25)
        _tcPlant6(1) = New Vector2(1, 0.25)
        _tcPlant6(2) = New Vector2(0.75, 0.5)
        _tcPlant6(3) = _tcPlant6(1)
        _tcPlant6(4) = New Vector2(1, 0.5)
        _tcPlant6(5) = _tcPlant6(2)

        _tcPlant6(6) = _tcPlant6(0)
        _tcPlant6(7) = _tcPlant6(1)
        _tcPlant6(8) = _tcPlant6(2)
        _tcPlant6(9) = _tcPlant6(3)
        _tcPlant6(10) = _tcPlant6(4)
        _tcPlant6(11) = _tcPlant6(5)
    End Sub
    Private Sub tcTree1()
        _tcTree1(0) = New Vector2(0, 0.5)
        _tcTree1(1) = New Vector2(0.25, 0.5)
        _tcTree1(2) = New Vector2(0, 0.75)
        _tcTree1(3) = _tcTree1(1)
        _tcTree1(4) = New Vector2(0.25, 0.75)
        _tcTree1(5) = _tcTree1(2)

        _tcTree1(6) = _tcTree1(0)
        _tcTree1(7) = _tcTree1(1)
        _tcTree1(8) = _tcTree1(2)
        _tcTree1(9) = _tcTree1(3)
        _tcTree1(10) = _tcTree1(4)
        _tcTree1(11) = _tcTree1(5)
    End Sub
    Private Sub tcTree2()
        _tcTree2(0) = New Vector2(0.25, 0.5)
        _tcTree2(1) = New Vector2(0.5, 0.5)
        _tcTree2(2) = New Vector2(0.25, 0.75)
        _tcTree2(3) = _tcTree2(1)
        _tcTree2(4) = New Vector2(0.5, 0.75)
        _tcTree2(5) = _tcTree2(2)

        _tcTree2(6) = _tcTree2(0)
        _tcTree2(7) = _tcTree2(1)
        _tcTree2(8) = _tcTree2(2)
        _tcTree2(9) = _tcTree2(3)
        _tcTree2(10) = _tcTree2(4)
        _tcTree2(11) = _tcTree2(5)
    End Sub
    Private Sub tcTree3()
        _tcTree3(0) = New Vector2(0.5, 0.5)
        _tcTree3(1) = New Vector2(0.75, 0.5)
        _tcTree3(2) = New Vector2(0.5, 0.75)
        _tcTree3(3) = _tcTree3(1)
        _tcTree3(4) = New Vector2(0.75, 0.75)
        _tcTree3(5) = _tcTree3(2)

        _tcTree3(6) = _tcTree3(0)
        _tcTree3(7) = _tcTree3(1)
        _tcTree3(8) = _tcTree3(2)
        _tcTree3(9) = _tcTree3(3)
        _tcTree3(10) = _tcTree3(4)
        _tcTree3(11) = _tcTree3(5)
    End Sub
    Private Sub tcTree4()
        _tcTree4(0) = New Vector2(0.75, 0.5)
        _tcTree4(1) = New Vector2(1, 0.5)
        _tcTree4(2) = New Vector2(0.75, 0.75)
        _tcTree4(3) = _tcTree4(1)
        _tcTree4(4) = New Vector2(1, 0.75)
        _tcTree4(5) = _tcTree4(2)

        _tcTree4(6) = _tcTree4(0)
        _tcTree4(7) = _tcTree4(1)
        _tcTree4(8) = _tcTree4(2)
        _tcTree4(9) = _tcTree4(3)
        _tcTree4(10) = _tcTree4(4)
        _tcTree4(11) = _tcTree4(5)
    End Sub
    Private Sub tcPalm1()
        _tcPalm1(0) = New Vector2(0, 0.75)
        _tcPalm1(1) = New Vector2(0.25, 0.75)
        _tcPalm1(2) = New Vector2(0, 1)
        _tcPalm1(3) = _tcPalm1(1)
        _tcPalm1(4) = New Vector2(0.25, 1)
        _tcPalm1(5) = _tcPalm1(2)

        _tcPalm1(6) = _tcPalm1(0)
        _tcPalm1(7) = _tcPalm1(1)
        _tcPalm1(8) = _tcPalm1(2)
        _tcPalm1(9) = _tcPalm1(3)
        _tcPalm1(10) = _tcPalm1(4)
        _tcPalm1(11) = _tcPalm1(5)
    End Sub
    Private Sub tcPalm2()
        _tcPalm2(0) = New Vector2(0.25, 0.75)
        _tcPalm2(1) = New Vector2(0.5, 0.75)
        _tcPalm2(2) = New Vector2(0.25, 1)
        _tcPalm2(3) = _tcPalm2(1)
        _tcPalm2(4) = New Vector2(0.5, 1)
        _tcPalm2(5) = _tcPalm2(2)

        _tcPalm2(6) = _tcPalm2(0)
        _tcPalm2(7) = _tcPalm2(1)
        _tcPalm2(8) = _tcPalm2(2)
        _tcPalm2(9) = _tcPalm2(3)
        _tcPalm2(10) = _tcPalm2(4)
        _tcPalm2(11) = _tcPalm2(5)
    End Sub
    Private Sub tcPalm3()
        _tcPalm3(0) = New Vector2(0.5, 0.75)
        _tcPalm3(1) = New Vector2(0.75, 0.75)
        _tcPalm3(2) = New Vector2(0.5, 1)
        _tcPalm3(3) = _tcPalm3(1)
        _tcPalm3(4) = New Vector2(0.75, 1)
        _tcPalm3(5) = _tcPalm3(2)

        _tcPalm3(6) = _tcPalm3(0)
        _tcPalm3(7) = _tcPalm3(1)
        _tcPalm3(8) = _tcPalm3(2)
        _tcPalm3(9) = _tcPalm3(3)
        _tcPalm3(10) = _tcPalm3(4)
        _tcPalm3(11) = _tcPalm3(5)
    End Sub
    Private Sub tcPalm4()
        _tcPalm4(0) = New Vector2(0.75, 0.75)
        _tcPalm4(1) = New Vector2(1, 0.75)
        _tcPalm4(2) = New Vector2(0.75, 1)
        _tcPalm4(3) = _tcPalm4(1)
        _tcPalm4(4) = New Vector2(1, 1)
        _tcPalm4(5) = _tcPalm4(2)

        _tcPalm4(6) = _tcPalm4(0)
        _tcPalm4(7) = _tcPalm4(1)
        _tcPalm4(8) = _tcPalm4(2)
        _tcPalm4(9) = _tcPalm4(3)
        _tcPalm4(10) = _tcPalm4(4)
        _tcPalm4(11) = _tcPalm4(5)
    End Sub

    Private Sub tcQuad23()
        _tcQuad23(0) = New Vector4(0, 0, 0, 0)
        _tcQuad23(1) = New Vector4(1, 0, 0, 0)
        _tcQuad23(2) = New Vector4(0, 1, 0, 0)
        _tcQuad23(3) = _tcQuad23(1)
        _tcQuad23(4) = New Vector4(1, 1, 0, 0)
        _tcQuad23(5) = _tcQuad23(2)

        _tcQuad23(6) = New Vector4(0, 0, 1, 0)
        _tcQuad23(7) = New Vector4(1, 0, 1, 0)
        _tcQuad23(8) = New Vector4(0, 1, 1, 0)
        _tcQuad23(9) = _tcQuad23(7)
        _tcQuad23(10) = New Vector4(1, 1, 1, 0)
        _tcQuad23(11) = _tcQuad23(8)

        _tcQuad23(12) = New Vector4(0, 0, 2, 0)
        _tcQuad23(13) = New Vector4(1, 0, 2, 0)
        _tcQuad23(14) = New Vector4(0, 1, 2, 0)
        _tcQuad23(15) = _tcQuad23(13)
        _tcQuad23(16) = New Vector4(1, 1, 2, 0)
        _tcQuad23(17) = _tcQuad23(14)

        _tcQuad23(18) = New Vector4(0, 0, 3, 0)
        _tcQuad23(19) = New Vector4(1, 0, 3, 0)
        _tcQuad23(20) = New Vector4(0, 1, 3, 0)
        _tcQuad23(21) = _tcQuad23(19)
        _tcQuad23(22) = New Vector4(1, 1, 3, 0)
        _tcQuad23(23) = _tcQuad23(20)
    End Sub
    Private Sub tcQuad11()
        _tcQuad11(0) = New Vector4(0, 0, 0, 0)
        _tcQuad11(1) = New Vector4(1, 0, 0, 0)
        _tcQuad11(2) = New Vector4(0, 1, 0, 0)
        _tcQuad11(3) = _tcQuad11(1)
        _tcQuad11(4) = New Vector4(1, 1, 0, 0)
        _tcQuad11(5) = _tcQuad11(2)
        _tcQuad11(6) = New Vector4(0, 0, 1, 0)
        _tcQuad11(7) = New Vector4(1, 0, 1, 0)
        _tcQuad11(8) = New Vector4(0, 1, 1, 0)
        _tcQuad11(9) = _tcQuad11(7)
        _tcQuad11(10) = New Vector4(1, 1, 1, 0)
        _tcQuad11(11) = _tcQuad11(8)
    End Sub
    Private Sub tcQuad5()
        _tcQuad5(0) = New Vector4(0, 0, 0, 1)
        _tcQuad5(1) = New Vector4(1, 0, 0, 1)
        _tcQuad5(2) = New Vector4(0, 1, 0, 1)
        _tcQuad5(3) = _tcQuad5(1)
        _tcQuad5(4) = New Vector4(1, 1, 0, 1)
        _tcQuad5(5) = _tcQuad5(2)
    End Sub
#End Region
#Region "Create ShaderHandles"
    Private Sub createShaderHandles()
        '//Effect
        _fx = Effect.FromMemory(dxDevice, My.Resources.shaderVegetation, ShaderFlags.PartialPrecision)

        '#Shader Parameter Handles
        _fxHandle(0) = _fx.GetParameter(Nothing, "World")
        _fxHandle(1) = _fx.GetParameter(Nothing, "ViewProjection")
        _fxHandle(2) = _fx.GetParameter(Nothing, "ColorMap")
        _fxHandle(3) = _fx.GetParameter(Nothing, "CamPos")
        _fxHandle(4) = _fx.GetParameter(Nothing, "GameTime")
        _fxHandle(5) = _fx.GetParameter(Nothing, "WindForce")

        '//EffectTechnique
        _fxTechnique(0) = _fx.GetTechnique("Grass")
        _fxTechnique(1) = _fx.GetTechnique("Vegetation")
        _fxTechnique(2) = _fx.GetTechnique("LowQuality")

    End Sub
#End Region
#Region "Create Vertexdeclaration"
    Private Sub createVertexdeclaration()
        Dim elements(4) As VertexElement
        elements(0) = New VertexElement(0, 0, DeclarationType.Float3, DeclarationMethod.Default, DeclarationUsage.Position, 0)
        elements(1) = New VertexElement(0, 12, DeclarationType.Float4, DeclarationMethod.Default, DeclarationUsage.TextureCoordinate, 0)
        elements(2) = New VertexElement(0, 28, DeclarationType.Float2, DeclarationMethod.Default, DeclarationUsage.TextureCoordinate, 1)
        elements(3) = New VertexElement(0, 36, DeclarationType.Float4, DeclarationMethod.Default, DeclarationUsage.TextureCoordinate, 2)
        elements(4) = VertexElement.VertexDeclarationEnd
        _vertexdec = New VertexDeclaration(dxDevice, elements)

        '#size of FloatValues = 4 bytes
        _vertexdecSizeOfBytes = 12 + 16 + 8 + 16
    End Sub
#End Region
#Region "Create VertexBuffers"
    Public Sub VBuffersCreate()
        dxEngine.LoadingStatus = "Loading Shader: shaderVegetation - VertexBuffer..."
        '#create vbuffer 
        If _bbVegetation.Count > 0 Then
            _vbVegetation = New VertexBuffer(dxDevice, _vertexdecSizeOfBytes * _bbVegetation.Count * 4 * 3, Usage.Dynamic Or Usage.WriteOnly, VertexFormat.None, Pool.Default)
        End If
        If _bbGrass.Count > 0 Then
            _vbGrass = New VertexBuffer(dxDevice, _vertexdecSizeOfBytes * _bbGrass.Count * 8 * 3, Usage.Dynamic Or Usage.WriteOnly, VertexFormat.None, Pool.Default)
        End If
        If _bbVegetation.Count > 0 Or _bbGrass.Count > 0 Then
            'Billboards only
            _vbLowQuality = New VertexBuffer(dxDevice, _vertexdecSizeOfBytes * (_bbGrass.Count + _bbVegetation.Count) * 2 * 3, Usage.Dynamic Or Usage.WriteOnly, VertexFormat.None, Pool.Default)
        End If
    End Sub
#End Region
#Region "Load Positions"
    Public Sub LoadPositions(ByVal source As String, ByVal type As VegetationType, Optional ByVal addPosX As Single = 0, Optional ByVal addPosY As Single = 0, Optional ByVal addPosZ As Single = 0, Optional ByVal Random As Boolean = False)
        dxEngine.LoadingStatus = "Loading shaderVegetation - Position " & type.ToString & " ..."
        Dim pos() As Vector3 = ImportPositions(source)
        For i = 0 To pos.Length - 1
            addbillboard(pos(i), type, addPosX, addPosY, addPosZ, Random)
        Next
    End Sub
    Private Sub addbillboard(ByVal pos As Vector3, ByVal type As VegetationType, ByVal addposX As Single, ByVal addposY As Single, ByVal addPosZ As Single, ByVal random As Boolean)
        Dim ScaleX As Single
        Dim ScaleY As Single
        Dim ScaleZ As Single
        Dim rndValue As Single

        rndValue = Rnd() ' Random Value
        If random Then
            addposX *= rndValue
            addPosZ *= rndValue
        End If

        Select Case CInt(type)
            Case Is = 0 'grass

                ScaleX = 1.6
                ScaleY = CSng(rndValue * 0.8 + 1.2)
                ScaleZ = 0.2
            Case Is = 15 'Random Plants

                ScaleX = CSng(rndValue + 0.8)
                ScaleY = CSng(rndValue + 1)
                ScaleZ = 0.2

            Case Is = 16 'Random Trees

                ScaleX = CSng(rndValue + 3)
                ScaleY = CSng(rndValue * 1.5 + 3.4)
                ScaleZ = 0.1

            Case Is = 17 ' Random Palms

                ScaleX = CSng(rndValue + 3)
                ScaleY = CSng(rndValue * 1.5 + 3.4)
                ScaleZ = 0.1

            Case Is = 18 ' Random jungel

                ScaleX = CSng(rndValue + 3.2)
                ScaleY = CSng(rndValue * 1.6 + 3.7)
                ScaleZ = 0.1

            Case Is > 10 'Palm

                ScaleX = CSng(rndValue + 3)
                ScaleY = CSng(rndValue * 1.5 + 3.7)
                ScaleZ = 0.1

            Case Is > 6 'Tree

                ScaleX = CSng(rndValue + 2.8)
                ScaleY = CSng(rndValue * 1.5 + 3.5)
                ScaleZ = 0.1

            Case Is > 0 'Plants

                ScaleX = 1.6
                ScaleY = CSng(rndValue * 0.6 + 2)
                ScaleZ = 0.2

        End Select

        If type = VegetationType.Grass Then
            _bbGrass.Add(New BillboardVegetation(pos.X + addposX, pos.Y + addposY, pos.Z + addPosZ, ScaleX, ScaleY, ScaleZ, clamp(rndValue, 0.5, 1), Rnd, type))
        Else
            _bbVegetation.Add(New BillboardVegetation(pos.X + addposX, pos.Y + addposY, pos.Z + addPosZ, ScaleX, ScaleY, ScaleZ, clamp(rndValue, 0.5, 1), Rnd, type))
        End If
    End Sub
#End Region
#Region "Properties"
    Public WriteOnly Property ColorMap() As Texture
        Set(ByVal value As Texture)
            _colormap = value
        End Set
    End Property

    Public WriteOnly Property LightDirection() As Vector3
        Set(ByVal value As Vector3)
            _lightdirection = value
        End Set
    End Property
    Public ReadOnly Property InfoText() As String
        Get
            Return vbCrLf & _
                   "Veg Triangles" & vbCrLf & _
                   "Veg Obejcts"
        End Get
    End Property
    Public ReadOnly Property InfoData() As String
        Get
            Dim c As Integer = _bbGrass.Count * 8 + _bbVegetation.Count * 4
            Dim r As Integer = _geometry.TriangleGrass + _geometry.TriangleVeg
            Return vbCrLf & _
                       ":  " & r & "/" & c & vbCrLf & _
                       ":  " & _bbRenderGrass.Count + _bbRenderVegetation.Count & "/" & _bbGrass.Count + _bbVegetation.Count
        End Get
    End Property
#End Region

#Region "FrustumCulling"
    Private Sub FrustumCullingGrass(ByVal Item As BillboardVegetation)
        With dxCamera
            '# Test center position by nearplane, kills all behind
            If .FrustumTestNear(Item.FrustumCenterBottom, Item.FrustumRad) = False Then Exit Sub

            If .FrustumTestLeft(Item.FrustumCenterBottom, Item.FrustumRad) = False Then Exit Sub
            If .FrustumTestRight(Item.FrustumCenterBottom, Item.FrustumRad) = False Then Exit Sub

            If .FrustumTestBottom(Item.FrustumCenterTop, Item.FrustumRad) = False Then Exit Sub
            'If .FrustumTestTop(Item.FrustumCenterBottom, Item.FrustumRad) = False Then Exit Sub
        End With

        Item.IsBillboard = True
        _bbRenderLowQuality.Add(Item)
        _geometry.TriangleLowQuality += 2

        Item.IsBillboard = False
        _bbRenderGrass.Add(Item)
        _geometry.TriangleGrass += 8
    End Sub

    Private Sub FrustumCullingVegetation(ByVal Item As BillboardVegetation)
        With dxCamera
            If .FrustumTestNear(Item.FrustumCenterBottom, Item.FrustumRad) = False Then Exit Sub

            If .FrustumTestLeft(Item.FrustumCenterBottom, Item.FrustumRad) = False Then Exit Sub
            If .FrustumTestRight(Item.FrustumCenterBottom, Item.FrustumRad) = False Then Exit Sub

            If .FrustumTestBottom(Item.FrustumCenterTop, Item.FrustumRad) = False Then Exit Sub

        End With

        Item.IsBillboard = True
        _bbRenderLowQuality.Add(Item)
        _geometry.TriangleLowQuality += 2

        'Distance Test for Billboarding
        If dxCamera.FrustumTestFarBillboard(Item.FrustumCenterBottom) Then
            Item.IsBillboard = False
            _geometry.TriangleVeg += 4
        Else
            Item.IsBillboard = True
            _geometry.TriangleVeg += 2
        End If
        _bbRenderVegetation.Add(Item)
    End Sub
#End Region
#Region "Function getVertexData"
    Private Function getVertexData(ByVal billboard As BillboardVegetation) As BillboardVegetationVertex()
        With billboard
            Dim tc() As Vector2 = Nothing

            If .Type = VegetationType.Grass Then

                Select Case .RndVal
                    Case Is > 0.75
                        tc = _tcGrass0
                    Case Is > 0.5
                        tc = _tcGrass1
                    Case Is > 0.25
                        tc = _tcGrass2
                    Case Else
                        tc = _tcGrass3
                End Select

                If .IsBillboard Then
                    For i = 0 To 5
                        vd5.SetValue(.VertexStruxture, i)
                        vd5(i).TexCoords0 = _tcQuad5(i)
                        vd5(i).TexCoords1 = tc(i)
                    Next
                    vd5(0).Scaling.W = .InvSclW
                    Return vd5
                Else
                    For i = 0 To 23
                        vd23.SetValue(.VertexStruxture, i)
                        vd23(i).TexCoords0 = _tcQuad23(i)
                        vd23(i).TexCoords1 = tc(i)
                    Next

                    vd23(0).Scaling.W = .InvSclW
                    vd23(6).Scaling.W = .InvSclW
                    vd23(12).Scaling.W = .InvSclW
                    vd23(18).Scaling.W = .InvSclW
                    Return vd23
                End If
            Else

                Select Case .Type
                    Case VegetationType.Palm1
                        tc = _tcPalm1
                    Case VegetationType.Palm2
                        tc = _tcPalm2
                    Case VegetationType.Palm3
                        tc = _tcPalm3
                    Case VegetationType.Palm4
                        tc = _tcPalm4
                    Case VegetationType.Plant1
                        tc = _tcPlant1
                    Case VegetationType.Plant2
                        tc = _tcPlant2
                    Case VegetationType.Plant3
                        tc = _tcPlant3
                    Case VegetationType.Plant4
                        tc = _tcPlant4
                    Case VegetationType.Plant5
                        tc = _tcPlant5
                    Case VegetationType.Plant6
                        tc = _tcPlant6
                    Case VegetationType.Tree1
                        tc = _tcTree1
                    Case VegetationType.Tree2
                        tc = _tcTree2
                    Case VegetationType.Tree3
                        tc = _tcTree3
                    Case VegetationType.Tree4
                        tc = _tcTree4
                    Case VegetationType.RandomPalm
                        Select Case .RndVal
                            Case Is > 0.75
                                tc = _tcPalm1
                            Case Is > 0.5
                                tc = _tcPalm2
                            Case Is > 0.25
                                tc = _tcPalm3
                            Case Is > 0
                                tc = _tcPalm4
                        End Select
                    Case VegetationType.RandomPlant
                        Select Case .RndVal
                            Case Is > 0.75
                                tc = _tcPlant1
                            Case Is > 0.6
                                tc = _tcPlant2
                            Case Is > 0.45
                                tc = _tcPlant3
                            Case Is > 0.3
                                tc = _tcPlant4
                            Case Is > 0.15
                                tc = _tcPlant5
                            Case Is > 0
                                tc = _tcPlant6
                        End Select
                    Case VegetationType.RandomTree
                        Select Case .RndVal
                            Case Is > 0.75
                                tc = _tcTree1
                            Case Is > 0.5
                                tc = _tcTree2
                            Case Is > 0.25
                                tc = _tcTree3
                            Case Is > 0
                                tc = _tcTree4
                        End Select
                    Case VegetationType.RandomJungle
                        Select Case .RndVal
                            Case Is > 0.84
                                tc = _tcPalm1
                            Case Is > 0.72
                                tc = _tcPalm2
                            Case Is > 0.6
                                tc = _tcPalm3
                            Case Is > 0.48
                                tc = _tcPalm4
                            Case Is > 0.36
                                tc = _tcTree1
                            Case Is > 0.24
                                tc = _tcTree2
                            Case Is > 0.12
                                tc = _tcTree3
                            Case Is > 0
                                tc = _tcTree4
                        End Select
                    Case Else
                        tc = Nothing
                End Select

                If .IsBillboard Then
                    For i = 0 To 5
                        vd5.SetValue(.VertexStruxture, i)
                        vd5(i).TexCoords0 = _tcQuad5(i)
                        vd5(i).TexCoords1 = tc(i)
                    Next
                    vd5(0).Scaling.W = .InvSclW
                    Return vd5

                Else
                    For i = 0 To 11
                        vd11.SetValue(.VertexStruxture, i)
                        vd11(i).TexCoords0 = _tcQuad11(i)
                        vd11(i).TexCoords1 = tc(i)
                    Next

                    vd11(0).Scaling.W = .InvSclW
                    vd11(6).Scaling.W = .InvSclW
                    Return vd11
                End If
            End If
        End With
        Return Nothing
    End Function
#End Region
#Region "Fill RenderLists"
    Private Sub FillRenderLists()
        _bbRenderVegetation.Clear()
        _bbRenderGrass.Clear()
        _bbRenderLowQuality.Clear()
        _geometry.TriangleGrass = 0
        _geometry.TriangleLowQuality = 0
        _geometry.TriangleVeg = 0

        'get only Visible Grass
        If _bbGrass.Count > 0 Then
            For i = 0 To _bbGrass.Count - 1
                FrustumCullingGrass(_bbGrass.Item(i))
            Next
        End If

        'get only Visible Vegetation and switch by distance from CrossBoards to Billboard
        If _bbVegetation.Count > 0 Then
            For i = 0 To _bbVegetation.Count - 1
                FrustumCullingVegetation(_bbVegetation.Item(i))
            Next
        End If
    End Sub

#End Region
#Region "Fill Vertexbuffers"
    Private Sub FillVbufferGrass()

        '#GrassBuffer
        If _bbRenderGrass.Count > 0 Then
            _bbRenderGrass.Sort(New SortBillboardVegByDistance)
            _stream = _vbGrass.Lock(0, 0, LockFlags.Discard)
            For i = 0 To _bbRenderGrass.Count - 1
                _stream.WriteRange(getVertexData(_bbRenderGrass(i)))
            Next
            _vbGrass.Unlock()
            _stream.Close()
            _bufferGrass = True
        End If
    End Sub
    Private Sub FillVbufferVegetation()

        '#VegetationBuffer
        If _bbRenderVegetation.Count > 0 Then
            _bbRenderVegetation.Sort(New SortBillboardVegByDistance)
            _stream = _vbVegetation.Lock(0, 0, LockFlags.Discard)
            For i = 0 To _bbRenderVegetation.Count - 1
                _stream.WriteRange(getVertexData(_bbRenderVegetation(i)))
            Next
            _vbVegetation.Unlock()
            _stream.Close()
            _bufferVegetation = True
        End If

    End Sub
    Private Sub FillVbufferLowQuality()

        '#Low Quality Billboardbuffer for reflection
        If _bbRenderLowQuality.Count > 0 Then
            _bbRenderLowQuality.Sort(New SortBillboardVegByDistance)
            _stream = _vbLowQuality.Lock(0, 0, LockFlags.Discard)
            For i = 0 To _bbRenderLowQuality.Count - 1
                _stream.WriteRange(getVertexData(_bbRenderLowQuality(i)))
            Next
            _vbLowQuality.Unlock()
            _stream.Close()
            _bufferLowQuality = True
        End If

    End Sub
#End Region

#Region "## Render ##"
    Public Sub Refresh()
        FillRenderLists()
    End Sub

    Public Sub Draw()
        FillVbufferGrass()
        FillVbufferVegetation()

        dxTransformWorld = Matrix.Identity

        '#Shader
        _fx.SetValue(_fxHandle(0), dxWorld.World)
        _fx.SetValue(_fxHandle(1), dxWorld.ViewProjection)
        _fx.SetTexture(_fxHandle(2), _colormap)
        _fx.SetValue(_fxHandle(3), dxCamera.currentPosition)
        _fx.SetValue(_fxHandle(4), dxGametime.Gametime)
        _fx.SetValue(_fxHandle(5), _windforce)

        dxDevice.VertexDeclaration = _vertexdec

        If _bufferVegetation Then
            'reset
            _bufferVegetation = False
            _fx.Technique = _fxTechnique(1)
            _fx.Begin()
            dxDevice.SetStreamSource(0, _vbVegetation, 0, _vertexdecSizeOfBytes)
            For i = 0 To 1
                _fx.BeginPass(i)
                dxDevice.DrawPrimitives(PrimitiveType.TriangleList, 0, _geometry.TriangleVeg)
                _fx.EndPass()
            Next
            _fx.End()
        End If

        If _bufferGrass Then
            'reset
            _bufferGrass = False
            '#Shader
            _fx.SetValue(_fxHandle(0), dxWorld.World)
            _fx.SetValue(_fxHandle(1), dxWorld.ViewProjection)
            _fx.SetTexture(_fxHandle(2), _colormap)
            _fx.SetValue(_fxHandle(3), dxCamera.currentPosition)
            _fx.SetValue(_fxHandle(4), dxGametime.Gametime)
            _fx.SetValue(_fxHandle(5), _windforce)
            _fx.Technique = _fxTechnique(0)
            _fx.Begin()
            dxDevice.SetStreamSource(0, _vbGrass, 0, _vertexdecSizeOfBytes)
            _fx.BeginPass(0)
            dxDevice.DrawPrimitives(PrimitiveType.TriangleList, 0, _geometry.TriangleGrass)
            _fx.EndPass()
            _fx.End()
        End If

    End Sub

    Public Sub DrawLowQuality()
        FillVbufferLowQuality()

        If _bufferLowQuality Then
            'reset
            _bufferLowQuality = False
            dxTransformWorld = Matrix.Identity

            dxDevice.VertexDeclaration = _vertexdec
            dxDevice.SetStreamSource(0, _vbLowQuality, 0, _vertexdecSizeOfBytes)

            '#Shader
            _fx.SetValue(_fxHandle(0), dxWorld.World)
            _fx.SetValue(_fxHandle(1), dxWorld.ViewProjection)
            _fx.SetTexture(_fxHandle(2), _colormap)
            _fx.SetValue(_fxHandle(3), dxCamera.currentPosition)
            _fx.SetValue(_fxHandle(4), dxGametime.Gametime)
            _fx.SetValue(_fxHandle(5), _windforce)
            _fx.Technique = _fxTechnique(2)
            _fx.Begin()
            _fx.BeginPass(0)
            dxDevice.DrawPrimitives(PrimitiveType.TriangleList, 0, _geometry.TriangleLowQuality)
            _fx.EndPass()
            _fx.End()
        End If
    End Sub
#End Region

End Class




